@@ -1,7 +1,7 @@
 /*
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2005, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2005, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -267,7 +267,7 @@ b_workload_run_mixed_workload(dbp, config)
 	CONFIG *config;
 {
 	DBT key, data;
-	size_t next_op, i, ioff, inscount;
+	size_t next_op, i, inscount;
 	char kbuf[KBUF_LEN];
 	struct bench_q operation_queue;
 
@@ -277,7 +277,6 @@ b_workload_run_mixed_workload(dbp, config)
 	srand(config->seed);
 	memset(&operation_queue, 0, sizeof(struct bench_q));
 
-	ioff = 0;
 	INIT_KEY(key, config);
 	memset(&data, 0, sizeof(data));
 	DB_BENCH_ASSERT(
