@@ -1,9 +1,8 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file EXAMPLES-LICENSE for license information.
  *
- * $Id$
  */
 
 package collections.ship.basic;
@@ -21,7 +20,7 @@ import java.io.Serializable;
  */
 public class PartKey implements Serializable {
 
-    private String number;
+    private final String number;
 
     public PartKey(String number) {
 
@@ -33,6 +32,7 @@ public class PartKey implements Serializable {
         return number;
     }
 
+    @Override
     public String toString() {
 
         return "[PartKey: number=" + number + ']';
