@@ -1,9 +1,8 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file EXAMPLES-LICENSE for license information.
  *
- * $Id$
  */
 
 package collections.ship.entity;
@@ -20,9 +19,9 @@ package collections.ship.entity;
  */
 public class Shipment {
 
-    private String partNumber;
-    private String supplierNumber;
-    private int quantity;
+    private final String partNumber;
+    private final String supplierNumber;
+    private final int quantity;
 
     public Shipment(String partNumber, String supplierNumber, int quantity) {
 
@@ -46,6 +45,7 @@ public class Shipment {
         return quantity;
     }
 
+    @Override
     public String toString() {
 
         return "[Shipment: part=" + partNumber +
