@@ -1,9 +1,8 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file EXAMPLES-LICENSE for license information.
  *
- * $Id$
  */
 
 package collections.ship.index;
@@ -21,8 +20,8 @@ import java.io.Serializable;
  */
 public class ShipmentKey implements Serializable {
 
-    private String partNumber;
-    private String supplierNumber;
+    private final String partNumber;
+    private final String supplierNumber;
 
     public ShipmentKey(String partNumber, String supplierNumber) {
 
@@ -40,6 +39,7 @@ public class ShipmentKey implements Serializable {
         return supplierNumber;
     }
 
+    @Override
     public String toString() {
 
         return "[ShipmentKey: supplier=" + supplierNumber +
