@@ -1,9 +1,8 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file EXAMPLES-LICENSE for license information.
  *
- * $Id$
  */
 
 package collections.ship.tuple;
@@ -20,10 +19,10 @@ package collections.ship.tuple;
  */
 public class Supplier {
 
-    private String number;
-    private String name;
-    private int status;
-    private String city;
+    private final String number;
+    private final String name;
+    private final int status;
+    private final String city;
 
     public Supplier(String number, String name, int status, String city) {
 
@@ -53,6 +52,7 @@ public class Supplier {
         return city;
     }
 
+    @Override
     public String toString() {
 
         return "[Supplier: number=" + number +
