@@ -1,9 +1,8 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file EXAMPLES-LICENSE for license information.
  *
- * $Id$
  */
 
 package collections.ship.tuple;
@@ -19,7 +18,7 @@ package collections.ship.tuple;
  */
 public class SupplierKey {
 
-    private String number;
+    private final String number;
 
     public SupplierKey(String number) {
 
@@ -31,6 +30,7 @@ public class SupplierKey {
         return number;
     }
 
+    @Override
     public String toString() {
 
         return "[SupplierKey: number=" + number + ']';
