@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2009, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2009, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  */
 using System;
@@ -38,16 +38,19 @@ namespace BerkeleyDB {
         private void Config(BTreeDatabaseConfig cfg) {
             base.Config(cfg);
             /* 
-             * Database.Config calls set_flags, but that doesn't get the BTree
+             * Database.Config calls set_flags, but that does not get the BTree
              * specific flags.  No harm in calling it again.
              */
             db.set_flags(cfg.flags);
 
-            if (cfg.BlobDir != null && cfg.Env == null)
-                db.set_blob_dir(cfg.BlobDir);
+	    if (cfg.BlobDir != null && cfg.Env == null)
+                db.set_ext_file_dir(cfg.BlobDir);
+
+            if (cfg.ExternalFileDir != null && cfg.Env == null)
+                db.set_ext_file_dir(cfg.ExternalFileDir);
 
             if (cfg.blobThresholdIsSet)
-                db.set_blob_threshold(cfg.BlobThreshold, 0);
+                db.set_ext_file_threshold(cfg.BlobThreshold, 0);
 
             if (cfg.BTreeCompare != null)
                 Compare = cfg.BTreeCompare;
@@ -109,13 +112,13 @@ namespace BerkeleyDB {
         /// </para>
         /// <para>
         /// If <see cref="DatabaseConfig.AutoCommit"/> is set, the operation
-        /// will be implicitly transaction protected. Note that transactionally
+        /// is implicitly transaction protected. Transactionally
         /// protected operations on a database object requires the object itself
         /// be transactionally protected during its open.
         /// </para>
         /// </remarks>
         /// <param name="Filename">
-        /// The name of an underlying file that will be used to back the
+        /// The name of an underlying file used to back the
         /// database. In-memory databases never intended to be preserved on disk
         /// may be created by setting this parameter to null.
         /// </param>
@@ -139,18 +142,18 @@ namespace BerkeleyDB {
         /// object that created it, in circumstances where doing so is safe. If
         /// <paramref name="Filename"/> is null and
         /// <paramref name="DatabaseName"/> is non-null, the database can be
-        /// opened by other threads of control and will be replicated to client
+        /// opened by other threads of control and be replicated to client
         /// sites in any replication group.
         /// </para>
         /// <para>
         /// If <see cref="DatabaseConfig.AutoCommit"/> is set, the operation
-        /// will be implicitly transaction protected. Note that transactionally
+        /// is implicitly transaction protected. Transactionally
         /// protected operations on a database object requires the object itself
         /// be transactionally protected during its open.
         /// </para>
         /// </remarks>
         /// <param name="Filename">
-        /// The name of an underlying file that will be used to back the
+        /// The name of an underlying file that used to back the
         /// database. In-memory databases never intended to be preserved on disk
         /// may be created by setting this parameter to null.
         /// </param>
@@ -179,15 +182,15 @@ namespace BerkeleyDB {
         /// </para>
         /// <para>
         /// If <paramref name="txn"/> is null, but
-        /// <see cref="DatabaseConfig.AutoCommit"/> is set, the operation will
-        /// be implicitly transaction protected. Note that transactionally
+        /// <see cref="DatabaseConfig.AutoCommit"/> is set, the operation
+        /// is implicitly transaction protected. Transactionally
         /// protected operations on a database object requires the object itself
-        /// be transactionally protected during its open. Also note that the
+        /// be transactionally protected during its open. The
         /// transaction must be committed before the object is closed.
         /// </para>
         /// </remarks>
         /// <param name="Filename">
-        /// The name of an underlying file that will be used to back the
+        /// The name of an underlying file used to back the
         /// database. In-memory databases never intended to be preserved on disk
         /// may be created by setting this parameter to null.
         /// </param>
@@ -219,20 +222,20 @@ namespace BerkeleyDB {
         /// object that created it, in circumstances where doing so is safe. If
         /// <paramref name="Filename"/> is null and
         /// <paramref name="DatabaseName"/> is non-null, the database can be
-        /// opened by other threads of control and will be replicated to client
+        /// opened by other threads of control and be replicated to client
         /// sites in any replication group.
         /// </para>
         /// <para>
         /// If <paramref name="txn"/> is null, but
-        /// <see cref="DatabaseConfig.AutoCommit"/> is set, the operation will
-        /// be implicitly transaction protected. Note that transactionally
+        /// <see cref="DatabaseConfig.AutoCommit"/> is set, the operation
+        /// is implicitly transaction protected. Transactionally
         /// protected operations on a database object requires the object itself
-        /// be transactionally protected during its open. Also note that the
+        /// be transactionally protected during its open. The
         /// transaction must be committed before the object is closed.
         /// </para>
         /// </remarks>
         /// <param name="Filename">
-        /// The name of an underlying file that will be used to back the
+        /// The name of an underlying file used to back the
         /// database. In-memory databases never intended to be preserved on disk
         /// may be created by setting this parameter to null.
         /// </param>
@@ -361,12 +364,22 @@ namespace BerkeleyDB {
         #region Properties
         // Sorted alpha by property name
         /// <summary>
-        /// The path of the directory where blobs are stored.
+        /// The path of the directory where external files are stored.
+        /// </summary>
+        public string ExternalFileDir {
+            get {
+                string dir;
+                db.get_ext_file_dir(out dir);
+                return dir;
+            }
+        }
+        /// <summary>
+        /// Deprecated.  Replaced by ExternalFileDir.
         /// </summary>
         public string BlobDir {
             get {
                 string dir;
-                db.get_blob_dir(out dir);
+                db.get_ext_file_dir(out dir);
                 return dir;
             }
         }
@@ -381,19 +394,31 @@ namespace BerkeleyDB {
 
         /// <summary>
         /// The threshold value in bytes beyond which data items are stored as
-        /// blobs.
+        /// external files.
         /// <para>
         /// Any data item that is equal to or larger in size than the
-        /// threshold value will automatically be stored as a blob.
+        /// threshold value is automatically stored as an external file.
         /// </para>
         /// <para>
-        /// A value of 0 indicates that blobs are not used by the database.
+        /// A value of 0 indicates that external files are not used by the
+	/// database.
         /// </para>
         /// </summary>
+        public uint ExternalFileThreshold {
+            get {
+                uint ret = 0;
+                db.get_ext_file_threshold(ref ret);
+                return ret;
+            }
+        }
+
+	/// <summary>
+        /// Deprecated.  Replaced by ExternalFileThreshold.
+        /// </summary>
         public uint BlobThreshold {
             get {
                 uint ret = 0;
-                db.get_blob_threshold(ref ret);
+                db.get_ext_file_threshold(ref ret);
                 return ret;
             }
         }
@@ -547,7 +572,7 @@ namespace BerkeleyDB {
         }
 
         /// <summary>
-        /// If false, empty pages will not be coalesced into higher-level pages.
+        /// If false, empty pages are not coalesced into higher-level pages.
         /// </summary>
         public bool ReverseSplit {
             get {
@@ -568,8 +593,8 @@ namespace BerkeleyDB {
         /// </summary>
         /// <remarks>
         /// If the operation occurs in a transactional database, the operation
-        /// will be implicitly transaction protected using multiple
-        /// transactions. These transactions will be periodically committed to
+        /// is implicitly transaction protected using multiple
+        /// transactions. These transactions are periodically committed to
         /// avoid locking large sections of the tree. Any deadlocks encountered
         /// cause the compaction operation to be retried from the point of the
         /// last transaction commit.
@@ -591,8 +616,8 @@ namespace BerkeleyDB {
         /// </para>
         /// <para>
         /// If <paramref name="txn"/> is null, but the operation occurs in a
-        /// transactional database, the operation will be implicitly transaction
-        /// protected using multiple transactions. These transactions will be
+        /// transactional database, the operation is implicitly transaction
+        /// protected using multiple transactions. These transactions are
         /// periodically committed to avoid locking large sections of the tree.
         /// Any deadlocks encountered cause the compaction operation to be
         /// retried from the point of the last transaction commit.
@@ -721,7 +746,7 @@ namespace BerkeleyDB {
         /// </param>
         /// <param name="isoDegree">
         /// The level of isolation for database reads.
-        /// <see cref="Isolation.DEGREE_ONE"/> will be silently ignored for 
+        /// <see cref="Isolation.DEGREE_ONE"/> is silently ignored for 
         /// databases which did not specify
         /// <see cref="DatabaseConfig.ReadUncommitted"/>.
         /// </param>
@@ -825,7 +850,7 @@ namespace BerkeleyDB {
         /// </param>
         /// <param name="BufferSize">
         /// The initial size of the buffer to fill with duplicate data items. If
-        /// the buffer is not large enough, it will be automatically resized.
+        /// the buffer is not large enough, it is automatically resized.
         /// </param>
         /// <exception cref="NotFoundException">
         /// A NotFoundException is thrown if <paramref name="recno"/> is not in
@@ -849,7 +874,7 @@ namespace BerkeleyDB {
         /// </param>
         /// <param name="BufferSize">
         /// The initial size of the buffer to fill with duplicate data items. If
-        /// the buffer is not large enough, it will be automatically resized.
+        /// the buffer is not large enough, it is automatically resized.
         /// </param>
         /// <param name="txn">
         /// <paramref name="txn"/> is a Transaction object returned from
@@ -880,7 +905,7 @@ namespace BerkeleyDB {
         /// </param>
         /// <param name="BufferSize">
         /// The initial size of the buffer to fill with duplicate data items. If
-        /// the buffer is not large enough, it will be automatically resized.
+        /// the buffer is not large enough, it is automatically resized.
         /// </param>
         /// <param name="txn">
         /// <paramref name="txn"/> is a Transaction object returned from
@@ -1029,7 +1054,7 @@ namespace BerkeleyDB {
         /// </param>
         /// <param name="isoDegree">
         /// The level of isolation for database reads.
-        /// <see cref="Isolation.DEGREE_ONE"/> will be silently ignored for 
+        /// <see cref="Isolation.DEGREE_ONE"/> is silently ignored for 
         /// databases which did not specify
         /// <see cref="DatabaseConfig.ReadUncommitted"/>.
         /// </param>
