@@ -1,13 +1,15 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2000, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2000, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  */
 
 package com.sleepycat.collections;
 
 import com.sleepycat.compat.DbCompat;
+import com.sleepycat.compat.DbCompat.OpReadOptions;
+import com.sleepycat.compat.DbCompat.OpResult;
 import com.sleepycat.db.Cursor;
 import com.sleepycat.db.CursorConfig;
 import com.sleepycat.db.DatabaseEntry;
@@ -229,52 +231,91 @@ final class DataCursor implements Cloneable {
      *
      * @return REPOS_EXACT, REPOS_NEXT or REPOS_EOF.
      */
-    int repositionRange(byte[] keyBytes,
-                        byte[] priKeyBytes,
-                        byte[] valueBytes,
+    int repositionRange(byte[] lastKeyBytes,
+                        byte[] lastPriKeyBytes,
+                        byte[] lastValueBytes,
                         boolean lockForWrite)
         throws DatabaseException {
 
-        LockMode lockMode = getLockMode(lockForWrite);
-        OperationStatus status = null;
+        OpReadOptions options = OpReadOptions.make(getLockMode(lockForWrite));
+        OpResult result;
 
         /* Use the given key/data byte arrays. */
-        setThangs(keyBytes, priKeyBytes, valueBytes);
+        setThangs(lastKeyBytes, lastPriKeyBytes, lastValueBytes);
 
         /* Position on or after the given key/data pair. */
-        if (view.dupsAllowed) {
-            status = cursor.getSearchBothRange(keyThang, primaryKeyThang,
-                                               valueThang, lockMode);
-        }
-        if (status != OperationStatus.SUCCESS) {
-            status = cursor.getSearchKeyRange(keyThang, primaryKeyThang,
-                                              valueThang, lockMode);
+
+        if (!view.dupsAllowed) {
+            /*
+             * No-dups is the simple case. Search for key >= lastKey, and then
+             * compare to see if we found lastKey again.
+             */
+            result = cursor.getSearchKeyRange(
+                keyThang, primaryKeyThang, valueThang, options);
+
+            if (!result.isSuccess()) {
+                return REPOS_EOF;
+            }
+
+            return KeyRange.equalBytes(
+                lastKeyBytes, 0, lastKeyBytes.length,
+                keyThang.getData(), keyThang.getOffset(), keyThang.getSize()) ?
+                REPOS_EXACT : REPOS_NEXT;
         }
 
-        /* Return the result of the operation. */
-        if (status == OperationStatus.SUCCESS) {
-            if (!KeyRange.equalBytes(keyBytes, 0, keyBytes.length,
-                                     keyThang.getData(),
-                                     keyThang.getOffset(),
-                                     keyThang.getSize())) {
-                return REPOS_NEXT;
-            }
-            if (view.dupsAllowed) {
-                DatabaseEntry thang = view.isSecondary() ? primaryKeyThang
-                                                         : valueThang;
-                byte[] bytes = view.isSecondary() ? priKeyBytes
-                                                  : valueBytes;
-                if (!KeyRange.equalBytes(bytes, 0, bytes.length,
-                                         thang.getData(),
-                                         thang.getOffset(),
-                                         thang.getSize())) {
-                    return REPOS_NEXT;
-                }
-            }
-            return REPOS_EXACT;
-        } else {
+        /*
+         * Duplicates are more complex.
+         *
+         * Search for key == lastKey && data >= lastData, and then compare to
+         * see if we found lastData again.
+         */
+        result = cursor.getSearchBothRange(
+            keyThang, primaryKeyThang, valueThang, options);
+
+        if (result.isSuccess()) {
+
+            DatabaseEntry thang =
+                view.isSecondary() ? primaryKeyThang : valueThang;
+
+            byte[] bytes =
+                view.isSecondary() ? lastPriKeyBytes : lastValueBytes;
+
+            return KeyRange.equalBytes(
+                bytes, 0, bytes.length,
+                thang.getData(), thang.getOffset(), thang.getSize()) ?
+                REPOS_EXACT : REPOS_NEXT;
+        }
+
+        /*
+         * The record with lastKey/lastData must have been deleted and there
+         * are no records with key == lastkey && data > lastData.
+         *
+         * Search for key >= lastKey, but keep in mind that we will probably
+         * land on the first record with lastKey.
+         */
+        result = cursor.getSearchKeyRange(
+            keyThang, primaryKeyThang, valueThang, options);
+
+        if (!result.isSuccess()) {
             return REPOS_EOF;
         }
+
+        /*
+         * If we are positioned on the first dup of lastKey, skip over its
+         * records to the first record with the next key.
+         */
+        if (KeyRange.equalBytes(lastKeyBytes, 0, lastKeyBytes.length,
+            keyThang.getData(), keyThang.getOffset(), keyThang.getSize())) {
+
+            result = cursor.getNextNoDup(keyThang, primaryKeyThang,
+                valueThang, options);
+
+            if (!result.isSuccess()) {
+                return REPOS_EOF;
+            }
+        }
+
+        return REPOS_NEXT;
     }
 
     /**
@@ -291,8 +332,8 @@ final class DataCursor implements Cloneable {
                             boolean lockForWrite)
         throws DatabaseException {
 
-        LockMode lockMode = getLockMode(lockForWrite);
-        OperationStatus status = null;
+        OpReadOptions options = OpReadOptions.make(getLockMode(lockForWrite));
+        OpResult result;
 
         /* Use the given key/data byte arrays. */
         setThangs(keyBytes, priKeyBytes, valueBytes);
@@ -300,14 +341,14 @@ final class DataCursor implements Cloneable {
         /* Position on the given key/data pair. */
         if (view.recNumRenumber) {
             /* getSearchBoth doesn't work with recno-renumber databases. */
-            status = cursor.getSearchKey(keyThang, primaryKeyThang,
-                                         valueThang, lockMode);
+            result = cursor.getSearchKey(keyThang, primaryKeyThang,
+                                         valueThang, options);
         } else {
-            status = cursor.getSearchBoth(keyThang, primaryKeyThang,
-                                          valueThang, lockMode);
+            result = cursor.getSearchBoth(keyThang, primaryKeyThang,
+                                          valueThang, options);
         }
 
-        return (status == OperationStatus.SUCCESS);
+        return result.isSuccess();
     }
 
     /**
@@ -406,8 +447,9 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        return cursor.getCurrent(keyThang, primaryKeyThang, valueThang,
-                                 getLockMode(lockForWrite));
+        return cursor.getCurrent(
+            keyThang, primaryKeyThang, valueThang,
+            OpReadOptions.make(getLockMode(lockForWrite))).status();
     }
 
     /**
@@ -420,8 +462,9 @@ final class DataCursor implements Cloneable {
         if (joinCursor != null) {
             return joinCursor.getNext(keyThang, valueThang, lockMode);
         } else {
-            return cursor.getFirst(keyThang, primaryKeyThang, valueThang,
-                                   lockMode);
+            return cursor.getFirst(
+                keyThang, primaryKeyThang, valueThang,
+                OpReadOptions.make(lockMode)).status();
         }
     }
 
@@ -435,8 +478,9 @@ final class DataCursor implements Cloneable {
         if (joinCursor != null) {
             return joinCursor.getNext(keyThang, valueThang, lockMode);
         } else {
-            return cursor.getNext(keyThang, primaryKeyThang, valueThang,
-                                  lockMode);
+            return cursor.getNext(
+                keyThang, primaryKeyThang, valueThang,
+                OpReadOptions.make(lockMode)).status();
         }
     }
 
@@ -447,14 +491,15 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         LockMode lockMode = getLockMode(lockForWrite);
+        OpReadOptions options = OpReadOptions.make(lockMode);
         if (joinCursor != null) {
             return joinCursor.getNext(keyThang, valueThang, lockMode);
         } else if (view.dupsView) {
             return cursor.getNext
-                (keyThang, primaryKeyThang, valueThang, lockMode);
+                (keyThang, primaryKeyThang, valueThang, options).status();
         } else {
             return cursor.getNextNoDup
-                (keyThang, primaryKeyThang, valueThang, lockMode);
+                (keyThang, primaryKeyThang, valueThang, options).status();
         }
     }
 
@@ -468,9 +513,9 @@ final class DataCursor implements Cloneable {
         if (view.dupsView) {
             return null;
         } else {
-            return cursor.getNextDup
-                (keyThang, primaryKeyThang, valueThang,
-                 getLockMode(lockForWrite));
+            return cursor.getNextDup(
+                keyThang, primaryKeyThang, valueThang,
+                OpReadOptions.make(getLockMode(lockForWrite))).status();
         }
     }
 
@@ -481,8 +526,9 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        return cursor.getLast(keyThang, primaryKeyThang, valueThang,
-                              getLockMode(lockForWrite));
+        return cursor.getLast(
+            keyThang, primaryKeyThang, valueThang,
+            OpReadOptions.make(getLockMode(lockForWrite))).status();
     }
 
     /**
@@ -492,8 +538,9 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        return cursor.getPrev(keyThang, primaryKeyThang, valueThang,
-                              getLockMode(lockForWrite));
+        return cursor.getPrev(
+            keyThang, primaryKeyThang, valueThang,
+            OpReadOptions.make(getLockMode(lockForWrite))).status();
     }
 
     /**
@@ -504,14 +551,12 @@ final class DataCursor implements Cloneable {
 
         checkNoJoinCursor();
         LockMode lockMode = getLockMode(lockForWrite);
+        OpReadOptions options = OpReadOptions.make(lockMode);
         if (view.dupsView) {
             return null;
-        } else if (view.dupsView) {
-            return cursor.getPrev
-                (keyThang, primaryKeyThang, valueThang, lockMode);
         } else {
-            return cursor.getPrevNoDup
-                (keyThang, primaryKeyThang, valueThang, lockMode);
+            return cursor.getPrevNoDup(
+                keyThang, primaryKeyThang, valueThang, options).status();
         }
     }
 
@@ -525,9 +570,9 @@ final class DataCursor implements Cloneable {
         if (view.dupsView) {
             return null;
         } else {
-            return cursor.getPrevDup
-                (keyThang, primaryKeyThang, valueThang,
-                 getLockMode(lockForWrite));
+            return cursor.getPrevDup(
+                keyThang, primaryKeyThang, valueThang,
+                OpReadOptions.make(getLockMode(lockForWrite))).status();
         }
     }
 
@@ -543,9 +588,9 @@ final class DataCursor implements Cloneable {
         if (view.dupsView) {
             if (view.useKey(key, value, primaryKeyThang, view.dupsRange)) {
                 KeyRange.copy(view.dupsKey, keyThang);
-                return cursor.getSearchBoth
-                    (keyThang, primaryKeyThang, valueThang,
-                     getLockMode(lockForWrite));
+                return cursor.getSearchBoth(
+                    keyThang, primaryKeyThang, valueThang,
+                    OpReadOptions.make(getLockMode(lockForWrite))).status();
             }
         } else {
             if (view.useKey(key, value, keyThang, range)) {
@@ -562,13 +607,13 @@ final class DataCursor implements Cloneable {
     private OperationStatus doGetSearchKey(boolean lockForWrite)
         throws DatabaseException {
 
-        LockMode lockMode = getLockMode(lockForWrite);
+        OpReadOptions options = OpReadOptions.make(getLockMode(lockForWrite));
         if (view.btreeRecNumAccess) {
-            return cursor.getSearchRecordNumber(keyThang, primaryKeyThang,
-                                                valueThang, lockMode);
+            return cursor.getSearchRecordNumber(
+                keyThang, primaryKeyThang, valueThang, options).status();
         } else {
-            return cursor.getSearchKey(keyThang, primaryKeyThang,
-                                       valueThang, lockMode);
+            return cursor.getSearchKey(
+                keyThang, primaryKeyThang, valueThang, options).status();
         }
     }
 
@@ -580,17 +625,17 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        LockMode lockMode = getLockMode(lockForWrite);
+        OpReadOptions options = OpReadOptions.make(getLockMode(lockForWrite));
         if (view.dupsView) {
             if (view.useKey(key, value, primaryKeyThang, view.dupsRange)) {
                 KeyRange.copy(view.dupsKey, keyThang);
-                return cursor.getSearchBothRange
-                    (keyThang, primaryKeyThang, valueThang, lockMode);
+                return cursor.getSearchBothRange(
+                    keyThang, primaryKeyThang, valueThang, options).status();
             }
         } else {
             if (view.useKey(key, value, keyThang, range)) {
-                return cursor.getSearchKeyRange
-                    (keyThang, primaryKeyThang, valueThang, lockMode);
+                return cursor.getSearchKeyRange(
+                    keyThang, primaryKeyThang, valueThang, options).status();
             }
         }
         return OperationStatus.NOTFOUND;
@@ -604,7 +649,7 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        LockMode lockMode = getLockMode(lockForWrite);
+        OpReadOptions options = OpReadOptions.make(getLockMode(lockForWrite));
         view.useValue(value, valueThang, null);
         if (view.dupsView) {
             if (view.useKey(key, value, primaryKeyThang, view.dupsRange)) {
@@ -612,8 +657,8 @@ final class DataCursor implements Cloneable {
                 if (otherThang == null) {
                     otherThang = new DatabaseEntry();
                 }
-                OperationStatus status = cursor.getSearchBoth
-                    (keyThang, primaryKeyThang, otherThang, lockMode);
+                OperationStatus status = cursor.getSearchBoth(
+                    keyThang, primaryKeyThang, otherThang, options).status();
                 if (status == OperationStatus.SUCCESS &&
                     KeyRange.equalBytes(otherThang, valueThang)) {
                     return status;
@@ -624,21 +669,20 @@ final class DataCursor implements Cloneable {
                 if (otherThang == null) {
                     otherThang = new DatabaseEntry();
                 }
-                OperationStatus status = cursor.getSearchKey(keyThang,
-                                                             primaryKeyThang,
-                                                             otherThang,
-                                                             lockMode);
+                OperationStatus status = cursor.getSearchKey(
+                    keyThang, primaryKeyThang, otherThang, options).status();
                 while (status == OperationStatus.SUCCESS) {
                     if (KeyRange.equalBytes(otherThang, valueThang)) {
                         return status;
                     }
-                    status = cursor.getNextDup(keyThang, primaryKeyThang,
-                                               otherThang, lockMode);
+                    status = cursor.getNextDup(
+                        keyThang, primaryKeyThang, otherThang,
+                        options).status();
                 }
                 /* if status != SUCCESS set range cursor to invalid? */
             } else {
-                return cursor.getSearchBoth(keyThang, null, valueThang,
-                                            lockMode);
+                return cursor.getSearchBoth(
+                    keyThang, null, valueThang, options).status();
             }
         }
         return OperationStatus.NOTFOUND;
@@ -705,8 +749,8 @@ final class DataCursor implements Cloneable {
             if (otherThang == null) {
                 otherThang = new DatabaseEntry();
             }
-            cursor.getCurrent(keyThang, primaryKeyThang, otherThang,
-                              LockMode.DEFAULT);
+            cursor.getCurrent(
+                keyThang, primaryKeyThang, otherThang, OpReadOptions.EMPTY);
             if (KeyRange.equalBytes(valueThang, otherThang)) {
                 return OperationStatus.SUCCESS;
             } else {
@@ -778,10 +822,9 @@ final class DataCursor implements Cloneable {
         } else {
             if (view.dupsAllowed) {
                 /* Unordered duplicates. */
-                OperationStatus status =
-                        cursor.getSearchBoth(keyThang, primaryKeyThang,
-                                             valueThang,
-                                             getLockMode(false));
+                OperationStatus status = cursor.getSearchBoth(
+                    keyThang, primaryKeyThang, valueThang,
+                    OpReadOptions.make(getLockMode(false))).status();
                 if (status == OperationStatus.SUCCESS) {
                     return OperationStatus.KEYEXIST;
                 } else {
