@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2001, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2001, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -21,6 +21,10 @@ public interface BtreePrefixCalculator {
     A database entry representing a database key.
     @param dbt2
     A database entry representing a database key.
+    @return the number of bytes of the second key parameter that would be
+    required by the Btree key comparison function to determine the second
+    key parameter's ordering relationship with respect to the first key
+    parameter
     */
     int prefix(Database db, DatabaseEntry dbt1, DatabaseEntry dbt2);
 }
