@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2000, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2000, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -28,6 +28,10 @@ public abstract class EventHandlerAdapter implements EventHandler {
     */
     public void handlePanicEvent() {}
     /**
+    See {@link com.sleepycat.db.EventHandler#handleRepAutoTakeoverEvent EventHandler.handleRepAutoTakeoverEvent} for details of this callback.
+   */
+    public void handleRepAutoTakeoverEvent() {}
+    /**
     See {@link com.sleepycat.db.EventHandler#handleRepAutoTakeoverFailedEvent EventHandler.handleRepAutoTakeoverFailedEvent} for details of this
     callback.
     */
@@ -73,6 +77,11 @@ public abstract class EventHandlerAdapter implements EventHandler {
     */
     public void handleRepInitDoneEvent() {}
     /**
+    see {@link com.sleepycat.db.EventHandler#handleRepInQueueFullEvent EventHandler.handleRepInQueueFullEvent}for details of this
+    callback.
+    */
+    public void handleRepInQueueFullEvent() {}
+    /**
     See {@link com.sleepycat.db.EventHandler#handleRepJoinFailureEvent EventHandler.handleRepJoinFailureEvent} for details of this
     callback.
     */
