@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  */
 
@@ -68,7 +68,6 @@ import com.sleepycat.persist.model.PrimaryKey;
  *
  * {@code PrimaryIndex<Long, Employee>} primaryIndex =
  *     store.getPrimaryIndex(Long.class, Employee.class);</pre>
- * </pre>
  *
  * <p>Note that {@code Long.class} is passed as the primary key class, but the
  * primary key field has the primitive type {@code long}.  When a primitive
@@ -106,7 +105,7 @@ import com.sleepycat.persist.model.PrimaryKey;
  * <p>The {@link #putNoOverwrite} method can be used to ensure that an existing
  * entity is not overwritten.  {@link #putNoOverwrite} returns true if the
  * entity was inserted, or false if an existing entity exists and no action was
- * taken.  For example:<p>
+ * taken.  For example:</p>
  *
  * <pre class="code">
  * boolean inserted;
@@ -252,15 +251,6 @@ public class PrimaryIndex<PK, E> extends BasicIndex<PK, E> {
         }
     }
 
-    /**
-     * Returns the underlying database for this index.
-     *
-     * @return the database.
-     */
-    public Database getDatabase() {
-        return db;
-    }
-
     /**
      * Returns the primary key class for this index.
      *
@@ -440,6 +430,7 @@ public class PrimaryIndex<PK, E> extends BasicIndex<PK, E> {
     public void putNoReturn(Transaction txn, E entity)
         throws DatabaseException {
 
+
         DatabaseEntry keyEntry = new DatabaseEntry();
         DatabaseEntry dataEntry = new DatabaseEntry();
         assignKey(entity, keyEntry);
@@ -494,6 +485,7 @@ public class PrimaryIndex<PK, E> extends BasicIndex<PK, E> {
     public boolean putNoOverwrite(Transaction txn, E entity)
         throws DatabaseException {
 
+
         DatabaseEntry keyEntry = new DatabaseEntry();
         DatabaseEntry dataEntry = new DatabaseEntry();
         assignKey(entity, keyEntry);
@@ -504,6 +496,7 @@ public class PrimaryIndex<PK, E> extends BasicIndex<PK, E> {
         return (status == OperationStatus.SUCCESS);
     }
 
+
     /**
      * If we are assigning primary keys from a sequence, assign the next key
      * and set the primary key field.
@@ -534,6 +527,7 @@ public class PrimaryIndex<PK, E> extends BasicIndex<PK, E> {
     public E get(Transaction txn, PK key, LockMode lockMode)
         throws DatabaseException {
 
+
         DatabaseEntry keyEntry = new DatabaseEntry();
         DatabaseEntry dataEntry = new DatabaseEntry();
         keyBinding.objectToEntry(key, keyEntry);
@@ -541,17 +535,23 @@ public class PrimaryIndex<PK, E> extends BasicIndex<PK, E> {
         OperationStatus status = db.get(txn, keyEntry, dataEntry, lockMode);
 
         if (status == OperationStatus.SUCCESS) {
-            if (entityBinding instanceof PersistEntityBinding) {
-                return (E)((PersistEntityBinding) entityBinding).
-                           entryToObjectWithPriKey(key, dataEntry);
-            } else {
-                return entityBinding.entryToObject(keyEntry, dataEntry);
-            }
+            return makeEntity(key, keyEntry, dataEntry);
         } else {
             return null;
         }
     }
 
+
+    private E makeEntity(PK key,
+                         DatabaseEntry keyEntry,
+                         DatabaseEntry dataEntry) {
+
+        return (entityBinding instanceof PersistEntityBinding) ?
+            (E)((PersistEntityBinding) entityBinding).
+                entryToObjectWithPriKey(key, dataEntry) :
+            entityBinding.entryToObject(keyEntry, dataEntry);
+    }
+
     public Map<PK, E> map() {
         return sortedMap();
     }
