@@ -1,7 +1,7 @@
 /*
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2010, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2010, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -54,8 +54,10 @@
 #define	DB_SEQ_NEXT		0x0000
 #define	DB_SEQ_CURRENT		0x0001
 
-#define	MSG_CREATE_FAIL	"Sequence create failed: "
-#define	MSG_MALLOC_FAIL	"Malloc failed during sequence operation."
+#define	MSG_CREATE_FAIL		"Sequence create failed: "
+#define	MSG_MALLOC_FAIL		"Malloc failed during sequence operation."
+#define	MSG_INTMPDB_FAIL	"Sequences do not support in-memory or"	\
+                                " temporary databases."
 
 #define CACHE_ENTRY_VALID(_e)						\
 	(_e != NULL &&							\
@@ -78,6 +80,7 @@ static int btreeSeqPutCookie(
     sqlite3_context *context, Btree *p, SEQ_COOKIE *cookie, u_int32_t flags);
 static int btreeSeqRemoveHandle(
     sqlite3_context *context, Btree *p, CACHED_DB *cache_entry);
+static void btreeSeqSetSeqName(SEQ_COOKIE *cookie, const char *name);
 static int btreeSeqStartTransaction(
     sqlite3_context *context, Btree *p, int is_write);
 
@@ -96,6 +99,7 @@ static void db_seq_create_func(
 		    "create_sequence()");
 		return;
 	}
+	log_msg(LOG_NORMAL, "db_seq_create_func(%s)", sqlite3_value_text(argv[0]));
 	/*
 	 * Ensure that the sequence name is OK with our static buffer
 	 * size. We need extra characters for "seq_" and "_db".
@@ -126,9 +130,8 @@ static void db_seq_create_func(
 	memset(&cookie, 0, sizeof(SEQ_COOKIE));
 	cookie.incr = 1;
 
-	sqlite3_snprintf(BT_MAX_SEQ_NAME, cookie.name, "seq_%s",
-	    sqlite3_value_text(argv[0]));
-	cookie.name_len = (int)strlen(cookie.name);
+	btreeSeqSetSeqName(&cookie, (const char*)sqlite3_value_text(argv[0]));
+	log_msg(LOG_NORMAL, "db_seq_drop_func(%s)", cookie.name);
 	if (pBt->dbStorage == DB_STORE_NAMED && btreeSeqExists(context, p,
 	    cookie.name) == 1) {
 		btreeSeqError(context, SQLITE_ERROR,
@@ -255,7 +258,7 @@ static void db_seq_create_func(
 
 	if ((rc = btreeSeqGetHandle(context, p, SEQ_HANDLE_CREATE, &cookie)) !=
 	    SQLITE_OK) {
-		if (rc != SQLITE_ERROR)
+		if (rc != DB_NOINTMP)
 			btreeSeqError(context, dberr2sqlite(rc, NULL),
 			    "Failed to create sequence %s. Error: %s",
 			    (const char *)sqlite3_value_text(argv[0]),
@@ -290,9 +293,8 @@ static void db_seq_drop_func(
 		return;
 	}
 
-	sqlite3_snprintf(BT_MAX_SEQ_NAME, cookie.name, "seq_%s",
-	    sqlite3_value_text(argv[0]));
-	cookie.name_len = (int)strlen(cookie.name);
+	btreeSeqSetSeqName(&cookie, (const char*)sqlite3_value_text(argv[0]));
+	log_msg(LOG_NORMAL, "db_seq_drop_func(%s)", cookie.name);
 	rc = btreeSeqGetHandle(context, p, SEQ_HANDLE_OPEN, &cookie);
 	
 	if (rc != SQLITE_OK) {
@@ -300,7 +302,7 @@ static void db_seq_drop_func(
 		if (rc == DB_NOTFOUND) 
 			btreeSeqError(context, dberr2sqlite(rc, NULL),
 			    "no such sequence: %s", cookie.name + 4);
-		else if (rc != SQLITE_ERROR)
+		else if (rc != DB_NOINTMP)
 			btreeSeqError(context, dberr2sqlite(rc, NULL),
 			"Fail to drop sequence %s. Error: %s",
 			cookie.name + 4, db_strerror(rc));
@@ -310,7 +312,7 @@ static void db_seq_drop_func(
 	sqlite3_mutex_enter(pBt->mutex);
 	mutex_held = 1;
 	cache_entry =
-	    sqlite3HashFind(&pBt->db_cache, cookie.name, cookie.name_len);
+	    sqlite3HashFind(&pBt->db_cache, cookie.name);
 
 	if (cache_entry == NULL)
 		goto done;
@@ -370,6 +372,8 @@ static void btreeSeqGetVal(
 	p = db->aDb[0].pBt;
 	pBt = p->pBt;
 	memset(&cookie, 0, sizeof(cookie));
+	log_msg(LOG_NORMAL, "btreeSeqGetVal(%s, %s)", name,
+		mode == DB_SEQ_NEXT ? "next" : "current");
 
 	if (!p->connected &&
 	    (rc = btreeOpenEnvironment(p, 1)) != SQLITE_OK) {
@@ -379,15 +383,14 @@ static void btreeSeqGetVal(
 		return;
 	}
 
-	sqlite3_snprintf(BT_MAX_SEQ_NAME, cookie.name, "seq_%s", name);
-	cookie.name_len = (int)strlen(cookie.name);
+	btreeSeqSetSeqName(&cookie, name);
 	rc = btreeSeqGetHandle(context, p, SEQ_HANDLE_OPEN, &cookie);
 
 	if (rc != SQLITE_OK) {
 		if (rc == DB_NOTFOUND) 
 			btreeSeqError(context, dberr2sqlite(rc, NULL),
 			    "no such sequence: %s", name);
-		else if (rc != SQLITE_ERROR)
+		else if (rc != DB_NOINTMP)
 			btreeSeqError(context, dberr2sqlite(rc, NULL),
 			    "Fail to get next value from seq %s. Error: %s",
 			    name, db_strerror(rc));
@@ -510,10 +513,20 @@ static int btreeSeqGetHandle(sqlite3_context *context, Btree *p,
 
 	/* Does not support in-memory db and temp db for now */
 	if (pBt->dbStorage != DB_STORE_NAMED) {
-		btreeSeqError(context, SQLITE_ERROR,
-		    "Sequences do not support in-memory or "
-		    "temporary databases.");
-		return (SQLITE_ERROR);
+		btreeSeqError(context, SQLITE_ERROR, MSG_INTMPDB_FAIL);
+		return (DB_NOINTMP);
+	}
+
+	/* Tell sqlite3VdbeHalt() that this step has a transaction to end. */
+	if (p->db->pVdbe->bIsReader == 0) {
+		p->db->pVdbe->bIsReader = 1;
+		p->db->nVdbeRead++;
+	}
+
+	/* Tell sqlite3VdbeHalt() that this step has a transaction to end. */
+	if (p->db->pVdbe->bIsReader == 0) {
+		p->db->pVdbe->bIsReader = 1;
+		p->db->nVdbeRead++;
 	}
 
 	/*
@@ -528,7 +541,7 @@ static int btreeSeqGetHandle(sqlite3_context *context, Btree *p,
 	 */
 	sqlite3_mutex_enter(pBt->mutex);
 	cache_entry =
-	    sqlite3HashFind(&pBt->db_cache, cookie->name, cookie->name_len);
+	    sqlite3HashFind(&pBt->db_cache, cookie->name);
 	sqlite3_mutex_leave(pBt->mutex);
 
 	if (CACHE_ENTRY_VALID(cache_entry)) {
@@ -573,7 +586,7 @@ static int btreeSeqGetHandle(sqlite3_context *context, Btree *p,
 	sqlite3_mutex_enter(pBt->mutex);
 	/* Check to see if someone beat us to adding the handle. */
 	cache_entry =
-	    sqlite3HashFind(&pBt->db_cache, cookie->name, cookie->name_len);
+	    sqlite3HashFind(&pBt->db_cache, cookie->name);
 	if (CACHE_ENTRY_VALID(cache_entry)) {
 		cookie->handle->close(cookie->handle, 0);
 		cookie->handle = (DB_SEQUENCE *)cache_entry->dbp;
@@ -608,8 +621,10 @@ static int btreeSeqGetHandle(sqlite3_context *context, Btree *p,
 	cache_entry->is_sequence = 1;
 	memcpy(cache_entry->cookie, cookie, sizeof(SEQ_COOKIE));
 	stale_db = sqlite3HashInsert(&pBt->db_cache, cache_entry->key,
-	    cookie->name_len, cache_entry);
+	    cache_entry);
 	if (stale_db) {
+		if (stale_db->cookie != NULL)
+			sqlite3_free(stale_db->cookie);
 		sqlite3_free(stale_db);
 		/*
 		 * Hash table out of memory when returned pointer is
@@ -617,8 +632,8 @@ static int btreeSeqGetHandle(sqlite3_context *context, Btree *p,
 		 */
 		if (stale_db == cache_entry) {
 			btreeSeqError(context, SQLITE_NOMEM, MSG_MALLOC_FAIL);
-			ret = SQLITE_NOMEM;
-			goto err;
+			sqlite3_mutex_leave(pBt->mutex);
+			return SQLITE_NOMEM;
 		}
 	}
 
@@ -644,7 +659,7 @@ static int btreeSeqRemoveHandle(
 	memcpy(&cookie, cache_entry->cookie, sizeof(cookie));
 
 	/* Remove the entry from the hash table. */
-	sqlite3HashInsert(&pBt->db_cache, cookie.name, cookie.name_len, NULL);
+	sqlite3HashInsert(&pBt->db_cache, cookie.name, NULL);
 
 	if (cookie.cache != 0) {
 		seq = (DB_SEQUENCE *)cache_entry->dbp;
@@ -863,6 +878,41 @@ static int btreeSeqPutCookie(
 	return (0);
 }
 
+/*
+ * According to the documentation the sequence name should be converted
+ * to lowercase unless it is surrounded by quotation marks.
+ * This function assumes that the sequence name fits in the buffer to which
+ * cookie.name points.
+ */
+static void btreeSeqSetSeqName(SEQ_COOKIE *cookie, const char *name)
+{
+	char lowercase[BT_MAX_SEQ_NAME];
+	int i;
+	size_t len;
+
+	if (name == NULL) {
+		strcpy(cookie->name, "seq_");
+		cookie->name_len = 4;
+		return;
+	}
+
+	len = strlen(name);
+	if (name[0] == '"' && name[len-1] == '"')
+		sqlite3_snprintf(
+		    BT_MAX_SEQ_NAME, cookie->name, "seq_%s", name);
+	else {
+		memset(lowercase, 0, BT_MAX_SEQ_NAME);
+		for (i = 0; i < len; i++) {
+			lowercase[i] = sqlite3UpperToLower[*(name + i)];
+		}
+		sqlite3_snprintf(
+		    BT_MAX_SEQ_NAME, cookie->name, "seq_%s", lowercase);
+	}
+	cookie->name_len = (int)strlen(cookie->name);
+
+	return;
+}
+
 /*
  * SQLite manages explicit transactions by setting a flag when a BEGIN; is
  * issued, then starting an actual transaction in the btree layer when the
@@ -874,7 +924,7 @@ static int btreeSeqPutCookie(
  *   write operations, and thus we need a valid statement_txn.
  * - In an explicit transaction, and the first statement. Start a txn and a
      statement txn.
- * - In an explicit transaction and not the first statemetn. Start a statement
+ * - In an explicit transaction and not the first statement. Start a statement
  *   transaction.
  *
  * The SQLite vdbe will take care of closing the statement transaction for us,
