@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1999, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1999, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -345,18 +345,31 @@ tcl_LogStat(interp, objc, objv, dbenv)
 {
 	DB_LOG_STAT *sp;
 	Tcl_Obj *res;
+	u_int32_t flag;
+	char *arg;
 	int result, ret;
 
+	flag = 0;
 	result = TCL_OK;
-	/*
-	 * No args for this.  Error if there are some.
-	 */
-	if (objc != 2) {
-		Tcl_WrongNumArgs(interp, 2, objv, NULL);
+
+	if (objc > 3) {
+		Tcl_WrongNumArgs(interp, 2, objv, "?-clear?");
 		return (TCL_ERROR);
 	}
+
+	if (objc == 3) {
+		arg = Tcl_GetStringFromObj(objv[2], NULL);
+		if (strcmp(arg, "-clear") == 0)
+			flag = DB_STAT_CLEAR;
+		else {
+ 			Tcl_SetResult(interp,
+			"db stat: unknown arg", TCL_STATIC);
+			return (TCL_ERROR);
+		}
+	}
+
 	_debug_check();
-	ret = dbenv->log_stat(dbenv, &sp, 0);
+	ret = dbenv->log_stat(dbenv, &sp, flag);
 	result = _ReturnSetup(interp, ret, DB_RETOK_STD(ret), "log stat");
 	if (result == TCL_ERROR)
 		return (result);
@@ -419,11 +432,13 @@ tcl_LogStatPrint(interp, objc, objv, dbenv)
 {
 	static const char *logstatprtopts[] = {
 		"-all",
+		"-alloc",
 		"-clear",
 		 NULL
 	};
 	enum logstatprtopts {
 		LOGSTATPRTALL,
+		LOGSTATPRTALLOC,
 		LOGSTATPRTCLEAR
 	};
 	u_int32_t flag;
@@ -444,6 +459,9 @@ tcl_LogStatPrint(interp, objc, objv, dbenv)
 		case LOGSTATPRTALL:
 			flag |= DB_STAT_ALL;
 			break;
+		case LOGSTATPRTALLOC:
+			flag |= DB_STAT_ALLOC;
+			break;
 		case LOGSTATPRTCLEAR:
 			flag |= DB_STAT_CLEAR;
 			break;
@@ -722,6 +740,7 @@ static const char *confwhich[] = {
 	"direct",
 	"dsync",
 	"inmemory",
+	"nosync",
 	"zero",
 	NULL
 };
@@ -731,6 +750,7 @@ enum logwhich {
 	LOGCONF_DIRECT,
 	LOGCONF_DSYNC,
 	LOGCONF_INMEMORY,
+	LOGCONF_NOSYNC,
 	LOGCONF_ZERO
 };
 
@@ -769,7 +789,7 @@ tcl_LogConfig(interp, dbenv, which, onoff)
 		wh = DB_LOG_AUTO_REMOVE;
 		break;
 	case LOGCONF_BLOB:
-		wh = DB_LOG_BLOB;
+		wh = DB_LOG_EXT_FILE;
 		break;
 	case LOGCONF_DIRECT:
 		wh = DB_LOG_DIRECT;
@@ -780,6 +800,9 @@ tcl_LogConfig(interp, dbenv, which, onoff)
 	case LOGCONF_INMEMORY:
 		wh = DB_LOG_IN_MEMORY;
 		break;
+	case LOGCONF_NOSYNC:
+		wh = DB_LOG_NOSYNC;
+		break;
 	case LOGCONF_ZERO:
 		wh = DB_LOG_ZERO;
 		break;
@@ -801,7 +824,7 @@ tcl_LogConfig(interp, dbenv, which, onoff)
 	}
 	ret = dbenv->log_set_config(dbenv, wh, on);
 	return (_ReturnSetup(interp, ret, DB_RETOK_STD(ret),
-	    "env rep_config"));
+	    "env log_config"));
 }
 
 /*
@@ -831,7 +854,7 @@ tcl_LogGetConfig(interp, dbenv, which)
 		wh = DB_LOG_AUTO_REMOVE;
 		break;
 	case LOGCONF_BLOB:
-		wh = DB_LOG_BLOB;
+		wh = DB_LOG_EXT_FILE;
 		break;
 	case LOGCONF_DIRECT:
 		wh = DB_LOG_DIRECT;
@@ -842,6 +865,9 @@ tcl_LogGetConfig(interp, dbenv, which)
 	case LOGCONF_INMEMORY:
 		wh = DB_LOG_IN_MEMORY;
 		break;
+	case LOGCONF_NOSYNC:
+		wh = DB_LOG_NOSYNC;
+		break;
 	case LOGCONF_ZERO:
 		wh = DB_LOG_ZERO;
 		break;
