@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2013, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  */
 
 #include "db_config.h"
@@ -12,12 +12,13 @@
 #include "dbinc/db_am.h"
 #include "dbinc/blob.h"
 #include "dbinc/fop.h"
-#include "dbinc/mp.h"
+#include "dbinc/txn.h"
 #include "dbinc_auto/sequence_ext.h"
 
 static int __blob_open_meta_db __P((
-    DB *, DB_TXN *, DB **, DB_SEQUENCE **, int));
-static int __blob_clean_dir __P((DB *, DB_TXN *, const char *, int));
+    DB *, DB_TXN *, DB **, DB_SEQUENCE **, int, int, int));
+static int __blob_clean_dir
+    __P((ENV *, DB_TXN *, const char *, const char *, int));
 static int __blob_copy_dir __P((DB *, const char *, const char *));
 
 #define	BLOB_ID_KEY "blob_id"
@@ -30,14 +31,14 @@ static int __blob_copy_dir __P((DB *, const char *, const char *));
  *	Create the name of the subdirectory in the blob directory
  * for the given database file and subdatabase ids.
  *
- * PUBLIC: int __blob_make_sub_dir __P((ENV *, char **, uintmax_t, uintmax_t));
+ * PUBLIC: int __blob_make_sub_dir __P((ENV *, char **, db_seq_t, db_seq_t));
  */
 int
 __blob_make_sub_dir(env, blob_sub_dir, file_id, db_id)
 	ENV *env;
 	char **blob_sub_dir;
-	uintmax_t file_id;
-	uintmax_t db_id;
+	db_seq_t file_id;
+	db_seq_t db_id;
 {
 	char fname[MAX_BLOB_PATH_SZ], dname[MAX_BLOB_PATH_SZ];
 	int ret;
@@ -50,21 +51,24 @@ __blob_make_sub_dir(env, blob_sub_dir, file_id, db_id)
 	if (db_id == 0 && file_id == 0)
 		return (0);
 
+	if (db_id < 0 || file_id < 0)
+		return (USR_ERR(env, EINVAL));
+
 	/* The master db has no subdb id. */
 	if (db_id != 0)
-		sprintf(dname,
+		(void)snprintf(dname, MAX_BLOB_PATH_SZ,
 		    "%s%llu", BLOB_DIR_PREFIX, (unsigned long long)db_id);
-	sprintf(fname, "%s%llu", BLOB_DIR_PREFIX,
-	    (unsigned long long)file_id);
+	(void)snprintf(fname, MAX_BLOB_PATH_SZ, "%s%llu",
+	    BLOB_DIR_PREFIX, (unsigned long long)file_id);
 
 	len = strlen(fname) + (db_id ? strlen(dname) : 0) + 3;
 	if ((ret = __os_malloc(env, len, blob_sub_dir)) != 0)
 		goto err;
 	if (db_id != 0)
-		sprintf(*blob_sub_dir, "%s%c%s%c", fname,
+		(void)sprintf(*blob_sub_dir, "%s%c%s%c", fname,
 		    PATH_SEPARATOR[0], dname, PATH_SEPARATOR[0]);
 	else
-		sprintf(*blob_sub_dir, "%s%c", fname, PATH_SEPARATOR[0]);
+		(void)sprintf(*blob_sub_dir, "%s%c", fname, PATH_SEPARATOR[0]);
 
 	return (0);
 
@@ -157,12 +161,14 @@ err:	if (blob_dir != NULL)
  *	the per-db db used to generate blob ids (__db.bl001).
  */
 static int
-__blob_open_meta_db(dbp, txn, meta_db, seq, file)
+__blob_open_meta_db(dbp, txn, meta_db, seq, file, create, force_txn)
 	DB *dbp;
 	DB_TXN *txn;
 	DB **meta_db;
 	DB_SEQUENCE **seq;
 	int file;
+	int create;
+	int force_txn;
 {
 #ifdef HAVE_64BIT_TYPES
 	ENV *env;
@@ -172,8 +178,8 @@ __blob_open_meta_db(dbp, txn, meta_db, seq, file)
 	DB_THREAD_INFO *ip;
 	DB_TXN *local_txn;
 	char *fullname, *fname, *dname, *path;
-	int free_paths, ret;
-	u_int32_t flags, oflags;
+	int free_paths, ret, use_txn;
+	u_int32_t flags;
 
 	flags = 0;
 	fullname = fname = NULL;
@@ -181,7 +187,7 @@ __blob_open_meta_db(dbp, txn, meta_db, seq, file)
 	blob_seq = NULL;
 	local_txn = NULL;
 	env = dbp->env;
-	free_paths = 0;
+	free_paths = use_txn = 0;
 	memset(&key, 0, sizeof(DBT));
 
 	/*
@@ -203,9 +209,7 @@ __blob_open_meta_db(dbp, txn, meta_db, seq, file)
 		    file ? NULL : dbp, &fname)) < 0)
 			goto err;
 		free_paths = 1;
-		if ((ret = __db_get_open_flags(dbp, &oflags)) != 0)
-			goto err;
-		if (oflags & DB_THREAD)
+		if (dbp->open_flags & DB_THREAD)
 			LF_SET(DB_THREAD);
 	}
 
@@ -213,35 +217,65 @@ __blob_open_meta_db(dbp, txn, meta_db, seq, file)
 		goto err;
 
 	path = fullname;
-#ifdef DB_WIN32
-	/*
-	 * Absolute paths on windows can result in it creating a "C" or "D"
-	 * directory in the working directory.
-	 */
-	if (__os_abspath(path))
-		path += 2;
-#endif
 	/*
 	 * Create the blob, database file, and database name directories. The
 	 * mkdir isn't logged, so __fop_create_recover needs to do this as well.
 	 */
-	if (__os_exists(env, fullname, NULL) != 0 &&
-	    (ret = __db_mkpath(env, path)) != 0)
-		goto err;
+	if (__os_exists(env, fullname, NULL) != 0) {
+	    if (!create) {
+		    ret = USR_ERR(env, ENOENT);
+		    goto err;
+	    } else if ((ret = __db_mkpath(env, path)) != 0)
+		    goto err;
+	}
 
-	/* Create the blob meta db if it doesn't exist. */
-	LF_SET(DB_CREATE);
 	if ((ret = __db_create_internal(&blob_meta_db, env, 0)) != 0)
 		goto err;
 
+	if (create) {
+		LF_SET(DB_CREATE);
+		/*
+		 * Make the pagesize of the meta database the same as that
+		 * of the dbp passed in.  For the environment-wide db this
+		 * will be the first database created.  For a per-db db
+		 * this will be the owning database.
+		 */
+		if (dbp->pgsize != 0) {
+			if ((ret = __db_set_pagesize(
+			    blob_meta_db, dbp->pgsize)) != 0)
+				goto err;
+		}
+	}
+
 	/* Disable blobs in the blob meta databases themselves. */
 	if ((ret = __db_set_blob_threshold(blob_meta_db, 0, 0)) != 0)
 		goto err;
 
+	/*
+	 * To avoid concurrency issues, the blob meta database is
+	 * opened and operated on in a local transaction.  One
+	 * exception is when the blob meta database is created in the
+	 * same txn as the parent db.  Then the blob meta database
+	 * shares the given txn, so if the txn is rolled back, the
+	 * creation of the blob meta database will also be rolled back.
+	 * Another exception is when the master in replication opens
+	 * the meta database to find the highest blob id. In that case
+	 * it needs to be able to roll back the open call so the log
+	 * is not propagated to the clients.
+	 */
+	if ((!file && IS_REAL_TXN(dbp->cur_txn)) || force_txn)
+		use_txn = 1;
+
 	ENV_GET_THREAD_INFO(env, ip);
-	if (IS_REAL_TXN(txn) &&
-	    (ret = __db_txn_auto_init(env, ip, &local_txn)) != 0)
-		goto err;
+	if (IS_REAL_TXN(txn)) {
+		if (use_txn)
+			local_txn = txn;
+		else {
+			if ((ret = __txn_begin(
+			    env, ip, NULL, &local_txn, DB_IGNORE_LEASE)) != 0)
+				goto err;
+		}
+	}
 	if ((ret = __db_open(blob_meta_db, ip, local_txn, fname, dname,
 	    DB_BTREE, flags | DB_INTERNAL_BLOB_DB, 0, PGNO_BASE_MD)) != 0)
 		goto err;
@@ -256,8 +290,8 @@ __blob_open_meta_db(dbp, txn, meta_db, seq, file)
 	if ((ret = __seq_open(blob_seq, local_txn, &key, flags)) != 0)
 		goto err;
 
-	if (local_txn != NULL &&
-	    (ret = __db_txn_auto_resolve(env, local_txn, 0, ret)) != 0) {
+	if (local_txn != NULL && use_txn == 0 &&
+	    (ret = __txn_commit(local_txn, 0)) != 0) {
 		local_txn = NULL;
 		goto err;
 	}
@@ -273,17 +307,17 @@ err:
 		__os_free(env, fullname);
 	if (fname != NULL && free_paths)
 		__os_free(env, fname);
-	if (local_txn != NULL)
-		(void)__db_txn_auto_resolve(env, local_txn, 0, ret);
+	if (local_txn != NULL && use_txn == 0)
+		(void)__txn_abort(local_txn);
 	if (blob_seq != NULL)
-		__seq_close(blob_seq, 0);
+		(void)__seq_close(blob_seq, 0);
 	if (blob_meta_db != NULL)
-		__db_close(blob_meta_db, NULL, 0);
+		(void)__db_close(blob_meta_db, NULL, 0);
 	return (ret);
 
 #else /*HAVE_64BIT_TYPES*/
 	__db_errx(dbp->env, DB_STR("0217",
-	    "library build did not include support for blobs"));
+	    "library build did not include support for external files"));
 	return (DB_OPNOTSUP);
 #endif
 }
@@ -298,19 +332,18 @@ err:
  * subdatabase id.
  *
  * PUBLIC: int __blob_generate_dir_ids
- * PUBLIC:	__P((DB *, DB_TXN *, uintmax_t *));
+ * PUBLIC:	__P((DB *, DB_TXN *, db_seq_t *));
  */
 int
 __blob_generate_dir_ids(dbp, txn, id)
 	DB *dbp;
 	DB_TXN *txn;
-	uintmax_t *id;
+	db_seq_t *id;
 {
 	DB *blob_meta_db;
 	DB_SEQUENCE *blob_seq;
 	int ret;
 	u_int32_t flags;
-	db_seq_t val;
 
 #ifdef HAVE_64BIT_TYPES
 	flags = 0;
@@ -318,34 +351,28 @@ __blob_generate_dir_ids(dbp, txn, id)
 	blob_seq = NULL;
 
 	if ((ret = __blob_open_meta_db(
-	    dbp, txn, &blob_meta_db, &blob_seq, 1)) != 0)
+	    dbp, txn, &blob_meta_db, &blob_seq, 1, 1, 0)) != 0)
 		goto err;
 
 	if (IS_REAL_TXN(txn))
 		LF_SET(DB_AUTO_COMMIT | DB_TXN_NOSYNC);
 
-	/*
-	 * __seq_get returns a signed 64 bit integer instead of uint, but
-	 * it will throw an error before it rolls over, so there is no
-	 * rollover danger here.
-	 */
 	DB_ASSERT(dbp->env, id != NULL);
 	if (*id == 0) {
-		if ((ret = __seq_get(blob_seq, 0, 1, &val, flags)) != 0)
+		if ((ret = __seq_get(blob_seq, 0, 1, id, flags)) != 0)
 			goto err;
-		*id = (uintmax_t)val;
 	}
 
 err:	if (blob_seq != NULL)
-		__seq_close(blob_seq, 0);
+		(void)__seq_close(blob_seq, 0);
 	if (blob_meta_db != NULL)
-		__db_close(blob_meta_db, NULL, 0);
+		(void)__db_close(blob_meta_db, NULL, 0);
 	return (ret);
 #else /*HAVE_64BIT_TYPES*/
 	COMPQUIET(dbp, NULL);
 	COMPQUIET(txn, NULL);
-	__db_errx(dbp->env, DB_STR("0218",
-	    "library build did not include support for blobs"));
+	__db_errx(dbp->env, DB_STR("0217",
+	    "library build did not include support for external files"));
 	return (DB_OPNOTSUP);
 #endif
 }
@@ -354,71 +381,158 @@ err:	if (blob_seq != NULL)
  * __blob_generate_id --
  * Generate a new blob ID.
  *
- * PUBLIC: int __blob_generate_id __P((DB *, DB_TXN *, uintmax_t *));
+ * PUBLIC: int __blob_generate_id __P((DB *, DB_TXN *, db_seq_t *));
  */
 int
 __blob_generate_id(dbp, txn, blob_id)
 	DB *dbp;
 	DB_TXN *txn;
-	uintmax_t *blob_id;
+	db_seq_t *blob_id;
 {
 #ifdef HAVE_64BIT_TYPES
+	DB_TXN *ltxn;
 	int ret;
 	u_int32_t flags;
-	db_seq_t val;
-
-	flags = 0;
+	flags = DB_IGNORE_LEASE;
+	ltxn = NULL;
 
 	if (dbp->blob_seq == NULL) {
 		if ((ret = __blob_open_meta_db(dbp, txn,
-		    &dbp->blob_meta_db, &dbp->blob_seq, 0)) != 0)
+		    &dbp->blob_meta_db, &dbp->blob_seq, 0, 1, 0)) != 0)
 			goto err;
 	}
 
-	if (IS_REAL_TXN(txn))
+	/*
+	 * If this is the opening transaction of the database, use it instead
+	 * of auto commit.  Otherwise it could deadlock with the transaction
+	 * used to open the blob meta database in __blob_open_meta_db.
+	 */
+	if (IS_REAL_TXN(dbp->cur_txn))
+		ltxn = txn;
+
+	if (IS_REAL_TXN(txn) && ltxn == NULL)
 		LF_SET(DB_AUTO_COMMIT | DB_TXN_NOSYNC);
 
-	/*
-	 * __seq_get returns a signed 64 bit integer instead of uint, but
-	 * it will throw an error before it rolls over, so there is no
-	 * rollover danger here.
-	 */
-	if ((ret = __seq_get(dbp->blob_seq, 0, 1, &val, flags)) != 0)
+	if ((ret = __seq_get(dbp->blob_seq, ltxn, 1, blob_id, flags)) != 0)
 		goto err;
-	*blob_id = (uintmax_t)val;
 
 err:	return (ret);
 #else /*HAVE_64BIT_TYPES*/
 	COMPQUIET(blob_id, NULL);
-	__db_errx(dbp->env, DB_STR("0219",
-	    "library build did not include support for blobs"));
+	__db_errx(dbp->env, DB_STR("0217",
+	    "library build did not include support for external files"));
 	return (DB_OPNOTSUP);
 #endif
 }
 
+/*
+ * __blob_highest_id
+ *
+ * Returns the highest id in the blob meta database.
+ *
+ * PUBLIC: int __blob_highest_id __P((DB *, DB_TXN *, db_seq_t *));
+ */
+int
+__blob_highest_id(dbp, txn, id)
+	DB *dbp;
+	DB_TXN *txn;
+	db_seq_t *id;
+{
+#ifdef HAVE_64BIT_TYPES
+	int ret;
+
+	*id = 0;
+	if (dbp->blob_sub_dir == NULL) {
+		if ((ret = __blob_make_sub_dir(dbp->env, &dbp->blob_sub_dir,
+		    dbp->blob_file_id, dbp->blob_sdb_id)) != 0)
+				goto err;
+	}
+	if (dbp->blob_seq == NULL) {
+		ret = __blob_open_meta_db(dbp, txn,
+		    &dbp->blob_meta_db, &dbp->blob_seq, 0, 0, 1);
+		/*
+		 * It is not an error if the blob meta database does not
+		 * exist.
+		 */
+		if (ret == ENOENT)
+			ret = 0;
+		if (ret != 0)
+			goto err;
+	}
+
+	ret = __seq_get(dbp->blob_seq, txn, 0, id, DB_CURRENT);
+err:
+	return (ret);
+#else /*HAVE_64BIT_TYPES*/
+	COMPQUIET(id, NULL);
+	__db_errx(dbp->env, DB_STR("0217",
+	    "library build did not include support for external files"));
+	return (DB_OPNOTSUP);
+#endif
+}
+
+/*
+ * __blob_calculate_dirs
+ *
+ * Use a blob id to to determine the path below the blob subdirectory in
+ * which the blob file is located.  Assumes enough space exists in the path
+ * variable to hold the path.
+ *
+ * PUBLIC: void __blob_calculate_dirs __P((db_seq_t, char *, int *, int *));
+ */
+void
+__blob_calculate_dirs(blob_id, path, len, depth)
+	db_seq_t blob_id;
+	char *path;
+	int *len;
+	int *depth;
+{
+	int i;
+	db_seq_t factor, tmp;
+
+	/* Calculate the subdirectories from the blob id. */
+	factor = 1;
+	for ((*depth) = 0, tmp = blob_id/BLOB_DIR_ELEMS;
+	    tmp != 0; tmp = tmp/BLOB_DIR_ELEMS, (*depth)++)
+		factor *= BLOB_DIR_ELEMS;
+
+	for (i = (*depth); i > 0; i--) {
+		tmp = (blob_id / factor) % BLOB_DIR_ELEMS;
+		factor /= BLOB_DIR_ELEMS;
+		(*len) += sprintf(path + (*len),
+			"%03llu%c", (unsigned long long)tmp, PATH_SEPARATOR[0]);
+	}
+}
+
 /*
  * __blob_id_to_path --
  * Generate the file name and blob specific part of the path for a particular
  * blob_id. The __db_appname API is used to generate a fully qualified path.
  * The caller must deallocate the path.
  *
- * PUBLIC: int __blob_id_to_path __P((ENV *, const char *, uintmax_t, char **));
+ * PUBLIC: int __blob_id_to_path __P((ENV *,
+ * PUBLIC:	 const char *, db_seq_t, char **, int));
  */
 int
-__blob_id_to_path(env, blob_sub_dir, blob_id, ppath)
+__blob_id_to_path(env, blob_sub_dir, blob_id, ppath, create)
 	ENV *env;
 	const char *blob_sub_dir;
-	uintmax_t blob_id;
+	db_seq_t blob_id;
 	char **ppath;
+	int create;
 {
 	char *path, *tmp_path;
-	int depth, i, name_len, ret;
+	int depth, name_len, ret;
 	size_t len;
-	uintmax_t factor, tmp;
 
 	name_len = 0;
 	path = tmp_path = *ppath = NULL;
 
+	if (blob_id < 1) {
+		ret = USR_ERR(env, EINVAL);
+		goto err;
+	}
+
 	len = MAX_BLOB_PATH_SZ + strlen(blob_sub_dir) + 1;
 	if ((ret = __os_malloc(env, len, &path)) != 0)
 		goto err;
@@ -426,34 +540,25 @@ __blob_id_to_path(env, blob_sub_dir, blob_id, ppath)
 	memset(path, 0, len);
 	name_len += sprintf(path, "%s", blob_sub_dir);
 
-	/* Populate the directories. */
-	factor = 1;
-	for (depth = 0, tmp = blob_id/BLOB_DIR_ELEMS;
-	    tmp != 0; tmp = tmp/BLOB_DIR_ELEMS, depth++)
-		factor *= BLOB_DIR_ELEMS;
-
-	for (i = depth; i > 0; i--) {
-		tmp = (blob_id / factor) % BLOB_DIR_ELEMS;
-		factor /= BLOB_DIR_ELEMS;
-		name_len += sprintf(path + name_len,
-			"%03llu%c", (unsigned long long)tmp, PATH_SEPARATOR[0]);
-	}
+	__blob_calculate_dirs(blob_id, path, &name_len, &depth);
 
 	/*
 	 * Populate the file name. Ensure there are 3 digits for each directory
 	 * level (even if they are 0).
 	 */
-	sprintf(path + name_len, "%s%0*llu",
+	(void)sprintf(path + name_len, "%s%0*llu",
 	    BLOB_FILE_PREFIX, (depth + 1) * 3, (unsigned long long)blob_id);
 
 	/* If this is the first file in the directory, ensure it exists. */
-	if (blob_id % BLOB_DIR_ELEMS == 0 && depth > 0) {
-		__db_appname(env, DB_APP_BLOB, path, NULL, &tmp_path);
+	if (blob_id % BLOB_DIR_ELEMS == 0 && depth > 0 && create) {
+		if ((ret = __db_appname(
+		    env, DB_APP_BLOB, path, NULL, &tmp_path)) != 0 )
+			goto err;
 
 		if ((ret = __db_mkpath(env, tmp_path)) != 0) {
 			__db_errx(env, DB_STR("0221",
-			    "Error creating blob directory."));
-			ret = EINVAL;
+			    "Error creating external file directory."));
+			ret = USR_ERR(env, EINVAL);
 			goto err;
 		}
 		__os_free(env, tmp_path);
@@ -471,48 +576,151 @@ err:
 	return (ret);
 }
 
+/*
+ * __blob_str_to_id
+ *
+ * If the given string is a positive number, it returns it as a signed
+ * 64 bit integer.  Otherwise the number is returned as 0.
+ *
+ * PUBLIC:  int __blob_str_to_id __P((ENV *, const char **, db_seq_t *));
+ */
+int
+__blob_str_to_id(env, path, id)
+	ENV *env;
+	const char **path;
+	db_seq_t *id;
+{
+	db_seq_t i;
+	const char *p;
+	char buf[2];
+
+	p = *path;
+	i = 10;
+	*id = 0;
+	buf[1] = '\0';
+	while (p[0] >= '0' && p[0] <= '9') {
+		*id *= i;
+		buf[0] = p[0];
+		*id += atoi(buf);
+		if (*id < 0) {
+			__db_errx(env, DB_STR("0246",
+			    "External file id integer overflow."));
+			return (USR_ERR(env, EINVAL));
+		}
+		p++;
+	}
+	*path = p;
+	return (0);
+}
+
+/*
+ * __blob_path_to_dir_ids --
+ * Get the file and subdatabase ids from a path to a blob file
+ * or a path in the blob directory structure.  Skips the
+ * subdatabase directory id if sdb_id is NULL.
+ *
+ * PUBLIC: int __blob_path_to_dir_ids
+ * PUBLIC:	__P((ENV *, const char *, db_seq_t *, db_seq_t *));
+ */
+int
+__blob_path_to_dir_ids(env, path, file_id, sdb_id)
+	ENV *env;
+	const char *path;
+	db_seq_t *file_id;
+	db_seq_t *sdb_id;
+{
+	int ret;
+	size_t len;
+	const char *p;
+
+	*file_id = 0;
+	if (sdb_id != NULL)
+		*sdb_id = 0;
+	ret = 0;
+	p = path;
+
+	/*
+	 * The blob file and subdatabase directories are of the form __db###,
+	 * so search the string for any directories that match that form.
+	 */
+	len = strlen(path);
+	do {
+		p = strstr(p, BLOB_DIR_PREFIX);
+		if (p == NULL || p > (path + len + 4))
+			return (ret);
+		p += 4;
+	} while (p[0] < '0' || p[0] > '9');
+
+	/* The file id should be next in the path. */
+	if ((ret = __blob_str_to_id(env, &p, file_id)) != 0)
+		return (ret);
+
+	/* Quit now if a subdatabase argument was not passed. */
+	if (sdb_id == NULL)
+		return (ret);
+
+	p = strstr(p, BLOB_DIR_PREFIX);
+	/* It is okay for the path not to include a sdb_id. */
+	if (p == NULL || p > (path + 4 + len))
+		return (ret);
+
+	p += 4;
+	ret = __blob_str_to_id(env, &p, sdb_id);
+
+	return (ret);
+}
+
 /*
  * __blob_salvage --
  *
  * Print a blob file during salvage.  The function assumes the DBT already has
  * a buffer large enough to hold "size" bytes.
  *
- * PUBLIC: int __blob_salvage __P((ENV *, uintmax_t, off_t, size_t,
- * PUBLIC:	uintmax_t, uintmax_t, DBT *));
+ * PUBLIC: int __blob_salvage __P((ENV *, db_seq_t, off_t, size_t,
+ * PUBLIC:	db_seq_t, db_seq_t, DBT *));
  */
 int
 __blob_salvage(env, blob_id, offset, size, file_id, sdb_id, dbt)
 	ENV *env;
-	uintmax_t blob_id;
+	db_seq_t blob_id;
 	off_t offset;
 	size_t size;
-	uintmax_t file_id;
-	uintmax_t sdb_id;
+	db_seq_t file_id;
+	db_seq_t sdb_id;
 	DBT *dbt;
 {
 	DB_FH *fhp;
 	char *blob_sub_dir, *dir, *path;
-	int ret;
+	int isdir, ret;
 	size_t bytes;
 
 	blob_sub_dir = dir = path = NULL;
 	fhp = NULL;
 
-	if (file_id == 0 && sdb_id == 0) {
-		ret = ENOENT;
+	if (blob_id < 1 || file_id < 0 ||
+	    sdb_id < 0 || (file_id == 0 && sdb_id == 0)) {
+		ret = USR_ERR(env, ENOENT);
 		goto err;
 	}
 
-	if ((ret = __blob_make_sub_dir(
-	    env, &blob_sub_dir, file_id, sdb_id)) != 0)
+	if ((ret = __blob_make_sub_dir(env, &blob_sub_dir,
+	    file_id, sdb_id)) != 0 || blob_sub_dir == NULL) {
+		if (ret == 0)
+			ret = USR_ERR(env, ENOENT);
 		goto err;
+	}
 
-	if ((ret = __blob_id_to_path(env, blob_sub_dir, blob_id, &dir)) != 0)
+	if ((ret = __blob_id_to_path(env, blob_sub_dir, blob_id, &dir, 0)) != 0)
 		goto err;
 
 	if ((ret = __db_appname(env, DB_APP_BLOB, dir, NULL, &path)) != 0)
 		goto err;
 
+	if ((__os_exists(env, path, &isdir)) != 0 || isdir != 0) {
+		ret = USR_ERR(env, ENOENT);
+		goto err;
+	}
+
 	if ((ret = __os_open(env, path, 0, DB_OSO_RDONLY, 0, &fhp)) != 0)
 		goto err;
 
@@ -527,7 +735,7 @@ __blob_salvage(env, blob_id, offset, size, file_id, sdb_id, dbt)
 		ret = EIO;
 
 err:	if (fhp != NULL)
-		__os_closehandle(env, fhp);
+		(void)__os_closehandle(env, fhp);
 	if (dir != NULL)
 		__os_free(env, dir);
 	if (path != NULL)
@@ -542,63 +750,65 @@ err:	if (fhp != NULL)
  *
  * Checks that a blob file for the given blob id exists, and is the given size.
  *
- * PUBLIC: int __blob_vrfy __P((ENV *, uintmax_t, off_t,
- * PUBLIC:	uintmax_t, uintmax_t, db_pgno_t, u_int32_t));
+ * PUBLIC: int __blob_vrfy __P((ENV *, db_seq_t, off_t,
+ * PUBLIC:	db_seq_t, db_seq_t, db_pgno_t, u_int32_t));
  */
 int
 __blob_vrfy(env, blob_id, blob_size, file_id, sdb_id, pgno, flags)
 	ENV *env;
-	uintmax_t blob_id;
+	db_seq_t blob_id;
 	off_t blob_size;
-	uintmax_t file_id;
-	uintmax_t sdb_id;
+	db_seq_t file_id;
+	db_seq_t sdb_id;
 	db_pgno_t pgno;
 	u_int32_t flags;
 {
 	DB_FH *fhp;
 	char *blob_sub_dir, *dir, *path;
-	int isdir, ret, t_ret;
+	int isdir, ret;
 	off_t actual_size;
 	u_int32_t mbytes, bytes;
 
 	blob_sub_dir = dir = path = NULL;
 	fhp = NULL;
 	isdir = 0;
+
+	if ((ret = __blob_make_sub_dir(
+	    env, &blob_sub_dir, file_id, sdb_id)) != 0 || blob_sub_dir == NULL) {
+		if (ret != ENOMEM)
+			ret = DB_VERIFY_BAD;
+		goto err;
+	}
+
 	ret = DB_VERIFY_BAD;
 
-	if ((t_ret = __blob_make_sub_dir(
-	    env, &blob_sub_dir, file_id, sdb_id)) != 0)
-		goto err;
-
-	if ((t_ret = __blob_id_to_path(
-	    env, blob_sub_dir, blob_id, &dir)) != 0) {
+	if (__blob_id_to_path(env, blob_sub_dir, blob_id, &dir, 0) != 0) {
 		EPRINT((env, DB_STR_A("0222",
-		    "Page %lu: Error getting path to blob file for %llu",
+		    "Page %lu: Error getting path to external file for %llu",
 		    "%lu %llu"), (u_long)pgno, (unsigned long long)blob_id));
 		goto err;
 	}
-	if ((t_ret = __db_appname(env, DB_APP_BLOB, dir, NULL, &path)) != 0) {
-		EPRINT((env, DB_STR_A("0223",
-		    "Page %lu: Error getting path to blob file for %llu",
+	if (__db_appname(env, DB_APP_BLOB, dir, NULL, &path) != 0) {
+		EPRINT((env, DB_STR_A("0222",
+		    "Page %lu: Error getting path to external file for %llu",
 		    "%lu %llu"), (u_long)pgno, (unsigned long long)blob_id));
 		goto err;
 	}
-	if ((t_ret = __os_exists(env, path, &isdir)) != 0 || isdir != 0) {
+	if ((__os_exists(env, path, &isdir)) != 0 || isdir != 0) {
 		EPRINT((env, DB_STR_A("0224",
-		    "Page %lu: blob file does not exist at %s",
+		    "Page %lu: external file does not exist at %s",
 		    "%lu %s"), (u_long)pgno, path));
 		goto err;
 	}
-	if ((t_ret = __os_open(env, path, 0, DB_OSO_RDONLY, 0, &fhp)) != 0) {
+	if (__os_open(env, path, 0, DB_OSO_RDONLY, 0, &fhp) != 0) {
 		EPRINT((env, DB_STR_A("0225",
-		    "Page %lu: Error opening blob file at %s",
+		    "Page %lu: Error opening external file at %s",
 		    "%lu %s"), (u_long)pgno, path));
 		goto err;
 	}
-	if ((t_ret = __os_ioinfo(
-	    env, path, fhp, &mbytes, &bytes, NULL)) != 0) {
+	if (__os_ioinfo(env, path, fhp, &mbytes, &bytes, NULL) != 0) {
 		EPRINT((env, DB_STR_A("0226",
-		    "Page %lu: Error getting blob file size at %s",
+		    "Page %lu: Error getting external file size at %s",
 		    "%lu %s"), (u_long)pgno, path));
 		goto err;
 	}
@@ -606,7 +816,7 @@ __blob_vrfy(env, blob_id, blob_size, file_id, sdb_id, pgno, flags)
 	actual_size = ((off_t)mbytes * (off_t)MEGABYTE) + bytes;
 	if (blob_size != actual_size) {
 		EPRINT((env, DB_STR_A("0227",
-"Page %lu: blob file size does not match size in database record: %llu %llu",
+"Page %lu: external file size does not match size in database record: %llu %llu",
 		    "%lu %llu %llu"), (u_long)pgno,
 		    (unsigned long long)actual_size,
 		    (unsigned long long)blob_size));
@@ -616,7 +826,7 @@ __blob_vrfy(env, blob_id, blob_size, file_id, sdb_id, pgno, flags)
 	ret = 0;
 
 err:	if (fhp != NULL)
-		__os_closehandle(env, fhp);
+		(void)__os_closehandle(env, fhp);
 	if (dir != NULL)
 		__os_free(env, dir);
 	if (path != NULL)
@@ -626,11 +836,40 @@ err:	if (fhp != NULL)
 	return (ret);
 }
 
+/*
+ * __blob_del_hierarchy --
+ *
+ * Deletes the entire blob directory.  Used by replication.
+ *
+ * PUBLIC: int __blob_del_hierarchy __P((ENV *));
+ */
+int
+__blob_del_hierarchy(env)
+	ENV *env;
+{
+	int ret;
+	char *blob_dir;
+
+	blob_dir = NULL;
+
+	if ((ret = __db_appname(env, DB_APP_BLOB, NULL, NULL, &blob_dir)) != 0)
+		goto err;
+
+	if ((ret = __blob_clean_dir(env, NULL, blob_dir, NULL, 0)) != 0)
+		goto err;
+
+err:	if (blob_dir != NULL)
+		__os_free(env, blob_dir);
+	return (ret);
+}
+
 /*
  * __blob_del_all --
  *
  * Deletes all the blob files and meta databases in a database's blob
- * directory.  Does not delete the directories.
+ * directory.  Does not delete the directories if the delete is transactionally
+ * protected, since there is no current way to undo a directory delete in case
+ * the operation is aborted.
  *
  * PUBLIC: int __blob_del_all __P((DB *, DB_TXN *, int));
  */
@@ -643,12 +882,11 @@ __blob_del_all(dbp, txn, istruncate)
 #ifdef HAVE_64BIT_TYPES
 	ENV *env;
 	char *path;
-	int close, isdir, ret;
+	int isdir, ret;
 
 	env = dbp->env;
 	path = NULL;
 	ret = 0;
-	close = 0;
 
 	if (dbp->blob_sub_dir == NULL) {
 		if ((ret = __blob_make_sub_dir(env, &dbp->blob_sub_dir,
@@ -657,7 +895,8 @@ __blob_del_all(dbp, txn, istruncate)
 	}
 
 	/* Do nothing if blobs are not enabled. */
-	if (dbp->blob_sub_dir == NULL || dbp->blob_threshold == 0)
+	if (dbp->blob_sub_dir == NULL ||
+	    (dbp->blob_file_id == 0 && dbp->blob_sdb_id == 0))
 		goto err;
 
 	if ((ret = __blob_get_dir(dbp, &path)) != 0)
@@ -666,11 +905,14 @@ __blob_del_all(dbp, txn, istruncate)
 	/* Close the blob meta data databases, they are about to be deleted. */
 	if (!istruncate) {
 		if (dbp->blob_seq != NULL) {
-			__seq_close(dbp->blob_seq, 0);
+			if ((ret = __seq_close(dbp->blob_seq, 0)) != 0)
+			    goto err;
 			dbp->blob_seq = NULL;
 		}
 		if (dbp->blob_meta_db != NULL) {
-			__db_close(dbp->blob_meta_db, NULL, 0);
+			if ((ret =
+			    __db_close(dbp->blob_meta_db, NULL, 0)) != 0)
+			    goto err;
 			dbp->blob_meta_db = NULL;
 		}
 	}
@@ -682,17 +924,23 @@ __blob_del_all(dbp, txn, istruncate)
 	if (__os_exists(env, path, &isdir) != 0)
 		goto err;
 
-	if ((ret = __blob_clean_dir(dbp, txn, path, istruncate)) != 0)
+	if ((ret = __blob_clean_dir(
+	    env, txn, path, dbp->blob_sub_dir, istruncate)) != 0)
 		goto err;
 
+	if (!IS_REAL_TXN(txn) && !istruncate) {
+		if ((ret = __os_rmdir(env, path)) != 0)
+			goto err;
+	}
+
 err:	if (path != NULL)
 		__os_free(env, path);
 	return (ret);
 
 #else /*HAVE_64BIT_TYPES*/
-        __db_errx(dbp->env, DB_STR("0220",
-            "library build did not include support for blobs"));
-        return (DB_OPNOTSUP);
+	__db_errx(dbp->env, DB_STR("0217",
+	    "library build did not include support for external files"));
+	return (DB_OPNOTSUP);
 #endif
 
 }
@@ -701,33 +949,36 @@ err:	if (path != NULL)
  * __blob_clean_dir --
  *
  * Delete all files in the given directory, and all files
- * in all sub-directories.  Does not remove directories.
+ * in all sub-directories.  Does not remove directories if the operation is
+ * transactionally protected.
  */
 static int
-__blob_clean_dir(dbp, txn, dir, istruncate)
-	DB *dbp;
+__blob_clean_dir(env, txn, dir, subdir, istruncate)
+	ENV *env;
 	DB_TXN *txn;
 	const char *dir;
+	const char *subdir;
 	int istruncate;
 {
 	DB *meta;
 	DB_THREAD_INFO *ip;
-	ENV *env;
-	char **dirs, *fname, full_path[DB_MAXPATHLEN];
+	char *blob_dir, **dirs, *fname, full_path[DB_MAXPATHLEN], *local_path;
 	int count, i, isdir, ret, t_ret;
 
-	env = dbp->env;
 	count = 0;
 	dirs = NULL;
 	fname = NULL;
 	meta = NULL;
 
 	/* Get a list of all files in the directory. */
-	if ((ret = __os_dirlist(env, dir, 1, &dirs, &count)) != 0)
+	if ((ret = __os_dirlist(env, dir, 1, &dirs, &count)) != 0) {
+		if (ret == ENOENT)
+			ret = 0;
 		goto err;
+	}
 
 	for (i = 0; i < count; i++) {
-		sprintf(full_path, "%s%c%s%c",
+		(void)sprintf(full_path, "%s%c%s%c",
 		    dir, PATH_SEPARATOR[0], dirs[i], '\0');
 
 		if (__os_exists(env, full_path, &isdir) != 0)
@@ -736,16 +987,25 @@ __blob_clean_dir(dbp, txn, dir, istruncate)
 		/* If it is a directory, clean it.  Else remove the file. */
 		if (isdir) {
 			if ((ret = __blob_clean_dir(
-			    dbp, txn, full_path, istruncate)) != 0)
+			    env, txn, full_path, subdir, istruncate)) != 0)
 				goto err;
+			/* Delete the top directory. */
+			if (!IS_REAL_TXN(txn)) {
+				if ((ret = __os_rmdir(env, full_path)) != 0)
+					goto err;
+			}
 		} else if (strcmp(dirs[i], BLOB_META_FILE_NAME) == 0 ) {
 			/* Ignore the meta db when truncating. */
 			if (istruncate)
 				continue;
-			__blob_make_meta_fname(env, dbp, &fname);
-			ENV_GET_THREAD_INFO(env, ip);
+			blob_dir = (env->dbenv->db_blob_dir != NULL ?
+			    env->dbenv->db_blob_dir : BLOB_DEFAULT_DIR);
+			if ((fname = strstr(full_path, blob_dir)) == NULL)
+				goto err;
+			fname += strlen(blob_dir) + 1;
 			if ((ret = __db_create_internal(&meta, env, 0)) != 0)
 				goto err;
+			ENV_GET_THREAD_INFO(env, ip);
 			if ((ret = __db_remove_int(meta,
 			    ip, txn, fname, NULL, 0)) != 0)
 				goto err;
@@ -767,9 +1027,11 @@ __blob_clean_dir(dbp, txn, dir, istruncate)
 			if (!IS_REAL_TXN(txn))
 				ret = __os_unlink(env, full_path, 0);
 			else {
-				ret = __fop_remove(env, txn, NULL,
-				    strstr(full_path, dbp->blob_sub_dir),
-				    NULL, DB_APP_BLOB, 0);
+				local_path = (subdir == NULL ? full_path :
+				    strstr(full_path, subdir));
+				if (local_path != NULL)
+					ret = __fop_remove(env, txn, NULL,
+					    local_path, NULL, DB_APP_BLOB, 0);
 			}
 			if (ret != 0)
 				goto err;
@@ -780,8 +1042,8 @@ err:	if (meta != NULL) {
 		    meta, NULL, DB_NOSYNC)) != 0 && ret == 0)
 			ret = t_ret;
 	}
-	if (fname != NULL)
-		__os_free(env, fname);
+	if (dirs != NULL)
+		__os_dirfree(env, dirs, count);
 
 	return (ret);
 }
@@ -790,21 +1052,23 @@ err:	if (meta != NULL) {
  * __blob_copy_all --
  *	Copy all files in the blob directory.
  *
- * PUBLIC: int __blob_copy_all __P((DB*, const char *));
+ * PUBLIC: int __blob_copy_all __P((DB*, const char *, u_int32_t));
  */
-int __blob_copy_all(dbp, target)
+int __blob_copy_all(dbp, target, flags)
 	DB *dbp;
 	const char *target;
+	u_int32_t flags;
 {
 	DB_THREAD_INFO *ip;
 	ENV *env;
-	char *fullname, new_target[DB_MAXPATHLEN], *metafname, *path;
+	char *blobdir, *fullname, *metafname, new_target[DB_MAXPATHLEN];
+	const char *path;
 	int ret;
 
-	fullname = NULL;
 	env = dbp->env;
+	blobdir = NULL;
+	fullname = NULL;
 	metafname = NULL;
-	path = NULL;
 	ret = 0;
 
 	/* Do nothing if blobs are not enabled. */
@@ -812,51 +1076,59 @@ int __blob_copy_all(dbp, target)
 		return (0);
 
 	/* Create the directory structure in the target directory. */
-	sprintf(new_target, "%s%c%s%c%c", target,
-	    PATH_SEPARATOR[0], BLOB_DEFAULT_DIR, PATH_SEPARATOR[0], '\0');
-	path = new_target;
-#ifdef DB_WIN32
+	if (env->dbenv->db_blob_dir != NULL)
+		path = env->dbenv->db_blob_dir;
+	else
+		path = BLOB_DEFAULT_DIR;
+
 	/*
-	 * Absolute paths on windows can result in it creating a "C" or "D"
-	 * directory in the working directory.
+	 * Default blob directory will be maintained in the target
+	 * directory only when it is backing up a single directory.
 	 */
-	if (__os_abspath(path))
-		path += 2;
-#endif
+	(void)snprintf(new_target, sizeof(new_target), "%s%c%s%c%c",
+	    target, PATH_SEPARATOR[0], LF_ISSET(DB_BACKUP_SINGLE_DIR) ?
+	    BLOB_DEFAULT_DIR : path, PATH_SEPARATOR[0], '\0');
+	path = new_target;
+
 	if ((ret = __db_mkpath(env, path)) != 0)
 		goto err;
 
 	/* Copy the directory id database. */
-	if ((ret = __blob_make_meta_fname(env, dbp, &metafname)) != 0)
+	if ((ret = __blob_make_meta_fname(env, NULL, &metafname)) != 0)
 		goto err;
-	ret = __db_appname(env, DB_APP_BLOB, metafname, NULL, &fullname);
-	__os_free(env, metafname);
-	metafname = NULL;
-	if (ret != 0)
+	if ((ret = __db_appname(env,
+	    DB_APP_BLOB, metafname, NULL, &fullname)) != 0)
 		goto err;
+	path = fullname;
+	/* Remove env home from the full path of directory id database. */
+	if (!__os_abspath(fullname) &&
+	    env->db_home != NULL && (env->db_home)[0] != '\0')
+		path += (strlen(env->db_home) + 1);
 	ENV_GET_THREAD_INFO(env, ip);
-	if ((ret = __db_dbbackup(
-	    dbp->dbenv, ip, fullname, new_target, 0, 0, NULL)) != 0)
-		    goto err;
 
-	if ((ret = __blob_get_dir(dbp, &path)) != 0)
+	if ((ret = __db_dbbackup(
+	    dbp->dbenv, ip, path, new_target, 0, 0, metafname)) != 0)
+		goto err;
+
+	if ((ret = __blob_get_dir(dbp, &blobdir)) != 0)
 		goto err;
 
 	/*
 	 * The blob directory may not exist if blobs were enabled,
 	 * but none were created.
 	 */
-	if (__os_exists(env, path, NULL) != 0)
+	if (__os_exists(env, blobdir, NULL) != 0)
 		goto err;
 
-	sprintf(new_target, "%s%c%s%c%s%c",
-	    target, PATH_SEPARATOR[0], BLOB_DEFAULT_DIR,
-	    PATH_SEPARATOR[0], dbp->blob_sub_dir, '\0');
-	if ((ret = __blob_copy_dir(dbp, path, new_target)) != 0)
+	(void)sprintf(new_target + strlen(new_target),
+	     "%s%c", dbp->blob_sub_dir, '\0');
+	if ((ret = __blob_copy_dir(dbp, blobdir, new_target)) != 0)
 		goto err;
 
-err:	if (path != NULL)
-		__os_free(env, path);
+err:	if (blobdir != NULL)
+		__os_free(env, blobdir);
+	if (metafname != NULL)
+		__os_free(env, metafname);
 	if (fullname != NULL)
 		__os_free(env, fullname);
 	return (ret);
@@ -892,7 +1164,7 @@ __blob_copy_dir(dbp, dir, target)
 		goto err;
 
 	for (i = 0; i < count; i++) {
-		sprintf(full_path, "%s%c%s%c",
+		(void)sprintf(full_path, "%s%c%s%c",
 		    dir, PATH_SEPARATOR[0], dirs[i], '\0');
 
 		if (__os_exists(env, full_path, &isdir) != 0)
@@ -904,7 +1176,7 @@ __blob_copy_dir(dbp, dir, target)
 		 * copy the file.
 		 */
 		if (isdir) {
-			sprintf(new_target,
+			(void)sprintf(new_target,
 			    "%s%c%s%c%c", target, PATH_SEPARATOR[0],
 			    dirs[i], PATH_SEPARATOR[0], '\0');
 			if ((ret = __blob_copy_dir(
@@ -912,7 +1184,7 @@ __blob_copy_dir(dbp, dir, target)
 				goto err;
 		} else {
 			if (strcmp(dirs[i], BLOB_META_FILE_NAME) == 0) {
-				sprintf(full_path, "%s%c%s%c",
+				(void)sprintf(full_path, "%s%c%s%c",
 				    dbp->blob_sub_dir,
 				    PATH_SEPARATOR[0], dirs[i], '\0');
 				if ((ret = __db_dbbackup(dbp->dbenv, ip,
@@ -926,5 +1198,9 @@ __blob_copy_dir(dbp, dir, target)
 			}
 		}
 	}
-err:	return (ret);
+
+err:
+	if (dirs != NULL)
+		__os_dirfree(env, dirs, count);
+	return (ret);
 }
