@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -638,15 +638,13 @@ __bamc_cmp(dbc, other_dbc, result)
 	DBC *dbc, *other_dbc;
 	int *result;
 {
-	ENV *env;
 	BTREE_CURSOR *bcp, *obcp;
 
-	env = dbc->env;
 	bcp = (BTREE_CURSOR *)dbc->internal;
 	obcp = (BTREE_CURSOR *)other_dbc->internal;
 
-	DB_ASSERT (env, bcp->pgno == obcp->pgno);
-	DB_ASSERT (env, bcp->indx == obcp->indx);
+	DB_ASSERT(dbc->env, bcp->pgno == obcp->pgno);
+	DB_ASSERT(dbc->env, bcp->indx == obcp->indx);
 
 	/* Check to see if both cursors have the same deleted flag. */
 	*result =
@@ -938,7 +936,7 @@ __bamc_get(dbc, key, data, flags, pgnop)
 	case DB_CURRENT:
 		/* It's not possible to return a deleted record. */
 		if (F_ISSET(cp, C_DELETED)) {
-			ret = DB_KEYEMPTY;
+			ret = DBC_ERR(dbc, DB_KEYEMPTY);
 			goto err;
 		}
 
@@ -979,7 +977,7 @@ __bamc_get(dbc, key, data, flags, pgnop)
 				goto err;
 			if (flags == DB_GET_BOTH) {
 				if (!exact) {
-					ret = DB_NOTFOUND;
+					ret = DBC_ERR(dbc, DB_NOTFOUND);
 					goto err;
 				}
 				break;
@@ -1000,7 +998,7 @@ __bamc_get(dbc, key, data, flags, pgnop)
 			    dbc, PGNO_INVALID, key, flags, &exact)) != 0)
 				return (ret);
 			if (!exact) {
-				ret = DB_NOTFOUND;
+				ret = DBC_ERR(dbc, DB_NOTFOUND);
 				goto err;
 			}
 
@@ -1047,7 +1045,7 @@ __bamc_get(dbc, key, data, flags, pgnop)
 		if ((ret = __bamc_next(dbc, 1, 0)) != 0)
 			goto err;
 		if (!IS_CUR_DUPLICATE(dbc, orig_pgno, orig_indx)) {
-			ret = DB_NOTFOUND;
+			ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 		break;
@@ -1077,7 +1075,7 @@ __bamc_get(dbc, key, data, flags, pgnop)
 		if ((ret = __bamc_prev(dbc)) != 0)
 			goto err;
 		if (!IS_CUR_DUPLICATE(dbc, orig_pgno, orig_indx)) {
-			ret = DB_NOTFOUND;
+			ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 		break;
@@ -1181,7 +1179,7 @@ __bam_bulk(dbc, data, flags)
 	db_indx_t *inp, indx, pg_keyoff;
 	int32_t  *endp, key_off, *offp, *saveoffp;
 	off_t blob_size;
-	uintmax_t blob_id;
+	db_seq_t blob_id;
 	u_int8_t *dbuf, *dp, *np;
 	u_int32_t key_size, pagesize, size, space;
 	int adj, is_key, need_pg, next_key, no_dup, rec_key, ret;
@@ -1275,10 +1273,9 @@ next_pg:
 		}
 
 		/*
-		 * Check to see if we have a new key.
-		 * If so, then see if we need to put the
-		 * key on the page.  If its already there
-		 * then we just point to it.
+		 * Check to see if we have a new key.  If so, then see if we
+		 * need to put the key on the page.  If it is already there then
+		 * we just point to it.
 		 */
 		if (is_key && pg_keyoff != inp[indx]) {
 			bk = GET_BKEYDATA(dbc->dbp, pg, indx);
@@ -1421,9 +1418,7 @@ get_key_space:
 			size = (u_int32_t)blob_size;
 			if (size > space)
 				goto back_up;
-			GET_BLOB_ID(dbc->env, bl, blob_id, ret);
-			if (ret != 0)
-				return (ret);
+			blob_id = (db_seq_t)bl.id;
 			if ((ret = __blob_bulk(dbc, size, blob_id, np)) != 0)
 				return (ret);
 			if (is_key) {
@@ -1531,7 +1526,15 @@ get_space:
 	 * If we are not fetching keys, we may have stepped to the
 	 * next key.
 	 */
-	if (ret == DB_BUFFER_SMALL || next_key || pg_keyoff == inp[indx])
+	if (ret == DB_NOTFOUND && next_key && cp->indx >= NUM_ENT(cp->page)) {
+		/*
+		 * We have run over the last record on the last page,
+		 * back up the index.
+		 */
+		cp->indx -= adj;
+		if (dbc->dbtype == DB_RECNO)
+			cp->recno--;
+	} else if (ret == DB_BUFFER_SMALL || next_key || pg_keyoff == inp[indx])
 		cp->indx = indx;
 	else
 		cp->indx = indx - P_INDX;
@@ -1794,12 +1797,12 @@ __bam_getbothc(dbc, data)
 		 * the current position;  if it doesn't, return DB_NOTFOUND.
 		 */
 		if ((ret = __bam_cmp(dbc, data, cp->page, cp->indx,
-		    dbp->dup_compare == NULL ? __bam_defcmp : dbp->dup_compare,
+		    dbp->dup_compare == NULL ? __dbt_defcmp : dbp->dup_compare,
 		    &cmp, NULL)) != 0)
 			return (ret);
 
 		if (cmp <= 0)
-			return (DB_NOTFOUND);
+			return (DBC_ERR(dbc, DB_NOTFOUND));
 
 		/* Discard the current page, we're going to do a full search. */
 		if ((ret = __memp_fput(mpf,
@@ -1822,7 +1825,7 @@ __bam_getbothc(dbc, data)
 	 */
 	if (cp->indx + P_INDX >= NUM_ENT(cp->page) ||
 	    !IS_DUPLICATE(dbc, cp->indx, cp->indx + P_INDX))
-		return (DB_NOTFOUND);
+		return (DBC_ERR(dbc, DB_NOTFOUND));
 	cp->indx += P_INDX;
 
 	return (__bam_getboth_finddatum(dbc, data, DB_GET_BOTH));
@@ -1836,7 +1839,7 @@ __bam_getbothc(dbc, data)
  *	data == NULL indicates the DB_SET_LTE flag
  *	data != NULL indicates the DB_GET_BOTH_LTE flag
  *
- *	Only works for a primary cursor - not an OPD cursor. Handles the
+ *	Only works for a primary cursor - not an OPD cursor.  Handles the
  *	OPD manipulation as well - no need to return to the caller to
  *	perform more OPD movements.
  */
@@ -2011,10 +2014,10 @@ __bam_getboth_finddatum(dbc, data, flags)
 	 */
 	if (dbp->dup_compare == NULL) {
 		for (;; cp->indx += P_INDX) {
-			if (!IS_CUR_DELETED(dbc)) {
+			if (!IS_DELETED(dbp, cp->page, cp->indx)) {
 				if ((ret = __bam_cmp(
 				    dbc, data, cp->page, cp->indx + O_INDX,
-				    __bam_defcmp, &cmp, NULL)) != 0)
+				    __dbt_defcmp, &cmp, NULL)) != 0)
 					return (ret);
 				if (cmp == 0)
 					return (0);
@@ -2024,7 +2027,8 @@ __bam_getboth_finddatum(dbc, data, flags)
 			    !IS_DUPLICATE(dbc, cp->indx, cp->indx + P_INDX))
 				break;
 		}
-		return (DB_NOTFOUND);
+
+		return (DBC_ERR(dbc, DB_NOTFOUND));
 	}
 
 	/*
@@ -2045,7 +2049,7 @@ __bam_getboth_finddatum(dbc, data, flags)
 		if (cmp == 0 || (cmp < 0 && flags == DB_GET_BOTH_RANGE))
 			return (0);
 		cp->indx = top;
-		return DB_NOTFOUND;
+		return (DBC_ERR(dbc, DB_NOTFOUND));
 	}
 
 	for (lim = (top - base) / (db_indx_t)P_INDX; lim != 0; lim >>= 1) {
@@ -2055,12 +2059,17 @@ __bam_getboth_finddatum(dbc, data, flags)
 			return (ret);
 		if (cmp == 0) {
 			/*
-			 * XXX
+			 * !!!
 			 * No duplicate duplicates in sorted duplicate sets,
 			 * so there can be only one.
 			 */
 			if (!IS_CUR_DELETED(dbc))
 				return (0);
+			/*
+			 * Advance base so that by the end of the loop, base is
+			 * always the smallest index greater than the data item.
+			 */
+			base = cp->indx + P_INDX;
 			break;
 		}
 		if (cmp > 0) {
@@ -2071,7 +2080,7 @@ __bam_getboth_finddatum(dbc, data, flags)
 
 	/* No match found; if we're looking for an exact match, we're done. */
 	if (flags == DB_GET_BOTH)
-		return (DB_NOTFOUND);
+		return (DBC_ERR(dbc, DB_NOTFOUND));
 
 	/*
 	 * Base is the smallest index greater than the data item, may be zero
@@ -2081,7 +2090,7 @@ __bam_getboth_finddatum(dbc, data, flags)
 	cp->indx = base;
 	while (cp->indx < top && IS_CUR_DELETED(dbc))
 		cp->indx += P_INDX;
-	return (cp->indx < top ? 0 : DB_NOTFOUND);
+	return (cp->indx < top ? 0 : DBC_ERR(dbc, DB_NOTFOUND));
 }
 
 /*
@@ -2114,7 +2123,7 @@ split:	ret = stack = 0;
 	switch (flags) {
 	case DB_CURRENT:
 		if (F_ISSET(cp, C_DELETED))
-			return (DB_NOTFOUND);
+			return (DBC_ERR(dbc, DB_NOTFOUND));
 		/* FALLTHROUGH */
 	case DB_AFTER:
 	case DB_BEFORE:
@@ -2512,7 +2521,7 @@ __bamc_next(dbc, initial_move, deleted_okay)
 		 */
 		if (cp->indx >= NUM_ENT(cp->page)) {
 			if ((pgno = NEXT_PGNO(cp->page)) == PGNO_INVALID)
-				return (DB_NOTFOUND);
+				return (DBC_ERR(dbc, DB_NOTFOUND));
 
 			ACQUIRE_CUR(dbc, lock_mode, pgno, 0, ret);
 			if (ret != 0)
@@ -2572,7 +2581,7 @@ __bamc_prev(dbc)
 		if (cp->indx == 0) {
 			if ((pgno =
 			    PREV_PGNO(cp->page)) == PGNO_INVALID)
-				return (DB_NOTFOUND);
+				return (DBC_ERR(dbc, DB_NOTFOUND));
 
 			ACQUIRE_CUR(dbc, lock_mode, pgno, 0, ret);
 			if (ret != 0)
@@ -2748,7 +2757,7 @@ __bamc_search(dbc, root_pgno, key, flags, exactp)
 			goto fast_miss;
 		if (cmp > 0) {
 			if (FLD_ISSET(sflags, SR_EXACT))
-				return (DB_NOTFOUND);
+				return (DBC_ERR(dbc, DB_NOTFOUND));
 			else
 				indx += P_INDX;
 		}
@@ -2761,7 +2770,7 @@ __bamc_search(dbc, root_pgno, key, flags, exactp)
 		    t->bt_compare, &cmp, NULL)) != 0)
 			goto fast_miss;
 		if (cmp < 0 && FLD_ISSET(sflags, SR_EXACT))
-			return (DB_NOTFOUND);
+			return (DBC_ERR(dbc, DB_NOTFOUND));
 		if (cmp <= 0)
 			goto fast_hit;
 	}
@@ -2785,7 +2794,7 @@ __bamc_search(dbc, root_pgno, key, flags, exactp)
 		indx = base;
 		if (indx > 0 && indx < NUM_ENT(h)) {
 			if (FLD_ISSET(sflags, SR_EXACT))
-				return (DB_NOTFOUND);
+				return (DBC_ERR(dbc, DB_NOTFOUND));
 			goto fast_hit;
 		}
 	}
@@ -3101,7 +3110,7 @@ __bam_opd_exists(dbc, pgno)
 	if (NUM_ENT(h) == 0)
 		ret = 0;
 	else
-		ret = DB_KEYEXIST;
+		ret = DBC_ERR(dbc, DB_KEYEXIST);
 
 	(void)__memp_fput(dbc->dbp->mpf, dbc->thread_info, h, dbc->priority);
 
