@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2011, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2011, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -24,6 +24,7 @@ static int backup_read_data_dir
     __P((DB_ENV *, DB_THREAD_INFO *, const char *, const char *, u_int32_t));
 static int backup_dir_clean
     __P((DB_ENV *, const char *, const char *, int *, u_int32_t));
+static int backup_lgconf_chk __P((DB_ENV *));
 static int __db_backup
     __P((DB_ENV *, const char *, DB_THREAD_INFO *, int, u_int32_t));
 
@@ -93,6 +94,11 @@ retry:	if ((ret = __db_create_internal(&dbp, dbenv->env, 0)) == 0 &&
 		}
 	}
 
+	/* Hot backup requires DB_LOG_BLOB/DB_LOG_EXT_FILE. */
+	if (ret == 0 && dbp->blob_threshold != 0 &&
+	    (ret = backup_lgconf_chk(dbenv)) != 0)
+		goto err;
+
 	if (full_path == NULL)
 		full_path = dbfile;
 	if (ret == 0) {
@@ -115,12 +121,12 @@ retry:	if ((ret = __db_create_internal(&dbp, dbenv->env, 0)) == 0 &&
 	/*
 	 * Copy blob files.  Since no locking is done here, it is possible
 	 * that a blob file may be copied in the middle of being written.
-	 * This is not a problem since hotbackup requires DB_LOG_BLOB and
-	 * catastrophic recovery, which will fix any inconsistances in the
-	 * blob files.
+	 * This is not a problem since hotbackup requires
+	 * DB_LOG_BLOB/DB_LOG_EXT_FILE and catastrophic recovery, which will
+	 * fix any inconsistances in the blob files.
 	 */
 	if (ret == 0 && dbp->blob_threshold != 0 &&
-	    (t_ret = __blob_copy_all(dbp, target)) != 0)
+	    (t_ret = __blob_copy_all(dbp, target, flags)) != 0)
 		ret= t_ret;
 
 #ifdef HAVE_QUEUE
@@ -132,7 +138,7 @@ retry:	if ((ret = __db_create_internal(&dbp, dbenv->env, 0)) == 0 &&
 		ret = __qam_backup_extents(dbp, ip, target, flags);
 #endif
 
-	if (dbp != NULL &&
+err:	if (dbp != NULL &&
 	    (t_ret = __db_close(dbp, NULL, DB_NOSYNC)) != 0 && ret == 0)
 		ret = t_ret;
 
@@ -164,7 +170,7 @@ backup_dir_clean(dbenv, backup_dir, log_dir, remove_maxp, flags)
 		if ((ret = __os_concat_path(buf,
 		    sizeof(buf), backup_dir, log_dir)) != 0) {
 			buf[sizeof(buf) - 1] = '\0';
-			__db_errx(env,  DB_STR_A("0717",
+			__db_errx(env,  DB_STR_A("0714",
 			    "%s: path too long", "%s"), buf);
 			return (EINVAL);
 		}
@@ -249,14 +255,14 @@ backup_data_copy(dbenv, file, from_dir, to_dir, log)
 	if ((ret = __os_concat_path(from,
 	    sizeof(from), from_dir, file)) != 0) {
 		from[sizeof(from) - 1] = '\0';
-		__db_errx(env, DB_STR_A("0728",
+		__db_errx(env, DB_STR_A("0714",
 		     "%s: path too long", "%s"), from);
 		goto err;
 	}
 	if ((ret = __os_concat_path(to,
 	    sizeof(to), to_dir, file)) != 0) {
 		to[sizeof(to) - 1] = '\0';
-		__db_errx(env, DB_STR_A("0729",
+		__db_errx(env, DB_STR_A("0714",
 		     "%s: path too long", "%s"), to);
 		goto err;
 	}
@@ -280,7 +286,7 @@ backup_data_copy(dbenv, file, from_dir, to_dir, log)
 				    from_dir, PATH_SEPARATOR[0], file);
 			goto done;
 		}
-		__db_err(env, ret, "%s", buf);
+		__db_err(env, ret, "%s", from);
 		goto err;
 	}
 
@@ -370,13 +376,16 @@ backup_read_data_dir(dbenv, ip, dir, backup_dir, flags)
 	ENV *env;
 	FILE *savefile;
 	int fcnt, ret;
-	size_t cnt;
+	size_t cnt, len;
 	const char *bd;
 	char **names, buf[DB_MAXPATHLEN], bbuf[DB_MAXPATHLEN];
+	char fullpath[DB_MAXPATHLEN];
 	void (*savecall) (const DB_ENV *, const char *, const char *);
 
 	env = dbenv->env;
 	memset(bbuf, 0, sizeof(bbuf));
+	memset(fullpath, 0, sizeof(fullpath));
+	len = 0;
 
 	bd = backup_dir;
 	if (!LF_ISSET(DB_BACKUP_SINGLE_DIR) && dir != env->db_home) {
@@ -389,7 +398,7 @@ backup_read_data_dir(dbenv, ip, dir, backup_dir, flags)
 		    strchr(PATH_SEPARATOR, bbuf[cnt - 1]) == NULL)) &&
 		    LF_ISSET(DB_CREATE))) {
 			bbuf[sizeof(bbuf) - 1] = '\0';
-			__db_errx(env, DB_STR_A("0720",
+			__db_errx(env, DB_STR_A("0714",
 			    "%s: path too long", "%s"), bbuf);
 			return (1);
 		}
@@ -415,15 +424,21 @@ backup_read_data_dir(dbenv, ip, dir, backup_dir, flags)
 		if ((ret = __os_concat_path(buf,
 		    sizeof(buf), env->db_home, dir)) != 0) {
 			buf[sizeof(buf) - 1] = '\0';
-			__db_errx(env, DB_STR_A("0722",
+			__db_errx(env, DB_STR_A("0714",
 			    "%s: path too long", "%s"), buf);
 			return (EINVAL);
 		}
+		/* Save the original dir. */
+		if (!LF_ISSET(DB_BACKUP_SINGLE_DIR)) {
+			(void)snprintf(fullpath, sizeof(fullpath),
+			    "%s%c%c", dir, PATH_SEPARATOR[0], '\0');
+			len = strlen(fullpath);
+		}
 		dir = buf;
 	}
 	/* Get a list of file names. */
 	if ((ret = __os_dirlist(env, dir, 0, &names, &fcnt)) != 0) {
-		__db_err(env, ret, DB_STR_A("0723", "%s: directory read",
+		__db_err(env, ret, DB_STR_A("0718", "%s: directory read",
 		    "%s"), dir);
 		return (ret);
 	}
@@ -467,7 +482,16 @@ backup_read_data_dir(dbenv, ip, dir, backup_dir, flags)
 		savefile = dbenv->db_errfile;
 		dbenv->db_errfile = NULL;
 
-		ret = __db_dbbackup(dbenv, ip, names[cnt], bd, flags, 0, NULL);
+		/*
+		 * If it is not backing up to a single directory, prefix
+		 * the file with 'dir' so that the file and directory structure
+		 * in the source and backup location will be the same.
+		 */
+		if (len != 0)
+			(void)snprintf(fullpath + len,
+			    sizeof(fullpath) - len, "%s%c", names[cnt], '\0');
+		ret = __db_dbbackup(dbenv, ip, names[cnt],
+		    backup_dir, flags, 0, len != 0 ? fullpath : NULL);
 
 		dbenv->db_errcall = savecall;
 		dbenv->db_errfile = savefile;
@@ -542,7 +566,7 @@ backup_read_log_dir(dbenv, backup_dir, copy_minp, flags)
 			    strchr(PATH_SEPARATOR, to[cnt - 1]) == NULL)) &&
 			    LF_ISSET(DB_CREATE))) {
 				to[sizeof(to) - 1] = '\0';
-				__db_errx(env, DB_STR_A("0733",
+				__db_errx(env, DB_STR_A("0714",
 				    "%s: path too long", "%s"), to);
 				goto err;
 			}
@@ -551,7 +575,7 @@ backup_read_log_dir(dbenv, backup_dir, copy_minp, flags)
 					to[cnt] = PATH_SEPARATOR[0];
 
 				if ((ret = __db_mkpath(env, to)) != 0) {
-					__db_err(env, ret, DB_STR_A("0734",
+					__db_err(env, ret, DB_STR_A("0721",
 					    "%s: cannot create", "%s"), to);
 					goto err;
 				}
@@ -564,7 +588,7 @@ backup_read_log_dir(dbenv, backup_dir, copy_minp, flags)
 			if ((ret = __os_concat_path(from,
 			    sizeof(from), env->db_home, logd)) != 0) {
 				from[sizeof(from) - 1] = '\0';
-				__db_errx(env, DB_STR_A("0732",
+				__db_errx(env, DB_STR_A("0714",
 				    "%s: path too long", "%s"), from);
 				goto err;
 			}
@@ -606,7 +630,7 @@ again:	aflag = DB_ARCH_LOG;
 		if ((ret = __os_concat_path(from,
 		    sizeof(from), logd, *names)) != 0) {
 			from[sizeof(from) - 1] = '\0';
-			__db_errx(env, DB_STR_A("0737",
+			__db_errx(env, DB_STR_A("0714",
 			    "%s: path too long", "%s"), from);
 			goto err;
 		}
@@ -623,7 +647,7 @@ again:	aflag = DB_ARCH_LOG;
 			if ((ret = __os_concat_path(to,
 			    sizeof(to), backupd, *names)) != 0) {
 				to[sizeof(to) - 1] = '\0';
-				__db_errx(env, DB_STR_A("0738",
+				__db_errx(env, DB_STR_A("0714",
 				    "%s: path too long", "%s"), to);
 				goto err;
 			}
@@ -644,7 +668,7 @@ again:	aflag = DB_ARCH_LOG;
 
 		if (update) {
 			if (FLD_ISSET(dbenv->verbose, DB_VERB_BACKUP))
-				__db_msg(env, DB_STR_A("0740",
+				__db_msg(env, DB_STR_A("0715",
 				    "removing %s", "%s"), from);
 			if ((ret = __os_unlink(env, from, 0)) != 0) {
 				__db_err(env, ret, DB_STR_A("0741",
@@ -690,9 +714,11 @@ __db_backup_pp(dbenv, target, flags)
 {
 	DB_THREAD_INFO *ip;
 	ENV *env;
+	u_int32_t bytes;
 	int remove_max, ret;
 
 	env = dbenv->env;
+	bytes = 0;
 	remove_max = 0;
 
 #undef	OKFLAGS
@@ -709,6 +735,17 @@ __db_backup_pp(dbenv, target, flags)
 		return (EINVAL);
 	}
 
+	if (LF_ISSET(DB_BACKUP_UPDATE) && LF_ISSET(DB_BACKUP_NO_LOGS)) {
+		__db_errx(env, DB_STR("5501",
+	"DB_BACKUP_UPDATE and DB_BACKUP_NO_LOGS cannot be used together."));
+		return (EINVAL);
+	}
+
+	/* Hot backup requires DB_LOG_BLOB/DB_LOG_EXT_FILE. */
+	if ((ret = __env_get_blob_threshold_int(env, &bytes)) != 0 ||
+	    (bytes != 0 && (ret = backup_lgconf_chk(dbenv)) != 0))
+		return (ret);
+
 	/*
 	 * If the target directory for the backup does not exist, create it
 	 * with mode read-write-execute for the owner.  Ignore errors here,
@@ -765,6 +802,19 @@ __db_backup(dbenv, target, ip, remove_max, flags)
 		goto end;
 	F_SET(dbenv, DB_ENV_HOTBACKUP);
 	if (!LF_ISSET(DB_BACKUP_UPDATE)) {
+		/*
+		 * Don't allow absolute path of blob directory when
+		 * it is not backing up to a single directory.
+		 */
+		if (!LF_ISSET(DB_BACKUP_SINGLE_DIR) &&
+		    dbenv->db_blob_dir != NULL &&
+		    __os_abspath(dbenv->db_blob_dir)) {
+			ret = USR_ERR(env, EINVAL);
+			__db_errx(env, DB_STR_A("0780",
+"external file directory '%s' is absolute path, not permitted unless backup is to a single directory",
+			"%s"), dbenv->db_blob_dir);
+			goto err;
+		}
 		if ((ret = backup_read_data_dir(dbenv,
 		    ip, env->db_home, target, flags)) != 0)
 			goto err;
@@ -777,10 +827,10 @@ __db_backup(dbenv, target, ip, remove_max, flags)
 			 */
 			if (!LF_ISSET(DB_BACKUP_SINGLE_DIR) &&
 			    __os_abspath(*dir)) {
+				ret = USR_ERR(env, EINVAL);
 				__db_errx(env, DB_STR_A("0725",
 "data directory '%s' is absolute path, not permitted unless backup is to a single directory",
 				    "%s"), *dir);
-				ret = EINVAL;
 				goto err;
 			}
 			if ((ret = backup_read_data_dir(
@@ -789,27 +839,64 @@ __db_backup(dbenv, target, ip, remove_max, flags)
 		}
 	}
 
-	/*
-	 * Copy all log files found in the log directory.
-	 * The log directory defaults to the home directory.
-	 */
-	if ((ret = backup_read_log_dir(dbenv, target, &copy_min, flags)) != 0)
-		goto err;
-	/*
-	 * If we're updating a snapshot, the lowest-numbered log file copied
-	 * into the backup directory should be less than, or equal to, the
-	 * highest-numbered log file removed from the backup directory during
-	 * cleanup.
-	 */
-	if (LF_ISSET(DB_BACKUP_UPDATE) && remove_max < copy_min &&
-	    remove_max != 0 && copy_min != 1) {
-		__db_errx(env, DB_STR_A("0743",
+	if (!LF_ISSET(DB_BACKUP_NO_LOGS)) {
+		/*
+		 * Copy all log files found in the log directory.
+		 * The log directory defaults to the home directory.
+		 * Don't allow absolute path of log directory when
+		 * it is not backing up to a single directory.
+		 */
+		if (!LF_ISSET(DB_BACKUP_SINGLE_DIR) &&
+		    dbenv->db_log_dir != NULL &&
+		    __os_abspath(dbenv->db_log_dir)) {
+			ret = USR_ERR(env, EINVAL);
+			__db_errx(env, DB_STR_A("0781",
+"log directory '%s' is absolute path, not permitted unless backup is to a single directory",
+			    "%s"), dbenv->db_log_dir);
+			goto err;
+		}
+		if ((ret = backup_read_log_dir(
+			dbenv, target, &copy_min, flags)) != 0)
+			goto err;
+		/*
+		 * If we're updating a snapshot, the lowest-numbered log file
+		 * copied into the backup directory should be less than,
+		 * or equal to, the highest-numbered log file removed from the
+		 * backup directory during cleanup.
+		 */
+		if (LF_ISSET(DB_BACKUP_UPDATE) && remove_max < copy_min &&
+		    remove_max != 0 && copy_min != 1) {
+			ret = USR_ERR(env, EINVAL);
+			__db_errx(env, DB_STR_A("0743",
 "the largest log file removed (%d) must be greater than or equal the smallest log file copied (%d)",
-		    "%d %d"), remove_max, copy_min);
-		ret = EINVAL;
+			    "%d %d"), remove_max, copy_min);
+		}
 	}
 
 err:	F_CLR(dbenv, DB_ENV_HOTBACKUP);
 	(void)__env_set_backup(env, 0);
 end:	return (ret);
 }
+
+/*
+ * __db_backup_fchk --
+ *	Log configure checking for backup when blob is enabled.
+ */
+static int
+backup_lgconf_chk(dbenv)
+	DB_ENV *dbenv;
+{
+	int lgconf, ret;
+
+	ret = 0;
+
+	if (LOGGING_ON(dbenv->env) && ((ret = __log_get_config(dbenv,
+	    DB_LOG_EXT_FILE, &lgconf)) != 0 || lgconf == 0)) {
+		if (ret == 0)
+			ret = USR_ERR(dbenv->env, EINVAL);
+		__db_errx(dbenv->env, DB_STR("0782",
+		    "Hot backup requires DB_LOG_EXT_FILE"));
+	}
+
+	return (ret);
+}
