@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  */
 /*
  * Copyright (c) 1990, 1993, 1994, 1995, 1996
@@ -46,10 +46,16 @@
 #include "dbinc/db_page.h"
 #include "dbinc/db_swap.h"
 #include "dbinc/btree.h"
+#include "dbinc/fop.h"
 #include "dbinc/hash.h"
 #include "dbinc/heap.h"
 #include "dbinc/qam.h"
 
+static int __db_convert_extent
+    __P((ENV *, const char *, u_int32_t, u_int32_t));
+static int __db_convert_extent_names __P((DB *, DBMETA *, char *, char ***));
+static int __db_swap __P((DB *, char *, u_int32_t, DB_FH *, PAGE *, int *));
+
 /*
  * __db_pgin --
  *	Primary page-swap routine.
@@ -538,16 +544,12 @@ __db_byteswap(dbp, pg, h, pagesize, pgin)
 
 			switch (HPAGE_TYPE(dbp, h, i)) {
 			case H_BLOB:
-				p = HBLOB_ID_LO(P_ENTRY(dbp, h, i));
-				SWAP32(p);			/* id lo */
-				SWAP32(p);			/* id hi */
-				SWAP32(p);			/* size lo */
-				SWAP32(p);			/* size hi */
-				p = HBLOB_FILE_ID_LO(P_ENTRY(dbp, h, i));
-				SWAP32(p);			/* file id lo */
-				SWAP32(p);			/* file id hi */
-				SWAP32(p);			/* sdb id lo */
-				SWAP32(p);			/* sdb id hi */
+				p = HBLOB_ID(P_ENTRY(dbp, h, i));
+				SWAP64(p);			/* id */
+				SWAP64(p);			/* size */
+				p = HBLOB_FILE_ID(P_ENTRY(dbp, h, i));
+				SWAP64(p);			/* file id */
+				SWAP64(p);			/* sdb id */
 				break;
 			case H_KEYDATA:
 				break;
@@ -623,14 +625,11 @@ __db_byteswap(dbp, pg, h, pagesize, pgin)
 			switch (B_TYPE(bk->type)) {
 			case B_BLOB:
 				bl = (BBLOB *)bk;
-				M_32_SWAP(bl->id_lo);		/* id lo */
-				M_32_SWAP(bl->id_hi);		/* id hi */
-				M_32_SWAP(bl->size_lo);		/* size lo */
-				M_32_SWAP(bl->size_hi);		/* size hi */
-				M_32_SWAP(bl->file_id_lo);	/* file id lo */
-				M_32_SWAP(bl->file_id_hi);	/* file id hi */
-				M_32_SWAP(bl->sdb_id_lo);	/* sdb id lo */
-				M_32_SWAP(bl->sdb_id_hi);	/* sdb id hi */
+				M_16_SWAP(bl->len);
+				M_64_SWAP(bl->id);		/* id */
+				M_64_SWAP(bl->size);		/* size */
+				M_64_SWAP(bl->file_id);		/* file id */
+				M_64_SWAP(bl->sdb_id);		/* sdb id */
 				break;
 			case B_KEYDATA:
 				M_16_SWAP(bk->len);
@@ -696,9 +695,11 @@ __db_byteswap(dbp, pg, h, pagesize, pgin)
 		}
 		break;
 	case P_HEAP:
-		for (i = 0; i < NUM_ENT(h); i++) {
+		for (i = 0; i <= HEAP_HIGHINDX(h); i++) {
 			if (pgin)
 				M_16_SWAP(inp[i]);
+			if (inp[i] == 0)
+				continue;
 
 			hh = (HEAPHDR *)P_ENTRY(dbp, h, i);
 			if ((u_int8_t *)hh >= pgend)
@@ -711,13 +712,9 @@ __db_byteswap(dbp, pg, h, pagesize, pgin)
 				M_16_SWAP(hsh->nextindx);
 			} else if (F_ISSET(hh, HEAP_RECBLOB)) {
 				bhdr = (HEAPBLOBHDR *)hh;
-				M_32_SWAP(bhdr->id_lo);		/* id lo */
-				M_32_SWAP(bhdr->id_hi);		/* id hi */
-				M_32_SWAP(bhdr->size_lo);	/* size lo */
-				M_32_SWAP(bhdr->size_hi);	/* size hi */
-				M_32_SWAP(bhdr->file_id_lo);	/* file id lo */
-				M_32_SWAP(bhdr->file_id_hi);	/* file id hi */
-				break;
+				M_64_SWAP(bhdr->id);		/* id */
+				M_64_SWAP(bhdr->size);		/* size */
+				M_64_SWAP(bhdr->file_id);	/* file id */
 			}
 
 			if (!pgin)
@@ -785,7 +782,10 @@ __db_pageswap(env, dbp, pp, len, pdata, pgin)
 
 	case P_HASHMETA:
 		return (__ham_mswap(env, pp));
-
+#ifdef HAVE_HEAP
+	case P_HEAPMETA:
+		return (__heap_mswap(env, pp));
+#endif
 	case P_QAMMETA:
 		return (__qam_mswap(env, pp));
 
@@ -861,14 +861,17 @@ __db_recordswap(op, size, hdr, data, pgin)
 	void *hdr, *data;
 	u_int32_t pgin;
 {
+	BBLOB *bl;
 	BKEYDATA *bk;
 	BOVERFLOW *bo;
 	BINTERNAL *bi;
+	DBT *dbt;
 	HEAPHDR *hh;
+	HEAPBLOBHDR bhdr;
 	HEAPSPLITHDR *hsh;
 	RINTERNAL *ri;
 	db_indx_t tmp;
-	u_int8_t *p, *end;
+	u_int8_t buf[HEAPBLOBREC_SIZE], *end, *p;
 
 	if (size == 0)
 		return;
@@ -881,6 +884,14 @@ __db_recordswap(op, size, hdr, data, pgin)
 		case B_KEYDATA:
 			M_16_SWAP(bk->len);
 			break;
+		case B_BLOB:
+			bl = (BBLOB *)bk;
+			M_16_SWAP(bl->len);
+			M_64_SWAP(bl->id);		/* id */
+			M_64_SWAP(bl->size);		/* size */
+			M_64_SWAP(bl->file_id);		/* file id */
+			M_64_SWAP(bl->sdb_id);		/* sdb id */
+			break;
 		case B_DUPLICATE:
 		case B_OVERFLOW:
 			bo = (BOVERFLOW *)hdr;
@@ -904,6 +915,7 @@ __db_recordswap(op, size, hdr, data, pgin)
 			} else
 				bo = (BOVERFLOW *)data;
 			M_32_SWAP(bo->pgno);
+			M_32_SWAP(bo->tlen);
 		}
 		break;
 	case P_IRECNO:
@@ -939,7 +951,7 @@ __db_recordswap(op, size, hdr, data, pgin)
 		/* These three record types include the full header. */
 		case H_OFFDUP:
 			p = (u_int8_t *)hdr;
-			p += SSZ(HOFFPAGE, pgno);
+			p += SSZ(HOFFDUP, pgno);
 			SWAP32(p);			/* pgno */
 			break;
 		case H_OFFPAGE:
@@ -949,12 +961,12 @@ __db_recordswap(op, size, hdr, data, pgin)
 			SWAP32(p);			/* tlen */
 			break;
 		case H_BLOB:
-			p = (u_int8_t *)hdr;
-			p += SSZ(HBLOB, id_lo);
-			SWAP32(p);			/* id low */
-			SWAP32(p);			/* id high */
-			SWAP32(p);			/* size low */
-			SWAP32(p);			/* size high */
+			p = HBLOB_ID(hdr);
+			SWAP64(p);			/* id */
+			SWAP64(p);			/* size */
+			p = HBLOB_FILE_ID(hdr);
+			SWAP64(p);			/* file id */
+			SWAP64(p);			/* sdb id */
 			break;
 		default:
 			DB_ASSERT(NULL, op != op);
@@ -968,9 +980,304 @@ __db_recordswap(op, size, hdr, data, pgin)
 			M_32_SWAP(hsh->tsize);
 			M_32_SWAP(hsh->nextpg);
 			M_16_SWAP(hsh->nextindx);
+		}else if (F_ISSET(hh, HEAP_RECBLOB)) {
+			/*
+			 * Heap blob records are broken into two parts when
+			 * logged, the shared header and the part that is
+			 * unique to blob records, which is stored in the
+			 * log data field.
+			 */
+			if (data != NULL) {
+				dbt = NULL;
+				if (pgin) {
+					dbt = data;
+					memcpy(buf + sizeof(HEAPHDR),
+					    dbt->data, HEAPBLOBREC_DSIZE);
+				} else {
+					memcpy(buf + sizeof(HEAPHDR),
+					    data, HEAPBLOBREC_DSIZE);
+				}
+				memcpy(&bhdr, buf, HEAPBLOBREC_SIZE);
+				M_64_SWAP(bhdr.id);		/* id */
+				M_64_SWAP(bhdr.size);		/* size */
+				M_64_SWAP(bhdr.file_id);	/* file id */
+				memcpy(buf, &bhdr, HEAPBLOBREC_SIZE);
+				if (pgin) {
+					memcpy(dbt->data,
+					    HEAPBLOBREC_DATA(buf),
+					    HEAPBLOBREC_DSIZE);
+				} else {
+					memcpy(data,
+					    HEAPBLOBREC_DATA(buf),
+					    HEAPBLOBREC_DSIZE);
+				}
+			}
+			break;
 		}
 		break;
 	default:
 		DB_ASSERT(NULL, op != op);
 	}
 }
+
+/*
+ * __db_swap --
+ *	Swap the byte order for a page. Used by __db_page_pass.
+ */
+static int
+__db_swap(dbp, real_name, flags, fhp, h, dirtyp)
+	DB *dbp;
+	char *real_name;
+	u_int32_t flags;
+	DB_FH *fhp;
+	PAGE *h;
+	int *dirtyp;
+{
+	COMPQUIET(real_name, NULL);
+	COMPQUIET(flags, 0);
+	COMPQUIET(fhp, NULL);
+	*dirtyp = 1;
+	return __db_pageswap(dbp->env, dbp,
+	    h, dbp->pgsize, NULL, !F_ISSET(dbp, DB_AM_SWAP));
+}
+
+static int (* const func_swap[P_PAGETYPE_MAX])
+    __P((DB *, char *, u_int32_t, DB_FH *, PAGE *, int *)) = {
+	NULL,			/* P_INVALID */
+	__db_swap,		/* __P_DUPLICATE */
+	__db_swap,		/* P_HASH_UNSORTED */
+	__db_swap,		/* P_IBTREE */
+	__db_swap,		/* P_IRECNO */
+	__db_swap,		/* P_LBTREE */
+	__db_swap,		/* P_LRECNO */
+	__db_swap,		/* P_OVERFLOW */
+	__db_swap,		/* P_HASHMETA */
+	__db_swap,		/* P_BTREEMETA */
+	__db_swap,		/* P_QAMMETA */
+	__db_swap,		/* P_QAMDATA */
+	__db_swap,		/* P_LDUP */
+	__db_swap,		/* P_HASH */
+	__db_swap,		/* P_HEAPMETA */
+	__db_swap,		/* P_HEAP */
+	__db_swap,		/* P_IHEAP */
+};
+
+/*
+ * __db_convert_pp --
+ *	DB->convert pre/post processing.
+ *
+ * PUBLIC: int __db_convert_pp __P((DB *, const char *, u_int32_t));
+ */
+int
+__db_convert_pp(dbp, fname, lorder)
+	DB *dbp;
+	const char *fname;
+	u_int32_t lorder;
+{
+	DB_THREAD_INFO *ip;
+	ENV *env;
+	int ret;
+
+	env = dbp->env;
+
+	ENV_ENTER(env, ip);
+	ret = __db_convert(dbp, fname, lorder);
+
+#ifdef HAVE_SLICES
+	if (ret == 0)
+		ret = __db_slice_process(dbp, fname, lorder,
+		    __db_convert_pp, "db_convert");
+#endif
+
+	ENV_LEAVE(env, ip);
+	return (ret);
+}
+
+/*
+ * __db_convert_extent --
+ *	Convert the byte order of each database extent (a queue or partition
+ *	extent).
+ */
+static int
+__db_convert_extent(env, fname, pagesize, flags)
+	ENV *env;
+	const char *fname;
+	u_int32_t pagesize;
+	u_int32_t flags;
+{
+	DB *dbp;
+	DB_FH *fhp;
+	char *real_name;
+	int ret, t_ret;
+
+	dbp = NULL;
+	fhp = NULL;
+	ret = t_ret = 0;
+
+	/* Get the real backing file name. */
+	if ((ret = __db_appname(env,
+	    DB_APP_DATA, fname, NULL, &real_name)) != 0)
+		return (ret);
+
+	/* Open the file. */
+	if ((ret = __os_open(env, real_name, 0, 0, 0, &fhp)) != 0) {
+		__db_err(env, ret, "%s", real_name);
+		goto err;
+	}
+
+	if ((ret = __db_create_internal(&dbp, env, 0)) != 0)
+		goto err;
+
+	dbp->pgsize = pagesize;
+	dbp->flags = flags;
+
+	if ((ret = __db_page_pass(dbp,
+	    real_name, 0, func_swap, fhp, DB_CONVERT)) != 0)
+		goto err;
+	ret = __os_fsync(env, fhp);
+
+err:
+	if (fhp != NULL &&
+	    (t_ret = __os_closehandle(env, fhp)) != 0 && ret == 0)
+		ret = t_ret;
+	if (dbp != NULL && (t_ret = __db_close(dbp, NULL, 0) != 0) && ret == 0)
+		ret = t_ret;
+	__os_free(env, real_name);
+
+	return (ret);
+}
+
+static int
+__db_convert_extent_names(dbp, mbuf, fname, namelistp)
+	DB *dbp;
+	DBMETA *mbuf;
+	char *fname;
+	char ***namelistp;
+{
+	ENV *env;
+
+	env = dbp->env;
+	*namelistp = NULL;
+
+	switch (mbuf->magic) {
+	case DB_BTREEMAGIC:
+	case DB_HASHMAGIC:
+#ifdef HAVE_PARTITION
+		if (dbp->p_internal != NULL) {
+			return __partition_extent_names(dbp, fname, namelistp);
+		}
+#endif
+		break;
+	case DB_QAMMAGIC:
+		if (F_ISSET(dbp, DB_AM_CHKSUM) &&
+		    ((QMETA*)mbuf)->page_ext != 0) {
+			return __qam_extent_names(env, fname, namelistp);
+		}
+		break;
+	case DB_HEAPMAGIC:
+	default:
+		break;
+	}
+
+	return (0);
+}
+
+/*
+ * __db_convert --
+ *	Convert the byte order of a database.
+ *
+ * PUBLIC: int __db_convert __P((DB *, const char *, u_int32_t));
+ */
+int
+__db_convert(dbp, fname, lorder)
+	DB *dbp;
+	const char *fname;
+	u_int32_t lorder;
+{
+	ENV *env;
+	DB_FH *fhp;
+	u_int8_t mbuf[DBMETASIZE];
+	char *real_name, **extent_names, **ename;
+	size_t len;
+	u_int32_t db_order;
+	int t_ret, ret;
+
+	env = dbp->env;
+	fhp = NULL;
+	extent_names = NULL;
+	real_name = NULL;
+	len = 0;
+	ret = t_ret = 0;
+
+	/* Get the real backing file name. */
+	if ((ret = __db_appname(env,
+	    DB_APP_DATA, fname, NULL, &real_name)) != 0)
+		return (ret);
+
+	/* Open the file. */
+	if ((ret = __os_open(env, real_name, 0, 0, 0, &fhp)) != 0) {
+		__db_err(env, ret, "%s", real_name);
+		goto err;
+	}
+
+	/* Read the metadata page. */
+	if ((ret = __fop_read_meta(env, real_name, mbuf, sizeof(mbuf),
+	    fhp, 0, &len)) != 0)
+		goto err;
+
+	switch (__db_needswap(((DBMETA *)mbuf)->magic)) {
+	case 0:
+		db_order = __db_isbigendian() ? 4321 : 1234;
+		F_SET(dbp, DB_AM_SWAP);
+		break;
+	case DB_SWAPBYTES:
+		db_order = __db_isbigendian() ? 1234 : 4321;
+		M_32_SWAP(((DBMETA *)mbuf)->magic);
+		M_32_SWAP(((DBMETA *)mbuf)->pagesize);
+		F_CLR(dbp, DB_AM_SWAP);
+		break;
+	default:
+		ret = USR_ERR(env, EINVAL);
+		goto err;
+	}
+
+	if (db_order != lorder) {
+		dbp->pgsize = ((DBMETA*)mbuf)->pagesize;
+		if (FLD_ISSET(((DBMETA *)mbuf)->metaflags, DBMETA_CHKSUM))
+			F_SET(dbp, DB_AM_CHKSUM);
+		if (((DBMETA*)mbuf)->encrypt_alg != 0) {
+			if (!CRYPTO_ON(dbp->env)) {
+				ret = USR_ERR(env, EINVAL);
+				__db_errx(env, DB_STR("0667",
+"Attempt to convert an encrypted database without providing a password."));
+				goto err;
+			}
+			F_SET(dbp, DB_AM_ENCRYPT);
+		}
+		if ((ret = __db_page_pass(dbp,
+		    real_name, 0, func_swap, fhp, DB_CONVERT)) != 0)
+			goto err;
+		ret = __os_fsync(env, fhp);
+
+		if ((ret = __db_convert_extent_names(dbp,
+		    (DBMETA*)mbuf, (char*)fname, &extent_names)) != 0)
+			goto err;
+		if (extent_names != NULL) {
+			for (ename = extent_names; *ename != NULL; ename++) {
+				if ((t_ret = __db_convert_extent(env, *ename,
+				    dbp->pgsize, dbp->flags)) != 0 && ret == 0)
+					ret = t_ret;
+			}
+		}
+	}
+
+err:	if (fhp != NULL &&
+	    (t_ret = __os_closehandle(env, fhp)) != 0 && ret == 0)
+		ret = t_ret;
+	if (real_name != NULL)
+		__os_free(env, real_name);
+	if (extent_names != NULL)
+		__os_free(env, extent_names);
+
+	return (ret);
+}
