@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  */
 /*
  * Copyright (c) 1990, 1993, 1994, 1995, 1996
@@ -583,7 +583,7 @@ err:	if (h != NULL && (t_ret = __memp_fput(mpf,
 
 	PERFMON4(dbp->env, alloc, free, dbp->fname, dbp->dname, pgno, ret);
 	/*
-	 * XXX
+	 * !!!
 	 * We have to unlock the caller's page in the caller!
 	 */
 	return (ret);
@@ -1330,8 +1330,9 @@ __db_haslock(env, locker, dbmfp, pgno, mode, type)
 }
 /*
  * __db_has_pagelock --
- *	Determine if this locker holds a particular page lock.
- *	Returns 0 if lock is held, non-zero otherwise.
+ *	Determine if this locker holds a particular page lock, and return an
+ *	error if it is missing a page lock that it should have.
+ *	Otherwise (TDS with the page locked, or DS or CDS) return 0.
  *
  * PUBLIC: #ifdef DIAGNOSTIC
  * PUBLIC: int __db_has_pagelock __P((ENV *, DB_LOCKER *,
@@ -1348,6 +1349,9 @@ __db_has_pagelock(env, locker, dbmfp, pagep, mode)
 {
 	int ret;
 
+	if (!FLD_ISSET(env->open_flags, DB_INIT_TXN))
+		return (0);
+
 	switch (pagep->type) {
 	case P_OVERFLOW:
 	case P_INVALID:
