@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2000, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2000, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -113,9 +113,15 @@ __db_verify_internal(dbp, fname, dname, handle, callback, flags)
 
 	ENV_ENTER(env, ip);
 
-	if ((ret = __db_verify_arg(dbp, dname, handle, flags)) == 0)
+	if ((ret = __db_verify_arg(dbp, dname, handle, flags)) == 0) {
 		ret = __db_verify(dbp, ip,
 		     fname, dname, handle, callback, NULL, NULL, flags);
+#ifdef HAVE_SLICES
+		if (ret == 0 && FLD_ISSET(dbp->open_flags, DB_SLICED))
+			ret = __db_slice_verify(dbp,
+			    fname, dname, handle, callback, flags);
+#endif
+	}
 
 	/* Db.verify is a DB handle destructor. */
 	if ((t_ret = __db_close(dbp, NULL, 0)) != 0 && ret == 0)
@@ -393,7 +399,7 @@ __db_verify(dbp, ip, name, subdb, handle, callback, lp, rp, flags)
 	flags = sflags;
 
 #ifdef HAVE_PARTITION
-	if (t_ret == 0 && dbp->p_internal != NULL)
+	if (t_ret == 0 && isbad == 0 && dbp->p_internal != NULL)
 		t_ret = __part_verify(dbp, vdp, name, handle, callback, flags);
 #endif
 
@@ -558,7 +564,7 @@ __db_vrfy_pagezero(dbp, vdp, fhp, name, flags)
 		    "Page %lu: metadata page corrupted", "%lu"),
 		    (u_long)PGNO_BASE_MD));
 		isbad = 1;
-		if (ret != DB_CHKSUM_FAIL) {
+		if (ret != DB_META_CHKSUM_FAIL) {
 			EPRINT((env, DB_STR_A("0523",
 			    "Page %lu: could not check metadata page", "%lu"),
 			    (u_long)PGNO_BASE_MD));
@@ -659,8 +665,7 @@ __db_vrfy_pagezero(dbp, vdp, fhp, name, flags)
 	 * 26: Meta-flags.
 	 */
 	if (meta->metaflags != 0) {
-		if (FLD_ISSET(meta->metaflags,
-		    ~(DBMETA_CHKSUM|DBMETA_PART_RANGE|DBMETA_PART_CALLBACK))) {
+		if (FLD_ISSET(meta->metaflags, ~DBMETA_ALLFLAGS)) {
 			isbad = 1;
 			EPRINT((env, DB_STR_A("0529",
 			    "Page %lu: bad meta-data flags value %#lx",
@@ -673,6 +678,8 @@ __db_vrfy_pagezero(dbp, vdp, fhp, name, flags)
 			F_SET(pip, VRFY_HAS_PART_RANGE);
 		if (FLD_ISSET(meta->metaflags, DBMETA_PART_CALLBACK))
 			F_SET(pip, VRFY_HAS_PART_CALLBACK);
+		if (FLD_ISSET(meta->metaflags, DBMETA_SLICED))
+			F_SET(pip, VRFY_HAS_SLICES);
 
 		if (FLD_ISSET(meta->metaflags,
 		    DBMETA_PART_RANGE | DBMETA_PART_CALLBACK) &&
@@ -687,10 +694,10 @@ __db_vrfy_pagezero(dbp, vdp, fhp, name, flags)
 	 * for now, just store it.
 	 */
 	if (swapped)
-	    M_32_SWAP(meta->free);
+		M_32_SWAP(meta->free);
 	freelist = meta->free;
 	if (swapped)
-	    M_32_SWAP(meta->last_pgno);
+		M_32_SWAP(meta->last_pgno);
 	vdp->meta_last_pgno = meta->last_pgno;
 
 	/*
@@ -1109,7 +1116,7 @@ __db_vrfy_structure(dbp, vdp, dbname, meta_pgno, lp, rp, flags)
 		    && i <= vdp->meta_last_pgno
 #endif
 		    ) {
-			EPRINT((env, DB_STR_A("0535",
+			EPRINT((env, DB_STR_A("0502",
 			    "Page %lu: totally zeroed page", "%lu"),
 			    (u_long)i));
 			isbad = 1;
@@ -1232,7 +1239,7 @@ __db_vrfy_common(dbp, vdp, h, pgno, flags)
 	case P_HEAPMETA:
 		break;
 	default:
-		EPRINT((env, DB_STR_A("0537", "Page %lu: bad page type %lu",
+		EPRINT((env, DB_STR_A("0528", "Page %lu: bad page type %lu",
 		    "%lu %lu"), (u_long)pgno, (u_long)h->type));
 		ret = DB_VERIFY_BAD;
 	}
@@ -1329,7 +1336,7 @@ __db_vrfy_datapage(dbp, vdp, h, pgno, flags)
 		}
 		if (!IS_VALID_PGNO(NEXT_PGNO(h)) || NEXT_PGNO(h) == pip->pgno) {
 			isbad = 1;
-			EPRINT((env, DB_STR_A("0540",
+			EPRINT((env, DB_STR_A("0538",
 			    "Page %lu: invalid next_pgno %lu", "%lu %lu"),
 			    (u_long)pip->pgno, (u_long)NEXT_PGNO(h)));
 		}
@@ -1519,10 +1526,9 @@ __db_vrfy_meta(dbp, vdp, meta, pgno, flags)
 
 	/* Flags */
 	if (meta->metaflags != 0) {
-		if (FLD_ISSET(meta->metaflags,
-		    ~(DBMETA_CHKSUM|DBMETA_PART_RANGE|DBMETA_PART_CALLBACK))) {
+		if (FLD_ISSET(meta->metaflags, ~DBMETA_ALLFLAGS)) {
 			isbad = 1;
-			EPRINT((env, DB_STR_A("0549",
+			EPRINT((env, DB_STR_A("0529",
 			    "Page %lu: bad meta-data flags value %#lx",
 			    "%lu %#lx"), (u_long)PGNO_BASE_MD,
 			    (u_long)meta->metaflags));
@@ -1533,6 +1539,8 @@ __db_vrfy_meta(dbp, vdp, meta, pgno, flags)
 			F_SET(pip, VRFY_HAS_PART_RANGE);
 		if (FLD_ISSET(meta->metaflags, DBMETA_PART_CALLBACK))
 			F_SET(pip, VRFY_HAS_PART_CALLBACK);
+		if (FLD_ISSET(meta->metaflags, DBMETA_SLICED))
+			F_SET(pip, VRFY_HAS_SLICES);
 	}
 
 	/*
@@ -1836,7 +1844,7 @@ __db_vrfy_orderchkonly(dbp, vdp, name, subdb, flags)
 	if ((ret = __db_get(mdbp,
 	    vdp->thread_info, NULL, &key, &data, 0)) != 0) {
 		if (ret == DB_NOTFOUND)
-			ret = ENOENT;
+			ret = USR_ERR(env, ENOENT);
 		goto err;
 	}
 
@@ -1933,6 +1941,13 @@ __db_vrfy_orderchkonly(dbp, vdp, name, subdb, flags)
 			goto err;
 		for (bucket = 0; bucket <= hmeta->max_bucket; bucket++) {
 			pgno = BS_TO_PAGE(bucket, hmeta->spares);
+			if (!IS_VALID_PGNO(pgno)) {
+				EPRINT((env, DB_STR_A("5502",
+				"Page %lu: invalid pgno found in hash bucket",
+					"%lu"), (u_long)pgno));
+				ret = DB_VERIFY_BAD;
+				continue;
+			}
 			while (pgno != PGNO_INVALID) {
 				if ((ret = __memp_fget(mpf, &pgno,
 				    vdp->thread_info, NULL, 0, &currpg)) != 0)
@@ -2130,10 +2145,8 @@ __db_salvage_leaf(dbp, vdp, pgno, h, handle, callback, flags)
 	int (*callback) __P((void *, const void *));
 	u_int32_t flags;
 {
-	ENV *env;
 
-	env = dbp->env;
-	DB_ASSERT(env, LF_ISSET(DB_SALVAGE));
+	DB_ASSERT(dbp->env, LF_ISSET(DB_SALVAGE));
 
 	/* If we got this page in the subdb pass, we can safely skip it. */
 	if (__db_salvage_isdone(vdp, pgno))
