@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1999, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1999, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -29,7 +29,7 @@ extern "C" {
 
 /* Complain about a totally zeroed page where we don't expect one. */
 #define	ZEROPG_ERR_PRINT(dbenv, pgno, str) do {				\
-	EPRINT(((dbenv), DB_STR_A("0501", 				\
+	EPRINT(((dbenv), DB_STR_A("0501",				\
 	    "Page %lu: %s is of inappropriate type %lu", "%lu %s %lu"),	\
 	    (u_long)(pgno), str, (u_long)P_INVALID));			\
 	EPRINT(((dbenv), DB_STR_A("0502",				\
@@ -166,22 +166,24 @@ struct __vrfy_pageinfo {
 	u_int32_t	refcount;
 	u_int32_t	olen;
 
-#define	VRFY_DUPS_UNSORTED	0x0001	/* Have to flag the negative! */
-#define	VRFY_HAS_CHKSUM		0x0002
-#define	VRFY_HAS_DUPS		0x0004
-#define	VRFY_HAS_DUPSORT	0x0008	/* Has the flag set. */
-#define	VRFY_HAS_PART_RANGE	0x0010	/* Has the flag set. */
-#define	VRFY_HAS_PART_CALLBACK	0x0020	/* Has the flag set. */
-#define	VRFY_HAS_RECNUMS	0x0040
-#define	VRFY_HAS_SUBDBS		0x0080
-#define	VRFY_INCOMPLETE		0x0100	/* Meta or item order checks incomp. */
-#define	VRFY_IS_ALLZEROES	0x0200	/* Hash page we haven't touched? */
-#define	VRFY_IS_FIXEDLEN	0x0400
-#define	VRFY_IS_RECNO		0x0800
-#define	VRFY_IS_RRECNO		0x1000
-#define	VRFY_OVFL_LEAFSEEN	0x2000
-#define	VRFY_HAS_COMPRESS	0x4000
-#define	VRFY_NONEXISTENT	0x8000
+#define	VRFY_DUPS_UNSORTED	0x00001	/* Have to flag the negative! */
+#define	VRFY_HAS_CHKSUM		0x00002
+#define	VRFY_HAS_DUPS		0x00004
+#define	VRFY_HAS_DUPSORT	0x00008	/* Has the flag set. */
+#define	VRFY_HAS_PART_RANGE	0x00010	/* Has the flag set. */
+#define	VRFY_HAS_PART_CALLBACK	0x00020	/* Has the flag set. */
+#define	VRFY_HAS_RECNUMS	0x00040
+#define	VRFY_HAS_SLICES		0x00080
+#define	VRFY_HAS_SUBDBS		0x00100
+#define	VRFY_INCOMPLETE		0x00200	/* Meta or item order checks incomp. */
+#define	VRFY_IS_ALLZEROES	0x00400	/* Hash page we haven't touched? */
+#define	VRFY_IS_FIXEDLEN	0x00800
+#define	VRFY_IS_RECNO		0x01000
+#define	VRFY_IS_RRECNO		0x02000
+#define	VRFY_ITEM_BAD		0x04000	/* The page has a bad inp or item. */
+#define	VRFY_OVFL_LEAFSEEN	0x08000
+#define	VRFY_HAS_COMPRESS	0x10000
+#define	VRFY_NONEXISTENT	0x20000
 	u_int32_t	flags;
 
 	LIST_ENTRY(__vrfy_pageinfo) links;
