@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1998, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1998, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -36,7 +36,13 @@ extern "C" {
 #define	DB_ASSERT(env, e)						\
 	((e) ? (void)0 : __db_assert(env, #e, __FILE__, __LINE__))
 #else
-#define	DB_ASSERT(env, e)	NOP_STATEMENT
+#define	DB_ASSERT(env, e)	((void)0)
+#endif
+
+#if defined(HAVE_ERROR_HISTORY) && defined(DIAGNOSTIC)
+#define	DB_DEBUG_MSG	__db_debug_msg
+#else
+#define	DB_DEBUG_MSG	if (0) __db_debug_msg
 #endif
 
 /*
@@ -73,13 +79,40 @@ typedef enum {
 	DB_ERROR_SYSTEM=2
 } db_error_set_t;
 
+/*
+ * Use these macros wherever an error condition is initially noticed, e.g., when
+ * setting a value to any of the user visible error return codes, whether
+ * defined by Berkeley DB or by the operating environment (EINVAL).
+ * saving the specific source of an instance of an error code, including the
+ * time, stack, db name, current LSN, etc. If the error turns out to be
+ * important, the deferred message text is added to the text produced by
+ * __db_err(), __db_errx, and __db_syserr(). The additional information can be
+ * useful for diagnosing the behavior of applications under error conditions.
+ * It is enabled by configuring with --enable-error_history. The current
+ * implmentation requires pthreads' version of thread local storage.
+ */
+#ifdef HAVE_ERROR_HISTORY
+#define	USR_ERR(env, errcode)		__db_diags((env), (errcode))
+#define	DBC_ERR(dbc, errcode)		__dbc_diags((dbc), (errcode))
+#define	MUTEX_ERR(env, mutex, errcode)	__mutex_diags((env), (mutex), (errcode))
+#define	DISCARD_HISTORY(env)		__db_deferred_discard()
+/* Save at most 10KB of error history in an API call. Adjust this as desired. */
+#define	DB_ERROR_HISTORY_SIZE		(10 * 1024)
+#else
+#define	USR_ERR(env, errcode)		(errcode)
+#define	DBC_ERR(dbc, errcode)		(errcode)
+#define	MUTEX_ERR(env, mutex, errcode)	(errcode)
+#define	DISCARD_HISTORY(env)		NOP_STATEMENT
+/* No space is needed when error history is disabled. */
+#define	DB_ERROR_HISTORY_SIZE		0
+#endif
+
 /*
  * Message handling.  Use a macro instead of a function because va_list
  * references to variadic arguments cannot be reset to the beginning of the
  * variadic argument list (and then rescanned), by functions other than the
  * original routine that took the variadic list of arguments.
  */
-#if defined(STDC_HEADERS) || defined(__cplusplus)
 #define	DB_REAL_ERR(dbenv, error, error_set, app_call, fmt) {		\
 	va_list __ap;							\
 									\
@@ -103,34 +136,9 @@ typedef enum {
 	    ((app_call) || F_ISSET((dbenv)->env, ENV_NO_OUTPUT_SET))))	\
 		__db_errfile(dbenv, error, error_set, fmt, __ap);	\
 	va_end(__ap);							\
+	DISCARD_HISTORY((dbenv)->env);					\
 }
-#else
-#define	DB_REAL_ERR(dbenv, error, error_set, app_call, fmt) {		\
-	va_list __ap;							\
-									\
-	/* Call the application's callback function, if specified. */	\
-	va_start(__ap);							\
-	if ((dbenv) != NULL && (dbenv)->db_errcall != NULL)		\
-		__db_errcall(dbenv, error, error_set, fmt, __ap);	\
-	va_end(__ap);							\
-									\
-	/*								\
-	 * If the application specified a file descriptor, write to it.	\
-	 * If we wrote to neither the application's callback routine or	\
-	 * its file descriptor, and it's an application error message	\
-	 * using {DbEnv,Db}.{err,errx} or the application has never	\
-	 * configured an output channel, default by writing to stderr.	\
-	 */								\
-	va_start(__ap);							\
-	if ((dbenv) == NULL ||						\
-	    (dbenv)->db_errfile != NULL ||				\
-	    ((dbenv)->db_errcall == NULL &&				\
-	    ((app_call) || F_ISSET((dbenv)->env, ENV_NO_OUTPUT_SET))))	\
-		 __db_errfile(env, error, error_set, fmt, __ap);	\
-	va_end(__ap);							\
-}
-#endif
-#if defined(STDC_HEADERS) || defined(__cplusplus)
+
 #define	DB_REAL_MSG(dbenv, fmt) {					\
 	va_list __ap;							\
 									\
@@ -141,42 +149,20 @@ typedef enum {
 	va_end(__ap);							\
 									\
 	/*								\
-	 * If the application specified a file descriptor, write to it.	\
-	 * If we wrote to neither the application's callback routine or	\
-	 * its file descriptor, write to stdout.			\
+	 * If the application specified a file descriptor and its value	\
+	 * is not NULL, write to it. If the application specified a	\
+	 * file descriptor and its value is NULL, write nothing.	\
+	 * The file descriptor is initialized to stdout, so if the	\
+	 * application doesn't override it and there isn't a callback	\
+	 * function then write to stdout.				\
 	 */								\
 	va_start(__ap, fmt);						\
 	if ((dbenv) == NULL ||						\
-	    (dbenv)->db_msgfile != NULL ||				\
-	    (dbenv)->db_msgcall == NULL) {				\
+	    ((dbenv)->db_msgcall == NULL &&				\
+	    (dbenv)->db_msgfile != NULL))				\
 		__db_msgfile(dbenv, fmt, __ap);				\
-	}								\
 	va_end(__ap);							\
 }
-#else
-#define	DB_REAL_MSG(dbenv, fmt) {					\
-	va_list __ap;							\
-									\
-	/* Call the application's callback function, if specified. */	\
-	va_start(__ap);							\
-	if ((dbenv) != NULL && (dbenv)->db_msgcall != NULL)		\
-		__db_msgcall(dbenv, fmt, __ap);				\
-	va_end(__ap);							\
-									\
-	/*								\
-	 * If the application specified a file descriptor, write to it.	\
-	 * If we wrote to neither the application's callback routine or	\
-	 * its file descriptor, write to stdout.			\
-	 */								\
-	va_start(__ap);							\
-	if ((dbenv) == NULL ||						\
-	    (dbenv)->db_msgfile != NULL ||				\
-	    (dbenv)->db_msgcall == NULL) {				\
-		__db_msgfile(dbenv, fmt, __ap);				\
-	}								\
-	va_end(__ap);							\
-}
-#endif
 
 /*
  * Debugging macro to log operations.
@@ -193,7 +179,7 @@ typedef enum {
 #define	LOG_OP(C, T, O, K, A, F) {					\
 	DB_LSN __lsn;							\
 	DBT __op;							\
-	if (DBC_LOGGING((C))) {						\
+	if ((C)->dbp->log_filename != NULL && DBC_LOGGING((C))) {	\
 		memset(&__op, 0, sizeof(__op));				\
 		__op.data = O;						\
 		__op.size = (u_int32_t)strlen(O) + 1;			\
@@ -217,6 +203,11 @@ typedef enum {
  * Hook for testing subdb locks.
  */
 #if CONFIG_TEST
+#define	DB_TEST_CRASH(field, val) do {				        \
+	if (field == (val))						\
+		exit(1);					        \
+} while (0)
+
 #define	DB_TEST_SUBLOCKS(env, flags) do {				\
 	if ((env)->test_abort == DB_TEST_SUBDB_LOCKS)			\
 		(flags) |= DB_LOCK_NOWAIT;				\
@@ -233,7 +224,7 @@ typedef enum {
 	if ((env)->test_abort == (val)) {				\
 		/* ABORT the TXN */					\
 		(env)->test_abort = 0;					\
-		(ret) = EINVAL;						\
+		(ret) = USR_ERR(env, EINVAL);				\
 		goto db_tr_err;						\
 	}								\
 } while (0)
@@ -254,7 +245,7 @@ typedef enum {
 	if (__env->test_abort == (val)) {				\
 		/* Abort the transaction. */				\
 		__env->test_abort = 0;					\
-		(ret) = EINVAL;						\
+		(ret) = USR_ERR(__env, EINVAL);				\
 		goto db_tr_err;						\
 	}								\
 } while (0)
@@ -270,6 +261,7 @@ typedef enum {
 	if ((val) != 0)							\
 		__os_yield((env), (u_long)(val), 0)
 #else
+#define	DB_TEST_CRASH(env, val)
 #define	DB_TEST_SUBLOCKS(env, flags)
 #define	DB_ENV_TEST_RECOVERY(env, val, ret, name)
 #define	DB_TEST_RECOVERY(dbp, val, ret, name)
