@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  */
 /*
  * $Id$
@@ -37,9 +37,16 @@ typedef struct __part_internal {
 
 #ifdef HAVE_PARTITION
 #define	PART_NAME	"__dbp.%s.%03d"
-#define	PART_LEN	(strlen("__dbp..")+3)
+/*
+ * Currently we only support no more than 1000000 partitions.
+ * If the limit is changed, the PART_DIGITS and PART_MAXIMUM
+ * should be changed accordingly.
+ */
+#define	PART_DIGITS	6
+#define	PART_MAXIMUM	1000000
+#define	PART_LEN	(sizeof("__dbp..") + PART_DIGITS)
 #define	PART_PREFIX	"__dbp."
-#define IS_PARTITION_DB_FILE(name)	(strncmp(name, PART_PREFIX,	\
+#define	IS_PARTITION_DB_FILE(name)	(strncmp(name, PART_PREFIX,	\
 					    sizeof(PART_PREFIX) - 1) == 0)
 
 #define	DB_IS_PARTITIONED(dbp)						\
