@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2001, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2001, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -19,6 +19,7 @@ extern "C" {
  * Names of client temp databases.
  */
 #define	REPFILEPREFIX	"__db.rep"
+#define	REPBLOBNAME	"__db.rep.blob.db"
 #define	REPDBNAME	"__db.rep.db"
 #define	REPPAGENAME     "__db.reppg.db"
 
@@ -32,8 +33,7 @@ extern "C" {
 #define	REPLSNHIST	"__db.lsn.history"
 #define	REPMEMBERSHIP	"__db.membership"
 #define	REPSYSDBPGSZ	1024
-#define IS_REP_FILE(name)	(strcmp(name, REPSYSDBNAME) == 0)
-				    
+#define	IS_REP_FILE(name)	(strcmp(name, REPSYSDBNAME) == 0)
 
 /* Current version of commit token format, and LSN history database format. */
 #define	REP_COMMIT_TOKEN_FMT_VERSION	1
@@ -42,43 +42,58 @@ extern "C" {
 /*
  * Message types
  */
-#define	REP_INVALID	0	/* Invalid message type. */
-#define	REP_ALIVE	1	/* I am alive message. */
-#define	REP_ALIVE_REQ	2	/* Request for alive messages. */
-#define	REP_ALL_REQ	3	/* Request all log records greater than LSN. */
-#define	REP_BULK_LOG	4	/* Bulk transfer of log records. */
-#define	REP_BULK_PAGE	5	/* Bulk transfer of pages. */
-#define	REP_DUPMASTER	6	/* Duplicate master detected; propagate. */
-#define	REP_FILE	7	/* Page of a database file. NOTUSED */
-#define	REP_FILE_FAIL	8	/* File requested does not exist. */
-#define	REP_FILE_REQ	9	/* Request for a database file. NOTUSED */
-#define	REP_LEASE_GRANT	10	/* Client grants a lease to a master. */
-#define	REP_LOG		11	/* Log record. */
-#define	REP_LOG_MORE	12	/* There are more log records to request. */
-#define	REP_LOG_REQ	13	/* Request for a log record. */
-#define	REP_MASTER_REQ	14	/* Who is the master */
-#define	REP_NEWCLIENT	15	/* Announces the presence of a new client. */
-#define	REP_NEWFILE	16	/* Announce a log file change. */
-#define	REP_NEWMASTER	17	/* Announces who the master is. */
-#define	REP_NEWSITE	18	/* Announces that a site has heard from a new
-				 * site; like NEWCLIENT, but indirect.  A
-				 * NEWCLIENT message comes directly from the new
-				 * client while a NEWSITE comes indirectly from
-				 * someone who heard about a NEWSITE.
-				 */
-#define	REP_PAGE	19	/* Database page. */
-#define	REP_PAGE_FAIL	20	/* Requested page does not exist. */
-#define	REP_PAGE_MORE	21	/* There are more pages to request. */
-#define	REP_PAGE_REQ	22	/* Request for a database page. */
-#define	REP_REREQUEST	23	/* Force rerequest. */
-#define	REP_START_SYNC	24	/* Tell client to begin syncing a ckp.*/
-#define	REP_UPDATE	25	/* Environment hotcopy information. */
-#define	REP_UPDATE_REQ	26	/* Request for hotcopy information. */
-#define	REP_VERIFY	27	/* A log record for verification. */
-#define	REP_VERIFY_FAIL	28	/* The client is outdated. */
-#define	REP_VERIFY_REQ	29	/* Request for a log record to verify. */
-#define	REP_VOTE1	30	/* Send out your information for an election. */
-#define	REP_VOTE2	31	/* Send a "you are master" vote. */
+#define	REP_INVALID		0	/* Invalid message type. */
+#define	REP_ALIVE		1	/* I am alive message. */
+#define	REP_ALIVE_REQ		2	/* Request for alive messages. */
+#define	REP_ALL_REQ		3	/* Request all log records greater than
+					 * LSN. */
+#define	REP_BLOB_ALL_REQ	4	/* Request all the given blob files. */
+#define	REP_BLOB_CHUNK		5	/* A piece of data contained in a blob
+					 * file. */
+#define	REP_BLOB_CHUNK_REQ	6	/* Request a piece of data from a blob
+					 * file. */
+#define	REP_BLOB_UPDATE		7	/* A list of blob files for a
+					 * database. */
+#define	REP_BLOB_UPDATE_REQ	8	/* Request blob files. */
+#define	REP_BULK_LOG		9	/* Bulk transfer of log records. */
+#define	REP_BULK_PAGE		10	/* Bulk transfer of pages. */
+#define	REP_DUPMASTER		11	/* Duplicate master detected;
+					 * propagate. */
+#define	REP_FILE		12	/* Page of a database file. NOTUSED */
+#define	REP_FILE_FAIL		13	/* File requested does not exist. */
+#define	REP_FILE_REQ		14	/* Request for a database file.
+					 * NOTUSED */
+#define	REP_LEASE_GRANT		15	/* Client grants a lease to a master. */
+#define	REP_LOG			16	/* Log record. */
+#define	REP_LOG_MORE		17	/* There are more log records to
+					 * request. */
+#define	REP_LOG_REQ		18	/* Request for a log record. */
+#define	REP_MASTER_REQ		19	/* Who is the master */
+#define	REP_NEWCLIENT		20	/* Announces the presence of a new
+					 * client. */
+#define	REP_NEWFILE		21	/* Announce a log file change. */
+#define	REP_NEWMASTER		22	/* Announces who the master is. */
+#define	REP_NEWSITE		23	/* Announces that a site has heard from
+					 * a new site; like NEWCLIENT, but
+					 * indirect.  A NEWCLIENT message comes
+					 * directly from the new client while a
+					 * NEWSITE comes indirectly from
+					 * someone who heard about a NEWSITE.*/
+#define	REP_PAGE		24	/* Database page. */
+#define	REP_PAGE_FAIL		25	/* Requested page does not exist. */
+#define	REP_PAGE_MORE		26	/* There are more pages to request. */
+#define	REP_PAGE_REQ		27	/* Request for a database page. */
+#define	REP_REREQUEST		28	/* Force rerequest. */
+#define	REP_START_SYNC		29	/* Tell client to begin syncing a ckp.*/
+#define	REP_UPDATE		30	/* Environment hotcopy information. */
+#define	REP_UPDATE_REQ		31	/* Request for hotcopy information. */
+#define	REP_VERIFY		32	/* A log record for verification. */
+#define	REP_VERIFY_FAIL		33	/* The client is outdated. */
+#define	REP_VERIFY_REQ		34	/* Request for a log record to
+					 * verify. */
+#define	REP_VOTE1		35	/* Send out your information for an
+					 * election. */
+#define	REP_VOTE2		36	/* Send a "you are master" vote. */
 /*
  * Maximum message number for conversion tables.  Update this
  * value as the largest message number above increases.
@@ -90,7 +105,7 @@ extern "C" {
  * NOTE: When changing messages above, the two tables for upgrade support
  * need adjusting.  They are in rep_util.c.
  */
-#define	REP_MAX_MSG	31
+#define	REP_MAX_MSG	36
 
 /*
  * This is the list of client-to-client requests messages.
@@ -99,10 +114,18 @@ extern "C" {
  */
 #define	REP_MSG_REQ(rectype)			\
     (rectype == REP_ALL_REQ ||			\
+    rectype == REP_BLOB_ALL_REQ ||		\
+    rectype == REP_BLOB_CHUNK_REQ ||		\
     rectype == REP_LOG_REQ ||			\
     rectype == REP_PAGE_REQ ||			\
     rectype == REP_VERIFY_REQ)
 
+/*
+ * This is the default value for the FD_SET_SIZE.  It is used when client
+ * doesn't provide any value or the client provided value is smaller
+ */
+#define	REPMGR_FD_SET_DEFAULT_SIZE	1024
+
 /*
  * Note that the version information should be at the beginning of the
  * structure, so that we can rearrange the rest of it while letting the
@@ -126,20 +149,24 @@ extern "C" {
 #define	DB_LOGVERSION_52	18
 #define	DB_LOGVERSION_53	19
 #define	DB_LOGVERSION_60	20
-#define	DB_LOGVERSION_MIN	DB_LOGVERSION_44
+#define	DB_LOGVERSION_60p1	21
+#define	DB_LOGVERSION_61	22
+#define	DB_LOGVERSION_62	23
+#define	DB_LOGVERSION_MIN	DB_LOGVERSION_47
 #define	DB_REPVERSION_INVALID	0
 #define	DB_REPVERSION_44	3
 #define	DB_REPVERSION_45	3
 #define	DB_REPVERSION_46	4
 #define	DB_REPVERSION_47	5
 #define	DB_REPVERSION_48	5
-#define	DB_REPVERSION_50	5
 #define	DB_REPVERSION_51	5
 #define	DB_REPVERSION_52	6
 #define	DB_REPVERSION_53	7
 #define	DB_REPVERSION_60	7
-#define	DB_REPVERSION		DB_REPVERSION_60
-#define	DB_REPVERSION_MIN	DB_REPVERSION_44
+#define	DB_REPVERSION_61	8
+#define	DB_REPVERSION_62	9
+#define	DB_REPVERSION		DB_REPVERSION_62
+#define	DB_REPVERSION_MIN	DB_REPVERSION_47
 
 /*
  * RPRINT - Replication diagnostic output
@@ -219,6 +246,7 @@ extern "C" {
  * Database types for __rep_client_dbinit
  */
 typedef enum {
+	REP_BLOB,	/* Blob file database. */
 	REP_DB,		/* Log record database. */
 	REP_PG		/* Pg database. */
 } repdb_t;
@@ -251,9 +279,9 @@ typedef enum {
 typedef enum {
 	SYNC_OFF,	/* No recovery. */
 	SYNC_LOG,	/* Recovery - log. */
-	SYNC_PAGE,	/* Recovery - pages. */
+	SYNC_PAGE,	/* Recovery - pages and blobs. */
 	SYNC_UPDATE,	/* Recovery - update. */
-	SYNC_VERIFY 	/* Recovery - verify. */
+	SYNC_VERIFY	/* Recovery - verify. */
 } repsync_t;
 
 /*
@@ -273,6 +301,17 @@ typedef struct {
 	u_int32_t data_gen;
 } VOTE1_CONTENT;
 
+/*
+ * In a 2 node scenario, it is important that the newly joining node wait
+ * for the peer connection to be fully established before proceeding for
+ * election. Otherwise, it might end up electing itself as MASTER with its
+ * own vote (inspite of the fact that a fully functional master exists).
+ * Normally it takes about 5-20ms for the connection to go through. Setting
+ * the timeout to 500ms to account for connection time variations associated
+ * with SSL.
+ */
+#define DB_REP_PEER_WAIT_TIMEOUT 500000
+
 /*
  * REP --
  * Shared replication structure.
@@ -315,6 +354,8 @@ typedef struct __rep { /* SHARED */
 					 * request a missing log record. */
 	db_timespec	max_gap;	/* Maximum time to wait before
 					 * requesting a missing log record. */
+	u_int32_t	initsites;	/* Number of sites for
+					 * set_memory_init(). */
 	/* Status change information */
 	u_int32_t	apply_th;	/* Number of callers in rep_apply. */
 	u_int32_t	arch_th;	/* Number of callers in log_archive. */
@@ -358,6 +399,16 @@ typedef struct __rep { /* SHARED */
 	u_int32_t	first_vers;	/* Log version of first log file. */
 	DB_LSN		last_lsn;	/* Latest LSN we need. */
 	/* These are protected by mtx_clientdb. */
+	db_seq_t	gap_bl_hi_id;	/* Last id in the blob gap. */
+	db_seq_t	gap_bl_hi_sid;	/* Last sid in the blob gap. */
+	off_t		gap_bl_hi_off;	/* Last offset in the blob gap. */
+	db_seq_t	last_blob_id;	/* Last id on the list to process. */
+	db_seq_t	last_blob_sid;	/* Last sid on the list to process. */
+	db_seq_t	prev_blob_id;	/* Previous last id on list. */
+	db_seq_t	prev_blob_sid;	/* Previous last sid on list. */
+	db_seq_t	highest_id;	/* Highest file id to request. */
+	u_int32_t	blob_more_files;/* More blob files to be processed. */
+	int		blob_sync;	/* Currently handling blobs. */
 	db_timespec	last_pg_ts;	/* Last page stored timestamp. */
 	db_pgno_t	ready_pg;	/* Next pg expected. */
 	db_pgno_t	waiting_pg;	/* First pg after gap. */
@@ -407,6 +458,7 @@ typedef struct __rep { /* SHARED */
 	int		self_eid;	/* Where to find the local site. */
 	u_int		siteinfo_seq;	/* Number of updates to this info. */
 	u_int32_t	min_log_file;	/* Earliest log needed by repgroup. */
+	DB_LSN		last_ckp_lsn;	/* The last checkpoint's ckp_lsn. */
 
 	pid_t		listener;
 	u_int		listener_nthreads; /* # of msg threads in listener. */
@@ -417,8 +469,12 @@ typedef struct __rep { /* SHARED */
 	db_timeout_t	connection_retry_wait;
 	db_timeout_t	heartbeat_frequency; /* Max period between msgs. */
 	db_timeout_t	heartbeat_monitor_timeout;
-	u_int32_t	inqueue_msg_max;
-	u_int32_t	inqueue_bulkmsg_max;
+	db_timeout_t	write_forward_timeout;
+	u_int32_t	inqueue_max_gbytes;
+	u_int32_t	inqueue_max_bytes;
+	u_int32_t	inqueue_rz_gbytes;
+	u_int32_t	inqueue_rz_bytes;
+	u_int32_t	inqueue_full_event_on;
 #endif  /* HAVE_REPLICATION_THREADS */
 
 	/* Statistics. */
@@ -438,10 +494,17 @@ typedef struct __rep { /* SHARED */
 #define	REP_C_AUTOTAKEOVER	0x00008		/* Auto listener take over. */
 #define	REP_C_BULK		0x00010		/* Bulk transfer. */
 #define	REP_C_DELAYCLIENT	0x00020		/* Delay client sync-up. */
-#define	REP_C_ELECTIONS		0x00040		/* Repmgr to use elections. */
-#define	REP_C_INMEM		0x00080		/* In-memory replication. */
-#define	REP_C_LEASE		0x00100		/* Leases configured. */
-#define	REP_C_NOWAIT		0x00200		/* Immediate error return. */
+#define	REP_C_DISABLE_POLL	0x00040		/* Avoid POLL for network io */
+#define	REP_C_DISABLE_SSL	0x00080		/* Avoid POLL for network io */
+#define	REP_C_ELECT_LOGLENGTH	0x00100		/* Log length wins election. */
+#define	REP_C_ELECTIONS		0x00200		/* Repmgr to use elections. */
+#define	REP_C_ENABLE_EPOLL	0x00400		/* Use EPOLL for network io */
+#define	REP_C_FORWARD_WRITES	0x00800		/* Repmgr write forwarding. */
+#define	REP_C_INMEM		0x01000		/* In-memory replication. */
+#define	REP_C_LEASE		0x02000		/* Leases configured. */
+#define	REP_C_NOWAIT		0x04000		/* Immediate error return. */
+#define	REP_C_PREFMAS_CLIENT	0x08000		/* Preferred master client. */
+#define	REP_C_PREFMAS_MASTER	0x10000		/* Preferred master site. */
 	u_int32_t	config;		/* Configuration flags. */
 
 	/* Election. */
@@ -449,6 +512,7 @@ typedef struct __rep { /* SHARED */
 #define	REP_E_PHASE1		0x00000002	/* In phase 1 of election. */
 #define	REP_E_PHASE2		0x00000004	/* In phase 2 of election. */
 #define	REP_E_TALLY		0x00000008	/* Tallied vote before elect. */
+#define	REP_E_PEER_CONN_WAIT	0x00000010	/* Election waiting for peer. */
 	u_int32_t	elect_flags;	/* Election flags. */
 
 	/* Lockout. */
@@ -472,15 +536,17 @@ typedef struct __rep { /* SHARED */
 #define	REP_F_CLIENT		0x00000008	/* Client replica. */
 #define	REP_F_DELAY		0x00000010	/* Delaying client sync-up. */
 #define	REP_F_GROUP_ESTD	0x00000020	/* Rep group is established. */
-#define	REP_F_INUPDREQ		0x00000040	/* Thread in rep_update_req. */
-#define	REP_F_LEASE_EXPIRED	0x00000080	/* Leases guaranteed expired. */
-#define	REP_F_MASTER		0x00000100	/* Master replica. */
-#define	REP_F_MASTERELECT	0x00000200	/* Master elect. */
-#define	REP_F_NEWFILE		0x00000400	/* Newfile in progress. */
-#define	REP_F_NIMDBS_LOADED	0x00000800	/* NIMDBs are materialized. */
-#define	REP_F_SKIPPED_APPLY	0x00001000	/* Skipped applying a record. */
-#define	REP_F_START_CALLED	0x00002000	/* Rep_start called. */
-#define	REP_F_SYS_DB_OP		0x00004000	/* Operation in progress. */
+#define	REP_F_HOLD_GEN		0x00000040	/* PrefMas startup hold gen. */
+#define	REP_F_INUPDREQ		0x00000080	/* Thread in rep_update_req. */
+#define	REP_F_LEASE_EXPIRED	0x00000100	/* Leases guaranteed expired. */
+#define	REP_F_MASTER		0x00000200	/* Master replica. */
+#define	REP_F_MASTERELECT	0x00000400	/* Master elect. */
+#define	REP_F_NEWFILE		0x00000800	/* Newfile in progress. */
+#define	REP_F_NIMDBS_LOADED	0x00001000	/* NIMDBs are materialized. */
+#define	REP_F_READONLY_MASTER	0x00002000	/* PrefMas readonly master. */
+#define	REP_F_SKIPPED_APPLY	0x00004000	/* Skipped applying a record. */
+#define	REP_F_START_CALLED	0x00008000	/* Rep_start called. */
+#define	REP_F_SYS_DB_OP		0x00010000	/* Operation in progress. */
 	u_int32_t	flags;
 } REP;
 
@@ -516,10 +582,10 @@ struct __rep_waiter {
  * locked out/set individually because they pertain to different pieces of
  * the BDB API, they are otherwise always checked and cleared together.
  */
-#define ISSET_LOCKOUT_BDB(R) 						\
+#define	ISSET_LOCKOUT_BDB(R)						\
     (FLD_ISSET((R)->lockout_flags, (REP_LOCKOUT_API | REP_LOCKOUT_OP)))
 
-#define CLR_LOCKOUT_BDB(R) 						\
+#define	CLR_LOCKOUT_BDB(R)						\
     (FLD_CLR((R)->lockout_flags, (REP_LOCKOUT_API | REP_LOCKOUT_OP)))
 
 /*
@@ -530,7 +596,7 @@ struct __rep_waiter {
  * areas and we want to manipulate it separately (see especially
  * in __rep_new_master).
  */
-#define CLR_RECOVERY_SETTINGS(R)					\
+#define	CLR_RECOVERY_SETTINGS(R)					\
 do {									\
 	(R)->sync_state = SYNC_OFF;					\
 	CLR_LOCKOUT_BDB(R);						\
@@ -571,7 +637,7 @@ do {									\
 
 #define	IS_CLIENT_PGRECOVER(env)					\
 	(IS_REP_CLIENT(env) &&						\
-	    (((env)->rep_handle->region)->sync_state ==  SYNC_PAGE))
+	    (((env)->rep_handle->region)->sync_state == SYNC_PAGE))
 
 /*
  * Macros to figure out if we need to do replication pre/post-amble processing.
@@ -579,7 +645,7 @@ do {									\
  * replication is running recovery or because it's a handle entirely owned by
  * the replication code (replication opens its own databases to track state).
  */
-#define REP_FLAGS_SET(env)						\
+#define	REP_FLAGS_SET(env)						\
 	((env)->rep_handle->region->flags != 0 ||			\
 	(env)->rep_handle->region->elect_flags != 0 ||			\
 	(env)->rep_handle->region->lockout_flags != 0)
@@ -610,6 +676,21 @@ do {									\
 	ZERO_LSN(rep->gen_base_lsn);					\
 } while (0)
 
+/* Macros to determine current replication configuration options. */
+#define	REP_CONFIG_IS_SET(env, flags)					\
+	(REP_ON(env) ?							\
+	FLD_ISSET(((env)->rep_handle->region)->config, flags) :	\
+	FLD_ISSET(((env)->rep_handle)->config, flags))
+#ifdef HAVE_REPLICATION_THREADS
+#define	PREFMAS_IS_SET(env)						\
+	(REP_CONFIG_IS_SET(env,					\
+	(REP_C_PREFMAS_MASTER | REP_C_PREFMAS_CLIENT)))
+#else
+#define	PREFMAS_IS_SET(env)	0
+#endif
+#define	IS_PREFMAS_MODE(env)						\
+	(REP_ON(env) && PREFMAS_IS_SET(env) &&				\
+	((env)->rep_handle->region)->config_nsites < 3)
 
 /*
  * Gap processing flags.  These provide control over the basic
@@ -619,12 +700,23 @@ do {									\
 #define	REP_GAP_REREQUEST	0x002	/* Gap request is a forced rerequest. */
 					/* REREQUEST is a superset of FORCE. */
 
+/*
+ * Internal options for rep_start_int().  These are used by preferred master
+ * mode to help coordinate between the sites during changes of master.
+ */
+#define	REP_START_FORCE_ROLECHG	0x001	/* Force role change to advance gen. */
+#define	REP_START_HOLD_CLIGEN	0x002	/* Hold client gen before doing
+					 * lsnhist match. */
+#define	REP_START_WAIT_LOCKMSG	0x004	/* Wait for REP_LOCKOUT_MSG. */
+
 /*
  * Flags indicating what kind of record we want to back up to, in the log.
  */
-#define	REP_REC_COMMIT		0x001 	/* Most recent commit record. */
-#define	REP_REC_PERM		0x002	/* Most recent perm record. */
+#define	REP_REC_COMMIT		0x001   /* Most recent commit record. */
+#define	REP_REC_PERM		0x002   /* Most recent perm record. */
 					/* PERM is a superset of COMMIT. */
+#define	REP_REC_PERM_DEL	0x004   /* Most recent PERM, or fail if a
+					 * file delete is found first. */
 
 /*
  * Permanent record types.
@@ -662,7 +754,7 @@ do {									\
  * region addresses.  This means that curinfo and its pointers to DBT data
  * must be recalculated for each process starting with the offset.
  */
-#define GET_CURINFO(rep, infop, curinfo)				\
+#define	GET_CURINFO(rep, infop, curinfo)				\
 do {									\
 	curinfo = R_ADDR(infop, rep->curinfo_off);			\
 	if ((curinfo)->uid.size > 0)					\
@@ -683,6 +775,19 @@ do {									\
 		(curinfo)->dir.data = NULL;				\
 } while (0)
 
+#if defined(HAVE_REPMGR_SSL)
+
+typedef struct __repmgr_ssl_config {
+	char	*repmgr_ca_cert_file;
+	char	*repmgr_ca_dir;
+	char	*repmgr_cert_file;
+	char	*repmgr_key_file;
+	char	*repmgr_key_file_passwd;
+	u_int	repmgr_ssl_verify_depth;
+} REPMGR_SSL_CONFIG;
+
+#endif
+
 /*
  * Per-process replication structure.
  *
@@ -759,6 +864,7 @@ struct __db_rep {
 
 	DB		*rep_db;	/* Bookkeeping database. */
 	DB		*lsn_db;	/* (Replicated) LSN history database. */
+	db_mutex_t	mtx_lsnhist;	/* LSN history database mutex. */
 
 	REP		*region;	/* In memory structure. */
 	u_int8_t	*bulk;		/* Shared memory bulk area. */
@@ -771,6 +877,7 @@ struct __db_rep {
 	DB_MPOOLFILE	*file_mpf;	/* Mpoolfile for current database. */
 	DB		*file_dbp;	/* This file's page info. */
 	DBC		*queue_dbc;	/* Dbc for a queue file. */
+	DB		*blob_dbp;	/* Blob file database. */
 
 	/*
 	 * Please change __rep_print_all (rep_stat.c) to track any changes made
@@ -798,8 +905,9 @@ struct __db_rep {
 	db_timeout_t	connection_retry_wait;
 	db_timeout_t	heartbeat_frequency; /* Max period between msgs. */
 	db_timeout_t	heartbeat_monitor_timeout;
-	u_int32_t	inqueue_msg_max;
-	u_int32_t	inqueue_bulkmsg_max;
+	db_timeout_t	write_forward_timeout;
+	u_int32_t	inqueue_max_gbytes;
+	u_int32_t	inqueue_max_bytes;
 
 	/* Thread synchronization. */
 	REPMGR_RUNNABLE *selector, **messengers, **elect_threads;
@@ -829,32 +937,41 @@ struct __db_rep {
 	CONNECTION_LIST	connections;
 	RETRY_Q_HEADER	retries;	/* Sites needing connection retry. */
 	struct {
-		int	size;
+		u_int32_t gbytes;
+		u_int32_t bytes;
 		STAILQ_HEAD(__repmgr_q_header, __repmgr_message) header;
 	} input_queue;
 
 	socket_t	listen_fd;
 	db_timespec	last_bcast;	/* Time of last broadcast msg. */
+	db_timespec	last_hbeat;	/* Time of last heartbeat (prefmas). */
 	db_timespec	l_listener_chk; /* Time to check local listener. */
 	db_timeout_t	l_listener_wait;/* Timeout to check local listener. */
 	db_timespec	m_listener_chk; /* Time to check master listener. */
 	db_timeout_t	m_listener_wait;/* Timeout to check master listener. */
 
+#if defined(HAVE_REPMGR_SSL)
+	REPMGR_SSL_CONFIG	repmgr_ssl_conf;
+	SSL_CTX		*repmgr_ssl_ctx;
+#endif
+
 	/*
 	 * Status of repmgr.  It is ready when repmgr is not yet started.  It
 	 * is running after repmgr is (re)started.  It is stopped if the env
-	 * of the running repmgr is closed, or the site is removed. 
+	 * of the running repmgr is closed, or the site is removed.
 	 */
 	enum { ready, running, stopped } repmgr_status;
 	int		new_connection;	  /* Since last master seek attempt. */
 	int		demotion_pending; /* We're being demoted to a view. */
 	int		takeover_pending; /* We've been elected master. */
+	int		rejoin_pending; /* Join group retry after rejection. */
 	int		gmdb_busy;
 	int		client_intent;	/* Will relinquish master role. */
 	int		gmdb_dirty;
 	int		have_gmdb;
 	int		seen_repmsg;
 	int		view_mismatch; /* View callback and gmdb don't match. */
+	int		abbrev_init;	/* For mixed version gmdb upgrade. */
 
 	/*
 	 * Flag to show what kind of transaction is currently in progress.
@@ -865,7 +982,7 @@ struct __db_rep {
 	 * we don't have a more proper direct channel to communicate information
 	 * between the originator of a transaction and the replication send()
 	 * function that has to wait for acks and decide what to do about them.
-	 */ 
+	 */
 	enum { none, gmdb_primary, gmdb_secondary } active_gmdb_update;
 	int		limbo_resolution_needed;
 
@@ -885,14 +1002,28 @@ struct __db_rep {
 	DB_LSN		durable_lsn;
 	DB		*gmdb;	/* Membership database handle. */
 	/*
-	 * Membership list restored from init file after crash during internal init.
+	 * Membership list restored from init file after crash
+	 * during internal init.
 	 */
 	u_int8_t	*restored_list;
 	size_t		restored_list_length;
 
+	/*
+	 * Preferred master mode indicator for a pending action.  A
+	 * master_switch is initiated when the preferred master site is
+	 * ready to take over as master.  A start_temp_master is initiated
+	 * when the client site needs to start as the temporary master.
+	 */
+	enum { no_action, master_switch, start_temp_master } prefmas_pending;
+	/* The LSN at the very beginning of preferred master site startup. */
+	DB_LSN prefmas_init_lsn;
+
 	/* Application's message dispatch call-back function. */
 	void  (*msg_dispatch) __P((DB_ENV *, DB_CHANNEL *,
 		DBT *, u_int32_t, u_int32_t));
+
+	/* Socket approval callback function. */
+	int	(*approval) __P((DB_ENV *, DB_REPMGR_SOCKET, int *, u_int32_t));
 #endif  /* HAVE_REPLICATION_THREADS */
 };
 
@@ -983,15 +1114,6 @@ struct __db_rep {
 /*
  * Control structure flags for replication communication infrastructure.
  */
-/*
- * Define old DB_LOG_ values that we must support here.  For reasons of
- * compatibility with old versions, these values must be reserved explicitly in
- * the list of flag values (below)
- */
-#define	DB_LOG_PERM_42_44	0x020
-#define	DB_LOG_RESEND_42_44	0x040
-#define	REPCTL_INIT_45		0x002	/* Back compatible flag value. */
-
 /*
  * Add new REPCTL flags to the end of this list to preserve compatibility
  * with old versions.
@@ -1001,14 +1123,11 @@ struct __db_rep {
 #define	REPCTL_GROUP_ESTD	0x004	/* Message from site in a group. */
 #define	REPCTL_INIT		0x008	/* Internal init message. */
 #define	REPCTL_LEASE		0x010	/* Lease related message. */
-			/*
-			 * Skip over reserved values 0x20
-			 * and 0x40, as explained above.
-			 */
+#define	REPCTL_PERM		0x020	/* Permanent message (e.g. commit). */
+#define	REPCTL_RESEND		0x040	/* Rerequested message. */
 #define	REPCTL_LOG_END		0x080	/* Approximate end of group-wide log. */
-#define	REPCTL_PERM		DB_LOG_PERM_42_44
-#define	REPCTL_RESEND		DB_LOG_RESEND_42_44
 #define	REPCTL_INMEM_ONLY	0x100	/* In-memory databases only. */
+#define	REPCTL_ENCRYPTED	0x200	/* Master is using encryption. */
 
 /*
  * File info flags for internal init.  The per-database (i.e., file) flag
@@ -1025,38 +1144,8 @@ struct __db_rep {
 #define	REPINFO_DB_LITTLEENDIAN	0x0001	/* File is little-endian lorder. */
 #define	REPINFO_PG_LITTLEENDIAN	0x0002	/* Page is little-endian lorder. */
 
-/*
- * Control message format for 4.6 release.  The db_timespec_t is
- * not a portable structure.  Therefore, in 4.6, replication among
- * mixed OSs such as Linux and Windows, which have different time_t
- * sizes, does not work.
- */
-typedef struct {
-	u_int32_t	rep_version;	/* Replication version number. */
-	u_int32_t	log_version;	/* Log version number. */
-
-	DB_LSN		lsn;		/* Log sequence number. */
-	u_int32_t	rectype;	/* Message type. */
-	u_int32_t	gen;		/* Generation number. */
-	db_timespec	msg_time;	/* Timestamp seconds for leases. */
-	u_int32_t	flags;		/* log_put flag value. */
-} REP_46_CONTROL;
-
-/*
- * Control message format for 4.5 release and earlier.
- */
-typedef struct {
-	u_int32_t	rep_version;	/* Replication version number. */
-	u_int32_t	log_version;	/* Log version number. */
-
-	DB_LSN		lsn;		/* Log sequence number. */
-	u_int32_t	rectype;	/* Message type. */
-	u_int32_t	gen;		/* Generation number. */
-	u_int32_t	flags;		/* log_put flag value. */
-} REP_OLD_CONTROL;
-
-#define	LEASE_REFRESH_MIN	30	/* Minimum number of refresh retries. */
-#define	LEASE_REFRESH_USEC	50000	/* Microseconds between refresh tries. */
+#define	LEASE_REFRESH_MIN	30     /* Minimum number of refresh retries. */
+#define	LEASE_REFRESH_USEC	50000  /* Microseconds between refresh tries. */
 
 /* Master granted lease information. */
 typedef struct __rep_lease_entry {
@@ -1066,18 +1155,6 @@ typedef struct __rep_lease_entry {
 	DB_LSN		lease_lsn;	/* Durable LSN lease applies to. */
 } REP_LEASE_ENTRY;
 
-/*
- * Old vote info where some fields were not fixed size.
- */
-typedef struct {
-	u_int32_t	egen;		/* Election generation. */
-	int		nsites;		/* Number of sites I've been in
-					 * communication with. */
-	int		nvotes;		/* Number of votes needed to win. */
-	int		priority;	/* My site's priority. */
-	u_int32_t	tiebreaker;	/* Tie-breaking quasi-random value. */
-} REP_OLD_VOTE_INFO;
-
 typedef struct {
 	u_int32_t	egen;		/* Voter's election generation. */
 	int		eid;		/* Voter's ID. */
@@ -1142,6 +1219,20 @@ typedef struct {
 	DBT		*objs;
 } linfo_t;
 
+/*
+ * Used to store information on the child transaction that opens a blob meta
+ * database.  In partial replication processing the child transaction of the
+ * blob meta database must be delayed until after processing the child
+ * transaction that opens the database that owns the BMD.
+ */
+typedef struct {
+	db_seq_t blob_file_id;
+	DB_LSN lsn;
+	u_int32_t child;
+	void *next;
+	void *prev;
+} DELAYED_BLOB_LIST;
+
 #if defined(__cplusplus)
 }
 #endif
