@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -9,6 +9,7 @@
 #include "db_config.h"
 
 #include "db_int.h"
+#include "dbinc/blob.h"
 #include "dbinc/db_page.h"
 #include "dbinc/log.h"
 #include "dbinc/txn.h"
@@ -171,6 +172,7 @@ __dbreg_setup(dbp, fname, dname, create_txnid)
 		F_SET(fnp, DBREG_EXCL);
 	fnp->txn_ref = 1;
 	fnp->mutex = dbp->mutex;
+	fnp->blob_file_id = dbp->blob_file_id;
 
 	dbp->log_filename = fnp;
 
@@ -741,6 +743,7 @@ __dbreg_failchk(env)
 			fnp->pid = 0;
 		} else {
 			F_SET(fnp, DB_FNAME_CLOSED);
+			MUTEX_UNLOCK(env, fnp->mutex);
 			if ((t_ret = __dbreg_close_id_int(env,
 			    fnp, DBREG_CLOSE, 1)) && ret == 0)
 				ret = t_ret;
@@ -773,6 +776,7 @@ __dbreg_log_close(env, fnp, txn, op)
 	DB_LOG *dblp;
 	DB_LSN r_unused;
 	int ret;
+	u_int32_t blob_file_lo, blob_file_hi;
 
 	dblp = env->lg_handle;
 	ret = 0;
@@ -788,10 +792,12 @@ __dbreg_log_close(env, fnp, txn, op)
 	memset(&fid_dbt, 0, sizeof(fid_dbt));
 	fid_dbt.data = fnp->ufid;
 	fid_dbt.size = DB_FILE_ID_LEN;
+	SET_LO_HI_VAR(fnp->blob_file_id, blob_file_lo, blob_file_hi);
 	if ((ret = __dbreg_register_log(env, txn, &r_unused,
 	    F_ISSET(fnp, DB_FNAME_DURABLE) ? 0 : DB_LOG_NOT_DURABLE,
 	    op, dbtp, &fid_dbt, fnp->id,
-	    fnp->s_type, fnp->meta_pgno, TXN_INVALID)) != 0) {
+	    fnp->s_type, fnp->meta_pgno, TXN_INVALID, blob_file_lo,
+	    blob_file_hi)) != 0) {
 		/*
 		 * We are trying to close, but the log write failed.
 		 * Unfortunately, close needs to plow forward, because
@@ -933,13 +939,11 @@ __dbreg_pluck_id(env, id)
 
 /*
  * __dbreg_log_id --
- *	Used for in-memory named files.  They are created in mpool and
- * are given id's early in the open process so that we can read and
- * create pages in the mpool for the files.  However, at the time that
- * the mpf is created, the file may not be fully created and/or its
- * meta-data may not be fully known, so we can't do a full dbregister.
- * This is a routine exported that will log a complete dbregister
- * record that will allow for both recovery and replication.
+ *	Log some kind of open of a file in a dbreg_register record.  In-memory
+ * named files are created in mpool and get a fileid early in the open process
+ * so that we can find and make pages for them.  However, at the time that the
+ * mpf is created, the file may not be fully created and/or its meta-data may
+ * not be fully known, so we can't do a full dbregister.
  *
  * PUBLIC: int __dbreg_log_id __P((DB *, DB_TXN *, int32_t, int));
  */
@@ -958,6 +962,7 @@ __dbreg_log_id(dbp, txn, id, needlock)
 	LOG *lp;
 	u_int32_t op;
 	int i, ret;
+	u_int32_t blob_file_lo, blob_file_hi;
 
 	env = dbp->env;
 	dblp = env->lg_handle;
@@ -965,15 +970,14 @@ __dbreg_log_id(dbp, txn, id, needlock)
 	fnp = dbp->log_filename;
 
 	/*
-	 * Verify that the fnp has been initialized, by seeing if it
-	 * has any non-zero bytes in it.
+	 * Detect when certain fields of the FNAME are not yet set; get them
+	 * from the dbp if necessary.  This is for replication of inmem dbs.
 	 */
 	for (i = 0; i < DB_FILE_ID_LEN; i++)
 		if (fnp->ufid[i] != 0)
 			break;
 	if (i == DB_FILE_ID_LEN)
 		memcpy(fnp->ufid, dbp->fileid, DB_FILE_ID_LEN);
-
 	if (fnp->s_type == DB_UNKNOWN)
 		fnp->s_type = dbp->type;
 
@@ -995,15 +999,17 @@ __dbreg_log_id(dbp, txn, id, needlock)
 	fid_dbt.data = dbp->fileid;
 	fid_dbt.size = DB_FILE_ID_LEN;
 
-	op = !F_ISSET(dbp, DB_AM_OPEN_CALLED) ? DBREG_PREOPEN :
+	op = !F2_ISSET(dbp, DB2_AM_MPOOL_OPENED) ? DBREG_PREOPEN :
 	    (F_ISSET(dbp, DB_AM_INMEM) ?
 	    (F2_ISSET(dbp, DB2_AM_EXCL) ? DBREG_XREOPEN : DBREG_REOPEN):
 	    (F2_ISSET(dbp, DB2_AM_EXCL) ? DBREG_XOPEN : DBREG_OPEN));
+	SET_LO_HI_VAR(fnp->blob_file_id, blob_file_lo, blob_file_hi);
 	ret = __dbreg_register_log(env, txn, &unused,
 	    F_ISSET(dbp, DB_AM_NOT_DURABLE) ? DB_LOG_NOT_DURABLE : 0,
 	    op | F_ISSET(fnp, DB_FNAME_DBREG_MASK),
 	    r_name.size == 0 ? NULL : &r_name, &fid_dbt, id,
-	    fnp->s_type, fnp->meta_pgno, fnp->create_txnid);
+	    fnp->s_type, fnp->meta_pgno, fnp->create_txnid,
+	    blob_file_lo, blob_file_hi);
 
 	if (needlock)
 		MUTEX_UNLOCK(env, lp->mtx_filelist);
