@@ -1,9 +1,9 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1999, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1999, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
- * $Id: env_method.c,v dabaaeb7d839 2010/08/03 17:28:53 mike $
+ * $Id$
  */
 
 #include "db_config.h"
@@ -20,6 +20,7 @@
 static int  __db_env_init __P((DB_ENV *));
 static void __env_err __P((const DB_ENV *, int, const char *, ...));
 static void __env_errx __P((const DB_ENV *, const char *, ...));
+static void __env_msg __P((const DB_ENV *, const char *, ...));
 static int  __env_get_create_dir __P((DB_ENV *, const char **));
 static int  __env_get_data_dirs __P((DB_ENV *, const char ***));
 static int  __env_get_data_len __P((DB_ENV *, u_int32_t *));
@@ -82,6 +83,11 @@ db_env_create(dbenvpp, flags)
 	if (flags != 0)
 		return (EINVAL);
 
+#ifdef HAVE_ERROR_HISTORY
+	/* Call thread local storage initializer at least once per process. */
+	__db_thread_init();
+#endif
+
 	/* Allocate the DB_ENV and ENV structures -- we always have both. */
 	if ((ret = __os_calloc(NULL, 1, sizeof(DB_ENV), &dbenv)) != 0)
 		return (ret);
@@ -185,6 +191,8 @@ __db_env_init(dbenv)
 	dbenv->get_errcall = __env_get_errcall;
 	dbenv->get_errfile = __env_get_errfile;
 	dbenv->get_errpfx = __env_get_errpfx;
+	dbenv->get_ext_file_dir = __env_get_blob_dir;
+	dbenv->get_ext_file_threshold = __env_get_blob_threshold_pp;
 	dbenv->get_feedback = __env_get_feedback;
 	dbenv->get_flags = __env_get_flags;
 	dbenv->get_home = __env_get_home;
@@ -214,8 +222,12 @@ __db_env_init(dbenv)
 	dbenv->get_mp_tablesize = __memp_get_mp_tablesize;
 	dbenv->get_msgcall = __env_get_msgcall;
 	dbenv->get_msgfile = __env_get_msgfile;
+	dbenv->get_msgpfx = __env_get_msgpfx;
 	dbenv->get_open_flags = __env_get_open_flags;
+	dbenv->get_region_dir = __memp_get_reg_dir;
 	dbenv->get_shm_key = __env_get_shm_key;
+	dbenv->get_slice_count = __env_get_slice_count;
+	dbenv->get_slices = __env_get_slices;
 	dbenv->get_thread_count = __env_get_thread_count;
 	dbenv->get_thread_id_fn = __env_get_thread_id_fn;
 	dbenv->get_thread_id_string_fn = __env_get_thread_id_string_fn;
@@ -253,6 +265,7 @@ __db_env_init(dbenv)
 	dbenv->memp_stat_print = __memp_stat_print_pp;
 	dbenv->memp_sync = __memp_sync_pp;
 	dbenv->memp_trickle = __memp_trickle_pp;
+	dbenv->msg = __env_msg;
 	dbenv->mutex_alloc = __mutex_alloc_pp;
 	dbenv->mutex_free = __mutex_free_pp;
 	dbenv->mutex_get_align = __mutex_get_align;
@@ -285,9 +298,9 @@ __db_env_init(dbenv)
 	dbenv->rep_set_config = __rep_set_config;
 	dbenv->rep_set_limit = __rep_set_limit;
 	dbenv->rep_set_nsites = __rep_set_nsites_pp;
-	dbenv->rep_set_priority = __rep_set_priority;
+	dbenv->rep_set_priority = __rep_set_priority_pp;
 	dbenv->rep_set_request = __rep_set_request;
-	dbenv->rep_set_timeout = __rep_set_timeout;
+	dbenv->rep_set_timeout = __rep_set_timeout_pp;
 	dbenv->rep_set_transport = __rep_set_transport_pp;
 	dbenv->rep_set_view = __rep_set_view;
 	dbenv->rep_start = __rep_start_pp;
@@ -301,6 +314,8 @@ __db_env_init(dbenv)
 	dbenv->repmgr_msg_dispatch = __repmgr_set_msg_dispatch;
 	dbenv->repmgr_set_ack_policy = __repmgr_set_ack_policy;
 	dbenv->repmgr_set_incoming_queue_max = __repmgr_set_incoming_queue_max;
+	dbenv->repmgr_set_socket = __repmgr_set_socket;
+	dbenv->repmgr_set_ssl_config = __repmgr_set_ssl_config_pp;
 	dbenv->repmgr_site = __repmgr_site;
 	dbenv->repmgr_site_by_eid = __repmgr_site_by_eid;
 	dbenv->repmgr_site_list = __repmgr_site_list_pp;
@@ -323,6 +338,8 @@ __db_env_init(dbenv)
 	dbenv->set_errfile = __env_set_errfile;
 	dbenv->set_errpfx = __env_set_errpfx;
 	dbenv->set_event_notify = __env_set_event_notify;
+	dbenv->set_ext_file_dir = __env_set_blob_dir;
+	dbenv->set_ext_file_threshold = __env_set_blob_threshold;
 	dbenv->set_feedback = __env_set_feedback;
 	dbenv->set_flags = __env_set_flags;
 	dbenv->set_intermediate_dir_mode = __env_set_intermediate_dir_mode;
@@ -351,7 +368,9 @@ __db_env_init(dbenv)
 	dbenv->set_mp_tablesize = __memp_set_mp_tablesize;
 	dbenv->set_msgcall = __env_set_msgcall;
 	dbenv->set_msgfile = __env_set_msgfile;
+	dbenv->set_msgpfx = __env_set_msgpfx;
 	dbenv->set_paniccall = __env_set_paniccall;
+	dbenv->set_region_dir = __memp_set_reg_dir;
 	dbenv->set_shm_key = __env_set_shm_key;
 	dbenv->set_thread_count = __env_set_thread_count;
 	dbenv->set_thread_id = __env_set_thread_id;
@@ -378,10 +397,16 @@ __db_env_init(dbenv)
 	dbenv->thread_id = __os_id;
 	dbenv->thread_id_string = __env_thread_id_string;
 
+	dbenv->mutex_failchk_timeout = US_PER_SEC;
+
 	env = dbenv->env;
 	__os_id(NULL, &env->pid_cache, NULL);
+	/*
+	 * The real envid of any existing environment is unknown until the
+	 * region is read. Setting all bits until then is a diagnostic aid.
+	 */
+	env->envid = ENVID_UNKNOWN;
 
-	env->db_ref = 0;
 	env->log_verify_wrap = __log_verify_wrap;
 	env->data_len = ENV_DEF_DATA_LEN;
 	TAILQ_INIT(&env->fdlist);
@@ -389,6 +414,7 @@ __db_env_init(dbenv)
 	if (!__db_isbigendian())
 		F_SET(env, ENV_LITTLEENDIAN);
 	F_SET(env, ENV_NO_OUTPUT_SET);
+	dbenv->db_msgfile = stdout;
 
 	return (0);
 }
@@ -398,15 +424,7 @@ __db_env_init(dbenv)
  *	DbEnv.err method.
  */
 static void
-#ifdef STDC_HEADERS
 __env_err(const DB_ENV *dbenv, int error, const char *fmt, ...)
-#else
-__env_err(dbenv, error, fmt, va_alist)
-	const DB_ENV *dbenv;
-	int error;
-	const char *fmt;
-	va_dcl
-#endif
 {
 	/* Message with error string, to stderr by default. */
 	DB_REAL_ERR(dbenv, error, DB_ERROR_SET, 1, fmt);
@@ -417,19 +435,23 @@ __env_err(dbenv, error, fmt, va_alist)
  *	DbEnv.errx method.
  */
 static void
-#ifdef STDC_HEADERS
 __env_errx(const DB_ENV *dbenv, const char *fmt, ...)
-#else
-__env_errx(dbenv, fmt, va_alist)
-	const DB_ENV *dbenv;
-	const char *fmt;
-	va_dcl
-#endif
 {
 	/* Message without error string, to stderr by default. */
 	DB_REAL_ERR(dbenv, 0, DB_ERROR_NOT_SET, 1, fmt);
 }
 
+/*
+ * __env_msg --
+ *	DbEnv.msg method.
+ */
+static void
+__env_msg(const DB_ENV *dbenv, const char *fmt, ...)
+{
+	/* Prnt the message, to stdout by default. */
+	DB_REAL_MSG(dbenv, fmt);
+}
+
 static int
 __env_get_home(dbenv, homep)
 	DB_ENV *dbenv;
@@ -508,10 +530,41 @@ __env_get_memory_init(dbenv, type, countp)
 	u_int32_t *countp;
 {
 	ENV *env;
+	REGENV *renv;
+	REGINFO *infop;
 
 	env = dbenv->env;
+	infop = env->reginfo;
+	if (F_ISSET(env, ENV_OPEN_CALLED))
+		renv = infop->primary;
+	else
+		renv = NULL;
 
 	switch (type) {
+	case DB_MEM_DATABASE:
+		if (F_ISSET(env, ENV_OPEN_CALLED)) {
+			MUTEX_LOCK(env, renv->mtx_regenv);
+			*countp = renv->initdatabases;
+			MUTEX_UNLOCK(env, renv->mtx_regenv);
+		} else
+			*countp = dbenv->db_init_databases;
+		break;
+	case DB_MEM_DATABASE_LENGTH:
+		if (F_ISSET(env, ENV_OPEN_CALLED)) {
+			MUTEX_LOCK(env, renv->mtx_regenv);
+			*countp = renv->initdblen;
+			MUTEX_UNLOCK(env, renv->mtx_regenv);
+		} else
+			*countp = dbenv->db_init_db_len;
+		break;
+	case DB_MEM_EXTFILE_DATABASE:
+		if (F_ISSET(env, ENV_OPEN_CALLED)) {
+			MUTEX_LOCK(env, renv->mtx_regenv);
+			*countp = renv->initextfiledbs;
+			MUTEX_UNLOCK(env, renv->mtx_regenv);
+		} else
+			*countp = dbenv->db_init_extfile_dbs;
+		break;
 	case DB_MEM_LOCK:
 		ENV_NOT_CONFIGURED(env,
 		    env->lk_handle, "DB_ENV->get_memory_init", DB_INIT_LOCK);
@@ -549,6 +602,15 @@ __env_get_memory_init(dbenv, type, countp)
 		else
 			*countp = dbenv->lg_fileid_init;
 		break;
+	case DB_MEM_REP_SITE:
+		ENV_NOT_CONFIGURED(env, env->rep_handle->region,
+		    "DB_ENV->get_memory_init", DB_INIT_REP);
+
+		if (REP_ON(env))
+			*countp = ((REP *)env->rep_handle->region)->initsites;
+		else
+			*countp = dbenv->rep_init_sites;
+		break;
 	case DB_MEM_TRANSACTION:
 		ENV_NOT_CONFIGURED(env,
 		    env->tx_handle, "DB_ENV->memory_init", DB_INIT_TXN);
@@ -563,6 +625,10 @@ __env_get_memory_init(dbenv, type, countp)
 		/* We always update thr_init when joining an env. */
 		*countp = dbenv->thr_init;
 		break;
+	default:
+		__db_errx(env, DB_STR("1608",
+		    "unknown type argument to DB_ENV->get_memory_init"));
+		return (EINVAL);
 	}
 
 	return (0);
@@ -635,22 +701,19 @@ __env_set_blob_threshold(dbenv, bytes, flags)
 	u_int32_t bytes;
 	u_int32_t flags;
 {
+	DB_ENV *slice;
+	DB_THREAD_INFO *ip;
 	ENV *env;
 	REGENV *renv;
 	REGINFO *infop;
-	DB_THREAD_INFO *ip;
+	int i;
 
 	env = dbenv->env;
 
-	if (__db_fchk(dbenv->env, "DB_ENV->set_blob_threshold", flags, 0) != 0)
+	if (__db_fchk(dbenv->env,
+	    "DB_ENV->set_ext_file_threshold", flags, 0) != 0)
 		return (EINVAL);
 
-	if (REP_ON(dbenv->env) && bytes != 0) {
-		__db_errx(dbenv->env,
-		    "Blobs are not supported with replication.");
-		return (EINVAL);
-	}
-
 	if (F_ISSET(env, ENV_OPEN_CALLED)) {
 		infop = env->reginfo;
 		renv = infop->primary;
@@ -659,6 +722,8 @@ __env_set_blob_threshold(dbenv, bytes, flags)
 		renv->blob_threshold = bytes;
 		MUTEX_UNLOCK(env, renv->mtx_regenv);
 		ENV_LEAVE(env, ip);
+		SLICE_FOREACH(dbenv, slice, i)
+			(void)__env_set_blob_threshold(slice, bytes, flags);
 	} else
 		dbenv->blob_threshold = bytes;
 
@@ -683,24 +748,40 @@ __env_set_memory_init(dbenv, type, count)
 
 	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_memory_init");
 	switch (type) {
+	case DB_MEM_DATABASE:
+		dbenv->db_init_databases = count;
+		break;
+	case DB_MEM_DATABASE_LENGTH:
+		dbenv->db_init_db_len = count;
+		break;
+	case DB_MEM_EXTFILE_DATABASE:
+		dbenv->db_init_extfile_dbs = count;
+		break;
 	case DB_MEM_LOCK:
 		dbenv->lk_init = count;
 		break;
-	case DB_MEM_LOCKOBJECT:
-		dbenv->lk_init_objects = count;
-		break;
 	case DB_MEM_LOCKER:
 		dbenv->lk_init_lockers = count;
 		break;
+	case DB_MEM_LOCKOBJECT:
+		dbenv->lk_init_objects = count;
+		break;
 	case DB_MEM_LOGID:
 		dbenv->lg_fileid_init = count;
 		break;
-	case DB_MEM_TRANSACTION:
-		dbenv->tx_init = count;
+	case DB_MEM_REP_SITE:
+		dbenv->rep_init_sites = count;
 		break;
 	case DB_MEM_THREAD:
 		dbenv->thr_init = count;
 		break;
+	case DB_MEM_TRANSACTION:
+		dbenv->tx_init = count;
+		break;
+	default:
+		__db_errx(env, DB_STR("1607",
+		    "unknown type argument to DB_ENV->set_memory_init"));
+		return (EINVAL);
 	}
 
 	return (0);
@@ -818,7 +899,7 @@ __env_set_blob_dir(dbenv, dir)
 
 	env = dbenv->env;
 
-	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_blob_dir");
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_ext_file_dir");
 
 	if (dbenv->db_blob_dir != NULL)
 		__os_free(env, dbenv->db_blob_dir);
@@ -921,6 +1002,8 @@ __env_set_encrypt(dbenv, passwd, flags)
 	 * We're going to need this often enough to keep around
 	 */
 	dbenv->passwd_len = strlen(dbenv->passwd) + 1;
+
+	dbenv->encrypt_flags = flags;
 	/*
 	 * The MAC key is for checksumming, and is separate from
 	 * the algorithm.  So initialize it here, even if they
@@ -954,7 +1037,7 @@ err:
 	COMPQUIET(passwd, NULL);
 	COMPQUIET(flags, 0);
 
-	__db_errx(dbenv->env, DB_STR("1557",
+	__db_errx(dbenv->env, DB_STR("1555",
 	    "library build did not include support for cryptography"));
 	return (DB_OPNOTSUP);
 #endif
@@ -986,25 +1069,24 @@ const FLAG_MAP EnvMap[] = {
 
 /*
  * __env_map_flags -- map from external to internal flags.
- * PUBLIC: void __env_map_flags __P((const FLAG_MAP *,
- * PUBLIC:       u_int, u_int32_t *, u_int32_t *));
+ * PUBLIC: void __env_map_flags
+ * PUBLIC:      __P((const FLAG_MAP *, u_int, u_int32_t, u_int32_t *));
  */
 void
-__env_map_flags(flagmap, mapsize, inflagsp, outflagsp)
+__env_map_flags(flagmap, mapsize, inflags, outflagsp)
 	const FLAG_MAP *flagmap;
 	u_int mapsize;
-	u_int32_t *inflagsp, *outflagsp;
+	u_int32_t inflags, *outflagsp;
 {
-
 	const FLAG_MAP *fmp;
 	u_int i;
 
 	for (i = 0, fmp = flagmap;
 	    i < mapsize / sizeof(flagmap[0]); ++i, ++fmp)
-		if (FLD_ISSET(*inflagsp, fmp->inflag)) {
+		if (FLD_ISSET(inflags, fmp->inflag)) {
 			FLD_SET(*outflagsp, fmp->outflag);
-			FLD_CLR(*inflagsp, fmp->inflag);
-			if (*inflagsp == 0)
+			FLD_CLR(inflags, fmp->inflag);
+			if (inflags == 0)
 				break;
 		}
 }
@@ -1030,6 +1112,12 @@ __env_fetch_flags(flagmap, mapsize, inflagsp, outflagsp)
 			FLD_SET(*outflagsp, fmp->inflag);
 }
 
+/*
+ * __env_get_flags --
+ *	Extract the DB_ENV->set_flags() values for the env.  Most come from the
+ *	flags in our DB_ENV handle, but the panic and hotbackup flags are
+ *	found by examining the shared region.
+ */
 static int
 __env_get_flags(dbenv, flagsp)
 	DB_ENV *dbenv;
@@ -1043,7 +1131,7 @@ __env_get_flags(dbenv, flagsp)
 	env = dbenv->env;
 	/* Some flags are persisted in the regions. */
 	if (env->reginfo != NULL &&
-	    ((REGENV *)env->reginfo->primary)->panic != 0)
+	    ((REGENV *)env->reginfo->primary)->envid != env->envid)
 		FLD_SET(*flagsp, DB_PANIC_ENVIRONMENT);
 
 	/* If the hotbackup counter is positive, set the flag indicating so. */
@@ -1073,11 +1161,13 @@ __env_set_flags(dbenv, flags, on)
 	int on;
 {
 	ENV *env;
+	DB_ENV *slice;
 	DB_THREAD_INFO *ip;
+	int i, mem_on, ret;
 	u_int32_t mapped_flags;
-	int mem_on, ret;
 
 	env = dbenv->env;
+	ret = 0;
 
 #define	OK_FLAGS							\
 	(DB_AUTO_COMMIT | DB_CDB_ALLDB | DB_DATABASE_LOCKING |		\
@@ -1160,13 +1250,17 @@ __env_set_flags(dbenv, flags, on)
 	}
 
 	mapped_flags = 0;
-	__env_map_flags(EnvMap, sizeof(EnvMap), &flags, &mapped_flags);
+	__env_map_flags(EnvMap, sizeof(EnvMap), flags, &mapped_flags);
 	if (on)
 		F_SET(dbenv, mapped_flags);
 	else
 		F_CLR(dbenv, mapped_flags);
 
-	return (0);
+	SLICE_FOREACH(dbenv, slice, i)
+		if ((ret = __env_set_flags(slice, flags, on)) != 0)
+			break;
+
+	return (ret);
 }
 
 /*
@@ -1336,6 +1430,34 @@ __env_get_intermediate_dir_mode(dbenv, modep)
 	return (0);
 }
 
+/*
+ * __env_set_home_dir --
+ *	Helper function for DB_CONFIG's "slice <ID> home <directory>" line.
+ *	This is *not* an API call. It is permitted only for slices.
+ *
+ * PUBLIC: int __env_set_home_dir __P((DB_ENV *, const char *));
+ */
+int
+__env_set_home_dir(dbenv, dir)
+	DB_ENV *dbenv;
+	const char *dir;
+{
+	ENV *env;
+
+	env = dbenv->env;
+
+#ifdef HAVE_SLICES
+	if (env->slice_container == NULL)
+		return (__env_not_sliced(env));
+	if (env->db_home != NULL)
+		__os_free(env, env->db_home);
+	return (__os_strdup(env, dir, &env->db_home));
+#else
+	COMPQUIET(dir, NULL);
+	return (__env_no_slices(env));
+#endif
+}
+
 /*
  * __env_set_metadata_dir --
  *	DB_ENV->set_metadata_dir.
@@ -1390,8 +1512,18 @@ __env_set_data_len(dbenv, data_len)
 	DB_ENV *dbenv;
 	u_int32_t data_len;
 {
+	DB_ENV *slice;
+	int i;
+
+	if (data_len == 0) {
+		__db_errx(dbenv->env, DB_STR("1593",
+"Maximum number of bytes to display for each key/data item can not be 0."));
+		return (EINVAL);
+	}
 
 	dbenv->env->data_len = data_len;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->env->data_len = data_len;
 	return (0);
 }
 
@@ -1487,12 +1619,16 @@ __env_set_errcall(dbenv, errcall)
 	DB_ENV *dbenv;
 	void (*errcall) __P((const DB_ENV *, const char *, const char *));
 {
+	DB_ENV *slice;
 	ENV *env;
+	int i;
 
 	env = dbenv->env;
 
 	F_CLR(env, ENV_NO_OUTPUT_SET);
 	dbenv->db_errcall = errcall;
+	SLICE_FOREACH(dbenv, slice, i)
+		__env_set_errcall(slice, errcall);
 }
 
 /*
@@ -1520,12 +1656,16 @@ __env_set_errfile(dbenv, errfile)
 	DB_ENV *dbenv;
 	FILE *errfile;
 {
+	DB_ENV *slice;
 	ENV *env;
+	int i;
 
 	env = dbenv->env;
 
 	F_CLR(env, ENV_NO_OUTPUT_SET);
 	dbenv->db_errfile = errfile;
+	SLICE_FOREACH(dbenv, slice, i)
+		__env_set_errfile(slice, errfile);
 }
 
 /*
@@ -1553,7 +1693,12 @@ __env_set_errpfx(dbenv, errpfx)
 	DB_ENV *dbenv;
 	const char *errpfx;
 {
+	DB_ENV *slice;
+	int i;
+
 	dbenv->db_errpfx = errpfx;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->db_errpfx = errpfx;
 }
 
 static int
@@ -1571,7 +1716,12 @@ __env_set_feedback(dbenv, feedback)
 	DB_ENV *dbenv;
 	void (*feedback) __P((DB_ENV *, int, int));
 {
+	DB_ENV *slice;
+	int i;
+
 	dbenv->db_feedback = feedback;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->db_feedback = feedback;
 	return (0);
 }
 
@@ -1594,11 +1744,18 @@ __env_get_thread_id_fn(dbenv, idp)
  *	DB_ENV->set_thread_id
  */
 static int
-__env_set_thread_id(dbenv, id)
+__env_set_thread_id(dbenv, id_func)
 	DB_ENV *dbenv;
-	void (*id) __P((DB_ENV *, pid_t *, db_threadid_t *));
+	void (*id_func) __P((DB_ENV *, pid_t *, db_threadid_t *));
 {
-	dbenv->thread_id = id;
+	DB_ENV *slice;
+	int i;
+
+	dbenv->thread_id = id_func;
+
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->thread_id = id_func;
+
 	return (0);
 }
 
@@ -1626,7 +1783,12 @@ __env_set_thread_id_string(dbenv, thread_id_string)
 	DB_ENV *dbenv;
 	char *(*thread_id_string) __P((DB_ENV *, pid_t, db_threadid_t, char *));
 {
+	DB_ENV *slice;
+	int i;
+
 	dbenv->thread_id_string = thread_id_string;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->thread_id_string = thread_id_string;
 	return (0);
 }
 
@@ -1644,7 +1806,7 @@ __env_get_isalive(dbenv, is_alivep)
 	env = dbenv->env;
 
 	if (F_ISSET(env, ENV_OPEN_CALLED) && env->thr_nbucket == 0) {
-		__db_errx(env, DB_STR("1562",
+		__db_errx(env, DB_STR("1504",
 	    "is_alive method specified but no thread region allocated"));
 		return (EINVAL);
 	}
@@ -1667,7 +1829,7 @@ __env_set_isalive(dbenv, is_alive)
 	env = dbenv->env;
 
 	if (F_ISSET(env, ENV_OPEN_CALLED) && env->thr_nbucket == 0) {
-		__db_errx(env, DB_STR("1563",
+		__db_errx(env, DB_STR("1504",
 	    "is_alive method specified but no thread region allocated"));
 		return (EINVAL);
 	}
@@ -1713,13 +1875,13 @@ __env_set_thread_count(dbenv, count)
  * __env_get_msgcall --
  *	{DB_ENV,DB}->get_msgcall.
  *
- * PUBLIC: void __env_get_msgcall
- * PUBLIC:     __P((DB_ENV *, void (**)(const DB_ENV *, const char *)));
+ * PUBLIC: void __env_get_msgcall __P((DB_ENV *,
+ * PUBLIC:		void (**)(const DB_ENV *, const char *, const char *)));
  */
 void
 __env_get_msgcall(dbenv, msgcallp)
 	DB_ENV *dbenv;
-	void (**msgcallp) __P((const DB_ENV *, const char *));
+	void (**msgcallp) __P((const DB_ENV *, const char *, const char *));
 {
 	if (msgcallp != NULL)
 		*msgcallp = dbenv->db_msgcall;
@@ -1729,15 +1891,20 @@ __env_get_msgcall(dbenv, msgcallp)
  * __env_set_msgcall --
  *	{DB_ENV,DB}->set_msgcall.
  *
- * PUBLIC: void __env_set_msgcall
- * PUBLIC:     __P((DB_ENV *, void (*)(const DB_ENV *, const char *)));
+ * PUBLIC: void __env_set_msgcall __P((DB_ENV *,
+ * PUBLIC:		void (*)(const DB_ENV *, const char *, const char *)));
  */
 void
 __env_set_msgcall(dbenv, msgcall)
 	DB_ENV *dbenv;
-	void (*msgcall) __P((const DB_ENV *, const char *));
+	void (*msgcall) __P((const DB_ENV *, const char *, const char *));
 {
+	DB_ENV *slice;
+	int i;
+
 	dbenv->db_msgcall = msgcall;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->db_msgcall = msgcall;
 }
 
 /*
@@ -1765,7 +1932,45 @@ __env_set_msgfile(dbenv, msgfile)
 	DB_ENV *dbenv;
 	FILE *msgfile;
 {
+	DB_ENV *slice;
+	int i;
+
 	dbenv->db_msgfile = msgfile;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->db_msgfile = msgfile;
+}
+
+/*
+ * __env_get_msgpfx --
+ *	{DB_ENV,DB}->get_msgpfx.
+ *
+ * PUBLIC: void __env_get_msgpfx __P((DB_ENV *, const char **));
+ */
+void
+__env_get_msgpfx(dbenv, msgpfxp)
+	DB_ENV *dbenv;
+	const char **msgpfxp;
+{
+	*msgpfxp = dbenv->db_msgpfx;
+}
+
+/*
+ * __env_set_msgpfx --
+ *	{DB_ENV,DB}->set_msgpfx.
+ *
+ * PUBLIC: void __env_set_msgpfx __P((DB_ENV *, const char *));
+ */
+void
+__env_set_msgpfx(dbenv, msgpfx)
+	DB_ENV *dbenv;
+	const char *msgpfx;
+{
+	DB_ENV *slice;
+	int i;
+
+	dbenv->db_msgpfx = msgpfx;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->db_msgpfx = msgpfx;
 }
 
 /*
@@ -1779,7 +1984,12 @@ __env_set_paniccall(dbenv, paniccall)
 	DB_ENV *dbenv;
 	void (*paniccall) __P((DB_ENV *, int));
 {
+	DB_ENV *slice;
+	int i;
+
 	dbenv->db_paniccall = paniccall;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->db_paniccall = paniccall;
 	return (0);
 }
 
@@ -1792,7 +2002,12 @@ __env_set_event_notify(dbenv, event_func)
 	DB_ENV *dbenv;
 	void (*event_func) __P((DB_ENV *, u_int32_t, void *));
 {
+	DB_ENV *slice;
+	int i;
+
 	dbenv->db_event_func = event_func;
+	SLICE_FOREACH(dbenv, slice, i)
+		slice->db_event_func = event_func;
 	return (0);
 }
 
@@ -1879,11 +2094,15 @@ __env_get_verbose(dbenv, which, onoffp)
 	case DB_VERB_REP_TEST:
 	case DB_VERB_REPMGR_CONNFAIL:
 	case DB_VERB_REPMGR_MISC:
+	case DB_VERB_REPMGR_SSL_ALL:
+	case DB_VERB_REPMGR_SSL_CONN:
+	case DB_VERB_REPMGR_SSL_IO:
+	case DB_VERB_SLICE:
 	case DB_VERB_WAITSFOR:
 		*onoffp = FLD_ISSET(dbenv->verbose, which) ? 1 : 0;
 		break;
 	default:
-		return (EINVAL);
+		return (USR_ERR(dbenv->env, EINVAL));
 	}
 	return (0);
 }
@@ -1900,6 +2119,9 @@ __env_set_verbose(dbenv, which, on)
 	u_int32_t which;
 	int on;
 {
+	DB_ENV *slice;
+	int i;
+
 	switch (which) {
 	case DB_VERB_BACKUP:
 	case DB_VERB_DEADLOCK:
@@ -1918,14 +2140,24 @@ __env_set_verbose(dbenv, which, on)
 	case DB_VERB_REP_TEST:
 	case DB_VERB_REPMGR_CONNFAIL:
 	case DB_VERB_REPMGR_MISC:
+	case DB_VERB_REPMGR_SSL_ALL:
+	case DB_VERB_REPMGR_SSL_CONN:
+	case DB_VERB_REPMGR_SSL_IO:
+	case DB_VERB_SLICE:
 	case DB_VERB_WAITSFOR:
 		if (on)
 			FLD_SET(dbenv->verbose, which);
 		else
 			FLD_CLR(dbenv->verbose, which);
+		SLICE_FOREACH(dbenv, slice, i) {
+			if (on)
+				FLD_SET(slice->verbose, which);
+			else
+				FLD_CLR(slice->verbose, which);
+		}
 		break;
 	default:
-		return (EINVAL);
+		return (USR_ERR(dbenv->env, EINVAL));
 	}
 	return (0);
 }
@@ -1944,7 +2176,7 @@ __db_mi_env(env, name)
 	__db_errx(env, DB_STR_A("1564",
 	    "%s: method not permitted when environment specified", "%s"),
 	    name);
-	return (EINVAL);
+	return (USR_ERR(env, EINVAL));
 }
 
 /*
@@ -1962,7 +2194,7 @@ __db_mi_open(env, name, after)
 	__db_errx(env, DB_STR_A("1565",
 	    "%s: method not permitted %s handle's open method", "%s %s"),
 	    name, after ? DB_STR_P("after") : DB_STR_P("before"));
-	return (EINVAL);
+	return (USR_ERR(env, EINVAL));
 }
 
 /*
@@ -2020,7 +2252,7 @@ __env_not_config(env, i, flags)
 	    "%s %s"), i, sub);
 	}
 
-	return (EINVAL);
+	return (USR_ERR(env, EINVAL));
 }
 
 /*
@@ -2036,9 +2268,15 @@ __env_get_timeout(dbenv, timeoutp, flags)
 	int ret;
 
 	ret = 0;
-	if (flags == DB_SET_REG_TIMEOUT) {
+	if (flags == DB_SET_REG_TIMEOUT)
 		*timeoutp = dbenv->envreg_timeout;
-	} else
+	else if (flags == DB_SET_MUTEX_FAILCHK_TIMEOUT)
+#ifdef HAVE_FAILCHK_BROADCAST
+		*timeoutp = dbenv->mutex_failchk_timeout;
+#else
+		ret = USR_ERR(dbenv->env, DB_OPNOTSUP);
+#endif
+	else
 		ret = __lock_get_env_timeout(dbenv, timeoutp, flags);
 	return (ret);
 }
@@ -2055,12 +2293,129 @@ __env_set_timeout(dbenv, timeout, flags)
 	db_timeout_t timeout;
 	u_int32_t flags;
 {
+	DB_ENV *slice;
+	int i;
 	int ret;
 
 	ret = 0;
 	if (flags == DB_SET_REG_TIMEOUT)
 		dbenv->envreg_timeout = timeout;
+	else if (flags == DB_SET_MUTEX_FAILCHK_TIMEOUT)
+#ifdef HAVE_FAILCHK_BROADCAST
+		dbenv->mutex_failchk_timeout = timeout;
+#else
+		ret = USR_ERR(dbenv->env, DB_OPNOTSUP);
+#endif
 	else
 		ret = __lock_set_env_timeout(dbenv, timeout, flags);
+	if (ret == 0)
+		SLICE_FOREACH(dbenv, slice, i)
+			if ((ret =
+			    __env_set_timeout(slice, timeout, flags)) != 0)
+				break;
 	return (ret);
 }
+
+/*
+ * __env_encrypt_adj_size --
+ *	Ajust the given size to the amount needed to meet the "chunk" needs of
+ *	the env's encryption algorithm. It is only intended for use as an
+ *	internal utility routine.
+ *
+ * PUBLIC: int __env_encrypt_adj_size __P((DB_ENV *, size_t, size_t *));
+ */
+int
+__env_encrypt_adj_size(dbenv, orig_size, ajustp)
+	DB_ENV *dbenv;
+	size_t orig_size;
+	size_t *ajustp;
+{
+#ifdef HAVE_CRYPTO
+	DB_CIPHER *db_cipher;
+#endif
+	ENV *env;
+
+	env = dbenv->env;
+#ifdef HAVE_CRYPTO
+	db_cipher = env->crypto_handle;
+	if (db_cipher != NULL)
+		*ajustp = db_cipher->adj_size(orig_size);
+	return (0);
+#else
+	COMPQUIET(orig_size, 0);
+	COMPQUIET(ajustp, NULL);
+	__db_errx(env, DB_STR("1555",
+	    "library build did not include support for cryptography"));
+	return (DB_OPNOTSUP);
+#endif
+}
+
+/*
+ * __env_encrypt --
+ *	Encrypt the given data using the environment's crypto handler.
+ *	It is only intended for use as an internal utility routine.
+ *
+ * PUBLIC: int __env_encrypt __P((DB_ENV *, u_int8_t *, u_int8_t *, size_t));
+ */
+int
+__env_encrypt(dbenv, iv, data, len)
+	DB_ENV *dbenv;
+	u_int8_t *iv;
+	u_int8_t *data;
+	size_t len;
+{
+#ifdef HAVE_CRYPTO
+	DB_CIPHER *db_cipher;
+#endif
+	ENV *env;
+
+	env = dbenv->env;
+#ifdef HAVE_CRYPTO
+	db_cipher = env->crypto_handle;
+	if (db_cipher != NULL)
+		return db_cipher->encrypt(env, db_cipher->data, iv, data, len);
+	return (0);
+#else
+	COMPQUIET(iv, NULL);
+	COMPQUIET(data, NULL);
+	COMPQUIET(len, 0);
+	__db_errx(env, DB_STR("1555",
+	    "library build did not include support for cryptography"));
+	return (DB_OPNOTSUP);
+#endif
+}
+
+/*
+ * __env_decrypt --
+ *	Decrypt the given data using the environment's crypto handler.
+ *	It is only intended for use as an internal utility routine.
+ *
+ * PUBLIC: int __env_decrypt __P((DB_ENV *, u_int8_t *, u_int8_t *, size_t));
+ */
+int
+__env_decrypt(dbenv, iv, data, len)
+	DB_ENV *dbenv;
+	u_int8_t *iv;
+	u_int8_t *data;
+	size_t len;
+{
+#ifdef HAVE_CRYPTO
+	DB_CIPHER *db_cipher;
+#endif
+	ENV *env;
+
+	env = dbenv->env;
+#ifdef HAVE_CRYPTO
+	db_cipher = env->crypto_handle;
+	if (db_cipher != NULL)
+		return db_cipher->decrypt(env, db_cipher->data, iv, data, len);
+	return (0);
+#else
+	COMPQUIET(iv, NULL);
+	COMPQUIET(data, NULL);
+	COMPQUIET(len, 0);
+	__db_errx(env, DB_STR("1555",
+	    "library build did not include support for cryptography"));
+	return (DB_OPNOTSUP);
+#endif
+}
