@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -19,12 +19,11 @@
 #include "dbinc/txn.h"
 
 static int	__db_log_corrupt __P((ENV *, DB_LSN *));
-static int	__env_init_rec_42 __P((ENV *));
-static int	__env_init_rec_43 __P((ENV *));
-static int	__env_init_rec_46 __P((ENV *));
 static int	__env_init_rec_47 __P((ENV *));
 static int	__env_init_rec_48 __P((ENV *));
 static int	__env_init_rec_53 __P((ENV *));
+static int	__env_init_rec_60 __P((ENV *));
+static int	__env_init_rec_60p1 __P((ENV *));
 static int	__log_earliest __P((ENV *, DB_LOGC *, int32_t *, DB_LSN *));
 
 static double	__lsn_diff __P((DB_LSN *, DB_LSN *, DB_LSN *, u_int32_t, int));
@@ -56,7 +55,6 @@ __db_apprec(env, ip, max_lsn, trunclsn, update, flags)
 	DB_TXNHEAD *txninfo;
 	DB_TXNREGION *region;
 	REGENV *renv;
-	REGINFO *infop;
 	__txn_ckp_args *ckp_args;
 	time_t now, tlow;
 	double nfiles;
@@ -77,20 +75,24 @@ __db_apprec(env, ip, max_lsn, trunclsn, update, flags)
 	ZERO_LSN(lsn);
 
 	/*
-	 * XXX
 	 * Get the log size.  No locking required because we're single-threaded
 	 * during recovery.
 	 */
 	log_size = ((LOG *)env->lg_handle->reginfo.primary)->log_size;
 
+	/*
+	 * When truly recovering (i.e., not replication) change the environment
+	 * magic from 0 (newly created) to recovery-in-progress.
+	 */
+	renv = env->reginfo->primary;
+	if (LF_ISSET(DB_RECOVER | DB_RECOVER_FATAL))
+		renv->magic = DB_REGION_MAGIC_RECOVER;
+
 	/*
 	 * If we need to, update the env handle timestamp.
 	 */
-	if (update && REP_ON(env)) {
-		infop = env->reginfo;
-		renv = infop->primary;
+	if (update && REP_ON(env))
 		(void)time(&renv->rep_timestamp);
-	}
 
 	/* Set in-recovery flags. */
 	F_SET(env->lg_handle, DBLOG_RECOVER);
@@ -126,10 +128,10 @@ __db_apprec(env, ip, max_lsn, trunclsn, update, flags)
 			if ((p = strchr(t2, '\n')) != NULL)
 				*p = '\0';
 
+			ret = USR_ERR(env, EINVAL);
 			__db_errx(env, DB_STR_A("1509",
 		    "Invalid recovery timestamp %s; earliest time is %s",
 			    "%s %s"), t1, t2);
-			ret = EINVAL;
 			goto err;
 		}
 	}
@@ -238,7 +240,7 @@ __db_apprec(env, ip, max_lsn, trunclsn, update, flags)
 			/* We have a recent checkpoint.  This is LSN (1). */
 			if ((ret = __txn_ckp_read(env,
 			    data.data, &ckp_args)) != 0) {
-				__db_errx(env, DB_STR_A("1511",
+				__db_errx(env, DB_STR_A("4506",
 				    "Invalid checkpoint record at [%ld][%ld]",
 				    "%ld %ld"), (u_long)ckp_lsn.file,
 				    (u_long)ckp_lsn.offset);
@@ -563,7 +565,7 @@ done:
 			if (ret == DB_NOTFOUND)
 				ret = 0;
 			else
-				__db_errx(env, DB_STR("1516",
+				__db_errx(env, DB_STR("1510",
 				    "First log record not found"));
 			goto err;
 		}
@@ -572,7 +574,7 @@ done:
 			/* We have a recent checkpoint.  This is LSN (1). */
 			if ((ret = __txn_ckp_read(env,
 			    data.data, &ckp_args)) != 0) {
-				__db_errx(env, DB_STR_A("1517",
+				__db_errx(env, DB_STR_A("4506",
 				    "Invalid checkpoint record at [%ld][%ld]",
 				    "%ld %ld"), (u_long)first_lsn.file,
 				    (u_long)first_lsn.offset);
@@ -628,6 +630,12 @@ err:	if (logc != NULL && (t_ret = __logc_close(logc)) != 0 && ret == 0)
 
 	dbenv->tx_timestamp = 0;
 
+	/*
+	 * Failure means that the env has panicked. Disable locking so that the
+	 * env can close without its mutexes calls causing additional panics.
+	 */
+	if (ret != 0)
+		F_SET(env->dbenv, DB_ENV_NOLOCKING);
 	F_CLR(env->lg_handle, DBLOG_RECOVER);
 	F_CLR(region, TXN_IN_RECOVERY);
 
@@ -725,7 +733,7 @@ __log_backup(env, logc, max_lsn, start_lsn)
 		 * done.  Break with DB_NOTFOUND.
 		 */
 		if (IS_ZERO_LSN(lsn)) {
-			ret = DB_NOTFOUND;
+			ret = USR_ERR(env, DB_NOTFOUND);
 			break;
 		}
 		__os_free(env, ckp_args);
@@ -927,7 +935,30 @@ __env_init_rec(env, version)
 	 */
 	if (version == DB_LOGVERSION)
 		goto done;
+
+	/*
+	 * DB_LOGVERSION_62 added a new value to the enumeration APPNAME,
+	 * but the change does not require upgrading the logs.
+	 */
+
+	/* DB_LOGVERSION_61 add the blob file id to the dbreg logs. */
+	if (version > DB_LOGVERSION_60p1)
+		goto done;
+	if ((ret = __env_init_rec_60p1(env)) != 0)
+		goto err;
+
+	/*
+	 * DB_LOGVERSION_60p1 changed the two u_int32_t offset fields in the
+	 * log for fop_write_file into a single int64.
+	 */
+	if (version > DB_LOGVERSION_60)
+		goto done;
+	if ((ret = __env_init_rec_60(env)) != 0)
+		goto err;
+
 	/* DB_LOGVERSION_53 changed the heap addrem log record. */
+	if (version > DB_LOGVERSION_53)
+		goto done;
 	if ((ret = __env_init_rec_53(env)) != 0)
 		goto err;
 	/*
@@ -951,8 +982,6 @@ __env_init_rec(env, version)
 		goto err;
 	if (version == DB_LOGVERSION_47)
 		goto done;
-	if ((ret = __env_init_rec_46(env)) != 0)
-		goto err;
 	/*
 	 * There are no log record/recovery differences between 4.4 and 4.5.
 	 * The log version changed due to checksum.  There are no log recovery
@@ -961,87 +990,19 @@ __env_init_rec(env, version)
 	 */
 	if (version >= DB_LOGVERSION_44)
 		goto done;
-	if ((ret = __env_init_rec_43(env)) != 0)
-		goto err;
 	if (version == DB_LOGVERSION_43)
 		goto done;
 	if (version != DB_LOGVERSION_42) {
+		ret = USR_ERR(env, EINVAL);
 		__db_errx(env, DB_STR_A("1523", "Unknown version %lu",
 		    "%lu"), (u_long)version);
-		ret = EINVAL;
 		goto err;
 	}
-	ret = __env_init_rec_42(env);
 
 done:
 err:	return (ret);
 }
 
-static int
-__env_init_rec_42(env)
-	ENV *env;
-{
-	int ret;
-
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __db_relink_42_recover, DB___db_relink_42)) != 0)
-		goto err;
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __db_pg_alloc_42_recover, DB___db_pg_alloc_42)) != 0)
-		goto err;
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __db_pg_free_42_recover, DB___db_pg_free_42)) != 0)
-		goto err;
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __db_pg_freedata_42_recover, DB___db_pg_freedata_42)) != 0)
-		goto err;
-#ifdef HAVE_HASH
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __ham_metagroup_42_recover, DB___ham_metagroup_42)) != 0)
-		goto err;
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __ham_groupalloc_42_recover, DB___ham_groupalloc_42)) != 0)
-		goto err;
-#endif
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __txn_ckp_42_recover, DB___txn_ckp_42)) != 0)
-		goto err;
-err:
-	return (ret);
-}
-
-static int
-__env_init_rec_43(env)
-	ENV *env;
-{
-	int ret;
-
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __bam_relink_43_recover, DB___bam_relink_43)) != 0)
-		goto err;
-	/*
-	 * We want to use the 4.2-based txn_regop record.
-	 */
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __txn_regop_42_recover, DB___txn_regop_42)) != 0)
-		goto err;
-err:
-	return (ret);
-}
-
-static int
-__env_init_rec_46(env)
-	ENV *env;
-{
-	int ret;
-
-	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
-	    __bam_merge_44_recover, DB___bam_merge_44)) != 0)
-		goto err;
-
-err:	return (ret);
-}
-
 static int
 __env_init_rec_47(env)
 	ENV *env;
@@ -1118,3 +1079,58 @@ __env_init_rec_53(env)
 err:
 	return (ret);
 }
+
+static int
+__env_init_rec_60(env)
+	ENV *env;
+{
+	int ret;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __fop_create_60_recover, DB___fop_create_60)) != 0)
+		goto err;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __fop_remove_60_recover, DB___fop_remove_60)) != 0)
+		goto err;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __fop_rename_60_recover, DB___fop_rename_60)) != 0)
+		goto err;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __fop_rename_noundo_60_recover, DB___fop_rename_noundo_60)) != 0)
+		goto err;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __fop_file_remove_60_recover, DB___fop_file_remove_60)) != 0)
+		goto err;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __fop_write_60_recover, DB___fop_write_60)) != 0)
+		goto err;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __fop_write_file_60_recover, DB___fop_write_file_60)) != 0)
+		goto err;
+err:
+	return (ret);
+}
+
+static int
+__env_init_rec_60p1(env)
+	ENV *env;
+{
+	int ret;
+
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __dbreg_register_42_recover, DB___dbreg_register_42)) != 0)
+		goto err;
+#ifdef HAVE_HEAP
+	if ((ret = __db_add_recovery_int(env, &env->recover_dtab,
+	    __heap_addrem_60_recover, DB___heap_addrem_60)) != 0)
+		goto err;
+#endif
+err:
+	return (ret);
+}
