@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2004, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2004, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -19,9 +19,10 @@
 #define	PID_LEN		(25)				/* PID entry length */
 
 #define	REGISTRY_LOCK(env, pos, nowait)					\
-	__os_fdlock(env, (env)->dbenv->registry, (off_t)(pos), 1, nowait)
+	__os_fdlock(env, (env)->dbenv->registry, (off_t)(pos),		\
+	    DB_LOCK_WRITE, nowait)
 #define	REGISTRY_UNLOCK(env, pos)					\
-	__os_fdlock(env, (env)->dbenv->registry, (off_t)(pos), 0, 0)
+	__os_fdlock(env, (env)->dbenv->registry, (off_t)(pos), DB_LOCK_NG, 0)
 #define	REGISTRY_EXCL_LOCK(env, nowait)					\
 	REGISTRY_LOCK(env, 1, nowait)
 #define	REGISTRY_EXCL_UNLOCK(env)					\
@@ -30,6 +31,7 @@
 static	int __envreg_add __P((ENV *, int *, u_int32_t));
 static	int __envreg_pid_compare __P((const void *, const void *));
 static	int __envreg_create_active_pid __P((ENV *, char *));
+static	int __envreg_add_active_pid __P((ENV*, char *));
 
 /*
  * Support for portable, multi-process database environment locking, based on
@@ -137,24 +139,9 @@ __envreg_register(env, need_recoveryp, flags)
 
 	if (FLD_ISSET(dbenv->verbose, DB_VERB_REGISTER))
 		__db_msg(env, DB_STR_A("1524",
-	"%lu: register environment", "%lu"), (u_long)pid);
+		    "%lu: register environment", "%lu"), (u_long)pid);
 
-	/* Build the path name and open the registry file. */
-	if ((ret = __db_appname(env,
-	    DB_APP_NONE, REGISTER_FILE, NULL, &pp)) != 0)
-		goto err;
-	if ((ret = __os_open(env, pp, 0,
-	    DB_OSO_CREATE, DB_MODE_660, &dbenv->registry)) != 0)
-		goto err;
-
-	/*
-	 * Wait for an exclusive lock on the file.
-	 *
-	 * !!!
-	 * We're locking bytes that don't yet exist, but that's OK as far as
-	 * I know.
-	 */
-	if ((ret = REGISTRY_EXCL_LOCK(env, 0)) != 0)
+	if ((ret = __envreg_registry_open(env, &pp, DB_OSO_CREATE)) != 0)
 		goto err;
 
 	/*
@@ -176,7 +163,6 @@ __envreg_register(env, need_recoveryp, flags)
 	/* Register this process. */
 	if ((ret = __envreg_add(env, need_recoveryp, flags)) != 0)
 		goto err;
-
 	/*
 	 * Release our exclusive lock if we don't need to run recovery.  If
 	 * we need to run recovery, ENV->open will call back into register
@@ -186,17 +172,13 @@ __envreg_register(env, need_recoveryp, flags)
 		goto err;
 
 	if (0) {
-err:		*need_recoveryp = 0;
-
+err:
 		/*
 		 * !!!
 		 * Closing the file handle must release all of our locks.
 		 */
-		if (dbenv->registry != NULL)
-			(void)__os_closehandle(env, dbenv->registry);
-		dbenv->registry = NULL;
+		(void)__envreg_registry_close(env);
 	}
-
 	if (pp != NULL)
 		__os_free(env, pp);
 
@@ -222,17 +204,16 @@ __envreg_add(env, need_recoveryp, flags)
 	size_t nr, nw;
 	u_int lcnt;
 	u_int32_t bytes, mbytes, orig_flags;
-	int need_recovery, ret, t_ret;
+	int need_failchk, ret, t_ret;
 	char *p, buf[PID_LEN + 10], pid_buf[PID_LEN + 10];
 
 	dbenv = env->dbenv;
-	need_recovery = 0;
+	need_failchk = t_ret = 0;
 	COMPQUIET(dead, 0);
 	COMPQUIET(p, NULL);
 	ip = NULL;
 
-	/* Get a copy of our process ID. */
-	dbenv->thread_id(dbenv, &pid, NULL);
+	pid = env->pid_cache;
 	snprintf(pid_buf, sizeof(pid_buf), PID_FMT, (u_long)pid);
 
 	if (FLD_ISSET(dbenv->verbose, DB_VERB_REGISTER))
@@ -269,7 +250,7 @@ kill_all:	/*
 		 * registering.
 		 */
 		if (nr != PID_LEN) {
-			need_recovery = 1;
+			need_failchk = 1;
 			break;
 		}
 
@@ -299,7 +280,7 @@ kill_all:	/*
 		}
 
 #if DB_ENVREG_KILL_ALL
-		if (need_recovery) {
+		if (need_failchk) {
 			pid = (pid_t)strtoul(buf, NULL, 10);
 			(void)kill(pid, SIGKILL);
 
@@ -318,7 +299,7 @@ kill_all:	/*
 				__db_msg(env, DB_STR_A("1530",
 				    "%02u: %s: FAILED", "%02u %s"), lcnt, p);
 
-			need_recovery = 1;
+			need_failchk = 1;
 			dead = pos;
 #if DB_ENVREG_KILL_ALL
 			goto kill_all;
@@ -331,34 +312,45 @@ kill_all:	/*
 				    "%02u: %s: LOCKED", "%02u %s"), lcnt, p);
 	}
 
+	/* Check for a panic; if so there's no need to call failchk. */
+	if ((t_ret = __env_attach(env, NULL, 0, 0)) != 0)
+		goto sig_proc;
+	infop = env->reginfo;
+	renv = infop->primary;
+	*need_recoveryp = renv->envid != env->envid;
+	(void)__env_detach(env, 0);
+	if (*need_recoveryp)
+		return (0);
+
 	/*
-	 * If we have to perform recovery...
+	 * If we have to perform failchk...
 	 *
 	 * Mark all slots empty.  Registry ignores empty slots we can't lock,
 	 * so it doesn't matter if any of the processes are in the middle of
 	 * exiting Berkeley DB -- they'll discard their lock when they exit.
 	 */
-	if (need_recovery) {
+	if (need_failchk) {
 		if (FLD_ISSET(dbenv->verbose, DB_VERB_REGISTER))
-			__db_msg(env, "%lu: recovery required", (u_long)pid);
+			__db_msg(env,
+			    "%lu: failchk recovery required", (u_long)pid);
 
 		if (LF_ISSET(DB_FAILCHK) || LF_ISSET(DB_FAILCHK_ISALIVE)) {
 			if (FLD_ISSET(dbenv->verbose, DB_VERB_REGISTER))
 				__db_msg(env,
 				    "%lu: performing failchk", (u_long)pid);
 
-			if (LF_ISSET(DB_FAILCHK_ISALIVE))
-				if ((ret = __envreg_create_active_pid(
-				    env, pid_buf)) != 0)
-					goto sig_proc;
+			if (LF_ISSET(DB_FAILCHK_ISALIVE) && (ret =
+			    __envreg_create_active_pid(env, pid_buf)) != 0)
+				goto sig_proc;
 
-			/* The environment will already exist, so we do not
+			/*
+			 * The environment will already exist, so we do not
 			 * want DB_CREATE set, nor do we want any recovery at
 			 * this point.  No need to put values back as flags is
 			 * passed in by value.  Save original dbenv flags in
 			 * case we need to recover/remove existing environment.
 			 * Set DB_ENV_FAILCHK before attach to help ensure we
-			 * dont block on a mutex held by the dead process.
+			 * don't block on a mutex held by the dead process.
 			 */
 			LF_CLR(DB_CREATE | DB_RECOVER | DB_RECOVER_FATAL);
 			orig_flags = dbenv->flags;
@@ -367,47 +359,56 @@ kill_all:	/*
 			if ((ret = __env_attach_regions(
 			    dbenv, flags, orig_flags, 0)) != 0)
 				goto sig_proc;
-			if ((t_ret =
-			    __env_set_state(env, &ip, THREAD_FAILCHK)) != 0 &&
-			    ret == 0)
+			if ((t_ret = __env_set_state(env,
+			   &ip, THREAD_FAILCHK)) != 0 && ret == 0)
 				ret = t_ret;
-			if ((t_ret =
-			    __env_failchk_int(dbenv)) != 0 && ret == 0)
+			if (ret == 0 && (t_ret = __env_failchk_int(dbenv)) != 0)
 				ret = t_ret;
+			if (FLD_ISSET(dbenv->verbose, DB_VERB_REGISTER))
+				__db_msg(env,
+				    "%lu: failchk returned %d, ret is %d",
+				    (u_long)pid, t_ret, ret);
 
 			/* Free active pid array if used. */
 			if (LF_ISSET(DB_FAILCHK_ISALIVE)) {
-				DB_GLOBAL(num_active_pids) = 0;
-				DB_GLOBAL(size_active_pids) = 0;
-				__os_free( env, DB_GLOBAL(active_pids));
+				env->num_active_pids = 0;
+				env->size_active_pids = 0;
+				__os_free(env, env->active_pids);
+				env->active_pids = NULL;
 			}
 
 			/* Detach from environment and deregister thread. */
-			if ((t_ret =
-			    __env_refresh(dbenv, orig_flags, 0)) != 0 &&
-			    ret == 0)
+			if ((t_ret = __env_refresh(dbenv,
+			    orig_flags, 0)) != 0 && ret == 0)
 				ret = t_ret;
+			F_CLR(env, ENV_OPEN_CALLED);
+
 			if (ret == 0) {
 				if ((ret = __os_seek(env, dbenv->registry,
-				    0, 0,(u_int32_t)dead)) != 0 ||
+				    0, 0, (u_int32_t)dead)) != 0 ||
 				    (ret = __os_write(env, dbenv->registry,
 				    PID_EMPTY, PID_LEN, &nw)) != 0)
 					return (ret);
-				need_recovery = 0;
+				need_failchk = 0;
 				goto add;
 			}
 
 		}
 		/* If we can't attach, then we cannot set DB_REGISTER panic. */
-sig_proc:	if (__env_attach(env, NULL, 0, 0) == 0) {
+sig_proc:
+		if (FLD_ISSET(dbenv->verbose, DB_VERB_REGISTER))
+			__db_msg(env, "%lu: sig_proc attaching errs %s/ret %s",
+			    (u_long)pid, db_strerror(t_ret), db_strerror(ret));
+		if (__env_attach(env, NULL, 0, 0) == 0) {
 			infop = env->reginfo;
 			renv = infop->primary;
-			/* Indicate DB_REGSITER panic.  Also, set environment
-			 * panic as this is the panic trigger mechanism in
-			 * the code that everything looks for.
+			/*
+			 * Indicate DB_REGISTER panic.  Also, set (or re-set)
+			 * environment panic as this is the panic trigger
+			 * mechanism in the code that everything looks for.
 			 */
 			renv->reg_panic = 1;
-			renv->panic = 1;
+			renv->envid = ENVID_PANIC;
 			(void)__env_detach(env, 0);
 		}
 
@@ -479,17 +480,85 @@ add:	if ((ret = __os_seek(env, dbenv->registry, 0, 0, 0)) != 0)
 			    (ret = __os_write(env,
 			    dbenv->registry, pid_buf, PID_LEN, &nw)) != 0)
 				return (ret);
+			/* Add the entry for this process. */
+			if ((ret = __envreg_add_active_pid(env, pid_buf)) != 0)
+				return (ret);
 			dbenv->registry_off = (u_int32_t)pos;
 			break;
 		}
 	}
 
-	if (need_recovery)
+	if (need_failchk)
 		*need_recoveryp = 1;
 
 	return (ret);
 }
 
+/*
+ * __envreg_unregister_pid --
+ *	Unregister a process by pid, optionally with its offset in the registry.
+ *
+ * Parameters:
+ *	If the caller knows the entry's loation in the registry, they can pass
+ *	it in via 'offset'. If not known, pass in 0.
+ *
+ * PUBLIC: int __envreg_unregister_pid __P((ENV *, pid_t, u_int32_t));
+ */
+int
+__envreg_unregister_pid(env, pid, offset)
+	ENV *env;
+	pid_t  pid;
+	u_int32_t offset;
+{
+	DB_FH *registry;
+	size_t nbytes;
+	int ret, t_ret;
+	char buf[PID_LEN];
+
+	registry = env->dbenv->registry;
+	if (offset != 0) {
+		/* Verify that the pid is at the specified offset. */
+		if ((ret = __os_io(env, DB_IO_READ,
+		    registry, 0, 0, offset, PID_LEN,
+		    (u_int8_t *)buf, &nbytes)) != 0)
+			goto err;
+		if (nbytes != PID_LEN || pid != (pid_t)strtoul(buf, NULL, 10)) {
+not_found:
+			ret = USR_ERR(env, DB_NOTFOUND);
+			__db_errx(env, "__envreg_unregister_pid: %lu not found",
+				    (u_long)pid);
+			goto err;
+		}
+	} else {
+		/*
+		 * The caller did not tell us where to find the process, so
+		 * search for it.
+		 */
+		if ((ret = __os_seek(env, registry, 0, 0, 0)) != 0)
+			goto err;
+		for (;;) {
+			if ((ret = __os_read(
+			    env, registry, buf, PID_LEN, &nbytes)) != 0)
+				goto err;
+			/*
+			 * A too-short record means that we reached EOF without
+			 * finding the process.
+			 */
+			if (nbytes != PID_LEN)
+				goto not_found;
+			if (pid == (pid_t)strtoul(buf, NULL, 10))
+				break;
+			offset += PID_LEN;
+		}
+	}
+	ret = __os_io(env, DB_IO_WRITE,
+	    registry, 0, 0, offset, PID_LEN, (u_int8_t *)PID_EMPTY, &nbytes);
+err:
+	if ((t_ret = __envreg_registry_close(env)) != 0 && ret == 0)
+		ret = t_ret;
+	return (ret);
+}
+
 /*
  * __envreg_unregister --
  *	Unregister a ENV handle.
@@ -502,7 +571,6 @@ __envreg_unregister(env, recovery_failed)
 	int recovery_failed;
 {
 	DB_ENV *dbenv;
-	size_t nw;
 	int ret, t_ret;
 
 	dbenv = env->dbenv;
@@ -524,10 +592,8 @@ __envreg_unregister(env, recovery_failed)
 	 * lock, and threads of control reviewing the register file ignore any
 	 * slots which they can't lock.
 	 */
-	if ((ret = __os_seek(env,
-	    dbenv->registry, 0, 0, dbenv->registry_off)) != 0 ||
-	    (ret = __os_write(
-	    env, dbenv->registry, PID_EMPTY, PID_LEN, &nw)) != 0)
+	if ((ret = __envreg_unregister_pid(env,
+	    env->pid_cache, dbenv->registry_off)) != 0)
 		goto err;
 
 	/*
@@ -543,14 +609,92 @@ __envreg_unregister(env, recovery_failed)
 	 * also releasing our slot lock, we could race.  That can't happen, I
 	 * don't think.
 	 */
-err:	if ((t_ret =
-	    __os_closehandle(env, dbenv->registry)) != 0 && ret == 0)
+err:
+	if (dbenv->registry != NULL &&
+	    (t_ret = __os_closehandle(env, dbenv->registry)) != 0 && ret == 0)
 		ret = t_ret;
 
 	dbenv->registry = NULL;
 	return (ret);
 }
 
+ /*
+ * __envreg_registry_open --
+ *	Open the registry file, possibly creating it if the open mode contains
+ *	DB_OSO_CREATE. Obtain an exclusive lock on the registry.
+ *
+ * PUBLIC: int __envreg_registry_open __P((ENV *, char **, u_int32_t));
+ */
+int
+__envreg_registry_open(env, namep, os_open_flags)
+	ENV *env;
+	char **namep;
+	u_int32_t os_open_flags;
+{
+	int ret;
+
+	ret = 0;
+
+	/* Build the path name and open the registry file. */
+	if ((ret = __db_appname(env,
+	    DB_APP_NONE, REGISTER_FILE, NULL, namep)) != 0) {
+		__db_err(env, ret,
+		    "__envreg_register_open: appname failed for %s",
+		    REGISTER_FILE);
+		goto err;
+	}
+	if ((ret = __os_open(env, *namep, 0,
+	    os_open_flags, DB_MODE_660, &env->dbenv->registry)) != 0) {
+		if (ret != ENOENT)
+			__db_err(env, ret,
+			    "__envreg_register_open failed for %s", *namep);
+		goto err;
+	}
+
+	/*
+	 * Wait for an exclusive lock on the file.
+	 *
+	 * !!!
+	 * We're locking bytes that don't yet exist, but that's OK as far as
+	 * I know.
+	 */
+	if ((ret = REGISTRY_EXCL_LOCK(env, 0)) != 0)
+		goto err;
+	if (FLD_ISSET(env->dbenv->verbose, DB_VERB_REGISTER))
+		__db_msg(env, "opened registry %s", *namep);
+	if (0) {
+err:
+		(void)__envreg_registry_close(env);
+		if (*namep != NULL) {
+			__os_free(env, *namep);
+			*namep = NULL;
+		}
+	}
+	return (ret);
+}
+
+/*
+ * __envreg_registry_close --
+ *	Close the registry file, if any. That also releases any registry lock.
+ *
+ * PUBLIC: int __envreg_registry_close __P((ENV *));
+ */
+int
+__envreg_registry_close(env)
+	ENV *env;
+{
+	DB_ENV *dbenv;
+	int ret;
+
+	ret = 0;
+	dbenv = env->dbenv;
+	if (dbenv->registry != NULL) {
+		ret = __os_closehandle(env, dbenv->registry);
+		dbenv->registry = NULL;
+	}
+	return (ret);
+}
+
 /*
  * __envreg_xunlock --
  *	Discard the exclusive lock held by the ENV handle.
@@ -610,6 +754,10 @@ __envreg_isalive(dbenv, pid, tid, flags )
 	db_threadid_t tid;
 	u_int32_t flags;
 {
+	ENV *env;
+
+	env = dbenv->env;
+
 	/* in this case we really do not care about tid, simply for lint */
 	DB_THREADID_INIT(tid);
 
@@ -617,15 +765,14 @@ __envreg_isalive(dbenv, pid, tid, flags )
 	if (!((flags == 0) || (flags == DB_MUTEX_PROCESS_ONLY)))
 		return (EINVAL);
 
-	if (DB_GLOBAL(active_pids) == NULL ||
-	    DB_GLOBAL(num_active_pids) == 0 || dbenv == NULL)
+	if (env->active_pids == NULL || env->num_active_pids == 0)
 		return (0);
 	/*
 	 * bsearch returns a pointer to an entry in active_pids if a match
 	 * is found on pid, else no match found it returns NULL.   This
 	 * routine will return a 1 if a match is found, else a 0.
 	 */
-	if (bsearch(&pid, DB_GLOBAL(active_pids), DB_GLOBAL(num_active_pids),
+	if (bsearch(&pid, env->active_pids, env->num_active_pids,
 	    sizeof(pid_t), __envreg_pid_compare))
 		return 1;
 
@@ -635,7 +782,8 @@ __envreg_isalive(dbenv, pid, tid, flags )
 /*
  * __envreg_create_active_pid --
  *	Create array of pids, if need more room in array then double size.
- *	Only add active pids from DB_REGISTER file into array.
+ *	Only add active pids from DB_REGISTER file into array. The given
+ *	active my_pid is also added into array.
  */
 static int
 __envreg_create_active_pid(env, my_pid)
@@ -646,14 +794,22 @@ __envreg_create_active_pid(env, my_pid)
 	char buf[PID_LEN + 10];
 	int    ret;
 	off_t  pos;
-	pid_t  pid, *tmparray;
-	size_t tmpsize, nr;
+	size_t nr;
 	u_int lcnt;
 
 	dbenv = env->dbenv;
 	pos = 0;
 	ret = 0;
 
+	/*
+	 * The process getting here has not been added to the DB_REGISTER
+	 * file yet, so include it as the first item in array
+	 */
+	if (env->num_active_pids == 0) {
+		if ((ret = __envreg_add_active_pid(env, my_pid)) != 0)
+			return (ret);
+	}
+
 	/*
 	 * Walk through DB_REGISTER file, we grab pid entries that are locked
 	 * as those represent processes that are still alive.   Ignore empty
@@ -678,53 +834,48 @@ __envreg_create_active_pid(env, my_pid)
 			if ((ret = REGISTRY_UNLOCK(env, pos)) != 0)
 				return (ret);
 		} else {
-			/* first, check to make sure we have room in arrary */
-			if (DB_GLOBAL(num_active_pids) + 1 >
-			    DB_GLOBAL(size_active_pids)) {
-				tmpsize =
-				   DB_GLOBAL(size_active_pids) * sizeof(pid_t);
-
-				/* start with 512, then double if must grow */
-				tmpsize = tmpsize>0 ? tmpsize*2 : 512;
-				if ((ret = __os_malloc
-				    (env, tmpsize, &tmparray )) != 0)
-					return (ret);
-
-				/* if array exists, then copy and free */
-				if (DB_GLOBAL(active_pids)) {
-					memcpy( tmparray,
-					    DB_GLOBAL(active_pids),
-					    DB_GLOBAL(num_active_pids) *
-					    sizeof(pid_t));
-					__os_free( env, DB_GLOBAL(active_pids));
-				}
-
-				DB_GLOBAL(active_pids) = tmparray;
-				DB_GLOBAL(size_active_pids) = tmpsize;
-
-				/*
-				 * The process getting here has not been added
-				 * to the DB_REGISTER file yet, so include it
-				 * as the first item in array
-				 */
-				if (DB_GLOBAL(num_active_pids) == 0) {
-					pid = (pid_t)strtoul(my_pid, NULL, 10);
-					DB_GLOBAL(active_pids)
-					   [DB_GLOBAL(num_active_pids)++] = pid;
-				}
-			}
-
-			/* insert into array */
-			pid = (pid_t)strtoul(buf, NULL, 10);
-			DB_GLOBAL(active_pids)
-			    [DB_GLOBAL(num_active_pids)++] = pid;
-
+			if ((ret = __envreg_add_active_pid(env, buf)) != 0)
+				return (ret);
 		}
 
 	}
 
 	/* lets sort the array to allow for binary search in isalive func */
-	qsort(DB_GLOBAL(active_pids), DB_GLOBAL(num_active_pids),
+	qsort(env->active_pids, env->num_active_pids,
 	    sizeof(pid_t), __envreg_pid_compare);
 	return (ret);
 }
+
+/*
+ * __envreg_add_active_pid --
+ *	Add an active pid into array, if need more room in array
+ *	then double size.
+ */
+static int
+__envreg_add_active_pid(env, pid)
+	ENV *env;
+	char *pid;
+{
+	int ret;
+	size_t tmpsize;
+
+	ret = 0;
+
+	/* Realloc() the array if it is cannot hold one more item. */
+	if (env->num_active_pids >= env->size_active_pids) {
+		tmpsize = env->size_active_pids * sizeof(pid_t);
+
+		/* start with 512, then double if must grow */
+		tmpsize = tmpsize > 0 ? tmpsize * 2 : 512;
+		if ((ret = __os_realloc(env, tmpsize, &env->active_pids)) != 0)
+			return (ret);
+
+		env->size_active_pids = tmpsize / sizeof(pid_t);
+	}
+
+	/* insert into array */
+	env->active_pids
+	    [env->num_active_pids++] = (pid_t)strtoul(pid, NULL, 10);
+
+	return (0);
+}
