@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2010, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2010, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -9,7 +9,6 @@
 #include "db_config.h"
 
 #include "db_int.h"
-#include "dbinc/blob.h"
 #include "dbinc/db_page.h"
 #include "dbinc/btree.h"
 #include "dbinc/heap.h"
@@ -25,7 +24,7 @@ static int  __heapc_get __P((DBC *, DBT *, DBT *, u_int32_t, db_pgno_t *));
 static int  __heapc_put __P((DBC *, DBT *, DBT *, u_int32_t, db_pgno_t *));
 static int  __heapc_reloc __P((DBC *, DBT *, DBT *));
 static int  __heapc_reloc_partial __P((DBC *, DBT *, DBT *));
-static int  __heapc_search __P((DBC *, HEAPPG *, unsigned,
+static void  __heapc_search __P((DBC *, HEAPPG *, unsigned,
 		int, db_indx_t *, int *));
 static int  __heapc_split __P((DBC *, DBT *, DBT *, int));
 
@@ -145,7 +144,7 @@ __heap_bulk(dbc, data, flags)
 	int is_key, ret;
 	int32_t *offp;
 	off_t blob_size;
-	uintmax_t blob_id;
+	db_seq_t blob_id;
 	u_int32_t data_size, key_size, needed, space;
 	u_int8_t *dbuf, *np;
 
@@ -189,6 +188,7 @@ __heap_bulk(dbc, data, flags)
 next_pg:
 	rid.indx = cp->indx;
 	rid.pgno = cp->pgno;
+	prev_rid = rid;
 	pg = cp->page;
 
 	/*
@@ -266,9 +266,7 @@ next_pg:
 				return (ret);
 		} else if (F_ISSET(hdr, HEAP_RECBLOB)) {
 			memcpy(&bhdr, hdr, HEAPBLOBREC_SIZE);
-			GET_BLOB_ID(dbp->env, bhdr, blob_id, ret);
-			if (ret != 0)
-				return (ret);
+			blob_id = (db_seq_t)bhdr.id;
 			if ((ret = __blob_bulk(
 			    dbc, data_size, blob_id, np)) != 0)
 				return (ret);
@@ -352,8 +350,9 @@ __heapc_del(dbc, flags)
 	HEAPPG *rpage;
 	HEAP_CURSOR *cp;
 	db_pgno_t region_pgno;
-	int oldspacebits, ret, spacebits, t_ret;
-	uintmax_t blob_id;
+	int ret, t_ret;
+	db_seq_t blob_id;
+	u_int32_t oldspacebits, spacebits;
 	u_int16_t data_size, size;
 
 	dbp = dbc->dbp;
@@ -405,9 +404,7 @@ start:	if (STD_LOCKING(dbc) && (ret = __db_lget(dbc,
 	/* Delete the blob file. */
 	if (F_ISSET(hdr, HEAP_RECBLOB)) {
 		memcpy(&bhdr, hdr, HEAPBLOBREC_SIZE);
-		GET_BLOB_ID(dbc->env, bhdr, blob_id, ret);
-		if (ret != 0)
-			return (ret);
+		blob_id = (db_seq_t)bhdr.id;
 		if ((ret = __blob_del(dbc, blob_id)) != 0)
 			return (ret);
 	}
@@ -450,7 +447,7 @@ start:	if (STD_LOCKING(dbc) && (ret = __db_lget(dbc,
 		    dbc->thread_info, NULL, DB_MPOOL_DIRTY, &rpage)) != 0)
 			goto err;
 		HEAP_SETSPACE(dbp, rpage,
-		    cp->pgno - region_pgno - 1, spacebits);
+		    (cp->pgno - region_pgno) - 1, spacebits);
 	}
 
 err:	DB_ASSERT(dbp->env, ret != DB_PAGE_NOTFOUND);
@@ -583,11 +580,12 @@ __heapc_get(dbc, key, data, flags, pgnop)
 	db_pgno_t pgno;
 	int cmp, np_inc, f_indx, found, getpage, indx, ret;
 	off_t blob_size;
-	uintmax_t blob_id;
+	db_seq_t blob_id;
 
 	dbp = dbc->dbp;
 	mpf = dbp->mpf;
 	cp = (HEAP_CURSOR *)dbc->internal;
+	pgno = PGNO_INVALID;
 	LOCK_INIT(meta_lock);
 	COMPQUIET(pgnop, NULL);
 
@@ -617,7 +615,7 @@ __heapc_get(dbc, key, data, flags, pgnop)
 		ACQUIRE_CUR(dbc, lock_type, cp->pgno, 0, 0, ret);
 		if (ret != 0) {
 			if (ret == DB_PAGE_NOTFOUND)
-				ret = DB_NOTFOUND;
+				ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 
@@ -629,7 +627,7 @@ __heapc_get(dbc, key, data, flags, pgnop)
 		hdr = (HEAPHDR *)P_ENTRY(dbp, dpage, cp->indx);
 		if (F_ISSET(hdr, HEAP_RECSPLIT) &&
 		    !F_ISSET(hdr, HEAP_RECFIRST)) {
-			ret = DB_NOTFOUND;
+			ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 
@@ -648,7 +646,7 @@ first:		pgno = FIRST_HEAP_DPAGE;
 			ACQUIRE_CUR(dbc, lock_type, pgno, 0, 0, ret);
 			if (ret != 0 ) {
 				if (ret == DB_PAGE_NOTFOUND)
-					ret = DB_NOTFOUND;
+					ret = DBC_ERR(dbc, DB_NOTFOUND);
 				goto err;
 			}
 			dpage = (HEAPPG *)cp->page;
@@ -691,7 +689,7 @@ last:		pgno = PGNO_BASE_MD;
 		while (!found) {
 			/* Don't look earlier than the first data page. */
 			if (pgno < FIRST_HEAP_DPAGE) {
-				ret = DB_NOTFOUND;
+				ret = DBC_ERR(dbc, DB_NOTFOUND);
 				goto err;
 			}
 
@@ -744,7 +742,7 @@ last:		pgno = PGNO_BASE_MD;
 		dpage = (HEAPPG *)cp->page;
 
 		if (np_inc == 1 && cp->indx >= HEAP_HIGHINDX(dpage))
-			/* At end of current page, must get next page */
+			/* At end of current page, must get next page. */
 			getpage = TRUE;
 		else if (np_inc == -1) {
 			/*
@@ -752,12 +750,15 @@ last:		pgno = PGNO_BASE_MD;
 			 * if already at the first slot.
 			 */
 			for (f_indx=0; (f_indx <= HEAP_HIGHINDX(dpage)) &&
-			   (HEAP_OFFSETTBL(dbp, dpage)[f_indx] == 0); f_indx++);
+			    (HEAP_OFFSETTBL(dbp, dpage)[f_indx] == 0); f_indx++)
+			{
+				/* No-op. */
+			}
 
 			/* At the beginning of current page, get new page */
 			if (cp->indx == 0 || cp->indx <= f_indx) {
 				if (cp->pgno == FIRST_HEAP_DPAGE) {
-					ret = DB_NOTFOUND;
+					ret = DBC_ERR(dbc, DB_NOTFOUND);
 					goto err;
 				}
 				getpage = TRUE;
@@ -766,9 +767,12 @@ last:		pgno = PGNO_BASE_MD;
 
 		while (!found) {
 			if (getpage) {
-				pgno = cp->pgno + np_inc;
+				if (np_inc == -1)
+					pgno = cp->pgno - 1;
+				else if (np_inc == 1)
+					pgno = cp->pgno + 1;
 				if (pgno < FIRST_HEAP_DPAGE) {
-					ret = DB_NOTFOUND;
+					ret = DBC_ERR(dbc, DB_NOTFOUND);
 					goto err;
 				}
 				/* Put current page/lock and get next one */
@@ -777,7 +781,7 @@ last:		pgno = PGNO_BASE_MD;
 					if (np_inc == 1 &&
 					    ret == DB_PAGE_NOTFOUND)
 						/* Beyond last page */
-						ret = DB_NOTFOUND;
+						ret = DBC_ERR(dbc, DB_NOTFOUND);
 					goto err;
 				}
 				dpage = (HEAPPG *)cp->page;
@@ -796,7 +800,7 @@ last:		pgno = PGNO_BASE_MD;
 					 * When searching, indx gets
 					 * bumped to 0
 					 */
-					cp->indx = -1;
+					cp->indx = UINT16_MAX;
 				else
 					/*
 					 * When searching, indx gets bumped to
@@ -813,7 +817,10 @@ last:		pgno = PGNO_BASE_MD;
 			 * record. HEAP_HIGHINDX always points to highest filled
 			 * entry on page.
 			 */
-			cp->indx += np_inc;
+			if (np_inc == -1)
+				cp->indx--;
+			else if (np_inc == 1)
+				cp->indx++;
 			__heapc_search(dbc,
 			    dpage, cp->indx, np_inc, &cp->indx, &found);
 			if (!found)
@@ -831,7 +838,7 @@ last:		pgno = PGNO_BASE_MD;
 		/* First make sure we're trying to get a data page. */
 		if (pgno == PGNO_BASE_MD ||
 		    pgno == HEAP_REGION_PGNO(dbp, pgno)) {
-			ret = DB_NOTFOUND;
+			ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 
@@ -840,7 +847,7 @@ last:		pgno = PGNO_BASE_MD;
 
 		if (ret != 0) {
 			if (ret == DB_PAGE_NOTFOUND)
-				ret = DB_NOTFOUND;
+				ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 		dpage = (HEAPPG *)cp->page;
@@ -849,14 +856,14 @@ last:		pgno = PGNO_BASE_MD;
 		if ((indx >  HEAP_HIGHINDX(dpage)) ||
 		    (HEAP_OFFSETTBL(dbp, dpage)[indx] == 0)) {
 			DISCARD(dbc, cp->page, cp->lock, 0, ret);
-			ret = DB_NOTFOUND;
+			ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 		hdr = (HEAPHDR *)P_ENTRY(dbp, dpage, indx);
 		if (F_ISSET(hdr, HEAP_RECSPLIT) &&
 		    !F_ISSET(hdr, HEAP_RECFIRST)) {
 			DISCARD(dbc, cp->page, cp->lock, 0, ret);
-			ret = DB_NOTFOUND;
+			ret = DBC_ERR(dbc, DB_NOTFOUND);
 			goto err;
 		}
 
@@ -873,9 +880,7 @@ last:		pgno = PGNO_BASE_MD;
 					goto err;
 			} else if (F_ISSET(hdr, HEAP_RECBLOB)) {
 				memcpy(&bhdr, hdr, HEAPBLOBREC_SIZE);
-				GET_BLOB_ID(dbc->env, bhdr, blob_id, ret);
-				if (ret != 0)
-					goto err;
+				blob_id = (db_seq_t)bhdr.id;
 				GET_BLOB_SIZE(dbc->env, bhdr, blob_size, ret);
 				if (ret != 0)
 					goto err;
@@ -892,11 +897,11 @@ last:		pgno = PGNO_BASE_MD;
 				    (void *)((u_int8_t *)hdr + sizeof(HEAPHDR));
 				tmp_val.size = hdr->size;
 			}
-			cmp = __bam_defcmp(dbp, &tmp_val, data, NULL);
+			cmp = __dbt_defcmp(dbp, &tmp_val, data, NULL);
 			if (F_ISSET(&tmp_val, DB_DBT_MALLOC))
 				__os_ufree(dbp->env, tmp_val.data);
 			if (cmp != 0) {
-				ret = DB_NOTFOUND;
+				ret = DBC_ERR(dbc, DB_NOTFOUND);
 				goto err;
 			}
 		}
@@ -904,7 +909,7 @@ last:		pgno = PGNO_BASE_MD;
 		break;
 	case DB_NEXT_DUP:
 	case DB_PREV_DUP:
-		ret = DB_NOTFOUND;
+		ret = DBC_ERR(dbc, DB_NOTFOUND);
 		goto err;
 	default:
 		/* DB_GET_RECNO, DB_JOIN_ITEM, DB_SET_RECNO are invalid */
@@ -939,28 +944,36 @@ err:	if (ret == 0 ) {
  * __heapc_search --
  *	Search a given a heap page, starting at a given index, for a viable heap
  *	record.  Return the index of the found record in indxp.
+ *
+ * Compiler Note:
+ *	The 'begin' parameter would be a db_indx_t except that unsigned K&R C
+ *	function parameters that are smaller than a full word are promoted to
+ *	signed integers. By declaring 'begin' as a natural unsigned, we avoid
+ *	a warning generated by gcc -Wall:
+ *		promoted type 'int' of K&R function parameter is not compatible
+ *		with the parameter type 'db_indx_t' ...
  */
-static int
-__heapc_search(dbc, dpage, start, dir, indxp, found)
+static void
+__heapc_search(dbc, dpage, begin, dir, indxp, found)
      DBC *dbc;
      HEAPPG *dpage;
-     unsigned start;
+     unsigned begin;
      int dir;
      db_indx_t *indxp;
      int *found;
 {
 	DB *dbp;
 	HEAPHDR *hdr;
-	unsigned indx;
+	db_indx_t indx;
 
 	dbp = dbc->dbp;
 	DB_ASSERT(dbp->env, dir == -1 || dir == 1);
 
 	*found = FALSE;
 	if (TYPE(dpage) != P_HEAP || NUM_ENT(dpage) == 0)
-		return (0);
+		return;
 
-	indx = start;
+	indx = (db_indx_t) begin;
 	for (;;) {
 		if (HEAP_OFFSETTBL(dbp, dpage)[indx] != 0) {
 			hdr = (HEAPHDR *)P_ENTRY(dbp, dpage, indx);
@@ -968,16 +981,18 @@ __heapc_search(dbc, dpage, start, dir, indxp, found)
 			    F_ISSET(hdr, HEAP_RECFIRST)) {
 				*found = TRUE;
 				*indxp = indx;
-				return (0);
+				break;
 			}
 		}
 		if ((dir == -1 && indx == 0) ||
 		    (dir == 1 && indx == HEAP_HIGHINDX(dpage)))
-			return (0);
+			break;
 
-		indx += dir;
+		if (dir == -1)
+			indx--;
+		else
+			indx++;
 	}
-	return (0);
 }
 
 #undef	IS_FIRST
@@ -1036,7 +1051,7 @@ __heapc_reloc_partial(dbc, key, data)
 			dlen = old_size - doff;
 		else
 			dlen = data->dlen;
-		data_size = old_size - dlen + data->size;
+		data_size = (old_size - dlen) + data->size;
 	}
 
 	/*
@@ -1097,8 +1112,8 @@ __heapc_reloc_partial(dbc, key, data)
 			 */
 			data_size = doff + (add_bytes ? data->size : 0);
 		else
-			data_size = old_hdr->size -
-				dlen + (add_bytes ? data->size : 0);
+			data_size = (old_hdr->size -
+				dlen) + (add_bytes ? data->size : 0);
 		data_size += remaining;
 
 		if (data_size > buflen) {
@@ -1142,7 +1157,7 @@ __heapc_reloc_partial(dbc, key, data)
 			if (doff + dlen < old_hdr->size) {
 				olddata += dlen;
 				memcpy(buf,
-				    olddata, old_hdr->size - doff - dlen);
+				    olddata, (old_hdr->size - doff) - dlen);
 				dlen = 0;
 			} else
 				/*
@@ -1255,7 +1270,7 @@ __heapc_reloc_partial(dbc, key, data)
 			size -= sizeof(db_indx_t);
 		/* Round down to a multiple of 4. */
 		size = DB_ALIGN(
-		    size - sizeof(u_int32_t) + 1, sizeof(u_int32_t));
+		    (size - sizeof(u_int32_t)) + 1, sizeof(u_int32_t));
 		DB_ASSERT(dbp->env, size >= sizeof(HEAPSPLITHDR));
 
 		/*
@@ -1501,7 +1516,7 @@ __heapc_reloc(dbc, key, data)
 			size -= sizeof(db_indx_t);
 		/* Round down to a multiple of 4. */
 		size = DB_ALIGN(
-		    size - sizeof(u_int32_t) + 1, sizeof(u_int32_t));
+		    (size - sizeof(u_int32_t)) + 1, sizeof(u_int32_t));
 		DB_ASSERT(dbp->env, size >= sizeof(HEAPSPLITHDR));
 		new_hdr.std_hdr.size =
 		    (u_int16_t)(size - sizeof(HEAPSPLITHDR));
@@ -1646,17 +1661,19 @@ __heapc_put(dbc, key, data, flags, pgnop)
 	HEAP_CURSOR *cp;
 	PAGE *rpage;
 	db_pgno_t region_pgno;
-	int oldspace, ret, space, t_ret;
+	int buf_alloc, ret, t_ret;
 	off_t blob_size;
-	uintmax_t blob_id, new_blob_id;
-	u_int32_t data_size, dlen, new_size, old_flags, old_size, tot_size;
-	u_int8_t *buf, *olddata, *src, *dest;
+	db_seq_t blob_id, new_blob_id;
+	u_int32_t data_size, dlen, new_size, old_flags, old_size;
+	u_int32_t oldspace, space, tot_size;
+	u_int8_t *buf, *olddata;
 
 	dbp = dbc->dbp;
 	mpf = dbp->mpf;
 	cp = (HEAP_CURSOR *)dbc->internal;
 	rpage = NULL;
-	buf = dest = src = NULL;
+	buf = NULL;
+	buf_alloc = 0;
 	dlen = 0;
 	blob_id = new_blob_id = 0;
 
@@ -1720,7 +1737,7 @@ __heapc_put(dbc, key, data, flags, pgnop)
 				dlen = tot_size - data->doff;
 			else
 				dlen = data->dlen;
-			data_size = tot_size - dlen + data->size;
+			data_size = (tot_size - dlen) + data->size;
 		}
 	} else if F_ISSET(old_hdr, HEAP_RECBLOB)
 		data_size = HEAPBLOBREC_DSIZE;
@@ -1756,6 +1773,7 @@ __heapc_put(dbc, key, data, flags, pgnop)
 		 */
 		if ((ret = __os_malloc(dbp->env, data_size, &buf)) != 0)
 			goto err;
+		buf_alloc = 1;
 		new_data.data = buf;
 
 		/*
@@ -1778,10 +1796,10 @@ __heapc_put(dbc, key, data, flags, pgnop)
 		buf += data->size;
 
 		/* Fill in remaining data from the old record, skipping dlen. */
-		if (data->doff < old_hdr->size) {
+		if ((data->doff + data->dlen) < old_hdr->size) {
 			olddata += data->doff + data->dlen;
-			memcpy(buf,
-			    olddata, old_hdr->size - data->doff - data->dlen);
+			memcpy(buf, olddata,
+			    (old_hdr->size - data->doff) - data->dlen);
 		}
 	} else if (F_ISSET(old_hdr, HEAP_RECBLOB)) {
 		data_size = HEAPBLOBREC_DSIZE;
@@ -1792,9 +1810,7 @@ __heapc_put(dbc, key, data, flags, pgnop)
 			new_data.data = HEAPBLOBREC_DATA(data->data);
 		} else {
 			memcpy(&bhdr, old_hdr, HEAPBLOBREC_SIZE);
-			GET_BLOB_ID(dbp->env, bhdr, blob_id, ret);
-			if (ret != 0)
-				goto err;
+			blob_id = (db_seq_t)bhdr.id;
 			GET_BLOB_SIZE(dbp->env, bhdr, blob_size, ret);
 			if (ret != 0)
 				goto err;
@@ -1858,14 +1874,14 @@ __heapc_put(dbc, key, data, flags, pgnop)
 		    dbc->thread_info, NULL, DB_MPOOL_DIRTY, &rpage)) != 0)
 			goto err;
 
-		HEAP_SETSPACE(dbp, rpage, cp->pgno - region_pgno - 1, space);
+		HEAP_SETSPACE(dbp, rpage, (cp->pgno - region_pgno) - 1, space);
 	}
 
 err:	DB_ASSERT(dbp->env, ret != DB_PAGE_NOTFOUND);
 	if (rpage != NULL && (t_ret = __memp_fput(mpf,
 	    dbc->thread_info, rpage, dbc->priority)) != 0 && ret == 0)
 		ret = t_ret;
-	if (F_ISSET(data, DB_DBT_PARTIAL) && !F_ISSET(old_hdr, HEAP_RECBLOB))
+	if (buf_alloc)
 		__os_free(dbp->env, new_data.data);
 
 	if (ret != 0 && LOCK_ISSET(cp->lock))
@@ -1893,17 +1909,21 @@ __heap_getpage(dbc, size, avail)
 	HEAP *h;
 	HEAPPG *rpage;
 	HEAP_CURSOR *cp;
-	db_pgno_t data_pgno, meta_pgno, region_pgno, start_region;
-	int i, lk_mode, max, p, ret, space, start, t_ret;
+	db_pgno_t data_pgno, i, max, meta_pgno, p, region_pgno, start;
+	db_pgno_t start_region;
+	int ret, t_ret;
+	u_int32_t lk_mode, space;
 
 	LOCK_INIT(meta_lock);
+	data_pgno = PGNO_INVALID;
 	dbp = dbc->dbp;
 	mpf = dbp->mpf;
 	cp = (HEAP_CURSOR *)dbc->internal;
 	h = dbp->heap_internal;
 	start_region = region_pgno = h->curregion;
 	max = HEAP_REGION_SIZE(dbp);
-	i = ret = t_ret = 0;
+	i = 0;
+	ret = t_ret = 0;
 
 	/*
 	 * The algorithm for finding a page:
@@ -1966,10 +1986,10 @@ find:	while ((ret = __memp_fget(mpf, &region_pgno,
 		max = h->maxpgno - region_pgno;
 	/*
 	 * Look in the bitmap for a page with sufficient free space.  We use i
-	 * in a slightly strange way.  Because the 2-bits in the bitmap are only
-	 * an estimate, there is a chance the data won't fit on the page we
-	 * choose.  In that case, we re-start the process and want to be able to
-	 * resume this loop where we left off.
+	 * in a slightly strange way.  Because the 2-bits in the bitmap are
+	 * only an estimate, there is a chance the data won't fit on the page
+	 * we choose.  In that case, we re-start the process and want to be
+	 * able to resume this loop where we left off.
 	 */
 	for (; i < max; i++) {
 		p = start + i;
@@ -1977,7 +1997,7 @@ find:	while ((ret = __memp_fget(mpf, &region_pgno,
 			p -= max;
 		if ((*avail = HEAP_SPACE(dbp, rpage, p)) > space)
 			continue;
-		data_pgno = region_pgno + p + 1;
+		data_pgno = (region_pgno + p) + 1;
 		ACQUIRE_CUR(dbc,
 		    DB_LOCK_WRITE, data_pgno, DB_LOCK_NOWAIT, 0, ret);
 		/*
@@ -2234,7 +2254,7 @@ check:		if (size + sizeof(db_indx_t) > HEAP_FREESPACE(dbp, cp->page)) {
 		}
 	}
 
-	h->curpgindx = data_pgno - region_pgno - 1;
+	h->curpgindx = (data_pgno - region_pgno) - 1;
 err:	DB_ASSERT(dbp->env, ret != DB_PAGE_NOTFOUND);
 	if (rpage != NULL && (t_ret = __memp_fput(mpf,
 	    dbc->thread_info, rpage, dbc->priority)) != 0 && ret == 0)
@@ -2266,11 +2286,11 @@ __heap_append(dbc, key, data)
 	HEAP_CURSOR *cp;
 	db_indx_t indx;
 	db_pgno_t region_pgno;
-	int is_blob, ret, space, t_ret;
+	int is_blob, ret, t_ret;
 	off_t blob_size;
-	uintmax_t blob_id;
+	db_seq_t blob_id;
 	u_int8_t avail;
-	u_int32_t data_size;
+	u_int32_t data_size, space;
 
 	dbp = dbc->dbp;
 	mpf = dbp->mpf;
@@ -2314,8 +2334,7 @@ __heap_append(dbc, key, data)
 		bhdr.std_hdr.size = HEAPBLOBREC_DSIZE;
 		SET_BLOB_SIZE(&bhdr, blob_size, HEAPBLOBHDR);
 		SET_BLOB_ID(&bhdr, blob_id, HEAPBLOBHDR);
-		SET_LO_HI(&bhdr,
-		    dbp->blob_file_id, HEAPBLOBHDR, file_id_lo, file_id_hi);
+		SET_BLOB_FILE_ID(&bhdr, dbp->blob_file_id, HEAPBLOBHDR);
 		tmp_dbt.data = &bhdr;
 		tmp_dbt.size = sizeof(HEAPHDR);
 		memset(&data_dbt, 0, sizeof(DBT));
@@ -2358,7 +2377,7 @@ __heap_append(dbc, key, data)
 		    dbc->thread_info, NULL, DB_MPOOL_DIRTY, &rpage)) != 0)
 			goto err;
 
-		HEAP_SETSPACE(dbp, rpage, cp->pgno - region_pgno - 1, space);
+		HEAP_SETSPACE(dbp, rpage, (cp->pgno - region_pgno) - 1, space);
 	}
 
 err:	DB_ASSERT(dbp->env, ret != DB_PAGE_NOTFOUND);
@@ -2394,8 +2413,8 @@ __heapc_split(dbc, key, data, is_first)
 	HEAP_CURSOR *cp;
 	db_indx_t indx;
 	db_pgno_t region_pgno;
-	int ret, spacebits, t_ret;
-	u_int32_t buflen, doff, left, size;
+	int ret, t_ret;
+	u_int32_t buflen, doff, left, size, spacebits;
 	u_int8_t availbits, *buf;
 
 	dbp = dbc->dbp;
@@ -2410,7 +2429,6 @@ __heapc_split(dbc, key, data, is_first)
 	ret = t_ret = 0;
 	indx = 0;
 	buf = NULL;
-	buflen = 0;
 
 	/*
 	 * Write the record to multiple pages, in chunks starting from the end.
@@ -2424,6 +2442,9 @@ __heapc_split(dbc, key, data, is_first)
 		left += data->doff;
 	}
 	hdrs.tsize = left;
+	buflen = 1;
+	if ((ret = __os_malloc(dbp->env, buflen, &buf)) != 0)
+		return (ret);
 	while (left > 0) {
 		size = DB_ALIGN(left + sizeof(HEAPSPLITHDR), sizeof(u_int32_t));
 		if (size < sizeof(HEAPSPLITHDR))
@@ -2438,8 +2459,10 @@ __heapc_split(dbc, key, data, is_first)
 		else
 			hdrs.std_hdr.flags |= HEAP_RECFIRST;
 
-		if ((ret = __heap_getpage(dbc, size, &availbits)) != 0)
+		if ((ret = __heap_getpage(dbc, size, &availbits)) != 0) {
+			__os_free(dbp->env, buf);
 			return (ret);
+		}
 
 		/*
 		 * size is the total number of bytes being written to the page.
@@ -2465,7 +2488,7 @@ __heapc_split(dbc, key, data, is_first)
 				size -= sizeof(db_indx_t);
 			/* Round down to a multiple of 4. */
 			size = DB_ALIGN(
-			    size - sizeof(u_int32_t) + 1, sizeof(u_int32_t));
+			    (size - sizeof(u_int32_t)) + 1, sizeof(u_int32_t));
 			DB_ASSERT(dbp->env, size >= sizeof(HEAPSPLITHDR));
 			hdrs.std_hdr.size =
 			    (u_int16_t)(size - sizeof(HEAPSPLITHDR));
@@ -2503,10 +2526,10 @@ __heapc_split(dbc, key, data, is_first)
 			 * page minus the bytes we're taking from data.
 			*/
 			t_data.data = buf;
-			memset(buf, '\0', t_data.size - left + doff);
-			buf += t_data.size - left + doff;
+			memset(buf, 0, (t_data.size - left) + doff);
+			buf += (t_data.size - left) + doff;
 			memcpy(buf, data->data, left - doff);
-			doff -= t_data.size - left + doff;
+			doff -= (t_data.size - left) + doff;
 			buf = t_data.data;
 		}
 		hdr_dbt.data = &hdrs;
@@ -2550,7 +2573,7 @@ __heapc_split(dbc, key, data, is_first)
 				goto err;
 
 			HEAP_SETSPACE(dbp,
-			    rpage, cp->pgno - region_pgno - 1, spacebits);
+			    rpage, (cp->pgno - region_pgno) - 1, spacebits);
 			ret = __memp_fput(mpf,
 			    dbc->thread_info, rpage, dbc->priority);
 			rpage = NULL;
