@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2010, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2010, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -147,7 +147,8 @@ __heap_stat_print(dbc, flags)
 	__db_dl(env,
 	    "Number of records in the database", (u_long)sp->heap_nrecs);
 	__db_dl(env,
-	    "Number of blobs in the database", (u_long)sp->heap_nblobs);
+	    "Number of external files in the database",
+	    (u_long)sp->heap_ext_files);
 	__db_dl(env, "Number of database pages", (u_long)sp->heap_pagecnt);
 	__db_dl(env, "Number of database regions", (u_long)sp->heap_nregions);
 	__db_dl(env,
@@ -201,13 +202,15 @@ __heap_stat_callback(dbc, h, cookie, putp)
 		 * We can't just use NUM_ENT, otherwise we'd mis-count split
 		 * records.
 		 */
-		for (i = 0; i < NUM_ENT(h); i++) {
+		for (i = 0; i <= HEAP_HIGHINDX(h); i++) {
 			hdr = (HEAPHDR *)P_ENTRY(dbp, h, i);
 			if (!F_ISSET(hdr, HEAP_RECSPLIT) ||
 			    F_ISSET(hdr, HEAP_RECFIRST))
 				sp->heap_nrecs++;
-			if (F_ISSET(hdr, HEAP_RECBLOB))
+			if (F_ISSET(hdr, HEAP_RECBLOB)) {
 				sp->heap_nblobs++;
+				sp->heap_ext_files++;
+			}
 		}
 		break;
 	case P_HEAPMETA: /* Fallthrough */
