@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2010, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2010, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -38,8 +38,16 @@ __heap_vrfy_meta(dbp, vdp, meta, pgno, flags)
 	HEAP *h;
 	VRFY_PAGEINFO *pip;
 	db_pgno_t last_pgno, max_pgno, npgs;
-	int isbad, ret;
-	uintmax_t blob_id;
+	int isbad, ret, t_ret;
+	db_seq_t blob_id;
+
+	if (dbp->type != DB_HEAP) {
+		EPRINT((dbp->env, DB_STR_A("1215",
+		    "Page %lu: invalid page type %u for %s database",
+		    "%lu %u %s"), (u_long)pgno, TYPE(meta),
+		    __db_dbtype_to_string(dbp->type)));
+		return (DB_VERIFY_FATAL);
+	}
 
 	if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 		return (ret);
@@ -103,13 +111,42 @@ __heap_vrfy_meta(dbp, vdp, meta, pgno, flags)
 		h->bytes = meta->bytes;
 	}
 
-	GET_LO_HI(dbp->env,
-	    meta->blob_file_lo, meta->blob_file_hi, blob_id, ret);
-	if (ret != 0) {
+/*
+ * Where 64-bit integer support is not available,
+ * return an error if the file has any blobs.
+ */
+	t_ret = 0;
+#ifdef HAVE_64BIT_TYPES
+	GET_BLOB_FILE_ID(dbp->env, meta, blob_id, t_ret);
+	if (t_ret != 0) {
 		isbad = 1;
 		EPRINT((dbp->env, DB_STR_A("1173",
-		    "Page %lu: blob file id overflow.", "%lu"), (u_long)pgno));
+		    "Page %lu: external file id overflow.", "%lu"),
+		    (u_long)pgno));
+		if (ret == 0)
+			ret = t_ret;
 	}
+	if (blob_id < 0) {
+		isbad = 1;
+		EPRINT((dbp->env, DB_STR_A("5503",
+			"Page %lu: invalid external file id.", "%lu"),
+			(u_long)pgno));
+	}
+#else /* HAVE_64BIT_TYPES */
+	/*
+	 * db_seq_t is an int on systems that do not have 64 integers types, so
+	 * this will compile and run.
+	 */
+	GET_BLOB_FILE_ID(dbp->env, meta, blob_id, t_ret);
+	if (t_ret != 0 || blob_id != 0) {
+		isbad = 1;
+		EPRINT((dbp->env, DB_STR_A("1200",
+	    "Page %lu: external files require 64 integer compiler support.",
+		    "%lu"), (u_long)pgno));
+		if (ret == 0)
+			ret = t_ret;
+	}
+#endif
 
 err:	if (LF_ISSET(DB_SALVAGE))
 		ret = __db_salvage_markdone(vdp, pgno);
@@ -134,13 +171,22 @@ __heap_vrfy(dbp, vdp, h, pgno, flags)
 {
 	HEAPBLOBHDR bhdr;
 	HEAPHDR *hdr;
-	int cnt, i, j, ret;
+	int i, j, ret;
 	off_t blob_size;
-	uintmax_t blob_id, file_id;
-	db_indx_t *offsets, *offtbl, end;
+	db_seq_t blob_id, file_id;
+	db_indx_t *offsets, offset, *offtbl, end;
+	u_int32_t cnt;
+
+	if (dbp->type != DB_HEAP) {
+		EPRINT((dbp->env, DB_STR_A("1215",
+		    "Page %lu: invalid page type %u for %s database",
+		    "%lu %u %s"), (u_long)pgno, TYPE(h),
+		    __db_dbtype_to_string(dbp->type)));
+		return (DB_VERIFY_BAD);
+	}
 
 	if ((ret = __db_vrfy_datapage(dbp, vdp, h, pgno, flags)) != 0)
-		goto err;
+		return (ret);
 
 	if (TYPE(h) == P_IHEAP)
 		/* Nothing to verify on a region page. */
@@ -170,6 +216,13 @@ __heap_vrfy(dbp, vdp, h, pgno, flags)
 			ret = DB_VERIFY_BAD;
 			goto err;
 		}
+		offset = offtbl[i];
+		if (offset >= dbp->pgsize) {
+			EPRINT((dbp->env, DB_STR_A("5508",
+			    "Page %lu: invalid offset.", "%lu"), (u_long)pgno));
+			ret = DB_VERIFY_BAD;
+			continue;
+		}
 		hdr = (HEAPHDR *)P_ENTRY(dbp, h, i);
 		if (!F_ISSET(hdr, HEAP_RECSPLIT) &&
 		    F_ISSET(hdr, HEAP_RECFIRST | HEAP_RECLAST)) {
@@ -185,36 +238,29 @@ __heap_vrfy(dbp, vdp, h, pgno, flags)
 			 * file size as is stored in the database record.
 			 */
 			memcpy(&bhdr, hdr, sizeof(HEAPBLOBHDR));
-			GET_BLOB_ID(dbp->env, bhdr, blob_id, ret);
-			if (ret != 0) {
-				EPRINT((dbp->env, DB_STR_A("1174",
-				    "Page %lu: blob id value has overflowed",
-				    "%lu"), (u_long)pgno));
+			blob_id = (db_seq_t)bhdr.id;
+			if (blob_id < 1) {
 				ret = DB_VERIFY_BAD;
+				EPRINT((dbp->env, DB_STR_A("1216",
+			"Page %lu: invalid external file id %lld at item %lu",
+				    "%lu %lld %lu"), (u_long)pgno,
+				    (long long)blob_id, (u_long)i));
 				goto err;
 			}
 			GET_BLOB_SIZE(dbp->env, bhdr, blob_size, ret);
-			if (ret != 0) {
+			if (ret != 0 || blob_size < 0) {
 				EPRINT((dbp->env, DB_STR_A("1175",
-			"Page %lu: blob file size value has overflowed",
+			"Page %lu: external file size value has overflowed",
 				    "%lu"), (u_long)pgno));
 				ret = DB_VERIFY_BAD;
 				goto err;
 			}
-			GET_LO_HI(dbp->env,
-			    bhdr.file_id_lo, bhdr.file_id_hi, file_id, ret);
-			if (ret != 0) {
-				EPRINT((dbp->env, DB_STR_A("1176",
-		"Page %lu: blob file id value has overflowed at item %lu",
-				    "%lu %lu"), (u_long)pgno, (u_long)i));
-				ret = DB_VERIFY_BAD;
-				goto err;
-			}
-			if (file_id == 0) {
+			file_id = (db_seq_t)bhdr.file_id;
+			if (file_id < 1) {
 				EPRINT((dbp->env, DB_STR_A("1177",
-			"Page %lu: invalid blob dir id %llu at item %lu",
-				    "%lu %llu, %lu"), (u_long)pgno,
-				    (unsigned long long)file_id, (u_long)i));
+		    "Page %lu: invalid external file dir id %lld at item %lu",
+				    "%lu %lld, %lu"), (u_long)pgno,
+				    (long long)file_id, (u_long)i));
 				ret = DB_VERIFY_BAD;
 				goto err;
 			}
@@ -240,7 +286,7 @@ __heap_vrfy(dbp, vdp, h, pgno, flags)
 	 * record.  We can't use the P_ENTRY macro because we've kept track of
 	 * the offsets, not the indexes.
 	 */
-	for (i = 0; i < cnt - 1; i++) {
+	for (i = 0; i < (int)cnt - 1; i++) {
 		hdr = (HEAPHDR *)((u_int8_t *)h + offsets[i]);
 		end = offsets[i] + HEAP_HDRSIZE(hdr) + hdr->size;
 		if (end > offsets[i+1]) {
@@ -397,7 +443,7 @@ __heap_salvage(dbp, vdp, pgno, h, handle, callback, flags)
 	off_t blob_size, blob_offset, remaining;
 	u_int32_t blob_buf_size;
 	u_int8_t *blob_buf;
-	uintmax_t blob_id, file_id;
+	db_seq_t blob_id, file_id;
 
 	COMPQUIET(flags, 0);
 	memset(&dbt, 0, sizeof(DBT));
@@ -413,7 +459,7 @@ __heap_salvage(dbp, vdp, pgno, h, handle, callback, flags)
 	 * when the first piece is encountered,
 	 */
 	for (i = 0; i <= HEAP_HIGHINDX(h); i++) {
-		if (offtbl[i] == 0)
+		if (offtbl[i] == 0 || offtbl[i] >= dbp->pgsize)
 			continue;
 		hdr = (HEAPHDR *)P_ENTRY(dbp, h, i);
 		if (F_ISSET(hdr, HEAP_RECSPLIT)) {
@@ -429,18 +475,20 @@ __heap_salvage(dbp, vdp, pgno, h, handle, callback, flags)
 			if ((ret =
 			    __os_malloc(env, dbt.size, &dbt.data)) != 0)
 				goto err;
-			__heap_safe_gsplit(dbp, vdp, h, i, &dbt);
+			if ((ret = __heap_safe_gsplit
+			    (dbp, vdp, h, i, &dbt)) != 0) {
+				err_ret = ret;
+				__os_free(env, dbt.data);
+				continue;
+			}
 		} else if (F_ISSET(hdr, HEAP_RECBLOB)) {
 			memcpy(&bhdr, hdr, HEAPBLOBREC_SIZE);
-			GET_BLOB_ID(env, bhdr, blob_id, ret);
-			if (ret != 0)
-				goto err;
+			blob_id = (db_seq_t)bhdr.id;
 			GET_BLOB_SIZE(env, bhdr, blob_size, ret);
-			if (ret != 0)
+			if (ret != 0 || blob_size < 0)
 				goto err;
-			GET_LO_HI(env,
-			    bhdr.file_id_lo, bhdr.file_id_hi, file_id, ret);
-			if (ret != 0)
+			file_id = (db_seq_t)bhdr.file_id;
+			if (file_id < 1)
 				goto err;
 			/* Read the blob, in pieces if it is too large.*/
 			blob_offset = 0;
@@ -570,7 +618,7 @@ __heap_safe_gsplit(dbp, vdp, h, i, dbt)
 
 err:	if (gotpg && (t_ret = __memp_fput(
 	    mpf, vdp->thread_info, h, DB_PRIORITY_UNCHANGED)) != 0 && ret == 0)
-		t_ret = ret;
+		ret = t_ret;
 	return (ret);
 }
 
