@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -29,12 +29,9 @@ __memp_env_create(dbenv)
 	 * state or turn off mutex locking, and so we can neither check
 	 * the panic state or acquire a mutex in the DB_ENV create path.
 	 *
-	 * We default to 32 8K pages.  We don't default to a flat 256K, because
-	 * we want to include the size of the buffer header which can vary
-	 * from system to system.
+	 * We default to 32 8K pages plus the overhead of the hash buckets.
 	 */
-	dbenv->mp_bytes =
-	    32 * ((8 * 1024) + sizeof(BH)) + 37 * sizeof(DB_MPOOL_HASH);
+	dbenv->mp_bytes = MEMP_SMALLCACHE_ADJUST(32 * 8 * 1024);
 	dbenv->mp_ncache = 1;
 
 	return (0);
@@ -171,7 +168,7 @@ __memp_set_cachesize(dbenv, gbytes, bytes, arg_ncache)
 	 */
 	if (gbytes == 0) {
 		if (bytes < 500 * MEGABYTE)
-			bytes += (bytes / 4) + 37 * sizeof(DB_MPOOL_HASH);
+			bytes = MEMP_SMALLCACHE_ADJUST(bytes);
 		if (bytes / ncache < DB_CACHESIZE_MIN)
 			bytes = ncache * DB_CACHESIZE_MIN;
 	}
@@ -180,7 +177,7 @@ __memp_set_cachesize(dbenv, gbytes, bytes, arg_ncache)
 		ENV_ENTER(env, ip);
 		ret = __memp_resize(env->mp_handle, gbytes, bytes);
 		ENV_LEAVE(env, ip);
-		return ret;
+		return (ret);
 	}
 
 	dbenv->mp_gbytes = gbytes;
@@ -307,12 +304,15 @@ __memp_set_mp_max_openfd(dbenv, maxopenfd)
 	DB_ENV *dbenv;
 	int maxopenfd;
 {
+	DB_ENV *slice;
 	DB_MPOOL *dbmp;
 	DB_THREAD_INFO *ip;
 	ENV *env;
 	MPOOL *mp;
+	int i, ret;
 
 	env = dbenv->env;
+	ret = 0;
 
 	ENV_NOT_CONFIGURED(env,
 	    env->mp_handle, "DB_ENV->set_mp_max_openfd", DB_INIT_MPOOL);
@@ -327,7 +327,10 @@ __memp_set_mp_max_openfd(dbenv, maxopenfd)
 		ENV_LEAVE(env, ip);
 	} else
 		dbenv->mp_maxopenfd = maxopenfd;
-	return (0);
+	SLICE_FOREACH(dbenv, slice, i)
+		if ((ret = __memp_set_mp_max_openfd(slice, maxopenfd)) != 0)
+			break;
+	return (ret);
 }
 
 /*
@@ -377,12 +380,15 @@ __memp_set_mp_max_write(dbenv, maxwrite, maxwrite_sleep)
 	int maxwrite;
 	db_timeout_t maxwrite_sleep;
 {
+	DB_ENV *slice;
 	DB_MPOOL *dbmp;
 	DB_THREAD_INFO *ip;
 	ENV *env;
 	MPOOL *mp;
+	int i, ret;
 
 	env = dbenv->env;
+	ret = 0;
 
 	ENV_NOT_CONFIGURED(env,
 	    env->mp_handle, "DB_ENV->set_mp_max_write", DB_INIT_MPOOL);
@@ -400,7 +406,11 @@ __memp_set_mp_max_write(dbenv, maxwrite, maxwrite_sleep)
 		dbenv->mp_maxwrite = maxwrite;
 		dbenv->mp_maxwrite_sleep = maxwrite_sleep;
 	}
-	return (0);
+	SLICE_FOREACH(dbenv, slice, i)
+		if ((ret = __memp_set_mp_max_write(slice,
+		    maxwrite, maxwrite_sleep)) != 0)
+			break;
+	return (ret);
 }
 
 /*
@@ -524,6 +534,40 @@ __memp_set_mp_pagesize(dbenv, mp_pagesize)
 	return (0);
 }
 
+/*
+ * __memp_get_reg_dir
+ *
+ * PUBLIC: int __memp_get_reg_dir __P((DB_ENV *, const char **));
+ */
+int
+__memp_get_reg_dir(dbenv, dirp)
+	DB_ENV *dbenv;
+	const char **dirp;
+{
+	*dirp = dbenv->db_reg_dir;
+	return (0);
+}
+
+/*
+ * __memp_set_reg_dir
+ *
+ * PUBLIC: int __memp_set_reg_dir __P((DB_ENV *, const char *));
+ */
+int
+__memp_set_reg_dir(dbenv, dir)
+	DB_ENV *dbenv;
+	const char *dir;
+{
+	ENV *env;
+
+	env = dbenv->env;
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_region_dir");
+
+	if (dbenv->db_reg_dir != NULL)
+		__os_free(env, dbenv->db_reg_dir);
+	return (__os_strdup(env, dir, &dbenv->db_reg_dir));
+}
+
 /*
  * PUBLIC: int __memp_get_mp_tablesize __P((DB_ENV *, u_int32_t *));
  */
@@ -629,7 +673,6 @@ __memp_set_mp_mtxcount(dbenv, mp_mtxcount)
  * PUBLIC: int __memp_nameop __P((ENV *,
  * PUBLIC:     u_int8_t *, const char *, const char *, const char *, int));
  *
- * XXX
  * Undocumented interface: DB private.
  */
 int
@@ -645,7 +688,7 @@ __memp_nameop(env, fileid, newname, fullold, fullnew, inmem)
 	MPOOLFILE *mfp;
 	roff_t newname_off;
 	u_int32_t bucket;
-	int locked, ret;
+	int locked, purge_dead, ret;
 	size_t nlen;
 	void *p;
 
@@ -662,6 +705,7 @@ __memp_nameop(env, fileid, newname, fullold, fullnew, inmem)
 	nhp = NULL;
 	p = NULL;
 	locked = ret = 0;
+	purge_dead = 0;
 
 	if (!MPOOL_ON(env))
 		goto fsop;
@@ -715,7 +759,7 @@ __memp_nameop(env, fileid, newname, fullold, fullnew, inmem)
 			    R_ADDR(dbmp->reginfo, mfp->path_off)) == 0)
 				break;
 		if (mfp != NULL) {
-			ret = EEXIST;
+			ret = USR_ERR(env, EEXIST);
 			goto err;
 		}
 	}
@@ -739,7 +783,7 @@ __memp_nameop(env, fileid, newname, fullold, fullnew, inmem)
 
 	if (mfp == NULL) {
 		if (inmem) {
-			ret = ENOENT;
+			ret = USR_ERR(env, ENOENT);
 			goto err;
 		}
 		goto fsop;
@@ -754,7 +798,7 @@ __memp_nameop(env, fileid, newname, fullold, fullnew, inmem)
 		 */
 		if (mfp->no_backing_file)
 			mfp->mpf_cnt--;
-		mfp->deadfile = 1;
+		__memp_mf_mark_dead(dbmp, mfp, &purge_dead);
 		MUTEX_UNLOCK(env, mfp->mutex);
 	} else {
 		/*
@@ -793,7 +837,7 @@ fsop:	/*
 			 */
 			DB_ASSERT(env, fullnew != NULL);
 			if (fullnew == NULL) {
-				ret = EINVAL;
+				ret = USR_ERR(env, EINVAL);
 				goto err;
 			}
 			ret = __os_rename(env, fullold, fullnew, 1);
@@ -813,6 +857,12 @@ err:	if (p != NULL) {
 		if (nhp != NULL && nhp != hp)
 			MUTEX_UNLOCK(env, nhp->mtx_hash);
 	}
+	/*
+	 * __memp_purge_dead_files() must be called when the hash bucket is
+	 * unlocked.
+	 */
+	if (purge_dead)
+		(void)__memp_purge_dead_files(env);
 	return (ret);
 }
 
