@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1999, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1999, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -23,7 +23,6 @@
 #define	pthread_cond_wait		_lwp_cond_wait
 #define	pthread_mutex_destroy(x)	0
 #define	pthread_mutex_lock		_lwp_mutex_lock
-#define	pthread_mutex_trylock		_lwp_mutex_trylock
 #define	pthread_mutex_unlock		_lwp_mutex_unlock
 #endif
 #ifdef HAVE_MUTEX_UI_THREADS
@@ -32,7 +31,6 @@
 #define	pthread_cond_wait		cond_wait
 #define	pthread_mutex_destroy		mutex_destroy
 #define	pthread_mutex_lock		mutex_lock
-#define	pthread_mutex_trylock		mutex_trylock
 #define	pthread_mutex_unlock		mutex_unlock
 #endif
 
@@ -63,6 +61,64 @@
 		(ret) = EAGAIN;						\
 } while (0)
 
+/*
+ * !!!
+ * Solaris bug workaround: pthread_cond_wait() sometimes returns ETIME  -- out
+ * of sheer paranoia, check both ETIME and ETIMEDOUT.  We believe this happens
+ * when the application uses SIGALRM for some purpose, e.g., the C library sleep
+ * call, and Solaris delivers the signal to the wrong LWP.
+ */
+#ifdef ETIME
+#define	ETIME_TO_ETIMEDOUT(ret)	((ret) == ETIME ? ETIMEDOUT : (ret))
+#else
+#define	ETIME_TO_ETIMEDOUT(ret)	(ret)
+#endif
+
+#if defined(HAVE_SHARED_LATCHES) && !defined(HAVE_MUTEX_HYBRID)
+#define	RET_SET_PTHREAD_LOCK(mutexp, ret) do {				\
+	if (F_ISSET(mutexp, DB_MUTEX_SHARED))				\
+		RET_SET((pthread_rwlock_wrlock(&(mutexp)->u.rwlock)),	\
+		    ret);						\
+	else								\
+		RET_SET((pthread_mutex_lock(&(mutexp)->u.m.mutex)), ret); \
+} while (0)
+#define	RET_SET_PTHREAD_TIMEDLOCK(mutexp, timespec, ret) do {		\
+	if (F_ISSET(mutexp, DB_MUTEX_SHARED))				\
+		RET_SET(pthread_rwlock_timedwrlock(&(mutexp)->u.rwlock, \
+		    (timespec)), ret);					\
+	else								\
+		RET_SET(pthread_mutex_timedlock(&(mutexp)->u.m.mutex,	\
+		    (timespec)), ret);					\
+} while (0)
+#define	RET_SET_PTHREAD_TRYLOCK(mutexp, ret) do {			\
+	if (F_ISSET(mutexp, DB_MUTEX_SHARED))				\
+		RET_SET(pthread_rwlock_trywrlock(&(mutexp)->u.rwlock),	\
+		    ret);						\
+	else								\
+		RET_SET(pthread_mutex_trylock(&(mutexp)->u.m.mutex),	\
+		    ret);						\
+} while (0)
+#define	RET_SET_PTHREAD_UNLOCK(mutexp, ret) do {			\
+	if (F_ISSET(mutexp, DB_MUTEX_SHARED))				\
+		RETRY_ON_EFAULT(pthread_rwlock_unlock(&(mutexp)->u.rwlock), \
+		    ret);						\
+	else								\
+		RETRY_ON_EFAULT(pthread_mutex_unlock(&(mutexp)->u.m.mutex), \
+		    ret);						\
+} while (0)
+#else
+#define	RET_SET_PTHREAD_LOCK(mutexp, ret)				\
+		RET_SET(pthread_mutex_lock(&(mutexp)->u.m.mutex), ret);
+#define	RET_SET_PTHREAD_TIMEDLOCK(mutexp, timespec, ret)		\
+		RET_SET(pthread_mutex_timedlock(&(mutexp)->u.m.mutex,	\
+		    (timespec)), ret);
+#define	RET_SET_PTHREAD_TRYLOCK(mutexp, ret)				\
+		RET_SET(pthread_mutex_trylock(&(mutexp)->u.m.mutex), ret);
+#define	RET_SET_PTHREAD_UNLOCK(mutexp, ret)				\
+		RETRY_ON_EFAULT(pthread_mutex_unlock(&(mutexp)->u.m.mutex), \
+		    ret);
+#endif
+
 /*
  * __db_pthread_mutex_init --
  *	Initialize a pthread mutex: either a native one or
@@ -104,18 +160,18 @@ __db_pthread_mutex_init(env, mutex, flags)
 		pthread_rwlockattr_t rwlockattr, *rwlockattrp = NULL;
 #ifndef HAVE_MUTEX_THREAD_ONLY
 		if (!LF_ISSET(DB_MUTEX_PROCESS_ONLY)) {
-			RET_SET((pthread_rwlockattr_init(&rwlockattr)), ret);
+			RET_SET(pthread_rwlockattr_init(&rwlockattr), ret);
 			if (ret != 0)
 				goto err;
-			RET_SET((pthread_rwlockattr_setpshared(
-			    &rwlockattr, PTHREAD_PROCESS_SHARED)), ret);
+			RET_SET(pthread_rwlockattr_setpshared(
+			    &rwlockattr, PTHREAD_PROCESS_SHARED), ret);
 			rwlockattrp = &rwlockattr;
 		}
 #endif
 
 		if (ret == 0)
-			RET_SET((pthread_rwlock_init(&mutexp->u.rwlock,
-			    rwlockattrp)), ret);
+			RET_SET(pthread_rwlock_init(&mutexp->u.rwlock,
+			    rwlockattrp), ret);
 		if (rwlockattrp != NULL)
 			(void)pthread_rwlockattr_destroy(rwlockattrp);
 
@@ -127,18 +183,18 @@ __db_pthread_mutex_init(env, mutex, flags)
 #endif
 #ifndef HAVE_MUTEX_THREAD_ONLY
 	if (!LF_ISSET(DB_MUTEX_PROCESS_ONLY)) {
-		RET_SET((pthread_mutexattr_init(&mutexattr)), ret);
+		RET_SET(pthread_mutexattr_init(&mutexattr), ret);
 		if (ret != 0)
 			goto err;
-		RET_SET((pthread_mutexattr_setpshared(
-		    &mutexattr, PTHREAD_PROCESS_SHARED)), ret);
+		RET_SET(pthread_mutexattr_setpshared(
+		    &mutexattr, PTHREAD_PROCESS_SHARED), ret);
 		mutexattrp = &mutexattr;
 	}
 #endif
 
 	if (ret == 0)
 		RET_SET(
-		    (pthread_mutex_init(&mutexp->u.m.mutex, mutexattrp)), ret);
+		    pthread_mutex_init(&mutexp->u.m.mutex, mutexattrp), ret);
 
 	if (mutexattrp != NULL)
 		(void)pthread_mutexattr_destroy(mutexattrp);
@@ -147,19 +203,19 @@ __db_pthread_mutex_init(env, mutex, flags)
 	if (LF_ISSET(DB_MUTEX_SELF_BLOCK)) {
 #ifndef HAVE_MUTEX_THREAD_ONLY
 		if (!LF_ISSET(DB_MUTEX_PROCESS_ONLY)) {
-			RET_SET((pthread_condattr_init(&condattr)), ret);
+			RET_SET(pthread_condattr_init(&condattr), ret);
 			if (ret != 0)
 				goto err;
 
 			condattrp = &condattr;
-			RET_SET((pthread_condattr_setpshared(
-			    &condattr, PTHREAD_PROCESS_SHARED)), ret);
+			RET_SET(pthread_condattr_setpshared(
+			    &condattr, PTHREAD_PROCESS_SHARED), ret);
 		}
 #endif
 
 		if (ret == 0)
-			RET_SET((pthread_cond_init(
-			    &mutexp->u.m.cond, condattrp)), ret);
+			RET_SET(pthread_cond_init(
+			    &mutexp->u.m.cond, condattrp), ret);
 
 		F_SET(mutexp, DB_MUTEX_SELF_BLOCK);
 		if (condattrp != NULL)
@@ -170,7 +226,6 @@ __db_pthread_mutex_init(env, mutex, flags)
 #endif
 #ifdef HAVE_MUTEX_SOLARIS_LWP
 	/*
-	 * XXX
 	 * Gcc complains about missing braces in the static initializations of
 	 * lwp_cond_t and lwp_mutex_t structures because the structures contain
 	 * sub-structures/unions and the Solaris include file that defines the
@@ -231,48 +286,79 @@ err:	if (ret != 0) {
  *	Test for failchk concerns here too, to avoid hanging on a dead pid/tid.
  */
 inline static int
-__db_pthread_mutex_prep(env, mutex, mutexp, exclusive)
+__db_pthread_mutex_prep(env, mutex, mutexp, ip, exclusive)
 	ENV *env;
 	db_mutex_t mutex;
 	DB_MUTEX *mutexp;
+	DB_THREAD_INFO *ip;
 	int exclusive;
 {
 	DB_ENV *dbenv;
-	DB_THREAD_INFO *ip;
+#ifdef HAVE_FAILCHK_BROADCAST
+	db_timespec timespec;
+#endif
 	int ret;
 
 	dbenv = env->dbenv;
 	PERFMON4(env,
 	    mutex, suspend, mutex, exclusive, mutexp->alloc_id, mutexp);
+
+	/*
+	 * If this is a failchk'ing env handle, then use this special path so
+	 * that the failchk thread does not get blocked by a dead process.
+	 */
 	if (F_ISSET(dbenv, DB_ENV_FAILCHK)) {
 		for (;;) {
 			RET_SET_PTHREAD_TRYLOCK(mutexp, ret);
 			if (ret != EBUSY)
 				break;
-			if (dbenv->is_alive(dbenv,
-			    mutexp->pid, mutexp->tid, 0) == 0) {
-				ret = __env_set_state(env, &ip, THREAD_VERIFY);
-				if (ret != 0 ||
-				    ip->dbth_state == THREAD_FAILCHK) {
-					ret = DB_RUNRECOVERY;
-				} else {
-					/*
-					 * Some thread other than the true
-					 * FAILCHK thread in this process is
-					 * asking for the mutex held by the
-					 * dead process/thread.  We will block
-					 * here until someone else does the
-					 * cleanup.  Same behavior as if we
-					 * hadn't gone down the 'if
-					 * DB_ENV_FAILCHK' path to start with.
-					 */
-				    RET_SET_PTHREAD_LOCK(mutexp, ret);
-				    break;
-				}
+			/* If the owner is still alive, try again. */
+			if (dbenv->is_alive(dbenv, mutexp->pid, mutexp->tid, 0))
+				continue;
+			/*
+			 * The owner is dead, but we don't yet know whether
+			 * this is the failchk thread or some other thread that
+			 * is sharing the same DB_ENV.  We treat it as the
+			 * failchk thread if the DB_THREAD_INFO is NULL (as
+			 * when opening the environment and the thread info
+			 * table is not yet accessible) or if the thread state
+			 * is THREAD_FAILCHK.
+			 */
+			if (ip == NULL || ip->dbth_state == THREAD_FAILCHK) {
+				ret = DB_RUNRECOVERY;
+				break;
 			}
+			/*
+			 * Some thread other than the true FAILCHK thread in
+			 * this process is asking for the mutex held by the dead
+			 * process/thread.  We will block in this function until
+			 * someone else does the cleanup.  This is the same
+			 * behavior as if we hadn't gone down the
+			 * 'if DB_ENV_FAILCHK' path to start with.
+			 */
+			goto lockit;
 		}
-	} else
-		RET_SET_PTHREAD_LOCK(mutexp, ret);
+	} else {
+lockit:
+#ifdef HAVE_FAILCHK_BROADCAST
+		if (dbenv->mutex_failchk_timeout != 0) {
+			timespecclear(&timespec);
+			__clock_set_expires(env,
+			    &timespec, dbenv->mutex_failchk_timeout);
+			do {
+				RET_SET_PTHREAD_TIMEDLOCK(mutexp,
+				    (struct timespec *)&timespec, ret);
+				ret = ETIME_TO_ETIMEDOUT(ret);
+				if (ret == ETIMEDOUT &&
+				    F_ISSET(mutexp, DB_MUTEX_OWNER_DEAD) &&
+				    !F_ISSET(dbenv, DB_ENV_FAILCHK))
+					ret = USR_ERR(env,
+					    __mutex_died(env, mutex));
+			} while  (ret == ETIMEDOUT);
+		} else
+#endif
+			RET_SET_PTHREAD_LOCK(mutexp, ret);
+	}
 
 	PERFMON4(env,
 	    mutex, resume, mutex, exclusive, mutexp->alloc_id, mutexp);
@@ -303,48 +389,84 @@ __db_pthread_mutex_condwait(env, mutex, mutexp, timespec)
 	db_timespec *timespec;
 {
 	int ret;
-
-#ifdef MUTEX_DIAG
-	printf("condwait %ld %x wait busy %x count %d\n",
-	    mutex, pthread_self(), MUTEXP_BUSY_FIELD(mutexp), mutexp->wait);
+#ifdef HAVE_FAILCHK_BROADCAST
+	DB_ENV *dbenv;
+	db_timespec failchk_timespec;
 #endif
+
 	PERFMON4(env, mutex, suspend, mutex, TRUE, mutexp->alloc_id, mutexp);
 
+#ifdef HAVE_FAILCHK_BROADCAST
+	dbenv = env->dbenv;
+
+	/*
+	 * If the failchk timeout would be soon than the timeout passed in,
+	 * argument, use the failchk timeout. The caller handles "short" waits.
+	 */
+	if (dbenv->mutex_failchk_timeout != 0) {
+		timespecclear(&failchk_timespec);
+		__clock_set_expires(env,
+		    &failchk_timespec, dbenv->mutex_failchk_timeout);
+		if (timespec == NULL ||
+		    timespeccmp(timespec, &failchk_timespec, >))
+			timespec = &failchk_timespec;
+	}
+#endif
+
 	if (timespec != NULL) {
-		RET_SET((pthread_cond_timedwait(&mutexp->u.m.cond,
-		    &mutexp->u.m.mutex, (struct timespec *) timespec)), ret);
+		RET_SET(pthread_cond_timedwait(&mutexp->u.m.cond,
+		    &mutexp->u.m.mutex, (struct timespec *) timespec), ret);
+		ret = ETIME_TO_ETIMEDOUT(ret);
+#ifdef HAVE_FAILCHK_BROADCAST
+		if (F_ISSET(mutexp, DB_MUTEX_OWNER_DEAD) &&
+		    !F_ISSET(dbenv, DB_ENV_FAILCHK)) {
+			ret = USR_ERR(env, __mutex_died(env, mutex));
+			goto err;
+		}
+#endif
+		/*
+		 * Return any timeout error unless the timeout was only for
+		 * the failure check.
+		 */
 		if (ret == ETIMEDOUT) {
-			ret = DB_TIMEOUT;
-			goto ret;
+#ifdef HAVE_FAILCHK_BROADCAST
+			if (timespec == &failchk_timespec)
+				ret = 0;
+			else
+#endif
+				ret = USR_ERR(env, DB_TIMEOUT);
+			goto err;
 		}
 	} else
-		RET_SET((pthread_cond_wait(&mutexp->u.m.cond,
-		    &mutexp->u.m.mutex)), ret);
-#ifdef MUTEX_DIAG
-	printf("condwait %ld %x wait returns %d busy %x\n",
-	    mutex, pthread_self(), ret, MUTEXP_BUSY_FIELD(mutexp));
+		RET_SET(pthread_cond_wait(&mutexp->u.m.cond,
+		    &mutexp->u.m.mutex), ret);
+#ifdef HAVE_FAILCHK_BROADCAST
+	if (ret == 0 && F_ISSET(mutexp, DB_MUTEX_OWNER_DEAD) &&
+	    !F_ISSET(dbenv, DB_ENV_FAILCHK)) {
+		ret = USR_ERR(env, __mutex_died(env, mutex));
+		goto err;
+	}
 #endif
 	/*
 	 * !!!
 	 * Solaris bug workaround: pthread_cond_wait() sometimes returns ETIME
-	 * -- out  of sheer paranoia, check both ETIME and ETIMEDOUT.  We
+	 * -- out of sheer paranoia, check both ETIME and ETIMEDOUT.  We
 	 * believe this happens when the application uses SIGALRM for some
 	 * purpose, e.g., the C library sleep call, and Solaris delivers the
-	 * signal to the wrong  LWP.
+	 * signal to the wrong LWP.
 	 */
 	if (ret != 0) {
-		if (ret == ETIMEDOUT ||
-#ifdef ETIME
-		    ret == ETIME ||
-#endif
+		if ((ret = ETIME_TO_ETIMEDOUT(ret)) == ETIMEDOUT ||
 		    ret == EINTR)
 			ret = 0;
-		else
+		else {
 			/* Failure, caller shouldn't condwait again. */
 			(void)pthread_mutex_unlock(&mutexp->u.m.mutex);
+			(void)MUTEX_ERR(env, mutex, ret);
+		}
 	}
 
-ret:
+err:
 	PERFMON4(env, mutex, resume, mutex, TRUE, mutexp->alloc_id, mutexp);
 
 	COMPQUIET(mutex, 0);
@@ -354,24 +476,32 @@ ret:
 
 #ifndef HAVE_MUTEX_HYBRID
 /*
- * __db_pthread_mutex_lock
+ * __db_pthread_mutex_lock --
  *	Lock on a mutex, blocking if necessary.
- *	Timeouts are supported only for self-blocking mutexes.
+ *
+ *	Timeouts are supported only for self-blocking mutexes. When both a
+ *	given timeout and a dbenv-wide failchk timeout are specified, the given
+ *	timeout takes precedence -- a process failure might not be noticed for a
+ *	little while.
  *
  *	Self-blocking shared latches are not supported.
  *
  * PUBLIC: #ifndef HAVE_MUTEX_HYBRID
- * PUBLIC: int __db_pthread_mutex_lock __P((ENV *, db_mutex_t, db_timeout_t));
+ * PUBLIC: int __db_pthread_mutex_lock
+ * PUBLIC:      __P((ENV *, db_mutex_t, db_timeout_t, u_int32_t flags));
  * PUBLIC: #endif
  */
 int
-__db_pthread_mutex_lock(env, mutex, timeout)
+__db_pthread_mutex_lock(env, mutex, timeout, flags)
 	ENV *env;
 	db_mutex_t mutex;
 	db_timeout_t timeout;
+	u_int32_t flags;
 {
 	DB_ENV *dbenv;
 	DB_MUTEX *mutexp;
+	DB_THREAD_INFO *ip;
+	db_timeout_t checktimeout;
 	db_timespec timespec;
 	int ret, t_ret;
 
@@ -380,12 +510,28 @@ __db_pthread_mutex_lock(env, mutex, timeout)
 	if (!MUTEX_ON(env) || F_ISSET(dbenv, DB_ENV_NOLOCKING))
 		return (0);
 
-	t_ret = 0;
+	ret = 0;
 	mutexp = MUTEXP_SET(env, mutex);
 
-	CHECK_MTX_THREAD(env, mutexp);
+	 /*
+	  * Fetch the current thread's state struct, if there is a thread hash
+	  * table and we are keeping track of this mutex.  We increment the
+	  * mtx_ctr before we know that this call succeeds, and decrement it on
+	  * failure or in __db_pthread_mutex_unlock.  By incrementing it before
+	  * the attempt, we detect crashing that occur during this function.
+	  * A crash while holding a non-counted mutex will not be detected by
+	  * failchk's mtx_ctr code: those have to be detected some other way.
+	  */
+	ip = NULL;
+	if (env->thr_hashtab != NULL && LF_ISSET(MUTEX_CTR)) {
+		if ((ret = __env_set_state(env, &ip, THREAD_CTR_VERIFY)) != 0)
+			return (__env_panic(env, ret));
+		if (ip != NULL) {
+			DB_ASSERT(env, ip->mtx_ctr < 20);
+			ip->mtx_ctr++;
+		}
+	}
 
-#if defined(HAVE_STATISTICS)
 	/*
 	 * We want to know which mutexes are contentious, but don't want to
 	 * do an interlocked test here -- that's slower when the underlying
@@ -398,64 +544,114 @@ __db_pthread_mutex_lock(env, mutex, timeout)
 	else
 		STAT_INC(env,
 		    mutex, set_nowait, mutexp->mutex_set_nowait, mutex);
+
+	checktimeout = timeout;
+#ifdef HAVE_FAILCHK_BROADCAST
+	if (checktimeout == 0 || checktimeout > dbenv->mutex_failchk_timeout)
+		checktimeout = dbenv->mutex_failchk_timeout;
 #endif
 
+	/*
+	 * Handle the trylock cases here. mutex_prep would block for
+	 * non-self-blocking mutexes.
+	 */
+	if (!LF_ISSET(MUTEX_WAIT) && !F_ISSET(mutexp, DB_MUTEX_SELF_BLOCK)) {
+#ifdef HAVE_SHARED_LATCHES
+		if (F_ISSET(mutexp, DB_MUTEX_SHARED))
+			ret = pthread_rwlock_trywrlock(&mutexp->u.rwlock);
+		else
+#endif
+			ret = pthread_mutex_trylock(&mutexp->u.m.mutex);
+		if (ret == 0)
+			goto success;
+		if (ret == EBUSY) {
+			ret = DB_LOCK_NOTGRANTED;
+			goto timeout;
+		}
+		goto err;
+	}
+
 	/* Single-thread the next block, except during the possible condwait. */
-	if ((ret = __db_pthread_mutex_prep(env, mutex, mutexp, TRUE)) != 0)
+	if ((ret = __db_pthread_mutex_prep(env, mutex, mutexp, ip, TRUE)) != 0)
 		goto err;
 
 	if (F_ISSET(mutexp, DB_MUTEX_SELF_BLOCK)) {
-		if (timeout != 0)
+		if (checktimeout != 0)
 			timespecclear(&timespec);
 		while (MUTEXP_IS_BUSY(mutexp)) {
 			/* Set expiration timer upon first need. */
-			if (timeout != 0 && !timespecisset(&timespec)) {
-				timespecclear(&timespec);
+			if (checktimeout != 0 && !timespecisset(&timespec))
 				__clock_set_expires(env, &timespec, timeout);
-			}
-			t_ret = __db_pthread_mutex_condwait(env,
+			ret = __db_pthread_mutex_condwait(env,
 			    mutex, mutexp, timeout == 0 ? NULL : &timespec);
-			if (t_ret != 0) {
-				if (t_ret == DB_TIMEOUT)
-					goto out;
-				ret = t_ret;
+			if (ret != 0) {
+				if (ret == DB_TIMEOUT)
+					goto timeout;
 				goto err;
 			}
 		}
-
-		F_SET(mutexp, DB_MUTEX_LOCKED);
-		dbenv->thread_id(dbenv, &mutexp->pid, &mutexp->tid);
-out:
-		/* #2471: HP-UX can sporadically return EFAULT. See above */
-		RETRY_ON_EFAULT(pthread_mutex_unlock(&mutexp->u.m.mutex), ret);
-		if (ret != 0)
-			goto err;
 	} else {
 #ifdef DIAGNOSTIC
 		if (F_ISSET(mutexp, DB_MUTEX_LOCKED)) {
 			char buf[DB_THREADID_STRLEN];
 			(void)dbenv->thread_id_string(dbenv,
 			    mutexp->pid, mutexp->tid, buf);
+			ret = MUTEX_ERR(env, mutex, EINVAL);
 			__db_errx(env, DB_STR_A("2022",
 		    "pthread lock failed: lock currently in use: pid/tid: %s",
 			    "%s"), buf);
-			ret = EINVAL;
 			goto err;
 		}
 #endif
-		F_SET(mutexp, DB_MUTEX_LOCKED);
-		dbenv->thread_id(dbenv, &mutexp->pid, &mutexp->tid);
+	}
+
+success:
+	F_SET(mutexp, DB_MUTEX_LOCKED);
+	dbenv->thread_id(dbenv, &mutexp->pid, &mutexp->tid);
+timeout:
+
+	/*
+	 * A failing attempt to lock a counted mutex does not keep the counter
+	 * incremented, otherwise (e.g., for DB_LOCK_NOTGRANTED) the counter
+	 * would be incorrect.
+	 */
+	if (ip != NULL && LF_ISSET(MUTEX_CTR) && ret != 0) {
+		DB_ASSERT(env, ip->mtx_ctr > 0);
+		ip->mtx_ctr--;
+	}
+	/*
+	 * Self-blocking pthread mutexes only keep the mutex locked for a short
+	 * time, during the acquire and release calls.  Release the mutex or
+	 * rwlock so that the condwait code above can be reached by waiters.
+	 * This complexity allows another thread (or process) to unlock this
+	 * BDB "mutex"; pthreads mutexes only support unlocking mutexes that
+	 * you have locked..
+	 */
+	if (F_ISSET(mutexp, DB_MUTEX_SELF_BLOCK)) {
+		/* #2471: HP-UX can sporadically return EFAULT. See above */
+		RET_SET_PTHREAD_UNLOCK(mutexp, t_ret);
+		if (t_ret != 0) {
+			ret = MUTEX_ERR(env, mutex, t_ret);
+			goto err;
+		}
 	}
 
 #ifdef DIAGNOSTIC
 	/*
-	 * We want to switch threads as often as possible.  Yield every time
-	 * we get a mutex to ensure contention.
+	 * For diagnostics we want to switch threads as often as possible.
+	 * Yield every time we get a mutex to ensure contention.
 	 */
 	if (F_ISSET(dbenv, DB_ENV_YIELDCPU))
 		__os_yield(env, 0, 0);
 #endif
-	return (t_ret);
+#ifdef MUTEX_DIAG
+	if (ret == 0) {
+		__os_gettime(env, &mutexp->mutex_history.when, 0);
+		__os_stack_text(env, mutexp->mutex_history.stacktext,
+		    sizeof(mutexp->mutex_history.stacktext), 12, 2);
+	}
+#endif
+	return (ret);
 
 err:
 	__db_err(env, ret, DB_STR("2023", "pthread lock failed"));
@@ -469,16 +665,22 @@ err:
  *	Take a shared lock on a mutex, blocking if necessary.
  *
  * PUBLIC: #if defined(HAVE_SHARED_LATCHES)
- * PUBLIC: int __db_pthread_mutex_readlock __P((ENV *, db_mutex_t));
+ * PUBLIC: int __db_pthread_mutex_readlock __P((ENV *, db_mutex_t, u_int32_t));
  * PUBLIC: #endif
  */
 int
-__db_pthread_mutex_readlock(env, mutex)
+__db_pthread_mutex_readlock(env, mutex, flags)
 	ENV *env;
 	db_mutex_t mutex;
+	u_int32_t flags;
 {
 	DB_ENV *dbenv;
 	DB_MUTEX *mutexp;
+	DB_THREAD_INFO *ip;
+	MUTEX_STATE *state;
+#ifdef HAVE_FAILCHK_BROADCAST
+	db_timespec timespec;
+#endif
 	int ret;
 
 	dbenv = env->dbenv;
@@ -489,9 +691,6 @@ __db_pthread_mutex_readlock(env, mutex)
 	mutexp = MUTEXP_SET(env, mutex);
 	DB_ASSERT(env, F_ISSET(mutexp, DB_MUTEX_SHARED));
 
-	CHECK_MTX_THREAD(env, mutexp);
-
-#if defined(HAVE_STATISTICS)
 	/*
 	 * We want to know which mutexes are contentious, but don't want to
 	 * do an interlocked test here -- that's slower when the underlying
@@ -505,15 +704,80 @@ __db_pthread_mutex_readlock(env, mutex)
 	else
 		STAT_INC(env,
 		    mutex, set_rd_nowait, mutexp->mutex_set_rd_nowait, mutex);
-#endif
+
+	ip = NULL;
+	state = NULL;
+	if (env->thr_hashtab != NULL) {
+		if ((ret = __env_set_state(env, &ip, THREAD_VERIFY)) != 0)
+			return (__env_panic(env, ret));
+		if ((ret = __mutex_record_lock(env,
+		    mutex, ip, MUTEX_ACTION_INTEND_SHARE, &state)) != 0)
+			return (ret);
+	}
 
 	PERFMON4(env, mutex, suspend, mutex, FALSE, mutexp->alloc_id, mutexp);
-	RET_SET((pthread_rwlock_rdlock(&mutexp->u.rwlock)), ret);
-	PERFMON4(env, mutex, resume, mutex, FALSE, mutexp->alloc_id, mutexp);
-	DB_ASSERT(env, !F_ISSET(mutexp, DB_MUTEX_LOCKED));
-	if (ret != 0)
-		goto err;
 
+#ifdef HAVE_FAILCHK_BROADCAST
+	if (dbenv->mutex_failchk_timeout != 0) {
+		do {
+			timespecclear(&timespec);
+			__clock_set_expires(env,
+			    &timespec, dbenv->mutex_failchk_timeout);
+			RET_SET(pthread_rwlock_timedrdlock(&mutexp->u.rwlock,
+			    (struct timespec *)&timespec), ret);
+			/*
+			 * Some pthreads can detect self-deadlock.  When not
+			 * waiting for the mutex, treat those as timeouts.
+			 */
+			if (ret == EDEADLK && !LF_ISSET(MUTEX_WAIT))
+				goto busy;
+			ret = ETIME_TO_ETIMEDOUT(ret);
+			if (F_ISSET(mutexp, DB_MUTEX_OWNER_DEAD) &&
+			    !F_ISSET(dbenv, DB_ENV_FAILCHK)) {
+				if (ret == 0)
+					RETRY_ON_EFAULT(pthread_rwlock_unlock(
+					    &mutexp->u.rwlock), ret);
+				ret = USR_ERR(env, __mutex_died(env, mutex));
+				goto err;
+			}
+		} while (ret == ETIMEDOUT);
+		if (ret != 0)
+			ret = USR_ERR(env, ret);
+	} else
+#endif
+	{
+		if (LF_ISSET(MUTEX_WAIT))
+			RET_SET(pthread_rwlock_rdlock(&mutexp->u.rwlock), ret);
+		else {
+			RET_SET(pthread_rwlock_tryrdlock(&mutexp->u.rwlock),
+			    ret);
+			if (ret == EBUSY || ret == EAGAIN) {
+#ifdef HAVE_FAILCHK_BROADCAST
+busy:
+#endif
+				ret = DB_LOCK_NOTGRANTED;
+			}
+		}
+	}
+
+	PERFMON4(env, mutex, resume, mutex, FALSE, mutexp->alloc_id, mutexp);
+	if (ret != 0 && ret != DB_LOCK_NOTGRANTED)
+		goto err;
+	if (state != NULL)
+		state->action = ret == 0 ?
+		    MUTEX_ACTION_SHARED : MUTEX_ACTION_UNLOCKED;
+#ifdef HAVE_FAILCHK_BROADCAST
+	if (F_ISSET(mutexp, DB_MUTEX_OWNER_DEAD) &&
+	    !F_ISSET(dbenv, DB_ENV_FAILCHK)) {
+		ret = USR_ERR(env, __mutex_died(env, mutex));
+		goto err;
+	}
+#endif
+#ifdef MUTEX_DIAG
+	__os_gettime(env, &mutexp->mutex_history.when, 0);
+	__os_stack_text(env, mutexp->mutex_history.stacktext,
+	    sizeof(mutexp->mutex_history.stacktext), 12, 2);
+#endif
 #ifdef DIAGNOSTIC
 	/*
 	 * We want to switch threads as often as possible.  Yield every time
@@ -522,9 +786,14 @@ __db_pthread_mutex_readlock(env, mutex)
 	if (F_ISSET(dbenv, DB_ENV_YIELDCPU))
 		__os_yield(env, 0, 0);
 #endif
-	return (0);
+	return (ret);
 
-err:	__db_err(env, ret, DB_STR("2024", "pthread readlock failed"));
+err:
+	if (state != NULL)
+		state->action = MUTEX_ACTION_UNLOCKED;
+	if (ret == EDEADLK)
+		return (ret);
+	__db_err(env, ret, DB_STR("2024", "pthread readlock failed"));
 	return (__env_panic(env, ret));
 }
 #endif
@@ -532,25 +801,31 @@ err:	__db_err(env, ret, DB_STR("2024", "pthread readlock failed"));
 #ifdef HAVE_MUTEX_HYBRID
 /*
  * __db_hybrid_mutex_suspend
- *	Suspend this thread until the mutex is free enough to give the caller a
- *	good chance of getting the mutex in the requested exclusivity mode.
+ *	Suspend this thread, usually until the mutex is free enough to give the
+ *	caller a good chance of getting the mutex in the requested exclusivity
+ *	mode. Return early if the timeout is reached or a dead mutex is found
+ *	to be dead.
  *
  *	The major difference between this and the old __db_pthread_mutex_lock()
- *	is the additional 'exclusive' parameter.
+ *	is the additional flags parameter.
  *
  * PUBLIC: #ifdef HAVE_MUTEX_HYBRID
- * PUBLIC: int __db_hybrid_mutex_suspend
- * PUBLIC:	__P((ENV *, db_mutex_t, db_timespec *, int));
+ * PUBLIC: int __db_hybrid_mutex_suspend  __P((ENV *,
+ * PUBLIC:	 db_mutex_t, db_timespec *, DB_THREAD_INFO *, int));
  * PUBLIC: #endif
  */
 int
-__db_hybrid_mutex_suspend(env, mutex, timespec, exclusive)
+__db_hybrid_mutex_suspend(env, mutex, timespec, ip, exclusive)
 	ENV *env;
 	db_mutex_t mutex;
 	db_timespec *timespec;
+	DB_THREAD_INFO *ip;
 	int exclusive;
 {
 	DB_MUTEX *mutexp;
+#ifdef HAVE_FAILCHECK_BROADCAST
+	db_timespec failchk_timespec;
+#endif
 	int ret, t_ret;
 
 	t_ret = 0;
@@ -560,7 +835,8 @@ __db_hybrid_mutex_suspend(env, mutex, timespec, exclusive)
 		DB_ASSERT(env, F_ISSET(mutexp, DB_MUTEX_SHARED));
 	DB_ASSERT(env, F_ISSET(mutexp, DB_MUTEX_SELF_BLOCK));
 
-	if ((ret = __db_pthread_mutex_prep(env, mutex, mutexp, exclusive)) != 0)
+	if ((ret = __db_pthread_mutex_prep(env,
+	    mutex, mutexp, ip, exclusive)) != 0)
 		goto err;
 
 	/*
@@ -571,7 +847,7 @@ __db_hybrid_mutex_suspend(env, mutex, timespec, exclusive)
 	 * before checking the wait counter.
 	 */
 	mutexp->wait++;
-	MUTEX_MEMBAR(mutexp->wait);
+	(void)MUTEX_MEMBAR(mutexp->wait);
 	while (exclusive ? MUTEXP_IS_BUSY(mutexp) :
 	    atomic_read(&mutexp->sharecount) == MUTEX_SHARE_ISEXCLUSIVE) {
 		t_ret = __db_pthread_mutex_condwait(env,
@@ -582,7 +858,7 @@ __db_hybrid_mutex_suspend(env, mutex, timespec, exclusive)
 			ret = t_ret;
 			goto err;
 		}
-		MUTEX_MEMBAR(mutexp->flags);
+		(void)MUTEX_MEMBAR(mutexp->flags);
 	}
 
 	mutexp->wait--;
@@ -617,18 +893,21 @@ err:
  * __db_pthread_mutex_unlock --
  *	Release a mutex, or, if hybrid, wake a thread up from a suspend.
  *
- * PUBLIC: int __db_pthread_mutex_unlock __P((ENV *, db_mutex_t));
+ * PUBLIC: int __db_pthread_mutex_unlock
+ * PUBLIC:     __P((ENV *, db_mutex_t, DB_THREAD_INFO *, u_int32_t));
  */
 int
-__db_pthread_mutex_unlock(env, mutex)
+__db_pthread_mutex_unlock(env, mutex, ip, flags)
 	ENV *env;
 	db_mutex_t mutex;
+	DB_THREAD_INFO *ip;
+	u_int32_t flags;
 {
 	DB_ENV *dbenv;
 	DB_MUTEX *mutexp;
 	int ret;
-#if defined(MUTEX_DIAG) && defined(HAVE_MUTEX_HYBRID)
-	int waiters;
+#ifndef HAVE_MUTEX_HYBRID
+	char description[DB_MUTEX_DESCRIBE_STRLEN];
 #endif
 
 	dbenv = env->dbenv;
@@ -637,20 +916,22 @@ __db_pthread_mutex_unlock(env, mutex)
 		return (0);
 
 	mutexp = MUTEXP_SET(env, mutex);
-#if defined(MUTEX_DIAG) && defined(HAVE_MUTEX_HYBRID)
-	waiters = mutexp->wait;
-#endif
+	if (env->thr_hashtab != NULL && ip == NULL &&
+	    (ret = __env_set_state(env, &ip, THREAD_CTR_VERIFY)) != 0)
+		return (__env_panic(env, ret));
 
-#if !defined(HAVE_MUTEX_HYBRID) && defined(DIAGNOSTIC)
+#if !defined(HAVE_MUTEX_HYBRID)
 	if (!F_ISSET(mutexp, DB_MUTEX_LOCKED | DB_MUTEX_SHARED)) {
-		__db_errx(env, DB_STR("2025",
-		    "pthread unlock failed: lock already unlocked"));
+		if (!PANIC_ISSET(env))
+			__db_errx(env, DB_STR("2069",
+			    "pthread unlock %s: already unlocked"),
+			    __mutex_describe(env, mutex, description));
 		return (__env_panic(env, EACCES));
 	}
 #endif
 	if (F_ISSET(mutexp, DB_MUTEX_SELF_BLOCK)) {
-		ret = __db_pthread_mutex_prep(env, mutex, mutexp, TRUE);
-		if (ret != 0)
+		if ((ret = __db_pthread_mutex_prep(env,
+		    mutex, mutexp, ip, TRUE)) != 0)
 			goto err;
 
 #ifdef HAVE_MUTEX_HYBRID
@@ -662,35 +943,37 @@ __db_pthread_mutex_unlock(env, mutex)
 
 		if (F_ISSET(mutexp, DB_MUTEX_SHARED))
 			RET_SET(
-			    (pthread_cond_broadcast(&mutexp->u.m.cond)), ret);
+			    pthread_cond_broadcast(&mutexp->u.m.cond), ret);
 		else
-			RET_SET((pthread_cond_signal(&mutexp->u.m.cond)), ret);
+			RET_SET(pthread_cond_signal(&mutexp->u.m.cond), ret);
 		if (ret != 0)
 			goto err;
+		if (ip != NULL && LF_ISSET(MUTEX_CTR)) {
+			DB_ASSERT(env, ip->mtx_ctr > 0);
+			ip->mtx_ctr--;
+		}
 	} else {
 #ifndef HAVE_MUTEX_HYBRID
-		F_CLR(mutexp, DB_MUTEX_LOCKED);
+		if (F_ISSET(mutexp, DB_MUTEX_LOCKED)) {
+			F_CLR(mutexp, DB_MUTEX_LOCKED);
+			if (ip != NULL && LF_ISSET(MUTEX_CTR)) {
+				DB_ASSERT(env, ip->mtx_ctr > 0);
+				ip->mtx_ctr--;
+			}
+		}
+		else if (ip != NULL &&
+		    (ret = __mutex_record_unlock(env, mutex, ip)) != 0)
+			goto err;
 #endif
 	}
 
 	/* See comment above; workaround for [#2471]. */
-#if defined(HAVE_SHARED_LATCHES) && !defined(HAVE_MUTEX_HYBRID)
-	if (F_ISSET(mutexp, DB_MUTEX_SHARED))
-		RETRY_ON_EFAULT(pthread_rwlock_unlock(&mutexp->u.rwlock), ret);
-	else
-#endif
-		RETRY_ON_EFAULT(pthread_mutex_unlock(&mutexp->u.m.mutex), ret);
+	RET_SET_PTHREAD_UNLOCK(mutexp, ret);
 
 err:	if (ret != 0) {
 		__db_err(env, ret, "pthread unlock failed");
 		return (__env_panic(env, ret));
 	}
-#if defined(MUTEX_DIAG) && defined(HAVE_MUTEX_HYBRID)
-	if (!MUTEXP_IS_BUSY(mutexp) && mutexp->wait != 0)
-		printf("unlock %ld %x busy %x waiters %d/%d\n",
-		    mutex, pthread_self(), ret,
-		    MUTEXP_BUSY_FIELD(mutexp), waiters, mutexp->wait);
-#endif
 	return (ret);
 }
 
@@ -733,13 +1016,13 @@ __db_pthread_mutex_destroy(env, mutex)
 		 * If there were dead processes waiting on the condition
 		 * we may not be able to destroy it.  Let failchk thread skip
 		 * this, unless destroy is required.
-		 * XXX What operating system resources might this leak?
+		 * !!! What operating system resources might this leak?
 		 */
 #ifdef HAVE_PTHREAD_RWLOCK_REINIT_OKAY
 		if (!failchk_thread)
 #endif
 			RET_SET(
-			    (pthread_rwlock_destroy(&mutexp->u.rwlock)), ret);
+			    pthread_rwlock_destroy(&mutexp->u.rwlock), ret);
 		/* For rwlocks, we're done - must not destroy rest of union */
 		return (ret);
 #endif
@@ -754,15 +1037,14 @@ __db_pthread_mutex_destroy(env, mutex)
 #ifdef HAVE_PTHREAD_COND_REINIT_OKAY
 		if (!failchk_thread)
 #endif
-			RET_SET((pthread_cond_destroy(&mutexp->u.m.cond)), ret);
+			RET_SET(pthread_cond_destroy(&mutexp->u.m.cond), ret);
 		if (ret != 0)
 			__db_err(env, ret, DB_STR("2026",
 			    "unable to destroy cond"));
 	}
-	RET_SET((pthread_mutex_destroy(&mutexp->u.m.mutex)), t_ret);
+	RET_SET(pthread_mutex_destroy(&mutexp->u.m.mutex), t_ret);
 	if (t_ret != 0 && !failchk_thread) {
-		__db_err(env, t_ret, DB_STR("2027",
-		    "unable to destroy mutex"));
+		__db_err(env, t_ret, DB_STR("2027", "unable to destroy mutex"));
 		if (ret == 0)
 			ret = t_ret;
 	}
