@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1999, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1999, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -47,6 +47,14 @@ __qam_vrfy_meta(dbp, vdp, meta, pgno, flags)
 	buf = NULL;
 	names = NULL;
 
+	if (dbp->type != DB_QUEUE) {
+		EPRINT((env, DB_STR_A("1215",
+		    "Page %lu: invalid page type %u for %s database",
+		    "%lu %u %s"), (u_long)pgno, TYPE(meta),
+		    __db_dbtype_to_string(dbp->type)));
+		return DB_VERIFY_FATAL;
+	}
+
 	if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 		return (ret);
 
@@ -83,7 +91,14 @@ __qam_vrfy_meta(dbp, vdp, meta, pgno, flags)
 	 * re_len:  If this is bad, we can't safely verify queue data pages, so
 	 * return DB_VERIFY_FATAL
 	 */
-	if (DB_ALIGN(meta->re_len + sizeof(QAMDATA) - 1, sizeof(u_int32_t)) *
+	if (meta->rec_page == 0) {
+		EPRINT((env, DB_STR_A("1214",
+		    "Page %lu: the number of records per page %lu is bad",
+		    "%lu %lu"), (u_long)pgno, (u_long)meta->rec_page));
+		ret = DB_VERIFY_FATAL;
+		goto err;
+	} else if (DB_ALIGN(meta->re_len +
+	    sizeof(QAMDATA) - 1, sizeof(u_int32_t)) *
 	    meta->rec_page + QPAGE_SZ(dbp) > dbp->pgsize) {
 		EPRINT((env, DB_STR_A("1147",
     "Page %lu: queue record length %lu too high for page size and recs/page",
@@ -325,6 +340,14 @@ __qam_vrfy_data(dbp, vdp, h, pgno, flags)
 	QAMDATA *qp;
 	db_recno_t i;
 
+	if (dbp->type != DB_QUEUE) {
+		EPRINT((dbp->env, DB_STR_A("1215",
+		    "Page %lu: invalid page type %u for %s database",
+		    "%lu %u %s"), (u_long)pgno, TYPE(h),
+		    __db_dbtype_to_string(dbp->type)));
+		return DB_VERIFY_BAD;
+	}
+
 	/*
 	 * Not much to do here, except make sure that flags are reasonable.
 	 *
@@ -526,7 +549,7 @@ begin:	for (; i <= stop; i++) {
 			if (F_ISSET(pip, VRFY_IS_ALLZEROES))
 				goto put;
 			if (pip->type != P_QAMDATA) {
-				EPRINT((env, DB_STR_A("1154",
+				EPRINT((env, DB_STR_A("1153",
 		    "Page %lu: queue database page of incorrect type %lu",
 				    "%lu %lu"), (u_long)i, (u_long)pip->type));
 				isbad = 1;
