@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -201,7 +201,7 @@ __rep_get_nsites(dbenv, n)
 }
 
 int
-__rep_set_priority(dbenv, priority)
+__rep_set_priority_pp(dbenv, priority)
 	DB_ENV *dbenv;
 	u_int32_t priority;
 {
@@ -219,7 +219,7 @@ __rep_get_priority(dbenv, priority)
 }
 
 int
-__rep_set_timeout(dbenv, which, timeout)
+__rep_set_timeout_pp(dbenv, which, timeout)
 	DB_ENV *dbenv;
 	int which;
 	db_timeout_t timeout;
@@ -432,4 +432,20 @@ __rep_txn_applied(env, ip, commit_info, timeout)
 	COMPQUIET(timeout, 0);
 	return (__db_norep(env));
 }
+
+size_t
+__rep_object_size(env)
+	ENV *env;
+{
+	COMPQUIET(env, NULL);
+	return (0);
+}
+
+size_t
+__rep_object_max(env)
+	ENV *env;
+{
+	COMPQUIET(env, NULL);
+	return (0);
+}
 #endif /* !HAVE_REPLICATION */
