@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2005, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2005, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -12,7 +12,8 @@
 
 static db_timeout_t __repmgr_compute_response_time __P((ENV *));
 static int __repmgr_elect __P((ENV *, u_int32_t, db_timespec *));
-static int __repmgr_elect_main __P((ENV *, REPMGR_RUNNABLE *));
+static int __repmgr_elect_main __P((ENV *,
+    DB_THREAD_INFO *, REPMGR_RUNNABLE *));
 static void *__repmgr_elect_thread __P((void *));
 
 /*
@@ -102,7 +103,7 @@ __repmgr_elect_thread(argsp)
 		RPRINT(env, (env,
 		    DB_VERB_REPMGR_MISC, "starting election thread"));
 
-	if (ret != 0 || (ret = __repmgr_elect_main(env, th)) != 0) {
+	if (ret != 0 || (ret = __repmgr_elect_main(env, ip, th)) != 0) {
 		__db_err(env, ret, "election thread failed");
 		RPRINT(env, (env,
 		    DB_VERB_REPMGR_MISC, "election thread is exiting"));
@@ -119,8 +120,9 @@ __repmgr_elect_thread(argsp)
 }
 
 static int
-__repmgr_elect_main(env, th)
+__repmgr_elect_main(env, ip, th)
 	ENV *env;
+	DB_THREAD_INFO *ip;
 	REPMGR_RUNNABLE *th;
 {
 	DB_REP *db_rep;
@@ -134,10 +136,13 @@ __repmgr_elect_main(env, th)
 	db_timespec failtime, now, repstart_time, target, wait_til;
 	db_timeout_t delay_time, response_time, tmp_time;
 	u_long sec, usec;
-	u_int32_t flags;
-	int done_repstart, ret, suppress_election;
+	u_int32_t flags, max_tries, tries;
+	int client_detected, done_repstart, lsnhist_match, master_detected;
+	int ret, suppress_election;
 	enum { ELECTION, REPSTART } action;
 
+	COMPQUIET(usec, 0);
+	COMPQUIET(max_tries, 0);
 	COMPQUIET(action, ELECTION);
 
 	db_rep = env->rep_handle;
@@ -191,6 +196,120 @@ __repmgr_elect_main(env, th)
 	db_rep->preferred_elect_thr = th;
 	UNLOCK_MUTEX(db_rep->mutex);
 
+	/*
+	 * In preferred master mode, the select thread signals when a
+	 * client has lost its connection to the master via prefmas_pending,
+	 * but the actual restart as temporary master is done here in an
+	 * election thread.
+	 */
+	if (IS_PREFMAS_MODE(env) && F_ISSET(rep, REP_F_CLIENT) &&
+	    db_rep->prefmas_pending == start_temp_master) {
+		db_rep->prefmas_pending = no_action;
+		RPRINT(env, (env, DB_VERB_REPMGR_MISC,
+		    "elect_main preferred master restart temp master"));
+		ret = __repmgr_become_master(env, 0);
+		goto out;
+	}
+
+	/* Get preferred master wait limits for detecting the other site. */
+	if (IS_PREFMAS_MODE(env) &&
+	    (ret = __repmgr_prefmas_get_wait(env, &max_tries, &usec)) != 0)
+		goto out;
+
+	/* Preferred master mode master site start-up. */
+	if (IS_PREFMAS_MODE(env) &&
+	    FLD_ISSET(rep->config, REP_C_PREFMAS_MASTER) &&
+	    LF_ISSET(ELECT_F_STARTUP)) {
+		RPRINT(env, (env, DB_VERB_REPMGR_MISC,
+		    "elect_main preferred master site startup"));
+		client_detected = FALSE;
+		lsnhist_match = FALSE;
+		tries = 0;
+		while (!client_detected && tries < max_tries) {
+			__os_yield(env, 0, usec);
+			tries++;
+			client_detected = __repmgr_prefmas_connected(env);
+		}
+		if (client_detected) {
+			RPRINT(env, (env, DB_VERB_REPMGR_MISC,
+			    "elect_main preferred master client detected"));
+			/*
+			 * Restart remote site as a client.  Depending on the
+			 * outcome of lsnhist_match below, this site will
+			 * either restart as master or it will start an
+			 * election.  In either case, the remote site should
+			 * be running as a client.
+			 *
+			 * Then perform the lsnhist_match comparison.
+			 */
+			if ((ret = __repmgr_restart_site_as_client(
+			    env, 1)) != 0 ||
+			    (ret = __repmgr_lsnhist_match(env,
+			    ip, 1, &lsnhist_match)) != 0)
+				goto out;
+			/*
+			 * An lsnhist_match means that we have a continuous
+			 * set of transactions and it is safe to call a
+			 * comparison election to preserve any temporary master
+			 * transactions that were committed while this site
+			 * was down.
+			 */
+			if (lsnhist_match) {
+				F_CLR(rep, REP_F_HOLD_GEN);
+				LF_SET(ELECT_F_IMMED);
+				LF_CLR(ELECT_F_STARTUP);
+				/* Continue on to election code below. */
+			}
+		}
+		/*
+		 * If we didn't detect a client within a reasonable time or
+		 * we failed the lsnhist_match (meaning we have conflicting
+		 * sets of transactions), we start this site as a master and
+		 * possibly force rollback of temporary master transactions.
+		 */
+		if (!client_detected || !lsnhist_match) {
+			RPRINT(env, (env, DB_VERB_REPMGR_MISC,
+			    "elect_main preferred master site start master"));
+			ret = __repmgr_become_master(env, 0);
+			F_CLR(rep, REP_F_HOLD_GEN);
+			goto out;
+		}
+	}
+
+	/* Preferred master mode client site start-up. */
+	if (IS_PREFMAS_MODE(env) &&
+	    FLD_ISSET(rep->config, REP_C_PREFMAS_CLIENT) &&
+	    LF_ISSET(ELECT_F_STARTUP)) {
+		RPRINT(env, (env, DB_VERB_REPMGR_MISC,
+		    "elect_main preferred master client site startup"));
+		master_detected = FALSE;
+		tries = 0;
+		while (!master_detected && tries < max_tries) {
+			__os_yield(env, 0, usec);
+			tries++;
+			master_detected = __repmgr_prefmas_connected(env);
+		}
+		/*
+		 * If we find the master, restart as client here so that we
+		 * send a newclient message after we are connected to the
+		 * master.  The master will send a newmaster message so that
+		 * we can start the client sync process.
+		 *
+		 * If we haven't found the master after the timeout, start as
+		 * temporary master.
+		 */
+		if (master_detected) {
+			RPRINT(env, (env, DB_VERB_REPMGR_MISC,
+			    "elect_main preferred master detected"));
+			ret = __repmgr_become_client(env);
+		} else {
+			RPRINT(env, (env, DB_VERB_REPMGR_MISC,
+			    "elect_main preferred master client start master"));
+			ret = __repmgr_become_master(env, 0);
+		}
+		goto out;
+	}
+
 	/*
 	 * The 'done_repstart' flag keeps track of which was our most recent
 	 * operation (repstart or election), so that we can alternate
@@ -272,7 +391,8 @@ __repmgr_elect_main(env, th)
 		suppress_election = IS_VIEW_SITE(env) ||
 		    (LF_ISSET(ELECT_F_STARTUP) ?
 		    db_rep->init_policy == DB_REP_CLIENT :
-		    !FLD_ISSET(rep->config, REP_C_ELECTIONS));
+		    !FLD_ISSET(rep->config, REP_C_ELECTIONS)) ||
+		    LF_ISSET(ELECT_F_CLIENT_RESTART);
 		repstart_time = db_rep->repstart_time;
 		target = suppress_election ? repstart_time : failtime;
 		TIMESPEC_ADD_DB_TIMEOUT(&target, rep->election_retry_wait);
@@ -355,7 +475,8 @@ __repmgr_elect_main(env, th)
 			DB_ASSERT(env, action == REPSTART);
 
 			db_rep->new_connection = FALSE;
-			if ((ret = __repmgr_repstart(env, DB_REP_CLIENT)) != 0)
+			if ((ret = __repmgr_repstart(env,
+			    DB_REP_CLIENT, 0)) != 0)
 				goto out;
 			done_repstart = TRUE;
 
@@ -434,19 +555,62 @@ __repmgr_elect(env, flags, failtimep)
 	REP *rep;
 	u_int32_t invitation, nsites, nvotes;
 	int ret, t_ret;
+	u_long peer_conn_waittime, sleeptime;
 
 	db_rep = env->rep_handle;
 	nsites = db_rep->region->config_nsites;
 	DB_ASSERT(env, nsites > 0);
 
+	rep = db_rep->region;
+
 	/*
 	 * With only 2 sites in the group, even a single failure could make it
 	 * impossible to get a majority.  So, fudge a little, unless the user
 	 * really wants strict safety.
 	 */
 	if (nsites == 2 &&
-	    !FLD_ISSET(db_rep->region->config, REP_C_2SITE_STRICT))
+	    !FLD_ISSET(rep->config, REP_C_2SITE_STRICT)) {
 		nvotes = 1;
+
+		/*
+		 * For 2 node scenario, wait for the peer connection
+		 * to complete. If a newly joining node gets to VOTE2,
+		 * before the peer connection completes it would declare
+		 * itself the winner.
+		 */
+		peer_conn_waittime = DB_REP_PEER_WAIT_TIMEOUT;
+		sleeptime = peer_conn_waittime/25;
+
+		REP_SYSTEM_LOCK(env);
+		if (rep->sites_avail == 0) {
+			FLD_SET(rep->elect_flags, REP_E_PEER_CONN_WAIT);
+
+			/* 
+			 * When peer connection is complete, REP_E_PEER_CONN_WAIT
+			 * is cleared in the code that processes version reponse
+			 * as part of handshake (read_version_response()).
+			 */
+ 			while (peer_conn_waittime > 0
+			    && peer_conn_waittime <= DB_REP_PEER_WAIT_TIMEOUT) {
+				if (!FLD_ISSET(rep->elect_flags,
+				    REP_E_PEER_CONN_WAIT) 
+				    || (rep->sites_avail > 0))
+					break;
+
+				REP_SYSTEM_UNLOCK(env);
+
+				/* pauses the thread for sleeptime */
+				__os_yield(env, 0, sleeptime);
+
+				peer_conn_waittime -= sleeptime;
+
+				REP_SYSTEM_LOCK(env);
+			}
+
+			FLD_CLR(rep->elect_flags, REP_E_PEER_CONN_WAIT);
+		}
+		REP_SYSTEM_UNLOCK(env);
+	}
 	else
 		nvotes = ELECTION_MAJORITY(nsites);
 
@@ -459,7 +623,6 @@ __repmgr_elect(env, flags, failtimep)
 		 * so, let's not spoil it by imposing our own full nsites count
 		 * on it.)
 		 */
-		rep = db_rep->region;
 		invitation = rep->nsites;
 		if (invitation == nsites || invitation == nsites - 1) {
 			nsites = invitation;
@@ -534,7 +697,7 @@ __repmgr_claim_victory(env)
 	int ret;
 
 	env->rep_handle->takeover_pending = FALSE;
-	if ((ret = __repmgr_become_master(env)) == DB_REP_UNAVAIL) {
+	if ((ret = __repmgr_become_master(env, 0)) == DB_REP_UNAVAIL) {
 		ret = 0;
 		RPRINT(env, (env, DB_VERB_REPMGR_MISC,
 		    "Won election but lost race with DUPMASTER client intent"));
