@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -105,6 +105,35 @@ __repmgr_set_incoming_queue_max(dbenv, messages, bulk_messages)
 	return (__db_norepmgr(dbenv));
 }
 
+/*
+ * PUBLIC: #ifndef HAVE_REPLICATION_THREADS
+ * PUBLIC: int __repmgr_get_incoming_queue_redzone __P((DB_ENV *,
+ * PUBLIC:     u_int32_t *, u_int32_t *));
+ * PUBLIC: #endif
+ */
+int __repmgr_get_incoming_queue_redzone(dbenv, gbytesp, bytesp)
+	DB_ENV *dbenv;
+	u_int32_t *gbytesp, *bytesp;
+{
+	COMPQUIET(gbytesp, NULL);
+	COMPQUIET(bytesp, NULL);
+	return (__db_norepmgr(dbenv));
+}
+
+/*
+ * PUBLIC: #ifndef HAVE_REPLICATION_THREADS
+ * PUBLIC: int __repmgr_get_incoming_queue_fullevent __P((DB_ENV *,
+ * PUBLIC:     int *));
+ * PUBLIC: #endif
+ */
+int __repmgr_get_incoming_queue_fullevent(dbenv, onoffp)
+	DB_ENV *dbenv;
+	int *onoffp;
+{
+	COMPQUIET(onoffp, NULL);
+	return (__db_norepmgr(dbenv));
+}
+
 /*
  * PUBLIC: #ifndef HAVE_REPLICATION_THREADS
  * PUBLIC: int __repmgr_site
@@ -294,4 +323,36 @@ __repmgr_init_recover(env, dtabp)
 	COMPQUIET(dtabp, NULL);
 	return (0);
 }
+
+/*
+ * PUBLIC: #ifndef HAVE_REPLICATION_THREADS
+ * PUBLIC: int __repmgr_set_socket __P((DB_ENV *,
+ * PUBLIC:     int (*)(DB_ENV *, DB_REPMGR_SOCKET, int *, u_int32_t)));
+ * PUBLIC: #endif
+ */
+int
+__repmgr_set_socket(dbenv, f_approval)
+	DB_ENV *dbenv;
+	int (*f_approval) __P((DB_ENV *, DB_REPMGR_SOCKET, int *, u_int32_t));
+{
+	COMPQUIET(f_approval, NULL);
+	return (__db_norepmgr(dbenv));
+}
+
+/*
+ * PUBLIC: #ifndef HAVE_REPLICATION_THREADS
+ * PUBLIC: int __repmgr_set_ssl_config_pp __P((DB_ENV *, int, char *value));
+ * PUBLIC: #endif
+ */
+int
+__repmgr_set_ssl_config_pp(dbenv, config_field, value)
+	DB_ENV *dbenv;
+	int config_field;
+	char *value;
+{
+	COMPQUIET(dbenv, NULL);
+	COMPQUIET(config_field, 0);
+	COMPQUIET(value, NULL);
+	return (__db_norepmgr(dbenv));
+}
 #endif /* !HAVE_REPLICATION_THREADS */
