@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2005, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2005, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -760,6 +760,13 @@ out:
 	return (ret);
 }
 
+int
+__repmgr_network_event_handler(env)
+	ENV *env;
+{
+	return (__repmgr_select_loop(env));
+}
+
 static int
 prepare_io(env, conn, info_)
 	ENV *env;
@@ -813,9 +820,20 @@ handle_completion(env, conn)
 	ENV *env;
 	REPMGR_CONNECTION *conn;
 {
-	int error, ret;
+	int error, ret, ssl_read_possible, ssl_write_possible;
+	int read_event_allowed, write_event_allowed;
+	int use_ssl_api;
 	WSANETWORKEVENTS events;
 
+	read_event_allowed = 0;
+	write_event_allowed = 0;
+	use_ssl_api = 0;
+
+#if defined(HAVE_REPMGR_SSL)
+	if (IS_REPMGR_SSL_ENABLED(env))
+		use_ssl_api = 1;
+#endif
+
 	if ((ret = WSAEnumNetworkEvents(conn->fd, conn->event_object, &events))
 	    == SOCKET_ERROR) {
 		error = net_errno;
@@ -823,30 +841,78 @@ handle_completion(env, conn)
 		goto report;
 	}
 
-	/* Check both writing and reading. */
-	if (events.lNetworkEvents & FD_CLOSE) {
-		error = events.iErrorCode[FD_CLOSE_BIT];
-		goto report;
+	read_event_allowed = events.lNetworkEvents & (FD_CLOSE | FD_READ);
+	write_event_allowed = events.lNetworkEvents & FD_WRITE;
+
+	if (use_ssl_api) {
+#if defined(HAVE_REPMGR_SSL)
+		ssl_read_possible = __repmgr_ssl_read_possible(conn,
+		    read_event_allowed, write_event_allowed);
+
+		ssl_write_possible = __repmgr_ssl_write_possible(conn,
+		    read_event_allowed, write_event_allowed);
+#endif
+	} else {
+		ssl_read_possible = read_event_allowed;
+		ssl_write_possible = write_event_allowed;
 	}
 
-	if (events.lNetworkEvents & FD_WRITE) {
+	/* Check both writing and reading. */
+	if (ssl_read_possible && (events.lNetworkEvents & FD_CLOSE)) {
+		error = events.iErrorCode[FD_CLOSE_BIT];
+
+		/*
+		 * There could be data for reading when we see FD_CLOSE,
+		 * so we should try reading in this case.
+		 */
+		if (error != 0)
+			goto report;
+		else if ((ret =
+		    __repmgr_read_from_site(env, conn)) != 0)
+			goto err;
+	}
+
+	if (ssl_write_possible && (events.lNetworkEvents & FD_WRITE)) {
 		if (events.iErrorCode[FD_WRITE_BIT] != 0) {
 			error = events.iErrorCode[FD_WRITE_BIT];
 			goto report;
 		} else if ((ret =
-			__repmgr_write_some(env, conn)) != 0)
+		    __repmgr_write_some(env, conn)) != 0)
 			goto err;
 	}
 
-	if (events.lNetworkEvents & FD_READ) {
+	if (use_ssl_api) {
+#if defined(HAVE_REPMGR_SSL)
+		if (ssl_write_possible && !(events.lNetworkEvents & FD_WRITE)) {
+			if ((ret = __repmgr_write_some(env, conn)) != 0)
+				goto err;
+
+			return (ret);
+		}
+#endif
+	}
+
+	if (ssl_read_possible && events.lNetworkEvents & FD_READ) {
 		if (events.iErrorCode[FD_READ_BIT] != 0) {
 			error = events.iErrorCode[FD_READ_BIT];
 			goto report;
 		} else if ((ret =
-			__repmgr_read_from_site(env, conn)) != 0)
+		    __repmgr_read_from_site(env, conn)) != 0)
 			goto err;
 	}
 
+	if (use_ssl_api) {
+#if defined(HAVE_REPMGR_SSL)
+		if (ssl_read_possible && !(events.lNetworkEvents
+		    & (FD_READ | FD_CLOSE))) {
+			if ((ret = __repmgr_read_from_site(env, conn)) != 0)
+				goto err;
+
+			return (ret);
+		}
+#endif
+	}
+
 	if (0) {
 report:
 		__repmgr_fire_conn_err_event(env, conn, error);
