@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  */
 /*
  * Copyright (c) 1995, 1996
@@ -85,7 +85,7 @@ static void __txn_set_txn_lsnp __P((DB_TXN *, DB_LSN **, DB_LSN **));
 
 /*
  * __txn_begin_pp --
- *	ENV->txn_begin pre/post processing.
+ *	DB_ENV->txn_begin pre/post processing.
  *
  * PUBLIC: int __txn_begin_pp __P((DB_ENV *, DB_TXN *, DB_TXN **, u_int32_t));
  */
@@ -99,6 +99,7 @@ __txn_begin_pp(dbenv, parent, txnpp, flags)
 	ENV *env;
 	int rep_check, ret;
 
+	*txnpp = NULL;
 	env = dbenv->env;
 
 	ENV_REQUIRES_CONFIG(env, env->tx_handle, "txn_begin", DB_INIT_TXN);
@@ -128,6 +129,8 @@ __txn_begin_pp(dbenv, parent, txnpp, flags)
 	}
 
 	ENV_ENTER(env, ip);
+	if (parent != NULL)
+		parent->thread_info = ip;
 
 	/* Replication accounts for top-level transactions. */
 	rep_check = IS_ENV_REPLICATED(env) &&
@@ -150,6 +153,38 @@ err:	ENV_LEAVE(env, ip);
 	return (ret);
 }
 
+/*
+ * __txn_allocate --
+ *	Allocate and initalize an empty DB_TXN from the heap.
+ *
+ *
+ * PUBLIC: int __txn_allocate __P((ENV *, DB_TXN **));
+ */
+ int
+ __txn_allocate(env, txnpp)
+	ENV *env;
+	DB_TXN **txnpp;
+{
+	DB_TXN *txn;
+	int ret;
+
+	if ((ret = __os_calloc(env, 1, sizeof(DB_TXN), &txn)) != 0) {
+		__db_errx(env, TxnAlloc);
+		return (ret);
+	}
+
+	txn->mgrp = env->tx_handle;
+	TAILQ_INIT(&txn->kids);
+	TAILQ_INIT(&txn->events);
+	STAILQ_INIT(&txn->logs);
+	TAILQ_INIT(&txn->my_cursors);
+	TAILQ_INIT(&txn->femfs);
+	txn->flags = TXN_MALLOC;
+
+	*txnpp = txn;
+	return (0);
+}
+
 /*
  * __txn_begin --
  *	ENV->txn_begin.
@@ -172,31 +207,21 @@ __txn_begin(env, ip, parent, txnpp, flags)
 	u_int32_t flags;
 {
 	DB_ENV *dbenv;
-	DB_LOCKREGION *region;
 	DB_TXN *txn;
 	TXN_DETAIL *ptd, *td;
 	int ret;
 
+	*txnpp = NULL;
+	dbenv = env->dbenv;
 	if (F_ISSET(env, ENV_FORCE_TXN_BULK))
 		flags |= DB_TXN_BULK;
 
-	*txnpp = NULL;
-	if ((ret = __os_calloc(env, 1, sizeof(DB_TXN), &txn)) != 0) {
-		__db_errx(env, TxnAlloc);
+	if ((ret = __txn_allocate(env, &txn)) != 0)
 		return (ret);
-	}
-
-	dbenv = env->dbenv;
-	txn->mgrp = env->tx_handle;
 	txn->parent = parent;
+	txn->begin_flags = flags;
 	if (parent != NULL && F_ISSET(parent, TXN_FAMILY))
 		parent = NULL;
-	TAILQ_INIT(&txn->kids);
-	TAILQ_INIT(&txn->events);
-	STAILQ_INIT(&txn->logs);
-	TAILQ_INIT(&txn->my_cursors);
-	TAILQ_INIT(&txn->femfs);
-	txn->flags = TXN_MALLOC;
 	txn->thread_info =
 	     ip != NULL ? ip : (parent != NULL ? parent->thread_info : NULL);
 
@@ -226,16 +251,11 @@ __txn_begin(env, ip, parent, txnpp, flags)
 		F_SET(txn, TXN_READ_UNCOMMITTED);
 	if (LF_ISSET(DB_TXN_FAMILY))
 		F_SET(txn, TXN_FAMILY | TXN_INFAMILY | TXN_READONLY);
+	if (LF_ISSET(DB_TXN_DISPATCH))
+		F_SET(txn, TXN_DISPATCH);
 	if (LF_ISSET(DB_TXN_SNAPSHOT) || F_ISSET(dbenv, DB_ENV_TXN_SNAPSHOT) ||
-	    (parent != NULL && F_ISSET(parent, TXN_SNAPSHOT))) {
-		if (IS_REP_CLIENT(env)) {
-			__db_errx(env, DB_STR("4572",
-		"DB_TXN_SNAPSHOT may not be used on a replication client"));
-			ret = (EINVAL);
-			goto err;
-		} else
-			F_SET(txn, TXN_SNAPSHOT);
-	}
+	    (parent != NULL && F_ISSET(parent, TXN_SNAPSHOT)))
+		F_SET(txn, TXN_SNAPSHOT);
 	if (LF_ISSET(DB_IGNORE_LEASE))
 		F_SET(txn, TXN_IGNORE_LEASE);
 
@@ -251,43 +271,23 @@ __txn_begin(env, ip, parent, txnpp, flags)
 
 	if ((ret = __txn_begin_int(txn)) != 0)
 		goto err;
-	td = txn->td;
 
 	if (parent != NULL) {
+		td = txn->td;
 		ptd = parent->td;
 		TAILQ_INSERT_HEAD(&parent->kids, txn, klinks);
 		SH_TAILQ_INSERT_HEAD(&ptd->kids, td, klinks, __txn_detail);
 	}
 
-	if (LOCKING_ON(env)) {
-		region = env->lk_handle->reginfo.primary;
-		if (parent != NULL) {
-			ret = __lock_inherit_timeout(env,
-			    parent->locker, txn->locker);
-			/* No parent locker set yet. */
-			if (ret == EINVAL) {
-				parent = NULL;
-				ret = 0;
-			}
-			if (ret != 0)
-				goto err;
-		}
-
-		/*
-		 * Parent is NULL if we have no parent
-		 * or it has no timeouts set.
-		 */
-		if (parent == NULL && region->tx_timeout != 0)
-			if ((ret = __lock_set_timeout(env, txn->locker,
-			    region->tx_timeout, DB_SET_TXN_TIMEOUT)) != 0)
-				goto err;
-	}
+	if (LOCKING_ON(env) && (ret = __txn_init_timeout(txn, parent)) != 0)
+		goto err;
 
 	*txnpp = txn;
 	PERFMON2(env, txn, begin, txn->txnid, flags);
 	return (0);
 
 err:
+	/* XXX This seems to leak the memory for 'txn->td'. XXX */
 	__os_free(env, txn);
 	return (ret);
 }
@@ -370,10 +370,11 @@ __txn_begin_int(txn)
 	inserted = 0;
 
 	TXN_SYSTEM_LOCK(env);
-	if (!F_ISSET(txn, TXN_COMPENSATE) && F_ISSET(region, TXN_IN_RECOVERY)) {
+	DB_TEST_CRASH(env->test_abort, DB_TEST_EXC_MUTEX);
+	if (!F_ISSET(txn, TXN_DISPATCH) && F_ISSET(region, TXN_IN_RECOVERY)) {
+		ret = USR_ERR(env, EINVAL);
 		__db_errx(env, DB_STR("4524",
 		    "operation not permitted during recovery"));
-		ret = EINVAL;
 		goto err;
 	}
 
@@ -401,6 +402,7 @@ __txn_begin_int(txn)
 	if (region->stat.st_nactive > region->stat.st_maxnactive)
 		STAT_SET(env, txn, maxnactive,
 		    region->stat.st_maxnactive, region->stat.st_nactive, id);
+	td->slice_details = INVALID_ROFF;
 #endif
 
 	td->txnid = id;
@@ -449,7 +451,7 @@ __txn_begin_int(txn)
 
 	/* Allocate a locker for this txn. */
 	if (LOCKING_ON(env) && (ret =
-		__lock_getlocker(env->lk_handle, id, 1, &txn->locker)) != 0)
+	    __lock_getlocker(env->lk_handle, id, 1, &txn->locker)) != 0)
 			goto err;
 
 	txn->abort = __txn_abort_pp;
@@ -549,8 +551,6 @@ __txn_continue(env, txn, td, ip, add_to_list)
 	txn->name = NULL;
 	txn->td = td;
 	td->xa_ref++;
-
-	/* This never seems to be used: txn->expire */
 	txn->txn_list = NULL;
 
 	TAILQ_INIT(&txn->kids);
@@ -630,12 +630,16 @@ __txn_commit_pp(txn, flags)
 	ENV *env;
 	int rep_check, ret, t_ret;
 
+	ret = 0;
 	env = txn->mgrp->env;
 	rep_check = IS_ENV_REPLICATED(env) &&
 	    txn->parent == NULL && IS_REAL_TXN(txn);
 
 	ENV_ENTER(env, ip);
-	ret = __txn_commit(txn, flags);
+
+ 	txn->thread_info = ip;
+	if ((t_ret = __txn_commit(txn, flags)) != 0 && ret == 0)
+		ret = t_ret;
 	if (rep_check && (t_ret = __op_rep_exit(env)) != 0 && ret == 0)
 		ret = t_ret;
 	ENV_LEAVE(env, ip);
@@ -663,13 +667,44 @@ __txn_commit(txn, flags)
 	DB_LSN token_lsn;
 	u_int32_t id;
 	int ret, t_ret;
+#ifdef HAVE_SLICES
+	DB_THREAD_INFO *ip;
+	DB_TXN *sl_txn;
+	db_slice_t i;
+#endif
 
 	env = txn->mgrp->env;
 	td = txn->td;
+	ret = 0;
 	PERFMON2(env, txn, commit, txn->txnid, flags);
 
 	DB_ASSERT(env, txn->xa_thr_status == TXN_XA_THREAD_NOTA ||
 	    td->xa_ref == 1);
+
+#ifdef HAVE_SLICES
+	if (txn->txn_slices != NULL) {
+
+		/*
+		 * Commit any slices' transactions before commiting in the
+		 * container. Remember errors, prioritizing panics above others.
+		 */
+		for (i = 0; i != env->dbenv->slice_cnt; i++) {
+			if ((sl_txn = txn->txn_slices[i]) == NULL)
+				continue;
+			DB_ASSERT(env, sl_txn->txn_container == txn);
+			txn->txn_slices[i] = NULL;
+			sl_txn->txn_container = NULL;
+			ENV_ENTER(env->slice_envs[i]->env, ip);
+			if ((t_ret = __txn_commit(sl_txn, flags)) != 0 &&
+			    ret == 0 && ret != DB_RUNRECOVERY)
+				ret = t_ret;
+			ENV_LEAVE(env->slice_envs[i]->env, ip);
+		}
+		__os_free(env, txn->txn_slices);
+		txn->txn_slices = NULL;
+	}
+#endif
+
 	/*
 	 * A common mistake in Berkeley DB programs is to mis-handle deadlock
 	 * return.  If the transaction deadlocked, they want abort, not commit.
@@ -801,8 +836,9 @@ __txn_commit(txn, flags)
 				if (ret == 0) {
 					DB_LSN s_lsn;
 
-					DB_ASSERT(env, __log_current_lsn_int(
-					    env, &s_lsn, NULL, NULL) == 0);
+					if ((ret = __log_current_lsn_int(
+					    env, &s_lsn, NULL, NULL)) != 0)
+						goto err;
 					DB_ASSERT(env, LOG_COMPARE(
 					    &td->visible_lsn, &s_lsn) <= 0);
 					COMPQUIET(s_lsn.file, 0);
@@ -890,23 +926,22 @@ err:	/*
 /*
  * __txn_close_cursors
  *	Close a transaction's registered cursors, all its cursors are
- *	guaranteed to be closed.
+ *	guaranteed to be closed, even when an error occurs.
  */
 static int
 __txn_close_cursors(txn)
 	DB_TXN *txn;
 {
-	int ret, tret;
+	int ret, t_ret;
 	DBC *dbc;
 
-	ret = tret = 0;
+	ret = t_ret = 0;
 	dbc = NULL;
 
 	if (txn == NULL)
 		return (0);
 
 	while ((dbc = TAILQ_FIRST(&txn->my_cursors)) != NULL) {
-
 		DB_ASSERT(dbc->env, txn == dbc->txn);
 
 		/*
@@ -919,21 +954,30 @@ __txn_close_cursors(txn)
 
 		/* Removed from the active queue here. */
 		if (F_ISSET(dbc, DBC_ACTIVE))
-			ret = __dbc_close(dbc);
+			t_ret = __dbc_close(dbc);
 
 		dbc->txn = NULL;
 
 		/* We have to close all cursors anyway, so continue on error. */
-		if (ret != 0) {
-			__db_err(dbc->env, ret, "__dbc_close");
-			if (tret == 0)
-				tret = ret;
+		if (t_ret != 0) {
+#if !defined(DIAGNOSTIC)
+			/*
+			 * In order to prevent overly alarming messages from
+			 * appearing in the application's error stream, we
+			 * suppress deadlock messages except when in a
+			 * diagnostic build.
+			 */
+			if (t_ret != DB_LOCK_DEADLOCK)
+#endif
+				__db_err(dbc->env, t_ret, "__dbc_close");
+			if (ret == 0)
+				ret = t_ret;
 		}
 	}
 	txn->my_cursors.tqh_first = NULL;
 	txn->my_cursors.tqh_last = NULL;
 
-	return (tret);/* Return the first error if any. */
+	return (ret);	/* Return the first error, if any. */
 }
 
 /*
@@ -1023,6 +1067,7 @@ __txn_abort_pp(txn)
 	    txn->parent == NULL && IS_REAL_TXN(txn);
 
 	ENV_ENTER(env, ip);
+ 	txn->thread_info = ip;
 	ret = __txn_abort(txn);
 	if (rep_check && (t_ret = __op_rep_exit(env)) != 0 && ret == 0)
 		ret = t_ret;
@@ -1047,7 +1092,7 @@ __txn_abort(txn)
 	REGINFO *infop;
 	TXN_DETAIL *td;
 	u_int32_t id;
-	int ret;
+	int deadlocked, ret;
 
 	env = txn->mgrp->env;
 	td = txn->td;
@@ -1056,7 +1101,7 @@ __txn_abort(txn)
 	 * it, however make sure that it is aborted when the last process
 	 * tries to abort it.
 	 */
-	if (txn->xa_thr_status != TXN_XA_THREAD_NOTA &&  td->xa_ref > 1) {
+	if (txn->xa_thr_status != TXN_XA_THREAD_NOTA && td->xa_ref > 1) {
 		td->status = TXN_NEED_ABORT;
 		return (0);
 	}
@@ -1064,10 +1109,13 @@ __txn_abort(txn)
 	PERFMON1(env, txn, abort, txn->txnid);
 	/*
 	 * Close registered cursors before the abort. Even if the call fails,
-	 * all cursors are closed.
+	 * all cursors are closed. We continue with the rest of the abort if
+	 * the close failed due to deadlock. Then, at the end of the function
+	 * we return the deadlock error, if the abort was successful otherwise.
 	 */
-	if ((ret = __txn_close_cursors(txn)) != 0)
-		return (__env_panic(env, ret));
+	if ((deadlocked = __txn_close_cursors(txn)) != 0 &&
+	    deadlocked != DB_LOCK_DEADLOCK)
+		return (__env_panic(env, deadlocked));
 
 	/* Ensure that abort always fails fatally. */
 	if ((ret = __txn_isvalid(txn, TXN_OP_ABORT)) != 0)
@@ -1079,16 +1127,43 @@ __txn_abort(txn)
 	 */
 	__txn_reset_fe_watermarks(txn);
 
+#ifdef HAVE_SLICES
+	if (txn->txn_slices != NULL) {
+		DB_TXN *sl_txn;
+		db_slice_t i;
+		int t_ret;
+
+		for (i = 0; i != env->dbenv->slice_cnt; i++) {
+			sl_txn = txn->txn_slices[i];
+			if (sl_txn == NULL)
+				continue;
+			txn->txn_slices[i] = NULL;
+			sl_txn->txn_container = NULL;
+			if ((t_ret = __txn_abort_pp(sl_txn)) != 0) {
+				__db_err(env, ret,
+				    "slice txn #%d failed abort!", i);
+				if (ret == 0)
+					ret = t_ret;
+			}
+		}
+		__os_free(env, txn->txn_slices);
+		txn->txn_slices = NULL;
+	} else
+		DB_ASSERT(env, txn->txn_container == NULL);
+#endif
 	/*
 	 * Try to abort any unresolved children.
 	 *
-	 * Abort either succeeds or panics the region.  As soon as we
-	 * see any failure, we just get out of here and return the panic
-	 * up.
+	 * Abort either succeeds, deadlocks, or panics the region.  As soon as
+	 * we see any non-deadlock failure (usually a panic) we return it.
 	 */
 	while ((kid = TAILQ_FIRST(&txn->kids)) != NULL)
-		if ((ret = __txn_abort(kid)) != 0)
-			return (ret);
+		if ((ret = __txn_abort(kid)) != 0) {
+			if (ret == DB_LOCK_DEADLOCK)
+				deadlocked = ret;
+			else
+				return (ret);
+		}
 
 	infop = env->reginfo;
 	renv = infop->primary;
@@ -1155,8 +1230,13 @@ done:	 if (DBENV_LOGGING(env) && td->status == TXN_PREPARED &&
 	    LOG_FLAGS(txn), TXN_ABORT, (int32_t)time(NULL), id, NULL)) != 0)
 		return (__env_panic(env, ret));
 
-	/* __txn_end always panics if it errors, so pass the return along. */
-	return (__txn_end(txn, 0));
+	if ((ret = __txn_end(txn, 0)) != 0)
+		return (ret);
+	/*
+	 * Everything has succeeded, except possibly that a deadlock was
+	 * remembered and deferred until now. Return success, or that error.
+	 */
+	return (deadlocked);
 }
 
 /*
@@ -1177,6 +1257,7 @@ __txn_discard(txn, flags)
 	    txn->parent == NULL && IS_REAL_TXN(txn);
 
 	ENV_ENTER(env, ip);
+ 	txn->thread_info = ip;
 	ret = __txn_discard_int(txn, flags);
 	if (rep_check && (t_ret = __op_rep_exit(env)) != 0 && ret == 0)
 		ret = t_ret;
@@ -1249,6 +1330,12 @@ __txn_prepare(txn, gid)
 	int ret;
 
 	env = txn->mgrp->env;
+	if (SLICES_ON(env)) {
+		ret = USR_ERR(env, EINVAL);
+		__db_errx(env, DB_STR("4577",
+		"Sliced environments do not support distributed transactions"));
+		return (ret);
+	}
 	td = txn->td;
 	PERFMON2(env, txn, prepare, txn->txnid, gid);
 	DB_ASSERT(env, txn->xa_thr_status == TXN_XA_THREAD_NOTA ||
@@ -1362,6 +1449,13 @@ __txn_set_name(txn, name)
 
 	mgr = txn->mgrp;
 	env = mgr->env;
+
+	if (name == NULL || strlen(name) == 0) {
+		__db_errx(env, DB_STR("4574",
+		    "DB_TXN->set_name: name cannot be empty."));
+		return (EINVAL);
+	}
+
 	td = txn->td;
 	len = strlen(name) + 1;
 
@@ -1477,21 +1571,11 @@ __txn_isvalid(txn, op)
 	txnop_t op;
 {
 	DB_TXNMGR *mgr;
-	DB_TXNREGION *region;
 	ENV *env;
 	TXN_DETAIL *td;
 
 	mgr = txn->mgrp;
 	env = mgr->env;
-	region = mgr->reginfo.primary;
-
-	/* Check for recovery. */
-	if (!F_ISSET(txn, TXN_COMPENSATE) &&
-	    F_ISSET(region, TXN_IN_RECOVERY)) {
-		__db_errx(env, DB_STR("4530",
-		    "operation not permitted during recovery"));
-		goto err;
-	}
 
 	/* Check for live cursors. */
 	if (txn->cursors != 0) {
@@ -1580,7 +1664,7 @@ err:	/*
 	 * handles are dead by definition when we return, and if you use
 	 * a cursor you forgot to close, we have no idea what will happen.
 	 */
-	return (__env_panic(env, EINVAL));
+	return (__env_panic(env, USR_ERR(env, EINVAL)));
 }
 
 /*
@@ -1606,6 +1690,13 @@ __txn_end(txn, is_commit)
 	region = mgr->reginfo.primary;
 	do_closefiles = 0;
 
+#ifdef HAVE_SLICES
+	if (txn->txn_container != NULL) {
+		DB_ASSERT(env, txn->txn_container->txn_slices[0] == txn);
+		txn->txn_container->txn_slices[0] = NULL;
+		txn->txn_container = NULL;
+	}
+#endif
 	/* Process commit events. */
 	if ((ret = __txn_doevents(env,
 	    txn, is_commit ? TXN_COMMIT : TXN_ABORT, 0)) != 0)
@@ -1671,9 +1762,18 @@ __txn_end(txn, is_commit)
 		    R_ADDR(&mgr->reginfo, td->name));
 		td->name = INVALID_ROFF;
 	}
-	if (td->nlog_slots != TXN_NSLOTS)
+	if (td->nlog_slots != TXN_NSLOTS) {
 		__env_alloc_free(&mgr->reginfo,
 		    R_ADDR(&mgr->reginfo, td->log_dbs));
+		td->log_dbs = INVALID_ROFF;
+	}
+#ifdef HAVE_STATISTICS
+	if (td->slice_details != INVALID_ROFF) {
+		__env_alloc_free(&mgr->reginfo,
+		    R_ADDR(&mgr->reginfo, td->slice_details));
+		td->slice_details = INVALID_ROFF;
+	}
+#endif
 
 	if (txn->parent != NULL) {
 		ptd = txn->parent->td;
@@ -1695,10 +1795,9 @@ __txn_end(txn, is_commit)
 			    nsnapshot, region->stat.st_nsnapshot, txn->txnid);
 			if (region->stat.st_nsnapshot >
 			    region->stat.st_maxnsnapshot)
-				    STAT_SET(env, txn, maxnsnapshot,
-					region->stat.st_maxnsnapshot,
-					region->stat.st_nsnapshot,
-					txn->txnid);
+				STAT_SET(env, txn, maxnsnapshot,
+				    region->stat.st_maxnsnapshot,
+				    region->stat.st_nsnapshot, txn->txnid);
 #endif
 			td = NULL;
 		}
@@ -1845,6 +1944,8 @@ __txn_undo(txn)
 	 */
 	for (ptxn = txn->parent; ptxn != NULL && ptxn->parent != NULL;)
 		ptxn = ptxn->parent;
+	if (ptxn != NULL)
+		txn->thread_info = ptxn->thread_info;
 
 	if (ptxn != NULL && ptxn->txn_list != NULL)
 		txnlist = ptxn->txn_list;
@@ -1922,13 +2023,12 @@ __txn_activekids(env, rectype, txn)
 	 * On a child commit, we know that there are children (i.e., the
 	 * committing child at the least.  In that case, skip this check.
 	 */
-	if (F_ISSET(txn, TXN_COMPENSATE) || rectype == DB___txn_child)
+	if (F_ISSET(txn, TXN_DISPATCH) || rectype == DB___txn_child)
 		return (0);
 
 	if (TAILQ_FIRST(&txn->kids) != NULL) {
-		__db_errx(env, DB_STR("4538",
-		    "Child transaction is active"));
-		return (EPERM);
+		__db_errx(env, DB_STR("4538", "Child transaction is active"));
+		return (USR_ERR(env, EPERM));
 	}
 	return (0);
 }
@@ -2171,5 +2271,139 @@ __txn_applied(env, ip, commit_info, timeout)
 	if (renv->envid == commit_info->envid &&
 	    LOG_COMPARE(&commit_info->lsn, &lsn) <= 0)
 		return (0);
-	return (DB_NOTFOUND);
+	return (USR_ERR(env, DB_NOTFOUND));
 }
+
+/*
+ * __txn_init_timeout --
+ *	See whether this new transaction should have a lock timeout period,
+ *	either inherited from its parent or __lock_set_env_timeout().
+ *
+ * PUBLIC: int __txn_init_timeout __P((DB_TXN *, DB_TXN *));
+ */
+int
+__txn_init_timeout(txn, parent)
+	DB_TXN *txn, *parent;
+{
+	DB_LOCKREGION *lock_region;
+	ENV *env;
+	int ret;
+
+	env = txn->mgrp->env;
+	lock_region = env->lk_handle->reginfo.primary;
+	ret = 0;
+	if (parent != NULL) {
+		ret = __lock_inherit_timeout(env, parent->locker, txn->locker);
+		/*
+		 * An error code except EINVAL is a real error here, so return
+		 * it. EINVAL means that no parent locker has been set yet; fall
+		 * through to check whether the lock region has a timeout.
+		 */
+		if (ret == 0 || ret != EINVAL)
+		    return (ret);
+	}
+
+	if (lock_region->tx_timeout != 0)
+		ret = __lock_set_timeout(env,
+		    txn->locker, lock_region->tx_timeout, DB_SET_TXN_TIMEOUT);
+
+	return (ret);
+}
+
+#ifdef HAVE_SLICES
+
+/*
+ * __txn_slice_begin --
+ *	Begin a transaction for a slice of a container if it does not already
+ *	have one, copying the relevent flags and other attributes from the
+ *	container's transaction.
+ *
+ *	It is okay if the container txn already has a txn in this slice.
+ *
+ *	The container's environment should have been enterered by the caller.
+ *	This enters and leaves the slice's environment if and only if it needs
+ *	to begin a transaction.
+ *
+ * PUBLIC: int __txn_slice_begin __P((DB_TXN *, DB_TXN **, db_slice_t));
+ */
+int
+__txn_slice_begin(txn, sl_txnpp, slice_index)
+	DB_TXN *txn;
+	DB_TXN **sl_txnpp;
+	db_slice_t slice_index;
+{
+	ENV *env;
+	TXN_DETAIL *td;
+	DB_TXNMGR *mgr;
+	int ret;
+#ifdef HAVE_STATISTICS
+	DB_TXN_ACTIVE_SLICE *slice_details;
+	size_t len;
+#endif
+
+	ret = 0;
+	*sl_txnpp = NULL;
+	mgr = txn->mgrp;
+	env = mgr->env;
+	td = txn->td;
+
+	CHECK_THREAD(env);
+	if (txn->txn_slices == NULL) {
+		if ((ret = __os_calloc(env, env->dbenv->slice_cnt,
+		    sizeof(DB_TXN *), &txn->txn_slices)) != 0)
+			return (ret);
+
+#ifdef HAVE_STATISTICS
+		len = (size_t)
+		    env->dbenv->slice_cnt * sizeof(DB_TXN_ACTIVE_SLICE);
+		TXN_SYSTEM_LOCK(env);
+		if ((ret = __env_alloc(&mgr->reginfo,
+		    len, &slice_details)) != 0) {
+			return (ret);
+		}
+		memset(slice_details, 0, len);
+		td->slice_details = R_OFFSET(&mgr->reginfo, slice_details);
+
+		TXN_SYSTEM_UNLOCK(env);
+#endif
+	}
+#ifdef HAVE_STATISTICS
+	else
+		slice_details = R_ADDR(&mgr->reginfo, td->slice_details);
+#endif
+
+	if ((*sl_txnpp = txn->txn_slices[slice_index]) == NULL) {
+		if ((ret = __txn_begin_pp(env->slice_envs[slice_index],
+		    NULL, sl_txnpp, txn->begin_flags)) != 0)
+			return (ret);
+		(*sl_txnpp)->txn_container = txn;
+		txn->txn_slices[slice_index] = *sl_txnpp;
+#ifdef HAVE_STATISTICS
+		slice_details[slice_index].txnid = (*sl_txnpp)->txnid;
+#endif
+	}
+
+	return (0);
+}
+
+/*
+ * __txn_multislice --
+ *	Return the error that this container's txn already has accessed a slice.
+ *
+ * PUBLIC: int __txn_multislice __P((DB_TXN *));
+ */
+ int
+ __txn_multislice(txn)
+	 DB_TXN *txn;
+{
+	ENV *env;
+	int ret;
+
+	env = txn->mgrp->env;
+	ret = USR_ERR(env, EINVAL);
+	__db_errx(env,
+	    DB_STR("4576", "A transaction tried to access a second slice"));
+	return (ret);
+}
+
+#endif
