@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 1996, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 1996, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  * $Id$
  */
@@ -58,12 +58,30 @@ __txn_open(env)
 	env->tx_handle = mgr;
 	return (0);
 
-err:	env->tx_handle = NULL;
-	if (mgr->reginfo.addr != NULL)
-		(void)__env_region_detach(env, &mgr->reginfo, 0);
+err:	(void)__mutex_free(env, &mgr->mutex);
+	(void)__txn_region_detach(env, mgr);
 
-	(void)__mutex_free(env, &mgr->mutex);
-	__os_free(env, mgr);
+	return (ret);
+}
+
+/*
+ * __txn_region_detach --
+ *
+ * PUBLIC: int __txn_region_detach __P((ENV *, DB_TXNMGR *));
+ */
+int
+__txn_region_detach(env, mgr)
+	ENV *env;
+	DB_TXNMGR *mgr;
+{
+	int ret;
+
+	ret = 0;
+	if (mgr != NULL) {
+		ret = __env_region_detach(env, &mgr->reginfo, 0);
+		__os_free(env, mgr);
+		env->tx_handle = NULL;
+	}
 	return (ret);
 }
 
@@ -259,10 +277,10 @@ __txn_env_refresh(env)
 			}
 		}
 		if (aborted) {
+			if (ret == 0)
+				ret = USR_ERR(env, EINVAL);
 			__db_errx(env, DB_STR("4511",
 	"Error: closing the transaction region with active transactions"));
-			if (ret == 0)
-				ret = EINVAL;
 		}
 	}
 
@@ -322,7 +340,7 @@ __txn_region_mutex_max(env)
 
 /*
  * __txn_region_size --
- *	 Return the amount of space needed for the txn region.
+ *	 Return the initial amount of space needed for the txn region.
  * PUBLIC:  size_t __txn_region_size __P((ENV *));
  */
 size_t
@@ -348,7 +366,9 @@ __txn_region_size(env)
 
 /*
  * __txn_region_max --
- *	 Return the additional amount of space needed for the txn region.
+ *	Return how much additional memory to reserve, so that all the configured
+ *	transaction-specific data structures can be allocated.
+ *
  * PUBLIC:  size_t __txn_region_max __P((ENV *));
  */
 size_t
@@ -395,16 +415,16 @@ __txn_id_set(env, cur_txnid, max_txnid)
 
 	ret = 0;
 	if (cur_txnid < TXN_MINIMUM) {
+		ret = USR_ERR(env, EINVAL);
 		__db_errx(env, DB_STR_A("4512",
 		    "Current ID value %lu below minimum", "%lu"),
 		    (u_long)cur_txnid);
-		ret = EINVAL;
 	}
 	if (max_txnid < TXN_MINIMUM) {
+		ret = USR_ERR(env, EINVAL);
 		__db_errx(env, DB_STR_A("4513",
 		    "Maximum ID value %lu below minimum", "%lu"),
 		    (u_long)max_txnid);
-		ret = EINVAL;
 	}
 	return (ret);
 }
