@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2009, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2009, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  */
 using System;
@@ -29,40 +29,47 @@ namespace CsharpAPITest
 		[Test]
 		public void TestBlob()
 		{
-			// Test opening the blob database without environment.
+			testName = "TestBlob";
+			SetUpTest(false);
+			// Test opening the external file database without
+			// environment.
 			TestBlobBtreeDatabase(0, null, 6, null, false);
 
 			/*
-			 * Test opening the blob database without environment
-			 * but specifying blob directory.
+			 * Test opening the external file database without
+			 * environment
+			 * but specifying external file directory.
 			 */
 			TestBlobBtreeDatabase(0, null, 6,
 			    testHome + "/DBBLOB", true);
 
-			// Test opening the blob database with environment.
+			// Test opening the external file database with
+			// environment.
 			TestBlobBtreeDatabase(3, "ENVBLOB", 6, null, false);
 
 			/*
-			 * Test opening the blob database with environment
-			 * and specifying blob directory.
+			 * Test opening the external file database with
+			 * environment and specifying external file directory.
 			 */
 			TestBlobBtreeDatabase(3, null, 6, "/DBBLOB", true);
 		}
 
 		/*
-		 * Test the blob database with or without environment.
+		 * Test the external file database with or without environment.
 		 * 1. Config and open the environment;
-		 * 2. Verify the environment blob configs;
+		 * 2. Verify the environment external file configs;
 		 * 3. Config and open the database;
-		 * 4. Verify the database blob configs;
-		 * 5. Insert and verify some blob data by database methods;
-		 * 6. Insert some blob data by cursor, update it and verify
+		 * 4. Verify the database external file configs;
+		 * 5. Insert and verify some external file data by database
+		 * methods;
+		 * 6. Insert some external file data by cursor, update it and
+		 * verify
 		 * the update by database stream and cursor;
 		 * 7. Verify the stats;
 		 * 8. Close all handles.
 		 * If "blobdbt" is true, set the data DatabaseEntry.Blob as
-		 * true, otherwise make the data DatabaseEntry reach the blob
-		 * threshold in size.
+		 * true, otherwise make the data DatabaseEntry reach the
+		 * external file threshold in size.
 		 */
 		void TestBlobBtreeDatabase(uint env_threshold,
 		    string env_blobdir, uint db_threshold,
@@ -71,16 +78,16 @@ namespace CsharpAPITest
 			if (env_threshold == 0 && db_threshold == 0)
 				return;
 
-			testName = "TestBlob";
-			SetUpTest(true);
 			string btreeDBName =
 			    testHome + "/" + testName + ".db";
 
+			Configuration.ClearDir(testHome);
 			BTreeDatabaseConfig cfg = new BTreeDatabaseConfig();
 			cfg.Creation = CreatePolicy.ALWAYS;
 			string blrootdir = "__db_bl";
 
-			// Open the environment and verify the blob configs.
+			// Open the environment and verify the external file
+			// configs.
 			if (env_threshold > 0)
 			{
 				DatabaseEnvironmentConfig envConfig =
@@ -91,36 +98,37 @@ namespace CsharpAPITest
 				envConfig.UseLogging = true;
 				envConfig.UseTxns = true;
 				envConfig.UseLocking = true;
-				envConfig.BlobThreshold = env_threshold;
+				envConfig.ExternalFileThreshold = env_threshold;
 				if (env_blobdir != null)
 				{
-					envConfig.BlobDir = env_blobdir;
+					envConfig.ExternalFileDir = env_blobdir;
 					blrootdir = env_blobdir;
 				}
 				DatabaseEnvironment env =
 				    DatabaseEnvironment.Open(
 				    testHome, envConfig);
 				if (env_blobdir == null)
-					Assert.IsNull(env.BlobDir);
+					Assert.IsNull(env.ExternalFileDir);
 				else
-					Assert.AreEqual(0, env.BlobDir.
+					Assert.AreEqual(0, env.ExternalFileDir.
 					    CompareTo(env_blobdir));
 				Assert.AreEqual(env_threshold,
-				    env.BlobThreshold);
+				    env.ExternalFileThreshold);
 				cfg.Env = env;
 				btreeDBName = testName + ".db";
 			}
 
-			// Open the database and verify the blob configs.
+			// Open the database and verify the external file
+			// configs.
 			if (db_threshold > 0)
-				cfg.BlobThreshold = db_threshold;
+				cfg.ExternalFileThreshold = db_threshold;
 			if (db_blobdir != null)
 			{
-				cfg.BlobDir = db_blobdir;
+				cfg.ExternalFileDir = db_blobdir;
 				/*
-				 * The blob directory setting in the database
- 				 * is effective only when it is opened without
-				 * an environment.
+				 * The external file directory setting in the
+ 				 * database is effective only when it is opened
+				 * without an environment.
 				 */
 				if (cfg.Env == null)
 					blrootdir = db_blobdir;
@@ -130,14 +138,15 @@ namespace CsharpAPITest
 			    BTreeDatabase.Open(btreeDBName, cfg);
 			Assert.AreEqual(
 			    db_threshold > 0 ? db_threshold : env_threshold,
-			    db.BlobThreshold);
+			    db.ExternalFileThreshold);
 			if (db_blobdir == null && cfg.Env == null)
-				Assert.IsNull(db.BlobDir);
+				Assert.IsNull(db.ExternalFileDir);
 			else
 				Assert.AreEqual(0,
-				    db.BlobDir.CompareTo(blrootdir));
+				    db.ExternalFileDir.CompareTo(blrootdir));
 
-			// Insert and verify some blob data by database methods.
+			// Insert and verify some external file data by
+			// database methods.
 			string[] records = {"a", "b", "c", "d", "e", "f", "g",
 			    "h", "i", "j", "k", "l", "m", "n", "o", "p", "q",
 			    "r", "s", "t", "u", "v", "w", "x", "y", "z"};
@@ -146,8 +155,8 @@ namespace CsharpAPITest
 			byte[] kdata, ddata;
 			string str;
 			KeyValuePair<DatabaseEntry, DatabaseEntry> pair;
-			ddbt.Blob = blobdbt;
-			Assert.AreEqual(blobdbt, ddbt.Blob);
+			ddbt.ExternalFile = blobdbt;
+			Assert.AreEqual(blobdbt, ddbt.ExternalFile);
 			for (int i = 0; i < records.Length; i++)
 			{
 				kdata = BitConverter.GetBytes(i);
@@ -175,15 +184,15 @@ namespace CsharpAPITest
 			}
 
 			/*
-			 * Insert some blob data by cursor, update it and verify
-			 * the update by database stream.
+			 * Insert some external file data by cursor, update it
+			 * and verify the update by database stream.
 			 */
 			kdata = BitConverter.GetBytes(records.Length);
 			ddata = Encoding.ASCII.GetBytes("abc");
 			kdbt.Data = kdata;
 			ddbt.Data = ddata;
-			ddbt.Blob = true;
-			Assert.IsTrue(ddbt.Blob);
+			ddbt.ExternalFile = true;
+			Assert.IsTrue(ddbt.ExternalFile);
 			pair = new KeyValuePair<
 			    DatabaseEntry, DatabaseEntry>(kdbt, ddbt);
 			CursorConfig dbcConfig = new CursorConfig();
@@ -239,7 +248,7 @@ namespace CsharpAPITest
 				txn.Commit();
 
 			/*
-			 * Verify the blob files are created
+			 * Verify the external file files are created
 			 * in the expected location.
 			 * This part of test code is disabled since
 			 * BTreeDatabase.BlobSubDir is not exposed to users.
@@ -255,7 +264,7 @@ namespace CsharpAPITest
 
 			// Verify the stats.
 			BTreeStats st = db.Stats();
-			Assert.AreEqual(records.Length + 1, st.nBlobRecords);
+			Assert.AreEqual(records.Length + 1, st.nExternalFiles);
 
 			// Close all handles.
 			db.Close();
@@ -263,7 +272,7 @@ namespace CsharpAPITest
 				cfg.Env.Close();
 
 			/*
-			 * Remove the default blob directory when it
+			 * Remove the default external file directory when it
 			 * is not under the test home.
 			 */
 			if (db_blobdir == null && cfg.Env == null)
@@ -1326,6 +1335,126 @@ ASCIIEncoding.ASCII.GetBytes(Configuration.RandomString(100)));
 			btreeDB.Close();
 		}
 
+		[Test]
+		public void TestMessageCall()
+		{
+			testName = "TestMessageCall";
+			SetUpTest(true);
+
+			// Configure and open an environment.
+			DatabaseEnvironmentConfig envConfig =
+			    new DatabaseEnvironmentConfig();
+			envConfig.Create = true;
+			envConfig.UseMPool = true;
+			DatabaseEnvironment env = DatabaseEnvironment.Open(
+			    testHome, envConfig);
+
+			// Configure and open a database.
+			BTreeDatabaseConfig DBConfig =
+			    new BTreeDatabaseConfig();
+			DBConfig.Env = env;
+			DBConfig.Creation = CreatePolicy.IF_NEEDED;
+
+			string DBFileName = testName + ".db";
+			BTreeDatabase db = BTreeDatabase.Open(DBFileName, DBConfig);
+
+			// Confirm message file does not exist.
+			string messageCallFile = testHome + "/" + "MessageCallFile";
+			Assert.AreEqual(false, File.Exists(messageCallFile));
+
+			string messageInfo = "Message come from db.set_msgcall!";
+
+			// Call set_msgcall() of env.
+			db.messageFeedback = new MessageFeedbackDelegate(Msgcall_fcn);
+			db.messageFeedback(null, messageInfo);
+
+			// Unconfigures the callback interface.
+			db.messageFeedback = null;
+
+			// Confirm message file exists now.
+			Assert.AreEqual(true, File.Exists(messageCallFile));
+
+			// Read the first line of message file.
+			string line = null;
+			System.IO.StreamReader file = new System.IO.StreamReader(@"" + messageCallFile);
+			line = file.ReadLine();
+
+			// Confirm the message file is not empty.
+			Assert.AreEqual(line, messageInfo);
+			file.Close();
+
+			// Close database and environment.
+			db.Close();
+			env.Close();
+		}
+
+		public void Msgcall_fcn(string msgpfx, string message)
+		{
+			string msgfile = testHome + "/" + "MessageCallFile";
+			FileStream fs = new FileStream(msgfile, FileMode.OpenOrCreate);
+			StreamWriter sw = new StreamWriter(fs);
+			if (msgpfx != null)
+				sw.Write(msgpfx + ": ");
+			sw.Write(message);
+			sw.Flush();
+			sw.Close();
+			fs.Close();
+		}
+
+		[Test]
+		public void TestMessageFile()
+		{
+			testName = "TestMessageFile";
+			SetUpTest(true);
+
+			// Configure and open an environment.
+			DatabaseEnvironmentConfig envConfig =
+			    new DatabaseEnvironmentConfig();
+			envConfig.Create = true;
+			envConfig.UseMPool = true;
+			DatabaseEnvironment env = DatabaseEnvironment.Open(
+			    testHome, envConfig);
+
+			// Configure and open a database.
+			BTreeDatabaseConfig DBConfig =
+			    new BTreeDatabaseConfig();
+			DBConfig.Env = env;
+			DBConfig.Creation = CreatePolicy.IF_NEEDED;
+
+			string DBFileName = testName + ".db";
+			BTreeDatabase db = BTreeDatabase.Open(DBFileName, DBConfig);
+
+			// Confirm message file does not exist.
+			string messageFile = testHome + "/" + "msgfile";
+			Assert.AreEqual(false, File.Exists(messageFile));
+
+			// Call set_msgfile() of db.
+			db.Msgfile = messageFile;
+
+			// Print db statistic to message file.
+			db.PrintStats(true);
+
+			// Confirm message file exists now.
+			Assert.AreEqual(true, File.Exists(messageFile));
+
+			db.Msgfile = "";
+			string line = null;
+
+			// Read the third line of message file.
+			System.IO.StreamReader file = new System.IO.StreamReader(@"" + messageFile);
+			line = file.ReadLine();
+			line = file.ReadLine();
+			line = file.ReadLine();
+
+			// Confirm the message file is not empty.
+			Assert.AreEqual(line, "DB handle information:");
+			file.Close();
+
+			// Close database and environment.
+			db.Close();
+			env.Close();
+		}
+
 		[Test]
 		public void TestNoWaitDbExclusiveLock()
 		{
@@ -2717,7 +2846,7 @@ ASCIIEncoding.ASCII.GetBytes(Configuration.RandomString(100)));
 			testName = "TestUpgrade";
 			SetUpTest(true);
 
-			string srcDBFileName = "../../bdb4.7.db";
+			string srcDBFileName = "../../../bdb4.7.db";
 			string testDBFileName = testHome + "/bdb4.7.db";
 
 			FileInfo srcDBFileInfo = new FileInfo(srcDBFileName);
@@ -2800,6 +2929,7 @@ ASCIIEncoding.ASCII.GetBytes(Configuration.RandomString(100)));
 			byte[] bigArray = new byte[10240];
 			db.Delete(new DatabaseEntry(bigArray));
 
+			db.Msgfile = testHome + "/" + testName+ ".log";
 			db.PrintStats();
 			db.PrintFastStats();
 
@@ -2905,6 +3035,7 @@ ASCIIEncoding.ASCII.GetBytes(Configuration.RandomString(100)));
 				stats = db.Stats(statsTxn, Isolation.DEGREE_THREE);
 			ConfirmStatsPart3Case1(stats);
 
+			db.Msgfile = home + "/" + name+ ".log";
 			db.PrintStats(true);
 			Assert.AreEqual(0, stats.EmptyPages);
 
@@ -2962,7 +3093,7 @@ ASCIIEncoding.ASCII.GetBytes(Configuration.RandomString(100)));
 			Assert.AreEqual(10, stats.MinKey);
 			Assert.AreEqual(2, stats.nPages);
 			Assert.AreEqual(4096, stats.PageSize);
-			Assert.AreEqual(9, stats.Version);
+			Assert.AreEqual(10, stats.Version);
 		}
 
 		public void ConfirmStatsPart2Case1(BTreeStats stats)
