@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2009, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2009, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  */
 using System;
@@ -1087,7 +1087,7 @@ namespace CsharpAPITest
 		 */
 		public static XmlElement TestSetUp(string testFixtureName, string testName)
 		{
-			XMLReader xmlReader = new XMLReader("../../AllTestData.xml");
+			XMLReader xmlReader = new XMLReader("../../../AllTestData.xml");
 			XmlElement xmlElem = xmlReader.GetXmlElement(testFixtureName, testName);
 			if (xmlElem == null)
 				throw new ConfigNotFoundException(testFixtureName + ":" + testName);
