@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  */
 
@@ -53,8 +53,7 @@ public class NegativeTest extends TxnTestCase {
 
     @Parameters
     public static List<Object[]> genParams() {
-        
-        return getTxnTypes(null, false);
+        return getTxnParams(null, false);
     }
     
     public NegativeTest(String type){
