@@ -1,7 +1,7 @@
 /*-
- * See the file LICENSE for redistribution information.
+ * Copyright (c) 2002, 2018 Oracle and/or its affiliates.  All rights reserved.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * See the file LICENSE for license information.
  *
  */
 
@@ -28,12 +28,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.junit.After;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
-import org.junit.runners.Parameterized.Parameters;
-
 import com.sleepycat.compat.DbCompat;
 import com.sleepycat.db.Database;
 import com.sleepycat.db.DatabaseConfig;
@@ -57,6 +51,12 @@ import com.sleepycat.persist.model.SecondaryKey;
 import com.sleepycat.persist.raw.RawStore;
 import com.sleepycat.util.test.TxnTestCase;
 
+import org.junit.After;
+import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.Parameterized;
+import org.junit.runners.Parameterized.Parameters;
+
 /**
  * Tests misc store and index operations that are not tested by IndexTest.
  *
@@ -69,8 +69,7 @@ public class OperationTest extends TxnTestCase {
 
     @Parameters
     public static List<Object[]> genParams() {
-        
-        return getTxnTypes(null, false);
+        return getTxnParams(null, false);
     }
     
     public OperationTest(String type){
