@@ -1,6 +1,6 @@
-# See the file LICENSE for redistribution information.
+# Copyright (c) 1999, 2018 Oracle and/or its affiliates.  All rights reserved.
 #
-# Copyright (c) 1999, 2013 Oracle and/or its affiliates.  All rights reserved.
+# See the file LICENSE for license information.
 #
 # $Id$
 #
@@ -23,14 +23,19 @@ set os_tbase 1
 # 1. db_reptest - Runs randomized configurations in a loop.
 # 2. basic_db_reptest - Runs a simple set configuration once,
 #	as a smoke test.
-# 3. restore_db_reptest 'dir' - Runs the configuration given in 'dir'
+# 3. master_switch_db_reptest - Runs db_reptest using a two-site
+#	replication group which periodically shuts down the master site,
+#	pauses, and restarts it as a client.
+# 4. restore_db_reptest 'dir' - Runs the configuration given in 'dir'
 #	in a loop.  The purpose is either to reproduce a problem
 #	that some configuration encountered, or test a fix.
-# 4. db_reptest_prof - Runs a single randomized configuration
+# 5. db_reptest_prof - Runs a single randomized configuration
 #	and generates gprof profiling information for that run.
-# 5. basic_db_reptest_prof - Runs a simple set configuration and
+# 6. basic_db_reptest_prof - Runs a simple set configuration and
 #	generates gprof profiling information for that run.
-# 6. restore_db_reptest_prof - Runs the configuration given in 'dir' and
+# 7. master_switch_db_reptest_prof - Runs the master switch configuration
+#	and generates gprof profiling information for that run.
+# 8. restore_db_reptest_prof - Runs the configuration given in 'dir' and
 #	generates gprof profiling information for one run.
 #
 
@@ -72,6 +77,47 @@ proc basic_db_reptest_prof { { basic 0 } } {
 	generate_profiles
 }
 
+#
+# Run db_reptest in master switch mode.  This mode creates a two-site
+# replication group with 2SITE_STRICT=off and bulk, then periodically
+# switches the master site.  The steps for a master switch are:
+# 1. Shut down the current master site
+# 2. Allow remaining client to elect itself master
+# 3. Allow the new master to run by itself for a while
+# 4. Restart previous master site as a client
+#
+# The input parameters are:
+#   switches: number of times to switch the master site
+#   databases: number of separate databases to create
+#   ops_per_txn: number of operations per transaction
+#   master_time: seconds a site should run as master
+#   down_time: seconds to wait before restarting previous master as client
+#
+proc master_switch_db_reptest { { switches 1 } { databases 1 } \
+    {ops_per_txn 100 } { master_time 180 } { down_time 60 } } {
+	global use
+	global util_path
+
+	if { [file exists $util_path/db_reptest] == 0 } {
+		puts "Skipping db_reptest.  Is it built?"
+		return
+	}
+
+	set use(ms_switches) $switches
+	set use(ms_databases) $databases
+	set use(ms_ops_per_txn) $ops_per_txn
+	set use(ms_master_time) $master_time
+	set use(ms_down_time) $down_time
+	db_reptest_int master_switch
+}
+
+proc master_switch_db_reptest_prof { { switches 1 } { databases 1 } \
+    {ops_per_txn 100 } { master_time 180 } { down_time 60 } } {
+	master_switch_db_reptest $switches $databases \
+	    $ops_per_txn $master_time $down_time
+	generate_profiles
+}
+
 #
 # Restore a configuration from the given directory and
 # run that configuration in a loop 'count' times or until
@@ -224,10 +270,13 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 	# after here depends on how many sites there are.
 	#
 	set use(nsites) [get_nsites $cfgtype $dirs(restore)]
-	set use(lease) [get_lease $cfgtype $dirs(restore)]
+	set use(twosite) [get_twosite $cfgtype $use(nsites)]
+	set use(pmkill) [get_pmkill $cfgtype $use(twosite)]
+	set use(lease) [get_lease $cfgtype $use(twosite) $dirs(restore)]
 	set use(peers) [get_peers $cfgtype]
 	set use(view) 0
 	set use(view_site) 0
+	set use(blobs) 0
 	#
 	# Get port information in case it needs to be converted for this
 	# run.  A conversion will happen for a restored run if the current
@@ -248,6 +297,7 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 	set kill_remover 0
 	set site_remove 0
 	set kill_self 0
+	set use(elect_loglength) 0
 	if { $use(nsites) > 2 } {
 		set use(kill) [get_kill $cfgtype \
 		    $dirs(restore) $use(nsites) baseport]
@@ -259,6 +309,7 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 			if { $kill_type == "DIE" || $kill_type == "REMOVE" } {
 				set kill_self $kill_site
 			}
+			set use(elect_loglength) [get_electloglength]
 		} else {
 			# If we are not doing a kill test, determine if
 			# we are doing a remove test.
@@ -275,23 +326,30 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 		if { $use(lease) } {
 			set use(master) 0
 		} else {
-			set use(master) [get_usemaster $cfgtype]
+			set use(master) [get_usemaster $cfgtype $use(twosite)]
 			if { $site_remove == $use(master) } {
 				set site_remove 0
 			}
 		}
-		set master_site [get_mastersite $cfgtype $use(master) $use(nsites)]
-		set noelect [get_noelect $use(master)]
+		set master_site [get_mastersite $cfgtype $use(master) \
+		    $use(nsites)]
+		set noelect [get_noelect $cfgtype $use(master)]
 		set master2_site [get_secondary_master \
 		    $noelect $master_site $kill_site $use(nsites)]
 		set use(view) [get_view $cfgtype NULL $master_site \
 		    $master2_site $use(nsites) $kill_self]
 		set use(view_site) [expr {abs($use(view))}]
-		set autotakeover_site [get_autotakeover $use(kill) \
-		    $site_remove $use(view) $use(view_site) $use(nsites)]
-		set workers [get_workers $cfgtype $use(lease)]
+		set autotakeover_site [get_autotakeover $cfgtype $use(kill) \
+		    $site_remove $use(view) $use(view_site) $use(nsites) \
+		    $use(pmkill)]
 		set dbtype [get_dbtype $cfgtype]
-		set runtime [get_runtime $cfgtype]
+		set use(blobs) [get_blobs $cfgtype $dbtype]
+		set workers [get_workers $cfgtype $use(lease) \
+		    $use(twosite) $use(blobs)]
+		set ipconfig [get_ipconfig $cfgtype]
+		set socketcb [get_socketcb $cfgtype]
+		set runtime [get_runtime $cfgtype $use(nsites) $use(lease) \
+		    $use(blobs)]
 		puts "Running: $use(nsites) sites, $runtime seconds."
 		puts -nonewline "Running: "
 		if { $use(createdir) } {
@@ -323,6 +381,11 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 		}
 		if { $use(lease) } {
 			puts "with leases."
+		} elseif { $cfgtype == "master_switch" } {
+			puts "master switches $use(ms_switches)\
+			    databases $use(ms_databases)\
+			    master time $use(ms_master_time)\
+			    down time $use(ms_down_time)."
 		} elseif { $use(master) } {
 			set master_text "master site $master_site"
 			if { $noelect } {
@@ -334,6 +397,12 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 				    "secondary master site $master2_site"]
 			}
 			puts "$master_text."
+		} elseif { $use(twosite) == "PREFMAS" } {
+			if { $use(pmkill) > 0 } {
+				puts "preferred master kill site $use(pmkill)."
+			} else {
+				puts "preferred master."
+			}
 		} else {
 			puts "no master."
 		}
@@ -348,7 +417,7 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 		reptest_cleanup $dirs(env.$i)
 		#
 		# If we are restoring the args, just read them from the
-		# saved location for this sites.  Otherwise build up
+		# saved location for this site.  Otherwise build up
 		# the args for each piece we need.
 		#
 		if { $cfgtype == "restore" } {
@@ -375,14 +444,24 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 				set runtime [parse_runtime $prog_args($i)]
 				puts "Runtime: $runtime"
 			}
+			set ipconfig [parse_ipconfig $prog_args($i)]
 		} else {
-			set nmsg [berkdb random_int 1 [expr $use(nsites) * 2]]
+			set nmsg_min 1
+			if { $use(twosite) == "PREFMAS" } {
+				set nmsg_min 2
+			}
+			set nmsg [berkdb random_int $nmsg_min \
+			    [expr $use(nsites) * 2]]
 			set prog_args($i) \
 			    "-v -c $workers -t $dbtype -T $runtime -m $nmsg "
 			set prog_args($i) \
 			    [concat $prog_args($i) "-h $dirs(home.$i)"]
 			set prog_args($i) \
 			    [concat $prog_args($i) "-o $use(nsites)"]
+			if { $use(blobs) != 0 } {
+				set prog_args($i) \
+				    [concat $prog_args($i) "-b $use(blobs)"]
+			}
 			#
 			# Add in if this site should remove itself.
 			#
@@ -408,7 +487,32 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 			#
 			# Add in if this site starts as master, client or view.
 			#
-			if { $i == $master_site } {
+			if { $use(twosite) == "PREFMAS" } {
+				set state($i) CLIENT
+				set prog_args($i) [concat $prog_args($i) "-P"]
+				if { $use(pmkill) == $i } {
+					set prog_args($i) \
+					    [concat $prog_args($i) "-k"]
+				}
+			} elseif { $cfgtype == "master_switch" } {
+				# Establish site's role.
+				if { $i == $master_site } {
+					set state($i) MASTER
+					set prog_args($i) \
+					    [concat $prog_args($i) "-M"]
+				} else {
+					set state($i) CLIENT
+					set prog_args($i) \
+					    [concat $prog_args($i) "-C"]
+				}
+				# Add master-switch-specific flags.
+				set prog_args($i) \
+				    [concat $prog_args($i) \
+"-W -S $use(ms_switches) -D $use(ms_databases) -O $use(ms_ops_per_txn)"]
+				set prog_args($i) \
+				    [concat $prog_args($i) \
+"-Y $use(ms_master_time) -y $use(ms_down_time)"]
+			} elseif { $i == $master_site } {
 				set state($i) MASTER
 				set prog_args($i) [concat $prog_args($i) "-M"]
 			} else {
@@ -454,19 +558,37 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 			if { $autotakeover_site == $i } {
 				set prog_args($i) [concat $prog_args($i) "-a"]
 			}
+			#
+			# Add in if this configuration is using something
+			# other than the default of IPv6 for all sites.
+			# Use 1 for IPv4 or 2 for mixed ipversions.
+			#
+			if { $ipconfig == "IPV4" } {
+				set prog_args($i) [concat $prog_args($i) "-I 1"]
+			} elseif { $ipconfig == "IPMIXED" } {
+				set prog_args($i) [concat $prog_args($i) "-I 2"]
+			}
+			#
+			# Add in if this configuration should set a socket
+			# callback.
+			#
+			if { $socketcb } {
+				set prog_args($i) [concat $prog_args($i) "-i"]
+			}
 		}
 		save_db_reptest $dirs(save) ARGS $i $prog_args($i)
 	}
 
 	# Now make the DB_CONFIG file for each site.
-	reptest_make_config $cfgtype dirs state use $portlist baseport
+	set hostlist [get_hosts $cfgtype $use(nsites) $ipconfig]
+	reptest_make_config $cfgtype dirs state use $hostlist $portlist baseport
 
 	# Run the test
 	run_db_reptest dirs $use(nsites) $runtime $use(lease)
 	puts "Test run complete.  Verify."
 
 	# Verify the test run.
-	verify_db_reptest $use(nsites) dirs use $kill_site $site_remove
+	verify_db_reptest $cfgtype $use(nsites) dirs use $kill_site $site_remove
 
 	# Show the summary files
 	print_summary
@@ -476,7 +598,8 @@ proc db_reptest_int { cfgtype { restoredir NULL } } {
 #
 # Make a DB_CONFIG file for all sites in the group
 #
-proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
+proc reptest_make_config { cfgtype dirsarr starr usearr \
+    hostlist portlist baseptarr } {
 	upvar $dirsarr dirs
 	upvar $starr state
 	upvar $baseptarr baseport
@@ -510,18 +633,13 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 	# 2site strict and ack policy must be the same on all sites.
 	#
 	if { $cfgtype == "random" } {
-		if { $use(nsites) == 2 } {
-			set strict [berkdb random_int 0 1]
-		} else {
-			set strict 0
-		}
 		if { $use(lease) } {
 			#
 			# 2site strict with leases must have ack policy of
 			# one because quorum acks are ignored in this case,
 			# resulting in lease expired panics on some platforms.
 			#
-			if { $strict } {
+			if { $use(twosite) == "STRICT" } {
 				set ackpolicy db_repmgr_acks_one
 			} else {
 				set ackpolicy db_repmgr_acks_quorum
@@ -557,6 +675,8 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 				set done 1
 			}
 		}
+	} elseif { $cfgtype == "master_switch" } {
+		set ackpolicy db_repmgr_acks_quorum
 	} else {
 		set ackpolicy db_repmgr_acks_one
 	}
@@ -639,9 +759,22 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 		# Add heartbeat_monitor.  Calculate value based on number of
 		# sites to reduce spurious heartbeat expirations.
 		#
-		lappend cfglist { "rep_set_timeout" \
-		    "db_rep_heartbeat_monitor \
-		    [expr $use(nsites) * 500000 * $os_tbase]" }
+		if { $cfgtype == "master_switch" } {
+			#
+			# Larger numbers of databases can cause more
+			# rerequests and longer waits.  Calculate this
+			# into the heartbeat monitor to avoid dupmasters
+			# and unexpected changes of master.
+			#
+			lappend cfglist { "rep_set_timeout" \
+			    "db_rep_heartbeat_monitor \
+			    [expr ($use(ms_databases) + $use(nsites)) \
+			    * 500000 * $os_tbase]" }
+		} else {
+			lappend cfglist { "rep_set_timeout" \
+			    "db_rep_heartbeat_monitor \
+			    [expr $use(nsites) * 500000 * $os_tbase]" }
+		}
 
 		#
 		# Add datadirs and the metadir, if needed.  If we are using
@@ -682,7 +815,13 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 			lappend cfglist $litem
 		}
 		#
-		# Others: limit size, bulk, 2site strict
+		# Set blob threshold.
+		#
+		if { $use(blobs) != 0 } {
+			lappend cfglist {"set_blob_threshold" "$use(blobs) 0" }
+		}
+		#
+		# Others: limit size, bulk, 2site strict, preferred master
 		#
 		if { $cfgtype == "random" } {
 			set limit_sz [berkdb random_int 15000 1000000]
@@ -692,16 +831,39 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 				    { "rep_set_config" "db_rep_conf_bulk" }
 			}
 			#
-			# 2site strict was set above for all sites but
-			# should only be used for sites in random configs.
+			# Preferred master and 2site strict were set above
+			# for all sites but should only be used for sites in
+			# random configs.
 			#
-			if { $strict } {
+			if { $use(twosite) == "PREFMAS" } {
+				if { $i == $known_master } {
+					lappend cfglist { "rep_set_config" \
+					    "db_repmgr_conf_prefmas_master" }
+				} else {
+					lappend cfglist { "rep_set_config" \
+					    "db_repmgr_conf_prefmas_client" }
+				}
+			}
+			if { $use(twosite) == "STRICT" ||
+			    $use(twosite) == "PREFMAS" } {
 				lappend cfglist { "rep_set_config" \
 				    "db_repmgr_conf_2site_strict" }
 			} else {
 				lappend cfglist { "rep_set_config" \
 				    "db_repmgr_conf_2site_strict off" }
 			}
+			if { $use(elect_loglength) } {
+				lappend cfglist { "rep_set_config" \
+				"db_rep_conf_elect_loglength" }
+			}
+		} elseif { $cfgtype == "master_switch" } {
+			set limit_sz [berkdb random_int 15000 1000000]
+			# Always turn on bulk for master switch.
+			lappend cfglist \
+			    { "rep_set_config" "db_rep_conf_bulk" }
+			# Always turn off 2site_strict for master switch.
+			lappend cfglist { "rep_set_config" \
+			    "db_repmgr_conf_2site_strict off" }
 		} else {
 			set limit_sz 100000
 		}
@@ -710,15 +872,16 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 		set litem [list repmgr_set_ack_policy $ackpolicy]
 		lappend cfglist $litem
 		#
-		# Now set up the local and remote ports.  If we are the
-		# known_master (either master or group creator) set the
+		# Now set up the local and remote hosts and ports.  If we are
+		# the known_master (either master or group creator) set the
 		# group creator flag on.
 		#
-		# Must use explicit 127.0.0.1 rather than localhost because
-		# localhost can be configured differently on different
+		# Must use explicit IPv6 or IPv4 loopback rather than localhost
+		# because localhost can be configured differently on different
 		# machines or platforms.  Use of localhost can cause 
 		# available_ports to return ports that are actually in use.
 		#
+		set lhost($i) [lindex $hostlist [expr $i - 1]]
 		set lport($i) [lindex $portlist [expr $i - 1]]
 		if { $i == $known_master } {
 			#
@@ -727,24 +890,28 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 			# on this ordering and these embedded spaces.
 			#
 			set litem [list repmgr_site \
-			    "127.0.0.1 $lport($i) db_local_site on \
+			    "$lhost($i) $lport($i) db_local_site on\
 			    db_group_creator on"]
 		} else {
 			set litem [list repmgr_site \
-			    "127.0.0.1 $lport($i) db_local_site on"]
+			    "$lhost($i) $lport($i) db_local_site on"]
 		}
 		lappend cfglist $litem
-		set rport($i) [get_rport $portlist $i $use(nsites) \
+		set rsites($i) [get_rsites $i $use(nsites) \
 		    $known_master $cfgtype]
 		#
 		# Declare all sites bootstrap helpers.
 		#
-		foreach p $rport($i) {
+		foreach p $rsites($i) {
 			if { $use(peers) } {
-				set litem [list repmgr_site "127.0.0.1 $p \
+				set litem [list repmgr_site \
+				    "[lindex $hostlist $p]\
+				    [lindex $portlist $p]\
 				    db_bootstrap_helper on db_repmgr_peer on"]
 			} else {
-				set litem [list repmgr_site "127.0.0.1 $p \
+				set litem [list repmgr_site \
+				    "[lindex $hostlist $p]\
+				    [lindex $portlist $p]\
 				    db_bootstrap_helper on"]
 			}
 			#
@@ -765,6 +932,9 @@ proc reptest_make_config { cfgtype dirsarr starr usearr portlist baseptarr } {
 			puts $cid "$carg $cval"
 		}
 		close $cid
+
+		setup_repmgr_ssl $dirs(env.$i)
+
 		set cid [open $dirs(env.$i)/DB_CONFIG r]
 		set cfg [read $cid]
 		close $cid
@@ -833,7 +1003,19 @@ proc run_db_reptest { dirsarr numsites runtime use_lease } {
 	}
 }
 
-proc verify_db_reptest { num_sites dirsarr usearr kill site_rem } {
+proc db_reptest_blob_cmp_func { a b } {
+	set big_e [big_endian]
+	if { $big_e } {
+		binary scan $a I a
+		binary scan $b I b
+	} else {
+		binary scan $a i a
+		binary scan $b i b
+	}
+	return [expr $a - $b]
+}
+
+proc verify_db_reptest { cfgtype num_sites dirsarr usearr kill site_rem } {
 	upvar $dirsarr dirs
 	upvar $usearr use
 
@@ -882,17 +1064,44 @@ proc verify_db_reptest { num_sites dirsarr usearr kill site_rem } {
 			    {eval $dirs(env.$i)/$datadir/am1.db}] 0
 			continue
 		}
+
+		set bt_cmp 0
+		set ham_cmp 0
+		set blob_dir_exists [file exists $dirs(env.$startenv)/__db_bl]
+		if { $use(blobs) != 0 || $blob_dir_exists } {
+			set dbam1 [eval berkdb_open -env $envbase \
+			    -rdonly am1.db]
+			set actype [$dbam1 get_type]
+			$dbam1 close
+			if { $actype == "btree" } {
+				set bt_cmp db_reptest_blob_cmp_func
+			} else {
+				set ham_cmp db_reptest_blob_cmp_func
+			}
+		}
+
 		set cmpenv [berkdb_env_noerr -home $dirs(env.$i)]
 		puts "Compare $dirs(env.$startenv) with $dirs(env.$i)"
 		#
 		# Compare 2 envs.  We assume the name of the database that
-		# db_reptest creates and know it is 'am1.db'.
+		# db_reptest creates and know it is 'am1.db' or 'amarr#.db'.
 		# We want as other args:
 		# 0 - compare_shared_portion
 		# 1 - match databases
 		# 0 - don't compare logs (for now)
-		rep_verify $dirs(env.$startenv) $envbase $dirs(env.$i) $cmpenv \
-		    0 1 0 am1.db $datadir
+		if { $cfgtype == "master_switch" } {
+			# Compare multiple databases for master_switch.
+			for { set j 0 } { $j < $use(ms_databases) } { incr j } {
+				rep_verify $dirs(env.$startenv) $envbase \
+				    $dirs(env.$i) $cmpenv 0 1 0 amarr$j.db \
+				    $datadir $bt_cmp $ham_cmp
+			}	
+		} else {
+			# Compare single database for all other cases.
+			rep_verify $dirs(env.$startenv) $envbase \
+			    $dirs(env.$i) $cmpenv 0 1 0 am1.db $datadir \
+			    $bt_cmp $ham_cmp
+		}
 		$cmpenv close
 	}
 	$envbase close
@@ -945,13 +1154,16 @@ puts "Getting random nsites between 2 and $maxsites.  Got $n, last_nsites $last_
 	if { $cfgtype == "basic1" } {
 		return 3
 	}
+	if { $cfgtype == "master_switch" } {
+		return 2
+	}
 	return -1
 }
 
 #
 # Run with master leases?  25%/75% (use a master lease 25% of the time).
 #
-proc get_lease { cfgtype restoredir } {
+proc get_lease { cfgtype twosite restoredir } {
 	#
 	# The number of sites must be the same for all.  Read the
 	# first site's saved DB_CONFIG file if we're restoring since
@@ -978,6 +1190,9 @@ proc get_lease { cfgtype restoredir } {
 		return $uselease
 	}
 	if { $cfgtype == "random" } {
+		if { $twosite == "PREFMAS" } {
+			return 0
+		}
 		set leases { 1 0 0 0 }
 		set len [expr [llength $leases] - 1]
 		set i [berkdb random_int 0 $len]
@@ -989,6 +1204,9 @@ proc get_lease { cfgtype restoredir } {
 	if { $cfgtype == "basic1" } {
 		return 0
 	}
+	if { $cfgtype == "master_switch" } {
+		return 0
+	}
 }
 
 #
@@ -1117,7 +1335,8 @@ proc get_kill { cfgtype restoredir num_sites basept } {
 		}
 		return [list $ktype $ksite $rsite]
 	}
-	if { $cfgtype == "basic0" || $cfgtype == "basic1" } {
+	if { $cfgtype == "basic0" || $cfgtype == "basic1" || \
+	    $cfgtype == "master_switch" } {
 		return $nokill
 	} else {
 		error "Get_kill: Invalid config type $cfgtype"
@@ -1187,12 +1406,16 @@ proc get_peers { cfgtype } {
 # Start with a master or all clients?  25%/75% (use a master 25%
 # of the time and have all clients 75%)
 #
-proc get_usemaster { cfgtype } {
+proc get_usemaster { cfgtype twosite } {
 	if { $cfgtype == "random" } {
-		set mst { 1 0 0 0 }
-		set len [expr [llength $mst] - 1]
-		set i [berkdb random_int 0 $len]
-		return [lindex $mst $i]
+		if { $twosite == "PREFMAS" } {
+			return 0
+		} else {
+			set mst { 1 0 0 0 }
+			set len [expr [llength $mst] - 1]
+			set i [berkdb random_int 0 $len]
+			return [lindex $mst $i]
+		}
 	}
 	if { $cfgtype == "basic0" } {
 		return 1
@@ -1200,6 +1423,9 @@ proc get_usemaster { cfgtype } {
 	if { $cfgtype == "basic1" } {
 		return 0
 	}
+	if { $cfgtype == "master_switch" } {
+		return 1
+	}
 }
 
 #
@@ -1221,13 +1447,16 @@ proc get_mastersite { cfgtype usemaster nsites } {
 	if { $cfgtype == "basic1" } {
 		return 0
 	}
+	if { $cfgtype == "master_switch" } {
+		return 1
+	}
 }
 
 #
 # If we are using a master, use no elections 20% of the time.
 #
-proc get_noelect { usemaster } {
-	if { $usemaster } {
+proc get_noelect { cfgtype usemaster } {
+	if { $cfgtype != "master_switch" && $usemaster } {
 		set noelect { 0 0 1 0 0 }
 		set len [expr [llength $noelect] - 1]
 		set i [berkdb random_int 0 $len]
@@ -1237,6 +1466,139 @@ proc get_noelect { usemaster } {
 	}
 }
 
+#
+# For 2-site repgroups, we want to evenly divide the test
+# configurations based on the following return values:
+#     NONE    2site_strict=off
+#     STRICT  2site_strict=on
+#     PREFMAS preferred master
+#
+proc get_twosite { cfgtype nsites } {
+	if { $cfgtype == "random" && $nsites == 2 } {
+		set i [berkdb random_int 0 2]
+		if { $i == 1 } {
+			return "STRICT"
+		}
+		if { $i == 2 } {
+			return "PREFMAS"
+		}
+	}
+	return "NONE"
+}
+
+#
+# For preferred master 2-site repgroups, we want half of the test
+# configurations to have a site kill itself and later come back.
+# Make the kill cases equally likely to kill one site or the other.
+#
+proc get_pmkill { cfgtype twosite } {
+	if { $cfgtype == "random" && $twosite == "PREFMAS" } {
+		# Decide whether to kill a site.
+		set pmk { 0 1 0 1 1 0 0 1 0 1 }
+		set len [expr [llength $pmk] - 1]
+		set i [berkdb random_int 0 $len]
+		if { [lindex $pmk $i] == 1 } {
+			# Decide which site to kill.
+			return [berkdb random_int 1 2]
+		}
+	}
+	return 0
+}
+
+#
+# Determine the IP version used by sites in the replication group.
+# Returns the following values:
+#     IPV6     All sites use IPv6
+#     IPV4     All sites use IPv4
+#     IPMIXED  Some sites use IPv6 and others use IPv4
+#
+proc get_ipconfig { cfgtype } {
+	if { $cfgtype == "random" } {
+		set i [berkdb random_int 0 3]
+		if { $i == 2 } {
+			return "IPV4"
+		}
+		if { $i == 3 } {
+			return "IPMIXED"
+		}
+	}
+	return "IPV6"
+}
+
+#
+# Returns an array of the host strings to be used by each site in the
+# replication group.  Each host string is either the IPv6 or IPv4
+# loopback.  The combination of IPv6 and IPv4 is determined by ipconfig.
+#
+proc get_hosts { cfgtype nsites ipconfig } {
+	set hosts {}
+	set v6str "::1"
+	set v4str "127.0.0.1"
+	set hoststr $v6str
+	if { $ipconfig == "IPV4" } {
+		set hoststr $v4str
+	}
+
+	if { $cfgtype == "random" || $cfgtype == "restore" || \
+	    $cfgtype == "master_switch" } {
+		# Fill in the initial array.
+		for { set i 1 } { $i <= $nsites } { incr i } {
+			lappend hosts $hoststr
+		}
+		#
+		# For a mixed configuration, the array was originally populated
+		# with IPv6 strings.  Now substitute the IPv4 string for the
+		# second element.  If there are 5 sites, also substitute the
+		# IPv4 string for the fourth element.
+		#
+		if { $ipconfig == "IPMIXED" } {
+			set hosts [lreplace $hosts 1 1 $v4str]
+			if { $nsites == 5 } {
+				set hosts [lreplace $hosts 3 3 $v4str]
+			}
+		}
+	}
+
+	#
+	# The basic_db_reptest options only use IPv6, so just create
+	# the basic host array they need.
+	#
+	if { $cfgtype == "basic0" || $cfgtype == "basic1" } {
+		for { set i 1 } { $i <= $nsites } { incr i } {
+			lappend hosts $hoststr
+		}
+	}
+
+	return $hosts
+}
+
+#
+# Set a socket callback 20% of the time, regardless of the other details
+# of the ip configuration.
+#
+proc get_socketcb { cfgtype } {
+	if { $cfgtype == "random" } {
+		set sockcb { 0 0 1 0 0 }
+		set len [expr [llength $sockcb] - 1]
+		set i [berkdb random_int 0 $len]
+		return [lindex $sockcb $i]
+	}
+	return 0
+}
+
+#
+# ELECT_LOGLENGTH is only significant in test cases where the master is killed.
+# The reason is that the repmgr group creator is automatically the master
+# without an election on initial startup.  Use ELECT_LOGLENGTH in 25% of the
+# cases of an election after the master is killed.
+#
+proc get_electloglength { } {
+	set electloglength { 0 0 1 0 }
+	set len [expr [llength $electloglength] - 1]
+	set i [berkdb random_int 0 $len]
+	return [lindex $electloglength $i]
+}
+
 #
 # If we are using no elections mode and we are going to kill the initial
 # master, select a different site to start up as master after the initial
@@ -1288,6 +1650,9 @@ proc get_view { cfgtype restoredir master_site second_master nsites kill_self} {
 	if { $cfgtype == "basic1" } {
 		return 3
 	}
+	if { $cfgtype == "master_switch" } {
+		return 0
+	}
 	if { $cfgtype == "random" } {
 		if { $nsites == 2 } {
 			return 0
@@ -1325,13 +1690,16 @@ proc get_view { cfgtype restoredir master_site second_master nsites kill_self} {
 #
 # Return a site number for autotakeover or 0 for no autotakeover.
 #
-proc get_autotakeover { kill remove view viewsite nsites } {
+proc get_autotakeover { cfgtype kill remove view viewsite nsites pmkill } {
+	if { $cfgtype == "master_switch" } {
+		return 0
+	}
 	set autotakeover 0
 	#
 	# Do not combine autotakeover with a kill or remove test because that
 	# would be too much disruption during a possibly short test run.
 	#
-	if { [llength $kill] == 0 && $remove == 0 } {
+	if { [llength $kill] == 0 && $remove == 0 && $pmkill == 0 } {
 		set at { 0 1 0 1 1 0 0 1 0 1 }
 		set len [expr [llength $at] - 1]
 		set i [berkdb random_int 0 $len]
@@ -1354,16 +1722,21 @@ proc get_autotakeover { kill remove view viewsite nsites } {
 # This is the number of worker threads performing the workload.
 # This is not the number of message processing threads.
 #
-# Scale back the number of worker threads if leases are in use.
-# The timing with leases can be fairly sensitive and since all sites
-# run on the local machine, too many workers on every site can
+# Scale back the number of worker threads if leases or blobs are in
+# use.  The timing with leases can be fairly sensitive and since all
+# sites run on the local machine, too many workers on every site can
 # overwhelm the system, causing lost messages and delays that make
 # the tests fail.  Rather than try to tweak timeouts, just reduce
 # the workloads a bit.
 #
-proc get_workers { cfgtype lease } {
+# Also scale back the number of worker threads for preferred master.
+# The timing can be sensitive when the preferred master takes over
+# after resyncing with the temporary master.  Too many workers
+# overwhelming the system can cause delays that make the test fail.
+#
+proc get_workers { cfgtype lease twosite blobs} {
 	if { $cfgtype == "random" } {
-		if { $lease } {
+		if { $lease || $twosite == "PREFMAS" || $blobs } {
 			return [berkdb random_int 2 4]
 		} else {
 			return [berkdb random_int 2 8]
@@ -1372,10 +1745,13 @@ proc get_workers { cfgtype lease } {
 	if { $cfgtype == "basic0" || $cfgtype == "basic1" } {
 		return 5
 	}
+	if { $cfgtype == "master_switch" } {
+		return [berkdb random_int 2 8]
+	}
 }
 
 proc get_dbtype { cfgtype } {
-	if { $cfgtype == "random" } {
+	if { $cfgtype == "random" || $cfgtype == "master_switch" } {
 		#
 		# 50% btree, 25% queue 12.5% hash 12.5% recno
 		# We favor queue only because there is special handling
@@ -1392,11 +1768,46 @@ proc get_dbtype { cfgtype } {
 	}
 }
 
-proc get_runtime { cfgtype } {
+# On random configurations return blobs 20% of the time if the method supports
+# blobs.  
+proc get_blobs { cfgtype dbtype } {
+	if { $cfgtype == "random" \
+	    && $dbtype != "queue" && $dbtype != "recno" } {
+		set threshold { 0 0 0 0 1024 }
+		set len [expr [llength $threshold] - 1]
+		set i [berkdb random_int 0 $len]
+		return [lindex $threshold $i]
+	}
+	if { $cfgtype == "basic1" } {
+		return 1024
+	}
+	return 0
+}
+
+proc get_runtime { cfgtype nsites useleases useblobs } {
 	global os_tbase
+	global use
 
 	if { $cfgtype == "random" } {
-		return [expr [berkdb random_int 100 500] * $os_tbase]
+		set min 100
+		if { $useleases && $useblobs } {
+			# Things are very slow when master leases are
+			# combined with blobs.  Master leases add delays to
+			# adding a site to the replication group and blobs
+			# have a longer internal init time.  A failure is most
+			# likely in a run that could kill the master site
+			# before all other sites have had time to complete
+			# their internal inits.
+			set min 300
+		} elseif { $nsites > 4 && $useleases} {
+			# Master leases really slow down the process of adding
+			# sites to the replication group.  With 5 sites it
+			# can take longer than the total test time when runtime
+			# is too small, causing the test to fail.  Set a higher
+			# minimum test time in this case.
+			set min 150
+		}
+		return [expr [berkdb random_int $min 500] * $os_tbase]
 	}
 	if { $cfgtype == "basic0" } {
 		return [expr 100 * $os_tbase]
@@ -1404,9 +1815,19 @@ proc get_runtime { cfgtype } {
 	if { $cfgtype == "basic1" } {
 		return [expr 150 * $os_tbase]
 	}
+	if { $cfgtype == "master_switch" } {
+		return [expr \
+		    ($use(ms_switches) + 1) * $use(ms_master_time) + 30]
+	}
 }
 
-proc get_rport { portlist i num_sites known_master cfgtype} {
+#
+# Returns a list of the site numbers to be used as the remote
+# bootstrap helpers for a given site.  The site numbers will
+# be used as indexes into the lists of host strings and port
+# numbers when writing the remote sites to DB_CONFIG.
+#
+proc get_rsites { i num_sites known_master cfgtype} {
 	global rporttype
 
 	if { $cfgtype == "random" && $rporttype == "NULL" } {
@@ -1415,7 +1836,7 @@ proc get_rport { portlist i num_sites known_master cfgtype} {
 		set rindex [berkdb random_int 0 $len]
 		set rporttype [lindex $types $rindex]
 	}
-	if { $cfgtype == "basic0" } {
+	if { $cfgtype == "basic0" || $cfgtype == "master_switch" } {
 		set rporttype onesite
 	}
 	if { $cfgtype == "basic1" } {
@@ -1428,16 +1849,16 @@ proc get_rport { portlist i num_sites known_master cfgtype} {
 	#
 	if { $rporttype == "forwcirc" } {
 		if { $i != $num_sites } {
-			return [list [lindex $portlist $i]]
+			return $i
 		} else {
-			return [list [lindex $portlist 0]]
+			return 0
 		}
 	}
 	if { $rporttype == "backcirc" } {
 		if { $i != 1 } {
-			return [list [lindex $portlist [expr $i - 2]]]
+			return [expr $i - 2]
 		} else {
-			return [list [lindex $portlist [expr $num_sites - 1]]]
+			return [expr $num_sites - 1]
 		}
 	}
 	#
@@ -1452,7 +1873,7 @@ proc get_rport { portlist i num_sites known_master cfgtype} {
 		if { $i == $known_master } {
 			return {}
 		}
-		return [lindex $portlist $helper_site]
+		return $helper_site
 	}
 	#
 	# This produces a fully connected configuration
@@ -1461,8 +1882,7 @@ proc get_rport { portlist i num_sites known_master cfgtype} {
 		set rlist {}
 		for { set site 1 } { $site <= $num_sites } { incr site } {
 			if { $site != $i } {
-				lappend rlist \
-				    [lindex $portlist [expr $site - 1]]
+				lappend rlist [expr $site - 1]
 			}
 		}
 		return $rlist
@@ -1547,11 +1967,13 @@ proc get_orig_baseport { cfgtype { restoredir NULL } } {
 	} else {
 		set cid [open $restoredir/DB_CONFIG.1 r]
 		set cfg [read $cid]
-		# Look for a number between "127.0.0.1" and "db_local_site on".
-		# The spaces after 127.0.0.1 and before db_local_site are
-		# significant in the pattern match.  Also accept localhost as
-		# input so that old configs can be run.
-		regexp {(127.0.0.1 |localhost )([0-9]+)( db_local_site on)} \
+		# Look for a number between "::1" or "127.0.0.1" and
+		# "db_local_site on".  The spaces after ::1/127.0.0.1 and
+		# before db_local_site are significant in the pattern match.
+		# Also accept localhost as input so that old configs can be
+		# run.
+		regexp \
+		    {(::1 |127.0.0.1 |localhost )([0-9]+)( db_local_site on)} \
 		    $cfg match p1 pnum
 		close $cid
 		return [expr $pnum - 1]
@@ -1559,14 +1981,14 @@ proc get_orig_baseport { cfgtype { restoredir NULL } } {
 }
 
 #
-# Convert DB_CONFIG file port numbers following "127.0.0.1 " to use a 
-# different baseport.  regsub -all substitutes all occurrences of pattern,
-# which is "127.0.0.1 " and a number.  The result of regsub contains a tcl
-# expression with the number (\2, the second part of the pattern), operators
-# and variable names, e.g.:
+# Convert DB_CONFIG file port numbers following "::1 " or "127.0.0.1 " to use 
+# a different baseport.  regsub -all substitutes all occurrences of pattern,
+# which is "::1 " or "127.0.0.1" and a number.  The result of regsub contains
+# a tcl expression with the number (\2, the second part of the pattern),
+# operators and variable names, e.g.:
 #     -K [expr 30104 - $baseport(orig) + $baseport(curr)]
 # and then subst evalutes the tcl expression.  Also accept localhost as
-# input so that old configs can be run.
+# input so that old configs can be run and convert localhost to "::1".
 #
 # Writes a converted copy of orig_file to new_file.
 #
@@ -1575,7 +1997,9 @@ proc convert_config_ports { orig_file new_file basept } {
 
 	set cid [open $orig_file r]
 	set cfg [read $cid]
-	regsub -all {(127.0.0.1 |localhost )([0-9]+)} $cfg \
+	regsub -all {(::1 |localhost )([0-9]+)} $cfg \
+	    {::1 [expr \2 - $baseport(orig) + $baseport(curr)]} cfg
+	regsub -all {(127.0.0.1 )([0-9]+)} $cfg \
 	    {127.0.0.1 [expr \2 - $baseport(orig) + $baseport(curr)]} cfg
 	set cfg [subst $cfg]
 	close $cid
@@ -1591,6 +2015,21 @@ proc parse_runtime { progargs } {
 	return $val
 }
 
+proc parse_ipconfig { progargs } {
+	set ipconfig "IPV6"
+	set val 0
+	set i [lsearch $progargs "-I"]
+	if { $i > -1 } {
+		set val [lindex $progargs [expr $i + 1]]
+	}
+	if { $val == 1 } {
+		set ipconfig "IPV4"
+	} elseif { $val == 2 } {
+		set ipconfig "IPMIXED"
+	}
+	return $ipconfig
+}
+
 proc print_summary { } {
 	source ./include.tcl
 
