@@ -1,6 +1,6 @@
-# See the file LICENSE for redistribution information.
+# Copyright (c) 2007, 2018 Oracle and/or its affiliates.  All rights reserved.
 #
-# Copyright (c) 2007, 2013 Oracle and/or its affiliates.  All rights reserved.
+# See the file LICENSE for license information.
 #
 # $Id$
 #
@@ -11,12 +11,16 @@
 # TEST	restart each client, processing transactions after each restart.
 # TEST	Verify all expected transactions are replicated.
 # TEST
+# TEST	Test flag -R of utility db_printlog.
+# TEST	
 # TEST	Run for btree only because access method shouldn't matter.
 # TEST
 proc repmgr007 { { niter 100 } { tnum "007" } args } {
 
 	source ./include.tcl
 
+	package require tls
+
 	if { $is_freebsd_test == 1 } {
 		puts "Skipping replication manager test on FreeBSD platform."
 		return
@@ -29,10 +33,19 @@ proc repmgr007 { { niter 100 } { tnum "007" } args } {
 	repmgr007_sub $method $niter $tnum $args
 }
 
+proc ::get_cert_pass {} {
+     return "someRandomPass"
+}
+
 proc repmgr007_sub { method niter tnum largs } {
+	source ./include.tcl
+
 	global testdir
+	global EXE
 	global rep_verbose
 	global verbose_type
+	global ipversion
+	global ssl_test_enabled
 	set nsites 3
 
 	set verbargs ""
@@ -42,6 +55,7 @@ proc repmgr007_sub { method niter tnum largs } {
 
 	env_cleanup $testdir
 	set ports [available_ports $nsites]
+	set hoststr [get_hoststr $ipversion]
 
 	set masterdir $testdir/MASTERDIR
 	set clientdir $testdir/CLIENTDIR
@@ -51,6 +65,10 @@ proc repmgr007_sub { method niter tnum largs } {
 	file mkdir $clientdir
 	file mkdir $clientdir2
 
+	setup_repmgr_ssl $masterdir
+	setup_repmgr_ssl $clientdir
+	setup_repmgr_ssl $clientdir2
+
 	# Open a master.
 	puts "\tRepmgr$tnum.a: Start a master."
 	set ma_envcmd "berkdb_env_noerr -create $verbargs \
@@ -58,7 +76,7 @@ proc repmgr007_sub { method niter tnum largs } {
 	    -txn -rep -thread"
 	set masterenv [eval $ma_envcmd]
 	$masterenv repmgr -ack all \
-	    -local [list 127.0.0.1 [lindex $ports 0]] \
+	    -local [list $hoststr [lindex $ports 0]] \
 	    -start master
 
 	# Open first client
@@ -67,9 +85,9 @@ proc repmgr007_sub { method niter tnum largs } {
 	    -errpfx CLIENT -home $clientdir -txn -rep -thread"
 	set clientenv [eval $cl_envcmd]
 	$clientenv repmgr -ack all \
-	    -local [list 127.0.0.1 [lindex $ports 1]] \
-	    -remote [list 127.0.0.1 [lindex $ports 0]] \
-	    -remote [list 127.0.0.1 [lindex $ports 2]] \
+	    -local [list $hoststr [lindex $ports 1]] \
+	    -remote [list $hoststr [lindex $ports 0]] \
+	    -remote [list $hoststr [lindex $ports 2]] \
 	    -start client
 	await_startup_done $clientenv
 
@@ -79,9 +97,9 @@ proc repmgr007_sub { method niter tnum largs } {
 	    -errpfx CLIENT2 -home $clientdir2 -txn -rep -thread"
 	set clientenv2 [eval $cl2_envcmd]
 	$clientenv2 repmgr -ack all \
-	    -local [list 127.0.0.1 [lindex $ports 2]] \
-	    -remote [list 127.0.0.1 [lindex $ports 0]] \
-	    -remote [list 127.0.0.1 [lindex $ports 1]] \
+	    -local [list $hoststr [lindex $ports 2]] \
+	    -remote [list $hoststr [lindex $ports 0]] \
+	    -remote [list $hoststr [lindex $ports 1]] \
 	    -start client
 	await_startup_done $clientenv2
 
@@ -100,9 +118,9 @@ proc repmgr007_sub { method niter tnum largs } {
 	# Open -recover to clear env region, including startup_done value.
 	set clientenv [eval $cl_envcmd -recover]
 	$clientenv repmgr -ack all \
-	    -local [list 127.0.0.1 [lindex $ports 1]] \
-	    -remote [list 127.0.0.1 [lindex $ports 0]] \
-	    -remote [list 127.0.0.1 [lindex $ports 2]] \
+	    -local [list $hoststr [lindex $ports 1]] \
+	    -remote [list $hoststr [lindex $ports 0]] \
+	    -remote [list $hoststr [lindex $ports 2]] \
 	    -start client
 	await_startup_done $clientenv
 
@@ -120,9 +138,9 @@ proc repmgr007_sub { method niter tnum largs } {
 	# Open -recover to clear env region, including startup_done value.
 	set clientenv2 [eval $cl2_envcmd -recover]
 	$clientenv2 repmgr -ack all \
-	    -local [list 127.0.0.1 [lindex $ports 2]] \
-	    -remote [list 127.0.0.1 [lindex $ports 0]] \
-	    -remote [list 127.0.0.1 [lindex $ports 1]] \
+	    -local [list $hoststr [lindex $ports 2]] \
+	    -remote [list $hoststr [lindex $ports 0]] \
+	    -remote [list $hoststr [lindex $ports 1]] \
 	    -start client
 	await_startup_done $clientenv2
 
@@ -137,12 +155,45 @@ proc repmgr007_sub { method niter tnum largs } {
 	# Test that repmgr won't crash on a small amount of unexpected
 	# input over its port.  
 	#
+	
+	set tls_setup_args [tcl_tls_setup_string]
+	
 	puts "\tRepmgr$tnum.k: Test that repmgr ignores unexpected input."
-	set msock [socket 127.0.0.1 [lindex $ports 0]]
+
+	if { $ssl_test_enabled == 1 } {
+		tls::init
+		set msock [tls::socket {*}$tls_setup_args $hoststr [lindex $ports 0] ]
+	} else {
+		set msock [socket $hoststr [lindex $ports 0]]
+	}
 	set garbage "abcdefghijklmnopqrstuvwxyz"
 	puts $msock $garbage
 	close $msock
 
+	puts "\tRepmgr$tnum.l: Test db_printlog with flag -R."
+	set binname db_printlog
+	set std_redirect "> /dev/null"
+	if { $is_windows_test } {
+		set std_redirect "> /nul"
+		append binname $EXE
+	}
+
+	set hpargs "-h $clientdir -R"
+	# Test flag R combination with other flags.
+	set flaglist [list "-N" "-r" "-V" "-D 100" ""]
+	foreach lgpflag $flaglist {
+		
+		set combinearg "$lgpflag $hpargs"
+
+		repmgr007_execmd "$binname $combinearg $std_redirect"
+		# Test with given start and end LSN.
+		repmgr007_execmd "$binname -b 0/0 $combinearg $std_redirect"
+		repmgr007_execmd\
+		    "$binname -e 1/1000 $combinearg $std_redirect"
+		repmgr007_execmd\
+		    "$binname -b 0/0 -e 1/1000 $combinearg $std_redirect"
+	}
+	
 	error_check_good client2_close [$clientenv2 close] 0
 	error_check_good client_close [$clientenv close] 0
 	error_check_good masterenv_close [$masterenv close] 0
@@ -156,3 +207,11 @@ proc repmgr007_sub { method niter tnum largs } {
 	close $maserrfile
 	error_check_good errchk [is_substr $maserr "unexpected msg type"] 1
 }
+
+proc repmgr007_execmd { execmd } {
+	source ./include.tcl
+	puts "\t\tRepmgr007: $util_path/$execmd"
+	if { [catch {eval exec $util_path/$execmd} result] } {
+		puts "FAIL: got $result while executing '$execmd'"
+	}
+}
