@@ -1,6 +1,6 @@
-# See the file LICENSE for redistribution information.
+# Copyright (c) 2007, 2018 Oracle and/or its affiliates.  All rights reserved.
 #
-# Copyright (c) 2007, 2013 Oracle and/or its affiliates.  All rights reserved.
+# See the file LICENSE for license information.
 #
 # $Id$
 #
@@ -34,6 +34,7 @@ proc repmgr012_sub { method niter tnum largs } {
 	source ./include.tcl
 	global rep_verbose
 	global verbose_type
+	global ipversion
 	set nsites 2
 
 	set verbargs ""
@@ -43,6 +44,7 @@ proc repmgr012_sub { method niter tnum largs } {
 
 	env_cleanup $testdir
 	set ports [available_ports $nsites]
+	set hoststr [get_hoststr $ipversion]
 
 	set masterdir $testdir/MASTERDIR
 	set clientdir $testdir/CLIENTDIR
@@ -50,13 +52,16 @@ proc repmgr012_sub { method niter tnum largs } {
 	file mkdir $masterdir
 	file mkdir $clientdir
 
+	setup_repmgr_ssl $masterdir
+	setup_repmgr_ssl $clientdir
+
 	# Open a master.
 	puts "\tRepmgr$tnum.a: Start a master."
 	set ma_envcmd "berkdb_env_noerr -create $verbargs \
 	    -errpfx MASTER -home $masterdir -txn -rep -thread"
 	set masterenv [eval $ma_envcmd]
 	$masterenv repmgr -ack all \
-	    -local [list 127.0.0.1 [lindex $ports 0]] \
+	    -local [list $hoststr [lindex $ports 0]] \
 	    -start master
 
 	# Open a client.
@@ -65,8 +70,8 @@ proc repmgr012_sub { method niter tnum largs } {
 	    -errpfx CLIENT -home $clientdir -txn -rep -thread"
 	set clientenv [eval $cl_envcmd]
 	$clientenv repmgr -ack all \
-	    -local [list 127.0.0.1 [lindex $ports 1]] \
-	    -remote [list 127.0.0.1 [lindex $ports 0]] \
+	    -local [list $hoststr [lindex $ports 1]] \
+	    -remote [list $hoststr [lindex $ports 0]] \
 	    -start client
 	await_startup_done $clientenv
 
@@ -83,9 +88,9 @@ proc repmgr012_sub { method niter tnum largs } {
 	# Timeouts are in microseconds, heartbeat monitor should be
 	# longer than heartbeat_send.
 	# If we have a machine that's really slow, like our AIX hosts
-	# are, double the times. 
+	# and QNX hosts are, double the times. 
 	puts "\tRepmgr$tnum.e: Set heartbeat timeouts."
-	if { $is_aix_test } {
+	if { $is_aix_test || $is_qnx_test } {
 		$masterenv repmgr -timeout {heartbeat_send 200000}
 		$clientenv repmgr -timeout {heartbeat_monitor 360000}
 	} else {
