@@ -1,6 +1,6 @@
-# See the file LICENSE for redistribution information.
+# Copyright (c) 2004, 2018 Oracle and/or its affiliates.  All rights reserved.
 #
-# Copyright (c) 2004, 2013 Oracle and/or its affiliates.  All rights reserved.
+# See the file LICENSE for license information.
 #
 # $Id$
 #
@@ -412,6 +412,10 @@ proc test_sequence_args_combine { tnum method env txnenv sargs filetype\
 		error_check_good err_txn_commit [$err_txn_id commit] 0
 		error_check_good err_env_close [$err_env close] 0
 	}
+
+	if { $is_qnx_test } {
+		fileremove -f $err_env_home
+	}
 }
 
 proc test_with_db_args { err_txn_id sargs env omethod testfile rdflag\
